@@ -210,16 +210,6 @@ nsLDAPConnection::Close()
   mInitListener = 0;
 
 }
-/** Get list of pending operation and store pointers to array
-  * \param userArg pointer to nsTArray<nsILDAPOperation*>
-  */
-PLDHashOperator
-GetListOfPendingOperations(const uint32_t &key, nsILDAPOperation *op, void *userArg)
-{
-  nsTArray<nsILDAPOperation*>* pending_operations = static_cast<nsTArray<nsILDAPOperation*>* >(userArg);
-  pending_operations->AppendElement(op);
-  return PL_DHASH_NEXT;
-}
 
 NS_IMETHODIMP
 nsLDAPConnection::Observe(nsISupports *aSubject, const char *aTopic,
@@ -235,7 +225,9 @@ nsLDAPConnection::Observe(nsISupports *aSubject, const char *aTopic,
     nsTArray<nsILDAPOperation*> pending_operations;
     {
       MutexAutoLock lock(mPendingOperationsMutex);
-      mPendingOperations.EnumerateRead(GetListOfPendingOperations, (void *) (&pending_operations));
+      for (auto iter = mPendingOperations.Iter(); !iter.Done(); iter.Next()) {
+        pending_operations.AppendElement(iter.UserData());
+      }
     }
     for (uint32_t i = 0; i < pending_operations.Length(); i++) {
       pending_operations[i]->AbandonExt();
@@ -385,7 +377,7 @@ nsLDAPConnection::RemovePendingOperation(uint32_t aOperationID)
   return NS_OK;
 }
 
-class nsOnLDAPMessageRunnable : public nsRunnable
+class nsOnLDAPMessageRunnable : public Runnable
 {
 public:
   nsOnLDAPMessageRunnable(nsLDAPMessage *aMsg, bool aClear)
@@ -394,7 +386,7 @@ public:
   {}
   NS_DECL_NSIRUNNABLE
 private:
-  nsRefPtr<nsLDAPMessage> m_msg;
+  RefPtr<nsLDAPMessage> m_msg;
   bool m_clear;
 };
 
@@ -445,7 +437,7 @@ nsLDAPConnection::InvokeMessageCallback(LDAPMessage *aMsgHandle,
   msg->mOperation = operation;
 
   // proxy the listener callback to the ui thread.
-  nsRefPtr<nsOnLDAPMessageRunnable> runnable =
+  RefPtr<nsOnLDAPMessageRunnable> runnable =
     new nsOnLDAPMessageRunnable(msg, aRemoveOpFromConnQ);
   // invoke the callback
   NS_DispatchToMainThread(runnable);
@@ -618,10 +610,7 @@ nsLDAPConnectionRunnable::nsLDAPConnectionRunnable(int32_t aOperationID,
 nsLDAPConnectionRunnable::~nsLDAPConnectionRunnable()
 {
   if (mConnection) {
-    nsCOMPtr<nsIThread> thread = do_GetMainThread();
-    nsILDAPConnection* forgettableConnection;
-    mConnection.forget(&forgettableConnection);
-    NS_ProxyRelease(thread, forgettableConnection, false);
+    NS_ReleaseOnMainThread(mConnection.forget());
   }
 }
 
@@ -636,7 +625,7 @@ NS_IMETHODIMP nsLDAPConnectionRunnable::Run()
 
   LDAPMessage *msgHandle;
   bool operationFinished = true;
-  nsRefPtr<nsLDAPMessage> msg;
+  RefPtr<nsLDAPMessage> msg;
 
   struct timeval timeout = { 0, 0 };
 
