@@ -16,7 +16,7 @@
 #include "plstr.h"
 #include "prmem.h"
 #include "nsIServiceManager.h"
-#include "nsIDOMWindow.h"
+#include "mozIDOMWindow.h"
 #include "nsIFilePicker.h"
 #include "plbase64.h"
 #include "nsIWindowWatcher.h"
@@ -73,18 +73,18 @@ const ExportAttributesTableStruct EXPORT_ATTRIBUTES_TABLE[] = {
   {kPriEmailProperty, 2104},
   {k2ndEmailProperty, 2105},
   {kScreenNameProperty, 2136},
-  {kPreferMailFormatProperty},
-  {kLastModifiedDateProperty},
+  {kPreferMailFormatProperty, 0},
+  {kLastModifiedDateProperty, 0},
   {kWorkPhoneProperty, 2106},
-  {kWorkPhoneTypeProperty},
+  {kWorkPhoneTypeProperty, 0},
   {kHomePhoneProperty, 2107},
-  {kHomePhoneTypeProperty},
+  {kHomePhoneTypeProperty, 0},
   {kFaxProperty, 2108},
-  {kFaxTypeProperty},
+  {kFaxTypeProperty, 0},
   {kPagerProperty, 2109},
-  {kPagerTypeProperty},
+  {kPagerTypeProperty, 0},
   {kCellularProperty, 2110},
-  {kCellularTypeProperty},
+  {kCellularTypeProperty, 0},
   {kHomeAddressProperty, 2111},
   {kHomeAddress2Property, 2112},
   {kHomeCityProperty, 2113},
@@ -110,11 +110,11 @@ const ExportAttributesTableStruct EXPORT_ATTRIBUTES_TABLE[] = {
   {kCustom3Property, 2133},
   {kCustom4Property, 2134},
   {kNotesProperty, 2135},
-  {kAnniversaryYearProperty},
-  {kAnniversaryMonthProperty},
-  {kAnniversaryDayProperty},
-  {kSpouseNameProperty},
-  {kFamilyNameProperty},
+  {kAnniversaryYearProperty, 0},
+  {kAnniversaryMonthProperty, 0},
+  {kAnniversaryDayProperty, 0},
+  {kSpouseNameProperty, 0},
+  {kFamilyNameProperty, 0},
 };
 
 //
@@ -522,13 +522,15 @@ NS_IMETHODIMP nsAbManager::MailListNameExists(const char16_t *name, bool *exist)
 
 enum ADDRESSBOOK_EXPORT_FILE_TYPE
 {
- LDIF_EXPORT_TYPE =  0,
- CSV_EXPORT_TYPE =   1,
- TAB_EXPORT_TYPE =   2,
- VCF_EXPORT_TYPE =   3,
+ CSV_EXPORT_TYPE      = 0,
+ CSV_EXPORT_TYPE_UTF8 = 1,
+ TAB_EXPORT_TYPE      = 2,
+ TAB_EXPORT_TYPE_UTF8 = 3,
+ VCF_EXPORT_TYPE      = 4,
+ LDIF_EXPORT_TYPE     = 5,
 };
 
-NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDirectory *aDirectory)
+NS_IMETHODIMP nsAbManager::ExportAddressBook(mozIDOMWindowProxy *aParentWin, nsIAbDirectory *aDirectory)
 {
   NS_ENSURE_ARG_POINTER(aParentWin);
 
@@ -543,38 +545,50 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
   rv = bundleService->CreateBundle("chrome://messenger/locale/addressbook/addressBook.properties", getter_AddRefs(bundle));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  nsString dirName;
+  aDirectory->GetDirName(dirName);
+  const char16_t *formatStrings[] = { dirName.get() };
+
   nsString title;
-  rv = bundle->GetStringFromName(MOZ_UTF16("ExportAddressBookTitle"), getter_Copies(title));
+  rv = bundle->FormatStringFromName(MOZ_UTF16("ExportAddressBookNameTitle"), formatStrings,
+                                    ArrayLength(formatStrings), getter_Copies(title));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = filePicker->Init(aParentWin, title, nsIFilePicker::modeSave);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsString filterString;
-  rv = bundle->GetStringFromName(MOZ_UTF16("LDIFFiles"), getter_Copies(filterString));
-  NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.ldi; *.ldif"));
+  // CSV: System charset and UTF-8.
+  rv = bundle->GetStringFromName(MOZ_UTF16("CSVFilesSysCharset"), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
-
-  rv = bundle->GetStringFromName(MOZ_UTF16("CSVFiles"), getter_Copies(filterString));
+  rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.csv"));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = bundle->GetStringFromName(MOZ_UTF16("CSVFilesUTF8"), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
-
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.csv"));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = bundle->GetStringFromName(MOZ_UTF16("TABFiles"), getter_Copies(filterString));
+  // Tab separated: System charset and UTF-8.
+  rv = bundle->GetStringFromName(MOZ_UTF16("TABFilesSysCharset"), getter_Copies(filterString));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.tab; *.txt"));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = bundle->GetStringFromName(MOZ_UTF16("TABFilesUTF8"), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
-
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.tab; *.txt"));
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = bundle->GetStringFromName(MOZ_UTF16("VCFFiles"), getter_Copies(filterString));
   NS_ENSURE_SUCCESS(rv, rv);
-
   rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.vcf"));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  rv = bundle->GetStringFromName(MOZ_UTF16("LDIFFiles"), getter_Copies(filterString));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = filePicker->AppendFilter(filterString, NS_LITERAL_STRING("*.ldi; *.ldif"));
+  NS_ENSURE_SUCCESS(rv, rv);
+
   int16_t dialogResult;
   filePicker->Show(&dialogResult);
 
@@ -621,6 +635,7 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
       break;
 
     case CSV_EXPORT_TYPE: // csv
+    case CSV_EXPORT_TYPE_UTF8:
       // If filename does not have the correct ext, add one.
       if (MsgFind(fileName, CSV_FILE_EXTENSION, true, fileName.Length() - strlen(CSV_FILE_EXTENSION)) == -1) {
 
@@ -628,10 +643,12 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
         fileName.AppendLiteral(CSV_FILE_EXTENSION);
         localFile->SetLeafName(fileName);
       }
-      rv = ExportDirectoryToDelimitedText(aDirectory, CSV_DELIM, CSV_DELIM_LEN, localFile);
+      rv = ExportDirectoryToDelimitedText(aDirectory, CSV_DELIM, CSV_DELIM_LEN, localFile,
+                                          exportType==CSV_EXPORT_TYPE_UTF8);
       break;
 
     case TAB_EXPORT_TYPE: // tab & text
+    case TAB_EXPORT_TYPE_UTF8:
       // If filename does not have the correct ext, add one.
       if ((MsgFind(fileName, TXT_FILE_EXTENSION, true, fileName.Length() - strlen(TXT_FILE_EXTENSION)) == -1) &&
           (MsgFind(fileName, TAB_FILE_EXTENSION, true, fileName.Length() - strlen(TAB_FILE_EXTENSION)) == -1)) {
@@ -640,7 +657,8 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
         fileName.AppendLiteral(TXT_FILE_EXTENSION);
         localFile->SetLeafName(fileName);
       }
-      rv = ExportDirectoryToDelimitedText(aDirectory, TAB_DELIM, TAB_DELIM_LEN, localFile);
+      rv = ExportDirectoryToDelimitedText(aDirectory, TAB_DELIM, TAB_DELIM_LEN, localFile,
+                                          exportType==TAB_EXPORT_TYPE_UTF8);
       break;
 
     case VCF_EXPORT_TYPE: // vCard
@@ -659,7 +677,11 @@ NS_IMETHODIMP nsAbManager::ExportAddressBook(nsIDOMWindow *aParentWin, nsIAbDire
 }
 
 nsresult
-nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const char *aDelim, uint32_t aDelimLen, nsIFile *aLocalFile)
+nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory,
+                                            const char *aDelim,
+                                            uint32_t aDelimLen,
+                                            nsIFile *aLocalFile,
+                                            bool useUTF8)
 {
   nsCOMPtr <nsISimpleEnumerator> cardsEnumerator;
   nsCOMPtr <nsIAbCard> card;
@@ -699,7 +721,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
       if (NS_FAILED(bundle->GetStringFromID(EXPORT_ATTRIBUTES_TABLE[i].plainTextStringID, getter_Copies(columnName))))
         columnName.AppendInt(EXPORT_ATTRIBUTES_TABLE[i].plainTextStringID);
 
-      rv = nsMsgI18NConvertFromUnicode(nsMsgI18NFileSystemCharset(),
+      rv = nsMsgI18NConvertFromUnicode(useUTF8 ? "UTF-8" : nsMsgI18NFileSystemCharset(),
                                        columnName, revisedName);
       NS_ENSURE_SUCCESS(rv,rv);
 
@@ -790,7 +812,7 @@ nsAbManager::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const ch
                 newValue.AppendLiteral("\"");
               }
 
-              rv = nsMsgI18NConvertFromUnicode(nsMsgI18NFileSystemCharset(),
+              rv = nsMsgI18NConvertFromUnicode(useUTF8 ? "UTF-8" : nsMsgI18NFileSystemCharset(),
                                                newValue, valueCStr);
               NS_ENSURE_SUCCESS(rv,rv);
 
@@ -1367,9 +1389,12 @@ nsAbManager::Handle(nsICommandLine* aCmdLine)
   nsCOMPtr<nsIWindowWatcher> wwatch (do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   NS_ENSURE_TRUE(wwatch, NS_ERROR_FAILURE);
 
-  nsCOMPtr<nsIDOMWindow> opened;
-  wwatch->OpenWindow(nullptr, "chrome://messenger/content/addressbook/addressbook.xul",
-                     "_blank", "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar", nullptr, getter_AddRefs(opened));
+  nsCOMPtr<mozIDOMWindowProxy> opened;
+  wwatch->OpenWindow(nullptr,
+                     "chrome://messenger/content/addressbook/addressbook.xul",
+                     "_blank",
+                     "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar",
+                     nullptr, getter_AddRefs(opened));
   aCmdLine->SetPreventDefault(true);
   return NS_OK;
 }
