@@ -14,7 +14,6 @@
 #include "nsIFile.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsQuickSort.h"
-#include "nsAutoPtr.h"
 #include "nsNativeCharsetUtils.h"
 #include "nsIMutableArray.h"
 #include "mozilla/Services.h"
@@ -151,7 +150,7 @@ public:
 
   nsCOMPtr<nsIFile> m_file;
   nsCOMPtr<nsIOutputStream> m_outputStream;
-  nsAutoPtr<char> m_dataBuffer;
+  char m_dataBuffer[FOUR_K];
   nsCOMPtr<nsIChannel> m_channel;
   nsCString m_templateUri;
   nsMessenger *m_messenger; // not ref counted
@@ -227,7 +226,7 @@ nsMessenger::~nsMessenger()
 
 NS_IMPL_ISUPPORTS(nsMessenger, nsIMessenger, nsISupportsWeakReference, nsIFolderListener)
 
-NS_IMETHODIMP nsMessenger::SetWindow(nsIDOMWindow *aWin, nsIMsgWindow *aMsgWindow)
+NS_IMETHODIMP nsMessenger::SetWindow(mozIDOMWindowProxy *aWin, nsIMsgWindow *aMsgWindow)
 {
   nsresult rv;
 
@@ -243,8 +242,8 @@ NS_IMETHODIMP nsMessenger::SetWindow(nsIDOMWindow *aWin, nsIMsgWindow *aMsgWindo
     rv = mailSession->AddFolderListener(this, nsIFolderListener::removed);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsPIDOMWindow> win( do_QueryInterface(aWin) );
-    NS_ENSURE_TRUE(win, NS_ERROR_FAILURE);
+    NS_ENSURE_TRUE(aWin, NS_ERROR_FAILURE);
+    nsCOMPtr<nsPIDOMWindowOuter> win = nsPIDOMWindowOuter::From(aWin);
 
     nsIDocShell *docShell = win->GetDocShell();
     nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(docShell));
@@ -438,7 +437,9 @@ nsMessenger::OpenURL(const nsACString& aURL)
 
   if (NS_SUCCEEDED(rv) && messageService)
   {
-    messageService->DisplayMessage(PromiseFlatCString(aURL).get(), mDocShell, mMsgWindow, nullptr, nullptr, nullptr);
+    nsCOMPtr<nsIURI> dummyNull;
+    messageService->DisplayMessage(PromiseFlatCString(aURL).get(), mDocShell,
+        mMsgWindow, nullptr, nullptr, getter_AddRefs(dummyNull));
     AddMsgUrlToNavigateHistory(aURL);
     mLastDisplayURI = aURL; // remember the last uri we displayed....
     return NS_OK;
@@ -469,7 +470,7 @@ NS_IMETHODIMP nsMessenger::LaunchExternalURL(const nsACString& aURL)
 }
 
 NS_IMETHODIMP
-nsMessenger::LoadURL(nsIDOMWindow *aWin, const nsACString& aURL)
+nsMessenger::LoadURL(mozIDOMWindowProxy *aWin, const nsACString& aURL)
 {
   nsresult rv;
 
@@ -563,6 +564,7 @@ nsMessenger::LoadURL(nsIDOMWindow *aWin, const nsACString& aURL)
   loadInfo->SetLoadType(nsIDocShellLoadInfo::loadNormal);
   AddMsgUrlToNavigateHistory(aURL);
   mNavigatingToUri.Truncate();
+  mLastDisplayURI = aURL; // Remember the last uri we displayed.
   return mDocShell->LoadURI(uri, loadInfo, 0, true);
 }
 
@@ -642,7 +644,7 @@ nsresult nsMessenger::SaveAttachment(nsIFile *aFile,
 
   // This instance will be held onto by the listeners, and will be released once 
   // the transfer has been completed.
-  nsRefPtr<nsSaveMsgListener> saveListener(new nsSaveMsgListener(aFile, this, aListener));
+  RefPtr<nsSaveMsgListener> saveListener(new nsSaveMsgListener(aFile, this, aListener));
   if (!saveListener)
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -712,10 +714,16 @@ nsresult nsMessenger::SaveAttachment(nsIFile *aFile,
                                                       getter_AddRefs(convertedListener));
       }
 #endif
+      nsCOMPtr<nsIURI> dummyNull;
       if (fetchService)
-        rv = fetchService->FetchMimePart(URL, fullMessageUri.get(), convertedListener, mMsgWindow, saveListener, nullptr);
+        rv = fetchService->FetchMimePart(URL, fullMessageUri.get(),
+                                         convertedListener, mMsgWindow,
+                                         saveListener, getter_AddRefs(dummyNull));
       else
-        rv = messageService->DisplayMessage(fullMessageUri.get(), convertedListener, mMsgWindow, nullptr, nullptr, nullptr);
+        rv = messageService->DisplayMessage(fullMessageUri.get(),
+                                            convertedListener, mMsgWindow,
+                                            nullptr, nullptr,
+                                            getter_AddRefs(dummyNull));
     } // if we got a message service
   } // if we created a url
 
@@ -1044,7 +1052,7 @@ nsMessenger::SaveAs(const nsACString& aURI, bool aAsFile,
 
     // After saveListener goes out of scope, the listener will be owned by
     // whoever the listener is registered with, usually a URL.
-    nsRefPtr<nsSaveMsgListener> saveListener = new nsSaveMsgListener(saveAsFile, this, nullptr);
+    RefPtr<nsSaveMsgListener> saveListener = new nsSaveMsgListener(saveAsFile, this, nullptr);
     if (!saveListener) {
       rv = NS_ERROR_OUT_OF_MEMORY;
       goto done;
@@ -1055,8 +1063,9 @@ nsMessenger::SaveAs(const nsACString& aURI, bool aAsFile,
 
     if (saveAsFileType == EML_FILE_TYPE)
     {
+      nsCOMPtr<nsIURI> dummyNull;
       rv = messageService->SaveMessageToDisk(PromiseFlatCString(aURI).get(), saveAsFile, false,
-        urlListener, nullptr,
+        urlListener, getter_AddRefs(dummyNull),
         true, mMsgWindow);
     }
     else
@@ -1120,8 +1129,9 @@ nsMessenger::SaveAs(const nsACString& aURI, bool aAsFile,
       if (NS_FAILED(rv))
         goto done;
 
+      nsCOMPtr<nsIURI> dummyNull;
       rv = messageService->DisplayMessage(urlString.get(), convertedListener, mMsgWindow,
-        nullptr, nullptr, nullptr);
+        nullptr, nullptr, getter_AddRefs(dummyNull));
     }
   }
   else
@@ -1160,9 +1170,10 @@ nsMessenger::SaveAs(const nsACString& aURI, bool aAsFile,
     if (NS_FAILED(rv))
       goto done;
 
+    nsCOMPtr<nsIURI> dummyNull;
     rv = messageService->SaveMessageToDisk(PromiseFlatCString(aURI).get(), tmpFile,
       needDummyHeader,
-      urlListener, nullptr,
+      urlListener, getter_AddRefs(dummyNull),
       canonicalLineEnding, mMsgWindow);
   }
 
@@ -1398,9 +1409,10 @@ nsMessenger::SaveMessages(uint32_t aCount,
     }
 
     // Ok, now save the message.
+    nsCOMPtr<nsIURI> dummyNull;
     rv = messageService->SaveMessageToDisk(aMessageUriArray[i],
                                            saveToFile, false,
-                                           urlListener, nullptr,
+                                           urlListener, getter_AddRefs(dummyNull),
                                            true, mMsgWindow);
     if (NS_FAILED(rv)) {
       NS_IF_RELEASE(saveListener);
@@ -1594,8 +1606,9 @@ NS_IMETHODIMP nsMessenger::SetDocumentCharset(const nsACString& aCharacterSet)
 
     if (NS_SUCCEEDED(rv) && messageService)
     {
+      nsCOMPtr<nsIURI> dummyNull;
       messageService->DisplayMessage(mLastDisplayURI.get(), mDocShell, mMsgWindow, nullptr,
-                                     PromiseFlatCString(aCharacterSet).get(), nullptr);
+                                     PromiseFlatCString(aCharacterSet).get(), getter_AddRefs(dummyNull));
     }
   }
 
@@ -1625,7 +1638,6 @@ nsSaveMsgListener::nsSaveMsgListener(nsIFile* aFile, nsMessenger *aMessenger, ns
   mCanceled = false;
   m_outputFormat = eUnknown;
   mInitialized = false;
-  m_dataBuffer = new char[FOUR_K];
 }
 
 nsSaveMsgListener::~nsSaveMsgListener()
@@ -1832,34 +1844,26 @@ nsSaveMsgListener::OnStopRequest(nsIRequest* request, nsISupports* aSupport,
   
   // rhp: If we are doing the charset conversion magic, this is different
   // processing, otherwise, its just business as usual.
-  //
-  if ( (m_doCharsetConversion) && (m_outputStream) )
+  // If we need text/plain, then we need to convert the HTML and then convert
+  // to the systems charset.
+  if (m_doCharsetConversion && m_outputStream)
   {
-    char        *conBuf = nullptr;
-    uint32_t    conLength = 0;
-    
-    // If we need text/plain, then we need to convert the HTML and then convert
-    // to the systems charset
-    //
-    if (m_outputFormat == ePlainText)
+    // For HTML, code is emitted immediately in OnDataAvailable.
+    MOZ_ASSERT(m_outputFormat == ePlainText,
+      "For HTML, m_doCharsetConversion shouldn't be set");
+    NS_ConvertUTF8toUTF16 utf16Buffer(m_msgBuffer);
+    ConvertBufToPlainText(utf16Buffer, false, false, false, false);
+
+    nsCString outCString;
+    rv = nsMsgI18NConvertFromUnicode(nsMsgI18NFileSystemCharset(),
+      utf16Buffer, outCString, false, true);
+    if (NS_SUCCEEDED(rv))
     {
-      NS_ConvertUTF8toUTF16 utf16Buffer(m_msgBuffer);
-      ConvertBufToPlainText(utf16Buffer, false, false);
-      rv = nsMsgI18NSaveAsCharset(TEXT_PLAIN, nsMsgI18NFileSystemCharset(),
-                                  utf16Buffer.get(), &conBuf);
-      if ( NS_SUCCEEDED(rv) && (conBuf) )
-        conLength = strlen(conBuf);
-    }
-    
-    if ( (NS_SUCCEEDED(rv)) && (conBuf) )
-    {
-      uint32_t      writeCount;
-      rv = m_outputStream->Write(conBuf, conLength, &writeCount);
-      if (conLength != writeCount)
+      uint32_t writeCount;
+      rv = m_outputStream->Write(outCString.get(), outCString.Length(), &writeCount);
+      if (outCString.Length() != writeCount)
         rv = NS_ERROR_FAILURE;
     }
-    
-    NS_Free(conBuf);
   }
  
   if (m_outputStream)
@@ -1961,7 +1965,7 @@ nsSaveMsgListener::OnDataAvailable(nsIRequest* request,
   if (!mInitialized)
     InitializeDownload(request);
 
-  if (m_dataBuffer && m_outputStream)
+  if (m_outputStream)
   {
     mProgress += count;
     uint64_t available;
@@ -2025,6 +2029,8 @@ nsMessenger::GetString(const nsString& aStringName, nsString& aValue)
 
   if (mStringBundle)
     rv = mStringBundle->GetStringFromName(aStringName.get(), getter_Copies(aValue));
+  else
+    rv = NS_ERROR_FAILURE;
 
   if (NS_FAILED(rv) || aValue.IsEmpty())
     aValue = aStringName;
