@@ -238,7 +238,18 @@ nsresult nsMsgGroupView::HashHdr(nsIMsgDBHdr *msgHdr, nsString& aHashKey)
     {
       nsIMsgCustomColumnHandler* colHandler = GetCurColumnHandler();
       if (colHandler)
-        rv = colHandler->GetSortStringForRow(msgHdr, aHashKey);
+      {
+        bool isString;
+        colHandler->IsString(&isString);
+        if (isString)
+          rv = colHandler->GetSortStringForRow(msgHdr, aHashKey);
+        else
+        {
+          uint32_t intKey;
+          rv = colHandler->GetSortLongForRow(msgHdr, &intKey);
+          aHashKey.AppendInt(intKey);
+        }
+      }
       break;
     }
     case nsMsgViewSortType::byCorrespondent:
@@ -460,15 +471,6 @@ NS_IMETHODIMP nsMsgGroupView::GetViewType(nsMsgViewTypeValue *aViewType)
     return NS_OK;
 }
 
-PLDHashOperator
-nsMsgGroupView::GroupTableCloner(const nsAString &aKey, nsIMsgThread* aGroupThread, void* aArg)
-{
-  nsMsgGroupView* view = static_cast<nsMsgGroupView*>(aArg);
-  view->m_groupsTable.Put(aKey, aGroupThread);
-  return PL_DHASH_NEXT;
-}
-
-
 NS_IMETHODIMP
 nsMsgGroupView::CopyDBView(nsMsgDBView *aNewMsgDBView, nsIMessenger *aMessengerInstance, 
                                        nsIMsgWindow *aMsgWindow, nsIMsgDBViewCommandUpdater *aCmdUpdater)
@@ -477,8 +479,11 @@ nsMsgGroupView::CopyDBView(nsMsgDBView *aNewMsgDBView, nsIMessenger *aMessengerI
   nsMsgGroupView* newMsgDBView = (nsMsgGroupView *) aNewMsgDBView;
 
   // If grouped, we need to clone the group thread hash table.
-  if (m_viewFlags & nsMsgViewFlagsType::kGroupBySort)
-    m_groupsTable.EnumerateRead(GroupTableCloner, newMsgDBView);
+  if (m_viewFlags & nsMsgViewFlagsType::kGroupBySort) {
+    for (auto iter = m_groupsTable.Iter(); !iter.Done(); iter.Next()) {
+      newMsgDBView->m_groupsTable.Put(iter.Key(), iter.UserData());
+    }
+  }
   return NS_OK;
 }
 
@@ -495,7 +500,7 @@ nsresult nsMsgGroupView::RebuildView(nsMsgViewFlagsTypeValue newFlags)
   {
     int32_t count;
     m_dayChanged = false;
-    nsAutoTArray<nsMsgKey, 1> preservedSelection;
+    AutoTArray<nsMsgKey, 1> preservedSelection;
     nsMsgKey curSelectedKey;
     SaveAndClearSelection(&curSelectedKey, preservedSelection);
     InternalClose();
@@ -519,7 +524,7 @@ nsresult nsMsgGroupView::RebuildView(nsMsgViewFlagsTypeValue newFlags)
     NS_ENSURE_SUCCESS(rv,rv);
 
     // now, restore our desired selection
-    nsAutoTArray<nsMsgKey, 1> keyArray;
+    AutoTArray<nsMsgKey, 1> keyArray;
     keyArray.AppendElement(curSelectedKey);
 
     return RestoreSelection(curSelectedKey, keyArray);
@@ -745,9 +750,13 @@ NS_IMETHODIMP nsMsgGroupView::GetCellProperties(int32_t aRow,
     rv = HashHdr(msgHdr, hashKey);
     if (NS_FAILED(rv))
       return NS_OK;
+
     nsCOMPtr<nsIMsgThread> msgThread;
     m_groupsTable.Get(hashKey, getter_AddRefs(msgThread));
     nsMsgGroupThread *groupThread = static_cast<nsMsgGroupThread *>(msgThread.get());
+    if (!groupThread)
+      return NS_OK;
+
     uint32_t numUnrMsg = 0;
     groupThread->GetNumUnreadChildren(&numUnrMsg);
     if (numUnrMsg > 0)
@@ -882,7 +891,18 @@ NS_IMETHODIMP nsMsgGroupView::CellTextForColumn(int32_t aRow,
         {
           nsIMsgCustomColumnHandler* colHandler = GetCurColumnHandler();
           if (colHandler)
-            rv = colHandler->GetSortStringForRow(msgHdr.get(), aValue);
+          {
+            bool isString;
+            colHandler->IsString(&isString);
+            if (isString)
+              rv = colHandler->GetSortStringForRow(msgHdr.get(), aValue);
+            else
+            {
+              uint32_t intKey;
+              rv = colHandler->GetSortLongForRow(msgHdr.get(), &intKey);
+              aValue.AppendInt(intKey);
+            }
+          }
           if (aValue.IsEmpty())
             aValue.AssignLiteral("*");
           break;
