@@ -14,6 +14,7 @@
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsPIDOMWindow.h"
 #include "nsIDocShell.h"
+#include "nsIDocument.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIObserverService.h"
@@ -277,7 +278,7 @@ nsresult nsMsgMailSession::GetTopmostMsgWindow(nsIMsgWindow **aMsgWindow)
     NS_ENSURE_SUCCESS(rv, rv);
 
     nsCOMPtr<nsISupports> windowSupports;
-    nsCOMPtr<nsPIDOMWindow> topMostWindow;
+    nsCOMPtr<nsPIDOMWindowOuter> topMostWindow;
     nsCOMPtr<nsIDOMDocument> domDocument;
     nsCOMPtr<nsIDOMElement> domElement;
     nsAutoString windowType;
@@ -295,7 +296,7 @@ nsresult nsMsgMailSession::GetTopmostMsgWindow(nsIMsgWindow **aMsgWindow)
       NS_ENSURE_SUCCESS(rv, rv);
       NS_ENSURE_TRUE(topMostWindow, NS_ERROR_FAILURE);
 
-      rv = topMostWindow->GetDocument(getter_AddRefs(domDocument));
+      domDocument = do_QueryInterface(topMostWindow->GetDoc());
       NS_ENSURE_SUCCESS(rv, rv);
       NS_ENSURE_TRUE(domDocument, NS_ERROR_FAILURE);
 
@@ -659,7 +660,7 @@ NS_IMETHODIMP nsMsgShutdownService::Observe(nsISupports *aSubject,
     nsCOMPtr<nsIMsgWindow> topMsgWindow;
     mailSession->GetTopmostMsgWindow(getter_AddRefs(topMsgWindow));
     
-    nsCOMPtr<nsIDOMWindow> internalDomWin;
+    nsCOMPtr<mozIDOMWindowProxy> internalDomWin;
     if (topMsgWindow)
       topMsgWindow->GetDomWindow(getter_AddRefs(internalDomWin));
     
