@@ -16,6 +16,7 @@
 #include "nsIURI.h"
 
 #include "nsPIDOMWindow.h"
+#include "mozIDOMWindow.h"
 #include "nsIContentViewer.h"
 #include "nsIMsgMessageService.h"
 #include "nsMsgUtils.h"
@@ -92,7 +93,7 @@ nsMsgPrintEngine::OnStateChange(nsIWebProgress* aWebProgress,
         // The mail msg doc is the last one to receive the STATE_STOP notification
         nsCOMPtr<nsISupports> container;
         docLoader->GetContainer(getter_AddRefs(container));
-        nsCOMPtr<nsIDOMWindow> domWindow(do_GetInterface(container));
+        nsCOMPtr<mozIDOMWindowProxy> domWindow(do_GetInterface(container));
         if (domWindow.get() != mMsgDOMWin.get()) {
           return NS_OK;
         }
@@ -160,7 +161,9 @@ nsMsgPrintEngine::OnStateChange(nsIWebProgress* aWebProgress,
       } 
       else 
       {
-        mWindow->Close();
+        if (mWindow) {
+          nsPIDOMWindowOuter::From(mWindow)->Close();
+        }
       }
     }
   }
@@ -210,24 +213,24 @@ nsMsgPrintEngine::OnSecurityChange(nsIWebProgress *aWebProgress,
 }
 
 NS_IMETHODIMP    
-nsMsgPrintEngine::SetWindow(nsIDOMWindow *aWin)
+nsMsgPrintEngine::SetWindow(mozIDOMWindowProxy *aWin)
 {
-	if (!aWin)
+  if (!aWin)
   {
     // It isn't an error to pass in null for aWin, in fact it means we are shutting
     // down and we should start cleaning things up...
-		return NS_OK;
+    return NS_OK;
   }
 
   mWindow = aWin;
 
-  nsCOMPtr<nsPIDOMWindow> win( do_QueryInterface(aWin) );
-  NS_ENSURE_TRUE(win, NS_ERROR_FAILURE);
+  NS_ENSURE_TRUE(mWindow, NS_ERROR_FAILURE);
+  nsCOMPtr<nsPIDOMWindowOuter> window = nsPIDOMWindowOuter::From(mWindow);
 
-  win->GetDocShell()->SetAppType(nsIDocShell::APP_TYPE_MAIL);
+  window->GetDocShell()->SetAppType(nsIDocShell::APP_TYPE_MAIL);
 
   nsCOMPtr<nsIDocShellTreeItem> docShellAsItem =
-    do_QueryInterface(win->GetDocShell());
+    do_QueryInterface(window->GetDocShell());
   NS_ENSURE_TRUE(docShellAsItem, NS_ERROR_FAILURE);
 
   nsCOMPtr<nsIDocShellTreeItem> rootAsItem;
@@ -246,8 +249,8 @@ nsMsgPrintEngine::SetWindow(nsIDOMWindow *aWin)
   return NS_OK;
 }
 
-/* void setParentWindow (in nsIDOMWindow ptr); */
-NS_IMETHODIMP nsMsgPrintEngine::SetParentWindow(nsIDOMWindow *ptr)
+/* void setParentWindow (in mozIDOMWindowProxy ptr); */
+NS_IMETHODIMP nsMsgPrintEngine::SetParentWindow(mozIDOMWindowProxy *ptr)
 {
   mParentWindow = ptr;
   return NS_OK;
@@ -261,12 +264,9 @@ nsMsgPrintEngine::ShowWindow(bool aShow)
 
   NS_ENSURE_TRUE(mWindow, NS_ERROR_NOT_INITIALIZED);
 
-  nsCOMPtr <nsPIDOMWindow> win = do_QueryInterface(mWindow, &rv);
-
-  NS_ENSURE_SUCCESS(rv,rv);
-
+  nsCOMPtr<nsPIDOMWindowOuter> window = nsPIDOMWindowOuter::From(mWindow);
   nsCOMPtr <nsIDocShellTreeItem> treeItem =
-    do_QueryInterface(win->GetDocShell(), &rv);
+    do_QueryInterface(window->GetDocShell(), &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 
   nsCOMPtr <nsIDocShellTreeOwner> treeOwner;
@@ -356,7 +356,7 @@ nsMsgPrintEngine::ShowProgressDialog(bool aIsForPrinting, bool& aDoNotify)
     }
     if (mPrintPromptService) 
     {
-      nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(mParentWindow));
+      nsCOMPtr<mozIDOMWindowProxy> domWin(do_QueryInterface(mParentWindow));
       if (!domWin) 
       {
         domWin = mWindow;
@@ -406,7 +406,8 @@ nsMsgPrintEngine::StartNextPrintOperation()
   if (mCurrentlyPrintingURI >= (int32_t)mURIArray.Length())
   {
     // This is the end...dum, dum, dum....my only friend...the end
-    mWindow->Close();
+    NS_ENSURE_TRUE(mWindow, NS_ERROR_FAILURE);
+    nsPIDOMWindowOuter::From(mWindow)->Close();
 
     // Tell the user we are done...
     nsString msg;
@@ -458,7 +459,7 @@ nsMsgPrintEngine::FireThatLoadOperationStartup(const nsString& uri)
 nsresult
 nsMsgPrintEngine::FireThatLoadOperation(const nsString& uri)
 {
-  nsresult rv;
+  nsresult rv = NS_ERROR_FAILURE;
   
   nsCString uriCStr;
   LossyCopyUTF16toASCII(uri, uriCStr);
@@ -485,8 +486,11 @@ nsMsgPrintEngine::FireThatLoadOperation(const nsString& uri)
     rv = GetMessageServiceFromURI(uriCStr, getter_AddRefs(messageService));
   }
 
-  if (NS_SUCCEEDED(rv) && messageService)
-    rv = messageService->DisplayMessageForPrinting(uriCStr.get(), mDocShell, nullptr, nullptr, nullptr);
+  if (NS_SUCCEEDED(rv) && messageService) {
+    nsCOMPtr<nsIURI> dummyNull;
+    rv = messageService->DisplayMessageForPrinting(uriCStr.get(), mDocShell, nullptr, nullptr,
+                                                   getter_AddRefs(dummyNull));
+  }
   //If it's not something we know about, then just load try loading it directly.
   else
   {
@@ -625,7 +629,9 @@ nsMsgPrintEngine::PrintMsgWindow()
         } 
         else 
         {
-          mWindow->Close();
+          if (mWindow) {
+            nsPIDOMWindowOuter::From(mWindow)->Close();
+          }
         }
       }
       else
@@ -644,7 +650,7 @@ nsMsgPrintEngine::PrintMsgWindow()
 //---------------------------------------------------------------
 
 //---------------------------------------------------------------
-class nsPrintMsgWindowEvent : public nsRunnable
+class nsPrintMsgWindowEvent : public mozilla::Runnable
 {
 public:
   nsPrintMsgWindowEvent(nsMsgPrintEngine *mpe)
@@ -659,11 +665,11 @@ public:
   }
 
 private:
-  nsRefPtr<nsMsgPrintEngine> mMsgPrintEngine;
+  RefPtr<nsMsgPrintEngine> mMsgPrintEngine;
 };
 
 //-----------------------------------------------------------
-class nsStartNextPrintOpEvent : public nsRunnable
+class nsStartNextPrintOpEvent : public mozilla::Runnable
 {
 public:
   nsStartNextPrintOpEvent(nsMsgPrintEngine *mpe)
@@ -678,7 +684,7 @@ public:
   }
 
 private:
-  nsRefPtr<nsMsgPrintEngine> mMsgPrintEngine;
+  RefPtr<nsMsgPrintEngine> mMsgPrintEngine;
 };
 
 //-----------------------------------------------------------
