@@ -15,7 +15,6 @@
 #include "nsMsgUtils.h"
 #include "nsITreeColumns.h"
 #include "nsIMsgMessageService.h"
-#include "nsAutoPtr.h"
 #include "nsArrayUtils.h"
 #include "nsIMutableArray.h"
 #include "nsMsgGroupThread.h"
@@ -75,23 +74,6 @@ NS_IMETHODIMP nsMsgSearchDBView::Open(nsIMsgFolder *folder,
     return rv;
 }
 
-
-PLDHashOperator
-nsMsgSearchDBView::ThreadTableCloner(const nsACString &aKey, nsIMsgThread* aThread, void* aArg)
-{
-  nsMsgSearchDBView* view = static_cast<nsMsgSearchDBView*>(aArg);
-  view->m_threadsTable.Put(aKey, aThread);
-  return PL_DHASH_NEXT;
-}
-
-PLDHashOperator
-nsMsgSearchDBView::MsgHdrTableCloner(const nsACString &aKey, nsIMsgDBHdr* aMsgHdr, void* aArg)
-{
-  nsMsgSearchDBView* view = static_cast<nsMsgSearchDBView*>(aArg);
-  view->m_hdrsTable.Put(aKey, aMsgHdr);
-  return PL_DHASH_NEXT;
-}
-
 NS_IMETHODIMP
 nsMsgSearchDBView::CloneDBView(nsIMessenger *aMessengerInstance, nsIMsgWindow *aMsgWindow, nsIMsgDBViewCommandUpdater *aCmdUpdater, nsIMsgDBView **_retval)
 {
@@ -134,8 +116,12 @@ nsMsgSearchDBView::CopyDBView(nsMsgDBView *aNewMsgDBView, nsIMessenger *aMesseng
   if (m_viewFlags & nsMsgViewFlagsType::kThreadedDisplay)
   {
     // We need to clone the thread and msg hdr hash tables.
-    m_threadsTable.EnumerateRead(ThreadTableCloner, newMsgDBView);
-    m_hdrsTable.EnumerateRead(MsgHdrTableCloner, newMsgDBView);
+    for (auto iter = m_threadsTable.Iter(); !iter.Done(); iter.Next()) {
+      newMsgDBView->m_threadsTable.Put(iter.Key(), iter.UserData());
+    }
+    for (auto iter = m_hdrsTable.Iter(); !iter.Done(); iter.Next()) {
+      newMsgDBView->m_hdrsTable.Put(iter.Key(), iter.UserData());
+    }
   }
   return NS_OK;
 }
@@ -555,7 +541,7 @@ void nsMsgSearchDBView::MoveThreadAt(nsMsgViewIndex threadIndex)
   bool threadIsExpanded = !(saveFlags & nsMsgMessageFlags::Elided);
   int32_t childCount = 0;
   nsMsgKey preservedKey;
-  nsAutoTArray<nsMsgKey, 1> preservedSelection;
+  AutoTArray<nsMsgKey, 1> preservedSelection;
   int32_t selectionCount;
   int32_t currentIndex;
   bool hasSelection = mTree && mTreeSelection &&
@@ -792,13 +778,13 @@ NS_IMETHODIMP nsMsgSearchDBView::DoCommand(nsMsgViewCommandTypeValue command)
   // ApplyCommandToIndices with the command and the indices in the
   // selection that are from that folder.
 
-  nsAutoArrayPtr<nsTArray<uint32_t> > indexArrays;
+  mozilla::UniquePtr<nsTArray<uint32_t>[]> indexArrays;
   int32_t numArrays;
-  rv = PartitionSelectionByFolder(indices, numIndices, getter_Transfers(indexArrays), &numArrays);
+  rv = PartitionSelectionByFolder(indices, numIndices, indexArrays, &numArrays);
   NS_ENSURE_SUCCESS(rv, rv);
   for (int32_t folderIndex = 0; folderIndex < numArrays; folderIndex++)
   {
-    rv = ApplyCommandToIndices(command, indexArrays[folderIndex].Elements(), indexArrays[folderIndex].Length());
+    rv = ApplyCommandToIndices(command, (indexArrays.get())[folderIndex].Elements(), indexArrays[folderIndex].Length());
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -895,7 +881,10 @@ nsMsgSearchDBView::CopyMessages(nsIMsgWindow *window, nsMsgViewIndex *indices, i
 }
 
 nsresult
-nsMsgSearchDBView::PartitionSelectionByFolder(nsMsgViewIndex *indices, int32_t numIndices, nsTArray<uint32_t> **indexArrays, int32_t *numArrays)
+nsMsgSearchDBView::PartitionSelectionByFolder(nsMsgViewIndex *indices,
+                                              int32_t numIndices,
+                                              mozilla::UniquePtr<nsTArray<uint32_t>[]> &indexArrays,
+                                              int32_t *numArrays)
 {
   nsMsgViewIndex i;
   int32_t folderIndex;
@@ -920,18 +909,18 @@ nsMsgSearchDBView::PartitionSelectionByFolder(nsMsgViewIndex *indices, int32_t n
   }
 
   int32_t numFolders = uniqueFoldersSelected.Count();
-  *indexArrays = new nsTArray<uint32_t>[numFolders];
+  indexArrays = mozilla::MakeUnique<nsTArray<uint32_t>[]>(numFolders);
   *numArrays = numFolders;
-  NS_ENSURE_TRUE(*indexArrays, NS_ERROR_OUT_OF_MEMORY);
+  NS_ENSURE_TRUE(indexArrays, NS_ERROR_OUT_OF_MEMORY);
   for (folderIndex = 0; folderIndex < numFolders; folderIndex++)
   {
-    (*indexArrays)[folderIndex].SetCapacity(numIndicesSelected[folderIndex]);
+    (indexArrays.get())[folderIndex].SetCapacity(numIndicesSelected[folderIndex]);
   }
   for (i = 0; i < (nsMsgViewIndex) numIndices; i++) 
   {
     nsIMsgFolder *curFolder = m_folders[indices[i]];
     int32_t folderIndex = uniqueFoldersSelected.IndexOf(curFolder);
-    (*indexArrays)[folderIndex].AppendElement(indices[i]);
+    (indexArrays.get())[folderIndex].AppendElement(indices[i]);
   }
   return NS_OK;
 }
@@ -1115,7 +1104,7 @@ NS_IMETHODIMP nsMsgSearchDBView::Sort(nsMsgViewSortTypeValue sortType, nsMsgView
     }
 
     nsMsgKey preservedKey;
-    nsAutoTArray<nsMsgKey, 1> preservedSelection;
+    AutoTArray<nsMsgKey, 1> preservedSelection;
     SaveAndClearSelection(&preservedKey, preservedSelection);
 
     nsresult rv = nsMsgDBView::Sort(sortType,sortOrder);
