@@ -1445,7 +1445,6 @@ NS_IMETHODIMP nsMsgIncomingServer::GetRetentionSettings(nsIMsgRetentionSettings
   nsMsgRetainByPreference retainByPreference;
   int32_t daysToKeepHdrs = 0;
   int32_t numHeadersToKeep = 0;
-  bool keepUnreadMessagesOnly = false;
   int32_t daysToKeepBodies = 0;
   bool cleanupBodiesByDays = false;
   bool applyToFlaggedMessages = false;
@@ -1456,8 +1455,6 @@ NS_IMETHODIMP nsMsgIncomingServer::GetRetentionSettings(nsIMsgRetentionSettings
      do_CreateInstance(NS_MSG_RETENTIONSETTINGS_CONTRACTID);
   if (retentionSettings)
   {
-    rv = GetBoolValue("keepUnreadOnly", &keepUnreadMessagesOnly);
-    NS_ENSURE_SUCCESS(rv, rv);
     rv = GetIntValue("retainBy", (int32_t*) &retainByPreference);
     NS_ENSURE_SUCCESS(rv, rv);
     rv = GetIntValue("numHdrsToKeep", &numHeadersToKeep);
@@ -1472,7 +1469,6 @@ NS_IMETHODIMP nsMsgIncomingServer::GetRetentionSettings(nsIMsgRetentionSettings
     NS_ENSURE_SUCCESS(rv, rv);
     retentionSettings->SetRetainByPreference(retainByPreference);
     retentionSettings->SetNumHeadersToKeep((uint32_t) numHeadersToKeep);
-    retentionSettings->SetKeepUnreadMessagesOnly(keepUnreadMessagesOnly);
     retentionSettings->SetDaysToKeepBodies(daysToKeepBodies);
     retentionSettings->SetDaysToKeepHdrs(daysToKeepHdrs);
     retentionSettings->SetCleanupBodiesByDays(cleanupBodiesByDays);
@@ -1489,20 +1485,16 @@ NS_IMETHODIMP nsMsgIncomingServer::SetRetentionSettings(nsIMsgRetentionSettings
   nsMsgRetainByPreference retainByPreference;
   uint32_t daysToKeepHdrs = 0;
   uint32_t numHeadersToKeep = 0;
-  bool keepUnreadMessagesOnly = false;
   uint32_t daysToKeepBodies = 0;
   bool cleanupBodiesByDays = false;
   bool applyToFlaggedMessages = false;
   settings->GetRetainByPreference(&retainByPreference);
   settings->GetNumHeadersToKeep(&numHeadersToKeep);
-  settings->GetKeepUnreadMessagesOnly(&keepUnreadMessagesOnly);
   settings->GetDaysToKeepBodies(&daysToKeepBodies);
   settings->GetDaysToKeepHdrs(&daysToKeepHdrs);
   settings->GetCleanupBodiesByDays(&cleanupBodiesByDays);
   settings->GetApplyToFlaggedMessages(&applyToFlaggedMessages);
-  nsresult rv = SetBoolValue("keepUnreadOnly", keepUnreadMessagesOnly);
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = SetIntValue("retainBy", retainByPreference);
+  nsresult rv = SetIntValue("retainBy", retainByPreference);
   NS_ENSURE_SUCCESS(rv, rv);
   rv = SetIntValue("numHdrsToKeep", numHeadersToKeep);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -2197,16 +2189,6 @@ NS_IMETHODIMP nsMsgIncomingServer::GetIsDeferredTo(bool *aIsDeferredTo)
 
 const long kMaxDownloadTableSize = 500;
 
-// aClosure is the server, from that we get the cutoff point, below which we evict
-// aData is the arrival index of the msg.
-/* static */PLDHashOperator nsMsgIncomingServer::evictOldEntries(nsCStringHashKey::KeyType aKey, int32_t &aData, void *aClosure)
-{
-  nsMsgIncomingServer *server = (nsMsgIncomingServer *) aClosure;
-  if (aData < server->m_numMsgsDownloaded - kMaxDownloadTableSize/2)
-    return PL_DHASH_REMOVE;
-  return server->m_downloadedHdrs.Count() > kMaxDownloadTableSize/2 ? PL_DHASH_NEXT : PL_DHASH_STOP;
-}
-
 // hash the concatenation of the message-id and subject as the hash table key,
 // and store the arrival index as the value. To limit the size of the hash table,
 // we just throw out ones with a lower ordinal value than the cut-off point.
@@ -2244,8 +2226,15 @@ NS_IMETHODIMP nsMsgIncomingServer::IsNewHdrDuplicate(nsIMsgDBHdr *aNewHdr, bool
     // Check if hash table is larger than some reasonable size
     // and if is it, iterate over hash table deleting messages
     // with an arrival index < number of msgs downloaded - half the reasonable size.
-    if (m_downloadedHdrs.Count() >= kMaxDownloadTableSize)
-      m_downloadedHdrs.Enumerate(evictOldEntries, this);
+    if (m_downloadedHdrs.Count() >= kMaxDownloadTableSize) {
+      for (auto iter = m_downloadedHdrs.Iter(); !iter.Done(); iter.Next()) {
+        if (iter.Data() < m_numMsgsDownloaded - kMaxDownloadTableSize/2) {
+          iter.Remove();
+        } else if (m_downloadedHdrs.Count() <= kMaxDownloadTableSize/2) {
+          break;
+        }
+      }
+    }
   }
   return NS_OK;
 }
