@@ -305,11 +305,15 @@ NS_MSG_BASE nsresult MsgDetectCharsetFromFile(nsIFile *aFile, nsACString &aChars
  * or may not work with certain end charsets which is why we
  * need that as an argument to the function. If charset is
  * unknown or deemed of no importance NULL could be passed.
- * @param[in/out] aConBuf      Variable with the text to convert
- * @param         formatFlowed Use format flowed?
- * @param         formatOutput Reformat the output?
+ * @param[in/out] aConBuf        Variable with the text to convert
+ * @param         formatFlowed   Use format flowed?
+ * @param         delsp          Use delsp=yes when flowed
+ * @param         formatOutput   Reformat the output?
+ & @param         disallowBreaks Disallow breaks when formatting
  */
-NS_MSG_BASE nsresult ConvertBufToPlainText(nsString &aConBuf, bool formatFlowed, bool formatOutput);
+NS_MSG_BASE nsresult
+ConvertBufToPlainText(nsString &aConBuf, bool formatFlowed, bool delsp,
+                                         bool formatOutput, bool disallowBreaks);
 
 /**
  * The following definitons exist for compatibility between the internal and
@@ -347,9 +351,9 @@ NS_MSG_BASE nsresult ConvertBufToPlainText(nsString &aConBuf, bool formatFlowed,
 #define MsgNewNotificationCallbacksAggregation(aCallbacks, aLoadGroup, aResult) \
         NS_NewNotificationCallbacksAggregation(aCallbacks, aLoadGroup, aResult)
 #define MsgGetAtom(aString) \
-        do_GetAtom(aString)
+        NS_Atomize(aString)
 #define MsgNewAtom(aString) \
-        NS_NewAtom(aString)
+        NS_Atomize(aString)
 #define MsgReplaceChar(aString, aNeedle, aReplacement) \
         (aString).ReplaceChar(aNeedle, aReplacement)
 #define MsgFind(str, what, ignore_case, offset) \
@@ -443,11 +447,11 @@ NS_MSG_BASE char *MsgEscapeHTML(const char *aString);
 NS_MSG_BASE char16_t *MsgEscapeHTML2(const char16_t *aBuffer, int32_t aLen);
 // Existing replacement for IsUTF8
 NS_MSG_BASE bool MsgIsUTF8(const nsACString& aString);
-/// Equivalent of NS_NewAtom(aUTF8String)
+/// Equivalent of NS_Atomize(aUTF8String)
 NS_MSG_BASE already_AddRefed<nsIAtom> MsgNewAtom(const char* aString);
 /// Replacement of NS_RegisterStaticAtoms
 NS_MSG_BASE nsIAtom* MsgNewPermanentAtom(const char* aString);
-/// Equivalent of do_GetAtom(aUTF8String)
+/// Equivalent of NS_Atomize(aUTF8String)
 inline already_AddRefed<nsIAtom> MsgGetAtom(const char* aUTF8String)
 {
   return MsgNewAtom(aUTF8String);
