@@ -16,7 +16,7 @@
 #include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIWindowWatcher.h"
-#include "nsIDOMWindow.h"
+#include "mozIDOMWindow.h"
 #include "nsIContentViewer.h"
 #include "nsIMsgWindow.h"
 #include "nsIDocShell.h"
@@ -49,6 +49,8 @@
 #include "nsLWBrkCIID.h"
 #include "mozilla/Services.h"
 #include "mimemoz2.h"
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
 
 #ifdef MSGCOMP_TRACE_PERFORMANCE
 #include "mozilla/Logging.h"
@@ -70,8 +72,6 @@
 
 #define DEFAULT_CHROME  "chrome://messenger/content/messengercompose/messengercompose.xul"
 
-#define PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS  "mail.compose.max_recycled_windows"
-
 #define PREF_MAILNEWS_REPLY_QUOTING_SELECTION            "mailnews.reply_quoting_selection"
 #define PREF_MAILNEWS_REPLY_QUOTING_SELECTION_MULTI_WORD "mailnews.reply_quoting_selection.multi_word"
 #define PREF_MAILNEWS_REPLY_QUOTING_SELECTION_ONLY_IF    "mailnews.reply_quoting_selection.only_if_chars"
@@ -116,45 +116,21 @@ nsMsgComposeService::nsMsgComposeService()
   mPreviousTime = mStartTime;
 #endif
 
-  mMaxRecycledWindows = 0;
-  mCachedWindows = nullptr;
 }
 
 NS_IMPL_ISUPPORTS(nsMsgComposeService,
                    nsIMsgComposeService,
-                   nsIObserver,
                    ICOMMANDLINEHANDLER,
                    nsISupportsWeakReference)
 
 nsMsgComposeService::~nsMsgComposeService()
 {
-  if (mCachedWindows)
-  {
-    DeleteCachedWindows();
-    delete [] mCachedWindows;
-  }
-
   mOpenComposeWindows.Clear();
 }
 
 nsresult nsMsgComposeService::Init()
 {
   nsresult rv = NS_OK;
-  // Register observers
-
-  // Register for quit application and profile change, we will need to clear the cache.
-  nsCOMPtr<nsIObserverService> observerService =
-    mozilla::services::GetObserverService();
-  if (observerService)
-  {
-    rv = observerService->AddObserver(this, "quit-application", true);
-    rv = observerService->AddObserver(this, "profile-do-change", true);
-  }
-
-  // Register some pref observer
-  nsCOMPtr<nsIPrefBranch> pbi = do_GetService(NS_PREFSERVICE_CONTRACTID);
-  if (pbi)
-    rv = pbi->AddObserver(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS, this, true);
 
   Reset();
 
@@ -169,41 +145,11 @@ nsresult nsMsgComposeService::Init()
 
 void nsMsgComposeService::Reset()
 {
-  nsresult rv = NS_OK;
-
-  if (mCachedWindows)
-  {
-    DeleteCachedWindows();
-    delete [] mCachedWindows;
-    mCachedWindows = nullptr;
-    mMaxRecycledWindows = 0;
-  }
-
   mOpenComposeWindows.Clear();
 
   nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
-  if(prefs)
-    rv = prefs->GetIntPref(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS, &mMaxRecycledWindows);
-  if (NS_SUCCEEDED(rv) && mMaxRecycledWindows > 0)
-  {
-    mCachedWindows = new nsMsgCachedWindowInfo[mMaxRecycledWindows];
-    if (!mCachedWindows)
-      mMaxRecycledWindows = 0;
-  }
-
-  rv = prefs->GetBoolPref("mailnews.logComposePerformance", &mLogComposePerformance);
-
-  return;
-}
-
-void nsMsgComposeService::DeleteCachedWindows()
-{
-  int32_t i;
-  for (i = 0; i < mMaxRecycledWindows; i ++)
-  {
-    CloseHiddenCachedWindow(mCachedWindows[i].window);
-    mCachedWindows[i].Clear();
-  }
+  if (prefs)
+    prefs->GetBoolPref("mailnews.logComposePerformance", &mLogComposePerformance);
 }
 
 // Function to open a message compose window and pass an nsIMsgComposeParams
@@ -233,38 +179,7 @@ nsMsgComposeService::OpenComposeWindowWithParams(const char *chrome,
     params->SetIdentity(identity);
   }
 
-  //if we have a cached window for the default chrome, try to reuse it...
-  if (!chrome || PL_strcasecmp(chrome, DEFAULT_CHROME) == 0)
-  {
-    MSG_ComposeFormat format;
-    params->GetFormat(&format);
-
-    bool composeHTML = true;
-    rv = DetermineComposeHTML(identity, format, &composeHTML);
-    if (NS_SUCCEEDED(rv))
-    {
-      int32_t i;
-      for (i = 0; i < mMaxRecycledWindows; i ++)
-      {
-        if (mCachedWindows[i].window && (mCachedWindows[i].htmlCompose == composeHTML) && mCachedWindows[i].listener)
-        {
-          /* We need to save the window pointer as OnReopen will call nsMsgComposeService::InitCompose which will
-             clear the cache entry if everything goes well
-          */
-          nsCOMPtr<nsIDOMWindow> domWindow(mCachedWindows[i].window);
-          nsCOMPtr<nsIXULWindow> xulWindow(mCachedWindows[i].xulWindow);
-          rv = ShowCachedComposeWindow(domWindow, xulWindow, true);
-          if (NS_SUCCEEDED(rv))
-          {
-            mCachedWindows[i].listener->OnReopen(params);
-            return NS_OK;
-          }
-        }
-      }
-    }
-  }
-
-  //Else, create a new one...
+  // Create a new window.
   nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
   if (!wwatch)
     return NS_ERROR_FAILURE;
@@ -276,7 +191,7 @@ nsMsgComposeService::OpenComposeWindowWithParams(const char *chrome,
   msgParamsWrapper->SetData(params);
   msgParamsWrapper->SetDataIID(&NS_GET_IID(nsIMsgComposeParams));
 
-  nsCOMPtr<nsIDOMWindow> newWindow;
+  nsCOMPtr<mozIDOMWindowProxy> newWindow;
   rv = wwatch->OpenWindow(0, chrome && *chrome ? chrome : DEFAULT_CHROME,
                  "_blank", "all,chrome,dialog=no,status,toolbar", msgParamsWrapper,
                  getter_AddRefs(newWindow));
@@ -284,63 +199,6 @@ nsMsgComposeService::OpenComposeWindowWithParams(const char *chrome,
   return rv;
 }
 
-void nsMsgComposeService::CloseHiddenCachedWindow(nsIDOMWindow *domWindow)
-{
-  if (domWindow)
-  {
-    nsCOMPtr<nsIDocShell> docshell;
-    nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(domWindow));
-    if (window)
-    {
-      nsCOMPtr<nsIDocShellTreeItem> treeItem =
-        do_QueryInterface(window->GetDocShell());
-
-      if (treeItem)
-      {
-        nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
-        treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
-        if (treeOwner)
-        {
-          nsCOMPtr<nsIBaseWindow> baseWindow;
-          baseWindow = do_QueryInterface(treeOwner);
-          if (baseWindow) {
-            // HACK ALERT: when we hid this window we fired the "xul-window-destroyed"
-            // notification for it. Now that it's being really-destroyed it will fire that
-            // notification *again* for itself. The appstartup code maintains an internal
-            // reference count of windows that block app shutdown: we want to increment that
-            // count without cancelling app shutdown (so don't use "xul-window-registered").
-            nsCOMPtr<nsIAppStartup> appStartup(do_GetService(NS_APPSTARTUP_CONTRACTID));
-            if (appStartup)
-              appStartup->EnterLastWindowClosingSurvivalArea();
-
-            baseWindow->Destroy();
-          }
-        }
-      }
-    }
-  }
-}
-
-NS_IMETHODIMP
-nsMsgComposeService::Observe(nsISupports *aSubject, const char *aTopic, const char16_t *someData)
-{
-  if (!strcmp(aTopic, "profile-do-change") || !strcmp(aTopic, "quit-application"))
-  {
-    DeleteCachedWindows();
-    return NS_OK;
-  }
-
-  if (!strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID))
-  {
-    nsDependentString prefName(someData);
-    if (prefName.EqualsLiteral(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS))
-      Reset();
-    return NS_OK;
-  }
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsMsgComposeService::DetermineComposeHTML(nsIMsgIdentity *aIdentity, MSG_ComposeFormat aFormat, bool *aComposeHTML)
 {
@@ -418,11 +276,11 @@ nsMsgComposeService::GetOrigWindowSelection(MSG_ComposeType type, nsIMsgWindow *
   nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(childAsItem, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDOMWindow> domWindow(do_GetInterface(childAsItem, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-  nsCOMPtr<nsISelection> sel;
-  rv = domWindow->GetSelection(getter_AddRefs(sel));
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<mozIDOMWindowProxy> domWindow(do_GetInterface(childAsItem));
+  NS_ENSURE_TRUE(domWindow, NS_ERROR_FAILURE);
+  nsCOMPtr<nsPIDOMWindowOuter> privateWindow = nsPIDOMWindowOuter::From(domWindow);
+  nsCOMPtr<nsISelection> sel = privateWindow->GetSelection();
+  NS_ENSURE_TRUE(sel, NS_ERROR_FAILURE);
 
   bool requireMultipleWords = true;
   nsAutoCString charsOnlyIf;
@@ -516,7 +374,6 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, nsIMsgDB
     if (type == nsIMsgCompType::Redirect)
       uriToOpen.Append("&redirect=true");
 
-    aMsgWindow->SetCharsetOverride(true);
     return LoadDraftOrTemplate(uriToOpen, type == nsIMsgCompType::ForwardInline || type == nsIMsgCompType::Draft ?
                                nsMimeOutput::nsMimeMessageDraftOrTemplate : nsMimeOutput::nsMimeMessageEditorTemplate,
                                identity, originalMsgURI, origMsgHdr, type == nsIMsgCompType::ForwardInline,
@@ -709,19 +566,10 @@ NS_IMETHODIMP nsMsgComposeService::OpenComposeWindowWithURI(const char * aMsgCom
 }
 
 NS_IMETHODIMP nsMsgComposeService::InitCompose(nsIMsgComposeParams *aParams,
-                                               nsIDOMWindow *aWindow,
+                                               mozIDOMWindowProxy *aWindow,
                                                nsIDocShell *aDocShell,
                                                nsIMsgCompose **_retval)
 {
-  // We need to remove the window from the cache.
-  int32_t i;
-  for (i = 0; i < mMaxRecycledWindows; i ++)
-    if (mCachedWindows[i].window == aWindow)
-    {
-      mCachedWindows[i].Clear();
-      break;
-    }
-
   nsresult rv;
   nsCOMPtr<nsIMsgCompose> msgCompose =
     do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv);
@@ -789,90 +637,6 @@ NS_IMETHODIMP nsMsgComposeService::TimeStamp(const char * label, bool resetTime)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMsgComposeService::IsCachedWindow(nsIDOMWindow *aCachedWindow, bool *aIsCachedWindow)
-{
-  NS_ENSURE_ARG_POINTER(aCachedWindow);
-  NS_ENSURE_ARG_POINTER(aIsCachedWindow);
-
-  int32_t i;
-  for (i = 0; i < mMaxRecycledWindows; i ++)
-    if (mCachedWindows[i].window.get() == aCachedWindow)
-    {
-      *aIsCachedWindow = true;
-      return NS_OK;
-    }
-
- *aIsCachedWindow = false;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsMsgComposeService::CacheWindow(nsIDOMWindow *aWindow, bool aComposeHTML, nsIMsgComposeRecyclingListener * aListener)
-{
-  NS_ENSURE_ARG_POINTER(aWindow);
-  NS_ENSURE_ARG_POINTER(aListener);
-
-  nsresult rv;
-  nsCOMPtr<nsPIDOMWindow> window(do_QueryInterface(aWindow, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIDocShellTreeItem> treeItem(do_QueryInterface(window->GetDocShell(), &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr <nsIDocShellTreeOwner> treeOwner;
-  rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsCOMPtr<nsIXULWindow> xulWindow(do_GetInterface(treeOwner, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  int32_t i;
-  int32_t sameTypeId = -1;
-  int32_t oppositeTypeId = -1;
-
-  for (i = 0; i < mMaxRecycledWindows; i ++)
-  {
-    if (!mCachedWindows[i].window)
-    {
-      rv = ShowCachedComposeWindow(aWindow, xulWindow, false);
-      if (NS_SUCCEEDED(rv))
-        mCachedWindows[i].Initialize(aWindow, xulWindow, aListener, aComposeHTML);
-
-      return rv;
-    }
-    else
-      if (mCachedWindows[i].htmlCompose == aComposeHTML)
-      {
-        if (sameTypeId == -1)
-          sameTypeId = i;
-      }
-      else
-      {
-        if (oppositeTypeId == -1)
-          oppositeTypeId = i;
-      }
-  }
-
-  /* Looks like the cache is full. In the case we try to cache a type (html or plain text) of compose window which is not
-     already cached, we should replace an opposite one with this new one. That would allow users to be able to take advantage
-     of the cached compose window when they switch from one type to another one
-  */
-  if (sameTypeId == -1 && oppositeTypeId != -1)
-  {
-    CloseHiddenCachedWindow(mCachedWindows[oppositeTypeId].window);
-    mCachedWindows[oppositeTypeId].Clear();
-
-    rv = ShowCachedComposeWindow(aWindow, xulWindow, false);
-    if (NS_SUCCEEDED(rv))
-      mCachedWindows[oppositeTypeId].Initialize(aWindow, xulWindow, aListener, aComposeHTML);
-
-    return rv;
-  }
-
-  return NS_ERROR_NOT_AVAILABLE;
-}
-
 class nsMsgTemplateReplyHelper final: public nsIStreamListener,
                                       public nsIUrlListener
 {
@@ -884,10 +648,10 @@ public:
 
   nsMsgTemplateReplyHelper();
 
-  nsCOMPtr <nsIMsgDBHdr> mHdrToReplyTo;
-  nsCOMPtr <nsIMsgDBHdr> mTemplateHdr;
-  nsCOMPtr <nsIMsgWindow> mMsgWindow;
-  nsCOMPtr <nsIMsgIncomingServer> mServer;
+  nsCOMPtr<nsIMsgDBHdr> mHdrToReplyTo;
+  nsCOMPtr<nsIMsgDBHdr> mTemplateHdr;
+  nsCOMPtr<nsIMsgWindow> mMsgWindow;
+  nsCOMPtr<nsIMsgIdentity> mIdentity;
   nsCString mTemplateBody;
   bool mInMsgBody;
   char mLastBlockChars[3];
@@ -919,10 +683,9 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStartRunningUrl(nsIURI *aUrl)
 
 NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult aExitCode)
 {
-
   NS_ENSURE_SUCCESS(aExitCode, aExitCode);
   nsresult rv;
-  nsCOMPtr<nsIDOMWindow> parentWindow;
+  nsCOMPtr<nsPIDOMWindowOuter> parentWindow;
   if (mMsgWindow)
   {
     nsCOMPtr<nsIDocShell> docShell;
@@ -931,18 +694,6 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult
     parentWindow = do_GetInterface(docShell);
     NS_ENSURE_TRUE(parentWindow, NS_ERROR_FAILURE);
   }
-  if ( NS_FAILED(rv) ) return rv ;
-  // get the MsgIdentity for the above key using AccountManager
-  nsCOMPtr <nsIMsgAccountManager> accountManager = do_GetService (NS_MSGACCOUNTMANAGER_CONTRACTID) ;
-  if (NS_FAILED(rv) || (!accountManager) ) return rv ;
-
-  nsCOMPtr <nsIMsgAccount> account;
-  nsCOMPtr <nsIMsgIdentity> identity;
-
-  rv = accountManager->FindAccountForServer(mServer, getter_AddRefs(account));
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = account->GetDefaultIdentity(getter_AddRefs(identity));
-  NS_ENSURE_SUCCESS(rv, rv);
 
   // create the compose params object
   nsCOMPtr<nsIMsgComposeParams> pMsgComposeParams (do_CreateInstance(NS_MSGCOMPOSEPARAMS_CONTRACTID, &rv));
@@ -958,16 +709,26 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult
   nsString body;
   nsString templateSubject, replySubject;
 
-  mTemplateHdr->GetMime2DecodedSubject(templateSubject);
   mHdrToReplyTo->GetMime2DecodedSubject(replySubject);
+  mTemplateHdr->GetMime2DecodedSubject(templateSubject);
+  nsString subject(NS_LITERAL_STRING("Auto: ")); // RFC 3834 3.1.5.
+  subject.Append(templateSubject);
   if (!replySubject.IsEmpty())
   {
-    templateSubject.Append(NS_LITERAL_STRING(" (was: "));
-    templateSubject.Append(replySubject);
-    templateSubject.Append(NS_LITERAL_STRING(")"));
+    subject.Append(NS_LITERAL_STRING(" (was: "));
+    subject.Append(replySubject);
+    subject.Append(NS_LITERAL_STRING(")"));
   }
-  compFields->SetSubject(templateSubject);
-  CopyASCIItoUTF16(mTemplateBody, body);
+
+  compFields->SetSubject(subject);
+  compFields->SetRawHeader("Auto-Submitted", NS_LITERAL_CSTRING("auto-replied"), nullptr);
+
+  nsCString charset;
+  rv = mTemplateHdr->GetCharset(getter_Copies(charset));
+  NS_ENSURE_SUCCESS(rv, rv);
+  compFields->SetCharacterSet(charset.get());
+  rv = nsMsgI18NConvertToUnicode(charset.get(), mTemplateBody, body);
+  NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "couldn't convert templ body to unicode");
   compFields->SetBody(body);
 
   nsCString msgUri;
@@ -975,12 +736,12 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult
   mHdrToReplyTo->GetFolder(getter_AddRefs(folder));
   folder->GetUriForMsg(mHdrToReplyTo, msgUri);
   // populate the compose params
-   // we use type new instead of reply - we might need to add a reply with template type.
-  pMsgComposeParams->SetType(nsIMsgCompType::New);
+  pMsgComposeParams->SetType(nsIMsgCompType::ReplyWithTemplate);
   pMsgComposeParams->SetFormat(nsIMsgCompFormat::Default);
-  pMsgComposeParams->SetIdentity(identity);
+  pMsgComposeParams->SetIdentity(mIdentity);
   pMsgComposeParams->SetComposeFields(compFields);
   pMsgComposeParams->SetOriginalMsgURI(msgUri.get());
+
   // create the nsIMsgCompose object to send the object
   nsCOMPtr<nsIMsgCompose> pMsgCompose (do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -992,7 +753,7 @@ NS_IMETHODIMP nsMsgTemplateReplyHelper::OnStopRunningUrl(nsIURI *aUrl, nsresult
 
   Release();
 
-  return pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, identity, nullptr, nullptr, nullptr) ;
+  return pMsgCompose->SendMsg(nsIMsgSend::nsMsgDeliverNow, mIdentity, nullptr, nullptr, nullptr) ;
 }
 
 NS_IMETHODIMP
@@ -1072,7 +833,6 @@ nsMsgTemplateReplyHelper::OnDataAvailable(nsIRequest* request,
       mInMsgBody = bodyOffset != 0;
       if (!mInMsgBody && readCount > 3) // still in msg hdrs
         strncpy(mLastBlockChars, readBuf + readCount - 3, 3);
-
     }
     mTemplateBody.Append(readBuf + bodyOffset);
   }
@@ -1083,13 +843,51 @@ nsMsgTemplateReplyHelper::OnDataAvailable(nsIRequest* request,
 NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(nsIMsgDBHdr *aMsgHdr, const char *templateUri,
                                              nsIMsgWindow *aMsgWindow, nsIMsgIncomingServer *aServer)
 {
-  // to reply with template, we need the message body of the template
+  // To reply with template, we need the message body of the template.
   // I think we're going to need to stream the template message to ourselves,
   // and construct the body, and call setBody on the compFields.
-  // We need the reply-to header of the msg we're replying to, so
-  // we're going to add that to the db if it's different from "from:"
-  // For imap, we could make adding a reply-to filter append
-  // reply-to to the custom headers...
+  nsresult rv;
+  nsCOMPtr <nsIMsgAccountManager> accountManager =
+    do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIMsgAccount> account;
+  rv = accountManager->FindAccountForServer(aServer, getter_AddRefs(account));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIArray> identities;
+  rv = account->GetIdentities(getter_AddRefs(identities));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsAutoCString recipients;
+  aMsgHdr->GetRecipients(getter_Copies(recipients));
+
+  nsAutoCString ccList;
+  aMsgHdr->GetCcList(getter_Copies(ccList));
+
+  // Go through the identities to see to whom this was addressed.
+  // In case we get no match, this is likely a list/bulk/bcc/spam mail and we
+  // shouldn't reply. RFC 3834 2.
+  nsCOMPtr<nsIMsgIdentity> identity; // identity to reply from
+  uint32_t count = 0;
+  identities->GetLength(&count);
+  for (uint32_t i = 0; i < count; i++)
+  {
+    nsCOMPtr<nsIMsgIdentity> anIdentity(do_QueryElementAt(identities, i, &rv));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsAutoCString identityEmail;
+    anIdentity->GetEmail(identityEmail);
+
+    if (recipients.Find(identityEmail, CaseInsensitiveCompare) != kNotFound ||
+        ccList.Find(identityEmail, CaseInsensitiveCompare) != kNotFound)
+    {
+      identity = anIdentity;
+      break;
+    }
+  }
+  if (!identity) // Found no match -> don't reply.
+    return NS_ERROR_ABORT;
 
   nsMsgTemplateReplyHelper *helper = new nsMsgTemplateReplyHelper;
   if (!helper)
@@ -1099,7 +897,14 @@ NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(nsIMsgDBHdr *aMsgHdr, const
 
   helper->mHdrToReplyTo = aMsgHdr;
   helper->mMsgWindow = aMsgWindow;
-  helper->mServer = aServer;
+  helper->mIdentity = identity;
+
+  nsAutoCString replyTo;
+  aMsgHdr->GetStringProperty("replyTo", getter_Copies(replyTo));
+  if (replyTo.IsEmpty())
+    aMsgHdr->GetAuthor(getter_Copies(replyTo));
+  if (replyTo.IsEmpty())
+    return NS_ERROR_FAILURE; // nowhere to send the reply
 
   nsCOMPtr <nsIMsgFolder> templateFolder;
   nsCOMPtr <nsIMsgDatabase> templateDB;
@@ -1109,7 +914,7 @@ NS_IMETHODIMP nsMsgComposeService::ReplyWithTemplate(nsIMsgDBHdr *aMsgHdr, const
     return NS_ERROR_FAILURE;
 
   nsAutoCString folderUri(Substring(templateUri, query)); 
-  nsresult rv = GetExistingFolder(folderUri, getter_AddRefs(templateFolder));
+  rv = GetExistingFolder(folderUri, getter_AddRefs(templateFolder));
   NS_ENSURE_SUCCESS(rv, rv);
   rv = templateFolder->GetMsgDatabase(getter_AddRefs(templateDB));
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1214,7 +1019,7 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
                                       true, forwardTo,
                                       false, aMsgWindow);
 
-  nsCOMPtr<nsIDOMWindow> parentWindow;
+  nsCOMPtr<mozIDOMWindowProxy> parentWindow;
   if (aMsgWindow)
   {
     nsCOMPtr<nsIDocShell> docShell;
@@ -1254,75 +1059,6 @@ nsMsgComposeService::ForwardMessage(const nsAString &forwardTo,
   return folder->AddMessageDispositionState(aMsgHdr, nsIMsgFolder::nsMsgDispositionState_Forwarded);
 }
 
-nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindow *aComposeWindow, nsIXULWindow *aXULWindow, bool aShow)
-{
-  nsresult rv = NS_OK;
-
-  nsCOMPtr<nsIObserverService> obs =
-    mozilla::services::GetObserverService();
-  NS_ENSURE_TRUE(obs, NS_ERROR_UNEXPECTED);
-
-  nsCOMPtr <nsPIDOMWindow> window = do_QueryInterface(aComposeWindow, &rv);
-
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsIDocShell *docShell = window->GetDocShell();
-
-  nsCOMPtr <nsIDocShellTreeItem> treeItem = do_QueryInterface(docShell, &rv);
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  nsCOMPtr <nsIDocShellTreeOwner> treeOwner;
-  rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
-  NS_ENSURE_SUCCESS(rv,rv);
-
-  if (treeOwner) {
-    // the window need to be sticky before we hide it.
-    nsCOMPtr<nsIContentViewer> contentViewer;
-    rv = docShell->GetContentViewer(getter_AddRefs(contentViewer));
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    rv = contentViewer->SetSticky(!aShow);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    // disable (enable) the cached window
-    nsCOMPtr<nsIBaseWindow> baseWindow;
-    baseWindow = do_QueryInterface(treeOwner, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    baseWindow->SetEnabled(aShow);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    nsCOMPtr<nsIWindowMediator> windowMediator =
-	         do_GetService(NS_WINDOWMEDIATOR_CONTRACTID, &rv);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    // if showing, reinstate the window with the mediator
-    if (aShow) {
-      rv = windowMediator->RegisterWindow(aXULWindow);
-      NS_ENSURE_SUCCESS(rv,rv);
-
-      obs->NotifyObservers(aXULWindow, "xul-window-registered", nullptr);
-    }
-
-    // hide (show) the cached window
-    rv = baseWindow->SetVisibility(aShow);
-    NS_ENSURE_SUCCESS(rv,rv);
-
-    // if hiding, remove the window from the mediator,
-    // so that it will be removed from the task list
-    if (!aShow) {
-      rv = windowMediator->UnregisterWindow(aXULWindow);
-      NS_ENSURE_SUCCESS(rv,rv);
-
-      obs->NotifyObservers(aXULWindow, "xul-window-destroyed", nullptr);
-    }
-  }
-  else {
-    rv = NS_ERROR_FAILURE;
-  }
-  return rv;
-}
-
 nsresult nsMsgComposeService::AddGlobalHtmlDomains()
 {
 
@@ -1616,8 +1352,10 @@ nsMsgComposeService::RunMessageThroughMimeDraft(
 
   // Now, just plug the two together and get the hell out of the way!
   nsCOMPtr<nsIStreamListener> streamListener = do_QueryInterface(mimeConverter);
+  nsCOMPtr<nsIURI> dummyNull;
   return messageService->DisplayMessage(PromiseFlatCString(aMsgURI).get(), streamListener,
-                                        aMsgWindow, nullptr, mailCharset.get(), nullptr);;
+                                        aMsgWindow, nullptr, mailCharset.get(),
+                                        getter_AddRefs(dummyNull));
 }
 
 NS_IMETHODIMP
@@ -1695,7 +1433,7 @@ nsMsgComposeService::Handle(nsICommandLine* aCmdLine)
     if (arg)
       arg->SetData(uristr);
 
-    nsCOMPtr<nsIDOMWindow> opened;
+    nsCOMPtr<mozIDOMWindowProxy> opened;
     wwatch->OpenWindow(nullptr, DEFAULT_CHROME, "_blank",
                        "chrome,dialog=no,all", arg, getter_AddRefs(opened));
 
