@@ -68,7 +68,7 @@ protected:
 
   nsCOMArray <nsIMsgFolder> m_foldersPendingListeners;
   nsCOMArray <nsIDBChangeListener> m_pendingListeners;
-  nsAutoTArray<nsMsgDatabase*, kInitialMsgDBCacheSize> m_dbCache;
+  AutoTArray<nsMsgDatabase*, kInitialMsgDBCacheSize> m_dbCache;
 };
 
 class nsMsgDBEnumerator : public nsISimpleEnumerator {
@@ -88,7 +88,7 @@ public:
 
     nsresult                        GetRowCursor();
     virtual nsresult                PrefetchNext();
-    nsRefPtr<nsMsgDatabase>         mDB;
+    RefPtr<nsMsgDatabase>         mDB;
     nsCOMPtr<nsIMdbTableRowCursor>  mRowCursor;
     mdb_pos                         mRowPos;
     nsCOMPtr<nsIMsgDBHdr>           mResultHdr;
@@ -288,11 +288,9 @@ protected:
   nsCOMPtr <nsIMsgDownloadSettings> m_downloadSettings;
 
   nsresult PurgeMessagesOlderThan(uint32_t daysToKeepHdrs,
-                                  bool keepUnreadMessagesOnly,
                                   bool applyToFlaggedMessages,
                                   nsIMutableArray *hdrsToDelete);
   nsresult PurgeExcessMessages(uint32_t numHeadersToKeep,
-                               bool keepUnreadMessagesOnly,
                                bool applyToFlaggedMessages,
                                nsIMutableArray *hdrsToDelete);
 
@@ -378,8 +376,8 @@ protected:
   void          ClearEnumerators();
   // all instantiated headers, but doesn't hold refs.
   PLDHashTable  *m_headersInUse;
-  static PLDHashNumber HashKey(PLDHashTable* aTable, const void* aKey);
-  static bool MatchEntry(PLDHashTable* aTable, const PLDHashEntryHdr* aEntry, const void* aKey);
+  static PLDHashNumber HashKey(const void* aKey);
+  static bool MatchEntry(const PLDHashEntryHdr* aEntry, const void* aKey);
   static void MoveEntry(PLDHashTable* aTable, const PLDHashEntryHdr* aFrom, PLDHashEntryHdr* aTo);
   static void ClearEntry(PLDHashTable* aTable, PLDHashEntryHdr* aEntry);
   static PLDHashTableOps gMsgDBHashTableOps;
@@ -424,7 +422,7 @@ public:
   }
 private:
   uint32_t m_cacheSize;
-  nsRefPtr<mozilla::mailnews::MsgDBReporter> mMemReporter;
+  RefPtr<mozilla::mailnews::MsgDBReporter> mMemReporter;
 };
 
 class nsMsgRetentionSettings : public nsIMsgRetentionSettings
@@ -439,7 +437,6 @@ protected:
   nsMsgRetainByPreference m_retainByPreference;
   uint32_t                m_daysToKeepHdrs;
   uint32_t                m_numHeadersToKeep;
-  bool                    m_keepUnreadMessagesOnly;
   bool                    m_useServerDefaults;
   bool                    m_cleanupBodiesByDays;
   uint32_t                m_daysToKeepBodies;
