@@ -137,7 +137,8 @@ static const PLDHashTableOps gTokenTableOps = {
     PLDHashTable::HashStringKey,
     PLDHashTable::MatchStringKey,
     PLDHashTable::MoveEntryStub,
-    PLDHashTable::ClearEntryStub
+    PLDHashTable::ClearEntryStub,
+    nullptr
 };
 
 TokenHash::TokenHash(uint32_t aEntrySize)
@@ -1402,20 +1403,20 @@ void nsBayesianFilter::classifyMessage(
     uint32_t traitCount = aProTraits.Length();
 
     // pro message counts per trait index
-    nsAutoTArray<uint32_t, kTraitAutoCapacity> numProMessages;
+    AutoTArray<uint32_t, kTraitAutoCapacity> numProMessages;
     // anti message counts per trait index
-    nsAutoTArray<uint32_t, kTraitAutoCapacity> numAntiMessages;
+    AutoTArray<uint32_t, kTraitAutoCapacity> numAntiMessages;
     // array of pro aliases per trait index
-    nsAutoTArray<uint32_t*, kTraitAutoCapacity > proAliasArrays;
+    AutoTArray<uint32_t*, kTraitAutoCapacity > proAliasArrays;
     // number of pro aliases per trait index
-    nsAutoTArray<uint32_t, kTraitAutoCapacity > proAliasesLengths;    
+    AutoTArray<uint32_t, kTraitAutoCapacity > proAliasesLengths;    
     // array of anti aliases per trait index
-    nsAutoTArray<uint32_t*, kTraitAutoCapacity> antiAliasArrays;
+    AutoTArray<uint32_t*, kTraitAutoCapacity> antiAliasArrays;
     // number of anti aliases per trait index
-    nsAutoTArray<uint32_t, kTraitAutoCapacity > antiAliasesLengths;    
+    AutoTArray<uint32_t, kTraitAutoCapacity > antiAliasesLengths;    
     // construct the outgoing listener arrays
-    nsAutoTArray<uint32_t, kTraitAutoCapacity> traits;
-    nsAutoTArray<uint32_t, kTraitAutoCapacity> percents;
+    AutoTArray<uint32_t, kTraitAutoCapacity> traits;
+    AutoTArray<uint32_t, kTraitAutoCapacity> percents;
     if (traitCount > kTraitAutoCapacity)
     {
       traits.SetCapacity(traitCount);
@@ -1526,7 +1527,7 @@ void nsBayesianFilter::classifyMessage(
         double distance = std::abs(prob - 0.5);
         if (distance >= .1)
         {
-          nsresult rv = setAnalysis(token, traitIndex, distance, prob);
+          mozilla::DebugOnly<nsresult> rv = setAnalysis(token, traitIndex, distance, prob);
           NS_ASSERTION(NS_SUCCEEDED(rv), "Problem in setAnalysis");
         }
       }
@@ -1534,7 +1535,7 @@ void nsBayesianFilter::classifyMessage(
 
     for (uint32_t traitIndex = 0; traitIndex < traitCount; traitIndex++)
     {
-      nsAutoTArray<TraitAnalysis, 1024> traitAnalyses;
+      AutoTArray<TraitAnalysis, 1024> traitAnalyses;
       // copy valid tokens into an array to sort
       for (uint32_t tokenIndex = 0; tokenIndex < tokenCount; tokenIndex++)
       {
@@ -1632,7 +1633,6 @@ void nsBayesianFilter::classifyMessage(
         for (uint32_t tokenIndex = 0; tokenIndex < usedTokenCount; tokenIndex++)
         {
           TraitAnalysis& ta = traitAnalyses[last - 1 - tokenIndex];
-          double S, H;
           int32_t chi_error;
           S = chi2P(-2.0 * sArray[tokenIndex], 2.0 * clueCount, &chi_error);
           if (!chi_error)
@@ -1722,8 +1722,8 @@ void nsBayesianFilter::classifyMessage(
   const char* messageURI,
   nsIJunkMailClassificationListener* aJunkListener)
 {
-  nsAutoTArray<uint32_t, 1> proTraits;
-  nsAutoTArray<uint32_t, 1> antiTraits;
+  AutoTArray<uint32_t, 1> proTraits;
+  AutoTArray<uint32_t, 1> antiTraits;
   proTraits.AppendElement(kJunkTrait);
   antiTraits.AppendElement(kGoodTrait);
   classifyMessage(tokens, messageURI, proTraits, antiTraits,
@@ -1866,8 +1866,8 @@ NS_IMETHODIMP nsBayesianFilter::ClassifyTraitsInMessages(
   nsIMsgWindow *aMsgWindow,
   nsIJunkMailClassificationListener *aJunkListener)
 {
-  nsAutoTArray<uint32_t, kTraitAutoCapacity> proTraits;
-  nsAutoTArray<uint32_t, kTraitAutoCapacity> antiTraits;
+  AutoTArray<uint32_t, kTraitAutoCapacity> proTraits;
+  AutoTArray<uint32_t, kTraitAutoCapacity> antiTraits;
   if (aTraitCount > kTraitAutoCapacity)
   {
     proTraits.SetCapacity(aTraitCount);
@@ -1928,8 +1928,8 @@ NS_IMETHODIMP nsBayesianFilter::SetMsgTraitClassification(
     nsIMsgWindow *aMsgWindow,
     nsIJunkMailClassificationListener *aJunkListener)
 {
-  nsAutoTArray<uint32_t, kTraitAutoCapacity> oldTraits;
-  nsAutoTArray<uint32_t, kTraitAutoCapacity> newTraits;
+  AutoTArray<uint32_t, kTraitAutoCapacity> oldTraits;
+  AutoTArray<uint32_t, kTraitAutoCapacity> newTraits;
   if (aOldCount > kTraitAutoCapacity)
     oldTraits.SetCapacity(aOldCount);
   if (aNewCount > kTraitAutoCapacity)
@@ -2015,8 +2015,8 @@ void nsBayesianFilter::observeMessage(
     if (aTraitListener)
     {
       // construct the outgoing listener arrays
-      nsAutoTArray<uint32_t, kTraitAutoCapacity> traits;
-      nsAutoTArray<uint32_t, kTraitAutoCapacity> percents;
+      AutoTArray<uint32_t, kTraitAutoCapacity> traits;
+      AutoTArray<uint32_t, kTraitAutoCapacity> percents;
       uint32_t newLength = newClassifications.Length();
       if (newLength > kTraitAutoCapacity)
       {
@@ -2057,8 +2057,8 @@ NS_IMETHODIMP nsBayesianFilter::SetMessageClassification(
     nsIMsgWindow *aMsgWindow,
     nsIJunkMailClassificationListener *aListener)
 {
-  nsAutoTArray<uint32_t, 1> oldClassifications;
-  nsAutoTArray<uint32_t, 1> newClassifications;
+  AutoTArray<uint32_t, 1> oldClassifications;
+  AutoTArray<uint32_t, 1> newClassifications;
 
   // convert between classifications and trait
   if (aOldClassification == nsIJunkMailPlugin::JUNK)
@@ -2090,8 +2090,8 @@ NS_IMETHODIMP nsBayesianFilter::DetailMessage(const char *aMsgURI,
     uint32_t aProTrait, uint32_t aAntiTrait,
     nsIMsgTraitDetailListener *aDetailListener, nsIMsgWindow *aMsgWindow)
 {
-  nsAutoTArray<uint32_t, 1> proTraits;
-  nsAutoTArray<uint32_t, 1> antiTraits;
+  AutoTArray<uint32_t, 1> proTraits;
+  AutoTArray<uint32_t, 1> antiTraits;
   proTraits.AppendElement(aProTrait);
   antiTraits.AppendElement(aAntiTrait);
 
