@@ -20,6 +20,7 @@
 #include "nsServiceManagerUtils.h"
 #include "nsMemory.h"
 #include "nsAlgorithm.h"
+#include "nsNSSComponent.h"
 #include "mozilla/RefPtr.h"
 #include "mozilla/Services.h"
 #include "mozilla/mailnews/MimeEncoder.h"
@@ -30,6 +31,7 @@
 
 using namespace mozilla::mailnews;
 using namespace mozilla;
+using namespace mozilla::psm;
 
 #define MK_MIME_ERROR_WRITING_FILE -1
 
@@ -58,7 +60,7 @@ static void
 GenerateGlobalRandomBytes(unsigned char *buf, int32_t len)
 {
   static bool      firstTime = true;
-  
+
   if (firstTime)
   {
     // Seed the random-number generator with current time so that
@@ -66,15 +68,15 @@ GenerateGlobalRandomBytes(unsigned char *buf, int32_t len)
     srand( (unsigned)PR_Now() );
     firstTime = false;
   }
-  
+
   for( int32_t i = 0; i < len; i++ )
     buf[i] = rand() % 10;
 }
-   
-char 
+
+char
 *mime_make_separator(const char *prefix)
 {
-  unsigned char rand_buf[13]; 
+  unsigned char rand_buf[13];
   GenerateGlobalRandomBytes(rand_buf, 12);
 
   return PR_smprintf("------------%s"
@@ -228,7 +230,7 @@ void nsMsgComposeSecure::SetError(nsIMsgSendReport *sendReport, const char16_t *
     return;
 
   mErrorAlreadyReported = true;
-  
+
   nsString errorString;
   nsresult res;
 
@@ -252,7 +254,7 @@ void nsMsgComposeSecure::SetErrorWithParam(nsIMsgSendReport *sendReport, const c
     return;
 
   mErrorAlreadyReported = true;
-  
+
   nsString errorString;
   nsresult res;
   const char16_t *params[1];
@@ -425,9 +427,11 @@ NS_IMETHODIMP nsMsgComposeSecure::BeginCryptoEncapsulation(nsIOutputStream * aSt
     PR_ASSERT(0);
 
   aIdentity->GetUnicharAttribute("signing_cert_name", mSigningCertName);
+  aIdentity->GetCharAttribute("signing_cert_dbkey", mSigningCertDBKey);
   aIdentity->GetUnicharAttribute("encryption_cert_name", mEncryptionCertName);
+  aIdentity->GetCharAttribute("encryption_cert_dbkey", mEncryptionCertDBKey);
 
-  rv = MimeCryptoHackCerts(aRecipients, sendReport, encryptMessages, signMessage);
+  rv = MimeCryptoHackCerts(aRecipients, sendReport, encryptMessages, signMessage, aIdentity);
   if (NS_FAILED(rv)) {
     goto FAIL;
   }
@@ -704,7 +708,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
           CRLF,
           mMultipartSignedBoundary,
           sig_content_desc_utf8.get());
-          
+
   if (!header) {
     rv = NS_ERROR_OUT_OF_MEMORY;
     goto FAIL;
@@ -718,7 +722,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (bool aOuter, nsIMsgSendR
     if (NS_FAILED(rv) || n < L) {
       // XXX This is -1, not an nsresult
       rv = static_cast<nsresult>(MK_MIME_ERROR_WRITING_FILE);
-    } 
+    }
   } else {
     /* If this is an inner block, feed it through the crypto stream. */
     rv = MimeCryptoWriteBlock (header, L);
@@ -864,7 +868,8 @@ nsresult nsMsgComposeSecure::MimeFinishEncryption (bool aSign, nsIMsgSendReport
 nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
                                                  nsIMsgSendReport *sendReport,
                                                  bool aEncrypt,
-                                                 bool aSign)
+                                                 bool aSign,
+                                                 nsIMsgIdentity *aIdentity)
 {
   nsCOMPtr<nsIX509CertDB> certdb = do_GetService(NS_X509CERTDB_CONTRACTID);
   nsresult res;
@@ -875,16 +880,74 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
   }
 
   PR_ASSERT(aEncrypt || aSign);
-  certdb->FindEmailEncryptionCert(mEncryptionCertName, getter_AddRefs(mSelfEncryptionCert));
-  certdb->FindEmailSigningCert(mSigningCertName, getter_AddRefs(mSelfSigningCert));
+
+  /*
+   Signing and encryption certs use the following (per-identity) preferences:
+   - "signing_cert_name"/"encryption_cert_name": a string specifying the
+     nickname of the certificate
+   - "signing_cert_dbkey"/"encryption_cert_dbkey": a Base64 encoded blob
+     specifying an nsIX509Cert dbKey (represents serial number
+     and issuer DN, which is considered to be unique for X.509 certificates)
+
+   When retrieving the prefs, we try (in this order):
+   1) *_cert_dbkey, if available
+   2) *_cert_name (for maintaining backwards compatibility with preference
+      attributes written by earlier versions)
+  */
+
+  RefPtr<SharedCertVerifier> certVerifier(GetDefaultCertVerifier());
+  NS_ENSURE_TRUE(certVerifier, NS_ERROR_UNEXPECTED);
+
+  UniqueCERTCertList builtChain;
+  if (!mEncryptionCertDBKey.IsEmpty()) {
+    certdb->FindCertByDBKey(mEncryptionCertDBKey.get(),
+                            getter_AddRefs(mSelfEncryptionCert));
+    if (mSelfEncryptionCert &&
+        (certVerifier->VerifyCert(mSelfEncryptionCert->GetCert(),
+                                  certificateUsageEmailRecipient,
+                                  mozilla::pkix::Now(),
+                                  nullptr, nullptr,
+                                  builtChain) != SECSuccess)) {
+      // not suitable for encryption, so unset cert and clear pref
+      mSelfEncryptionCert = nullptr;
+      mEncryptionCertDBKey.Truncate();
+      aIdentity->SetCharAttribute("encryption_cert_dbkey",
+                                   mEncryptionCertDBKey);
+    }
+  }
+  if (!mSelfEncryptionCert) {
+    certdb->FindEmailEncryptionCert(mEncryptionCertName,
+                                    getter_AddRefs(mSelfEncryptionCert));
+  }
+
+  // same procedure for the signing cert
+  if (!mSigningCertDBKey.IsEmpty()) {
+    certdb->FindCertByDBKey(mSigningCertDBKey.get(),
+                            getter_AddRefs(mSelfSigningCert));
+    if (mSelfSigningCert &&
+        (certVerifier->VerifyCert(mSelfSigningCert->GetCert(),
+                                  certificateUsageEmailSigner,
+                                  mozilla::pkix::Now(),
+                                  nullptr, nullptr,
+                                  builtChain) != SECSuccess)) {
+      // not suitable for signing, so unset cert and clear pref
+      mSelfSigningCert = nullptr;
+      mSigningCertDBKey.Truncate();
+      aIdentity->SetCharAttribute("signing_cert_dbkey", mSigningCertDBKey);
+    }
+  }
+  if (!mSelfSigningCert) {
+    certdb->FindEmailSigningCert(mSigningCertName,
+                                 getter_AddRefs(mSelfSigningCert));
+  }
 
   // must have both the signing and encryption certs to sign
-  if ((mSelfSigningCert == nullptr) && aSign) {
+  if (!mSelfSigningCert && aSign) {
     SetError(sendReport, MOZ_UTF16("NoSenderSigningCert"));
     return NS_ERROR_FAILURE;
   }
 
-  if ((mSelfEncryptionCert == nullptr) && aEncrypt) {
+  if (!mSelfEncryptionCert && aEncrypt) {
     SetError(sendReport, MOZ_UTF16("NoSenderEncryptionCert"));
     return NS_ERROR_FAILURE;
   }
@@ -916,7 +979,7 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
       nsCString mailbox_lowercase;
       ToLowerCase(mailboxes[i], mailbox_lowercase);
       nsCOMPtr<nsIX509Cert> cert;
-      res = certdb->FindCertByEmailAddress(nullptr, mailbox_lowercase.get(),
+      res = certdb->FindCertByEmailAddress(mailbox_lowercase.get(),
                                            getter_AddRefs(cert));
       if (NS_FAILED(res)) {
         // Failure to find a valid encryption cert is fatal.
@@ -943,7 +1006,7 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 
       mCerts->AppendElement(cert, false);
     }
-    
+
     if (!already_added_self_cert) {
       mCerts->AppendElement(mSelfEncryptionCert, false);
     }
@@ -998,7 +1061,7 @@ NS_IMETHODIMP nsMsgComposeSecure::MimeCryptoWriteBlock (const char *buf, int32_t
 
       memcpy(mBuffer+mBufferedBytes, inputBytesIterator, bytesToAppend);
       mBufferedBytes += bytesToAppend;
-      
+
       inputBytesIterator += bytesToAppend;
       inputBytesLeft -= bytesToAppend;
 
