@@ -19,7 +19,7 @@ var selectedModuleName = null;
 var addInterface = null ;
 var newFeedAcctCreated = false;
 
-const nsISupportsString = Components.interfaces.nsISupportsString;
+var nsISupportsString = Components.interfaces.nsISupportsString;
 
 function OnLoadImportDialog()
 {
@@ -389,8 +389,8 @@ function AddModuleToList(moduleName, index)
   var item = document.createElement('listitem');
   item.setAttribute('label', moduleName);
 
-  // Temporarily skip Eudora and Outlook Import which are busted (Bug 1175055).
-  if (moduleName == "Eudora" || moduleName == "Outlook") {
+  // Temporarily skip Outlook Import which are busted (Bug 1175055).
+  if (moduleName == "Outlook") {
     item.setAttribute('list-index', -1);
     item.setAttribute('disabled', true);
     item.setAttribute('tooltiptext', "Currently disabled due to bug 1175055");
