@@ -990,7 +990,7 @@ bool nsImportGenericMail::CreateFolder(nsIMsgFolder **ppFolder)
  * the folder, the main thread hands it the next folder.
  */
 
-class GetSubFoldersRunnable : public nsRunnable
+class GetSubFoldersRunnable : public mozilla::Runnable
 {
 public:
   GetSubFoldersRunnable(nsIMsgFolder *aFolder);
@@ -1006,18 +1006,19 @@ GetSubFoldersRunnable::GetSubFoldersRunnable(nsIMsgFolder *aFolder) :
 
 NS_IMETHODIMP GetSubFoldersRunnable::Run()
 {
-  return m_folder->GetSubFolders(nullptr);
+  nsCOMPtr<nsISimpleEnumerator> dummy;
+  return m_folder->GetSubFolders(getter_AddRefs(dummy));
 }
 
 
 nsresult ProxyGetSubFolders(nsIMsgFolder *aFolder)
 {
-  nsRefPtr<GetSubFoldersRunnable> getSubFolders =
+  RefPtr<GetSubFoldersRunnable> getSubFolders =
     new GetSubFoldersRunnable(aFolder);
   return NS_DispatchToMainThread(getSubFolders, NS_DISPATCH_SYNC);
 }
 
-class GetChildNamedRunnable : public nsRunnable
+class GetChildNamedRunnable : public mozilla::Runnable
 {
 public:
   GetChildNamedRunnable(nsIMsgFolder *aFolder, const nsAString& aName, nsIMsgFolder **aChild);
@@ -1044,12 +1045,12 @@ NS_IMETHODIMP GetChildNamedRunnable::Run()
 nsresult ProxyGetChildNamed(nsIMsgFolder *aFolder, const nsAString & aName,
                             nsIMsgFolder **aChild)
 {
-  nsRefPtr<GetChildNamedRunnable> getChildNamed =
+  RefPtr<GetChildNamedRunnable> getChildNamed =
     new GetChildNamedRunnable(aFolder, aName, aChild);
   return NS_DispatchToMainThread(getChildNamed, NS_DISPATCH_SYNC);
 }
 
-class GetParentRunnable : public nsRunnable
+class GetParentRunnable : public mozilla::Runnable
 {
 public:
   GetParentRunnable(nsIMsgFolder *aFolder, nsIMsgFolder **aParent);
@@ -1072,12 +1073,12 @@ NS_IMETHODIMP GetParentRunnable::Run()
 
 nsresult ProxyGetParent(nsIMsgFolder *aFolder, nsIMsgFolder **aParent)
 {
-  nsRefPtr<GetParentRunnable> getParent =
+  RefPtr<GetParentRunnable> getParent =
     new GetParentRunnable(aFolder, aParent);
   return NS_DispatchToMainThread(getParent, NS_DISPATCH_SYNC);
 }
 
-class ContainsChildNamedRunnable : public nsRunnable
+class ContainsChildNamedRunnable : public mozilla::Runnable
 {
 public:
   ContainsChildNamedRunnable(nsIMsgFolder *aFolder, const nsAString& aName, bool *aResult);
@@ -1104,13 +1105,13 @@ NS_IMETHODIMP ContainsChildNamedRunnable::Run()
 nsresult ProxyContainsChildNamed(nsIMsgFolder *aFolder, const nsAString &aName,
                                  bool *aResult)
 {
-  nsRefPtr<ContainsChildNamedRunnable> containsChildNamed =
+  RefPtr<ContainsChildNamedRunnable> containsChildNamed =
     new ContainsChildNamedRunnable(aFolder, aName, aResult);
   return NS_DispatchToMainThread(containsChildNamed, NS_DISPATCH_SYNC);
 }
 
 
-class GenerateUniqueSubfolderNameRunnable : public nsRunnable
+class GenerateUniqueSubfolderNameRunnable : public mozilla::Runnable
 {
 public:
   GenerateUniqueSubfolderNameRunnable(nsIMsgFolder *aFolder,
@@ -1144,12 +1145,12 @@ nsresult ProxyGenerateUniqueSubfolderName(nsIMsgFolder *aFolder,
                                           nsAString& aName)
 
 {
-  nsRefPtr<GenerateUniqueSubfolderNameRunnable> generateUniqueSubfolderName =
+  RefPtr<GenerateUniqueSubfolderNameRunnable> generateUniqueSubfolderName =
     new GenerateUniqueSubfolderNameRunnable(aFolder, aPrefix, aOtherFolder, aName);
   return NS_DispatchToMainThread(generateUniqueSubfolderName, NS_DISPATCH_SYNC);
 }
 
-class CreateSubfolderRunnable : public nsRunnable
+class CreateSubfolderRunnable : public mozilla::Runnable
 {
 public:
   CreateSubfolderRunnable(nsIMsgFolder *aFolder, const nsAString& aName);
@@ -1173,12 +1174,12 @@ NS_IMETHODIMP CreateSubfolderRunnable::Run()
 
 nsresult ProxyCreateSubfolder(nsIMsgFolder *aFolder, const nsAString &aName)
 {
-  nsRefPtr<CreateSubfolderRunnable> createSubfolder =
+  RefPtr<CreateSubfolderRunnable> createSubfolder =
     new CreateSubfolderRunnable(aFolder, aName);
   return NS_DispatchToMainThread(createSubfolder, NS_DISPATCH_SYNC);
 }
 
-class ForceDBClosedRunnable : public nsRunnable
+class ForceDBClosedRunnable : public mozilla::Runnable
 {
 public:
   ForceDBClosedRunnable(nsIMsgFolder *aFolder);
@@ -1199,7 +1200,7 @@ NS_IMETHODIMP ForceDBClosedRunnable::Run()
 
 nsresult ProxyForceDBClosed(nsIMsgFolder *aFolder)
 {
-  nsRefPtr<ForceDBClosedRunnable> forceDBClosed =
+  RefPtr<ForceDBClosedRunnable> forceDBClosed =
     new ForceDBClosedRunnable(aFolder);
   return NS_DispatchToMainThread(forceDBClosed, NS_DISPATCH_SYNC);
 }
