@@ -29,7 +29,7 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIDocShell.h"
-#include "nsIDOMWindow.h"
+#include "mozIDOMWindow.h"
 #include "nsEmbedCID.h"
 #include "nsMsgUtils.h"
 #include "nsMsgBaseCID.h"
@@ -37,8 +37,10 @@
 #include "nsIPop3Service.h"
 #include "nsMsgLocalCID.h"
 #include "mozilla/Services.h"
+#include "mozilla/Logging.h"
 
 extern PRLogModuleInfo *POP3LOGMODULE;
+#define POP3LOG(str) "%s sink: [this=%p] " str, POP3LOGMODULE->name, this
 
 NS_IMPL_ISUPPORTS(nsPop3Sink, nsIPop3Sink)
 
@@ -60,7 +62,8 @@ nsPop3Sink::nsPop3Sink()
 
 nsPop3Sink::~nsPop3Sink()
 {
-    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug, ("Calling ReleaseFolderLock from ~nsPop3Sink"));
+    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+            (POP3LOG("Calling ReleaseFolderLock from ~nsPop3Sink")));
     ReleaseFolderLock();
 }
 
@@ -228,12 +231,14 @@ nsPop3Sink::BeginMailDelivery(bool uidlDownload, nsIMsgWindow *aMsgWindow, bool*
   m_folder->GetLocked(&isLocked);
   if(!isLocked)
   {
-    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug, ("BeginMailDelivery acquiring semaphore"));
+    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+            (POP3LOG("BeginMailDelivery acquiring semaphore")));
     m_folder->AcquireSemaphore(supports);
   }
   else
   {
-    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug, ("BeginMailDelivery folder locked"));
+    MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+            (POP3LOG("BeginMailDelivery folder locked")));
     return NS_MSG_FOLDER_BUSY;
   }
   m_uidlDownload = uidlDownload;
@@ -278,7 +283,8 @@ nsPop3Sink::EndMailDelivery(nsIPop3Protocol *protocol)
   if (m_newMailParser)
     m_newMailParser->UpdateDBFolderInfo();
 
-  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug, ("Calling ReleaseFolderLock from EndMailDelivery"));
+  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+          (POP3LOG("Calling ReleaseFolderLock from EndMailDelivery")));
   nsresult rv = ReleaseFolderLock();
   NS_ASSERTION(NS_SUCCEEDED(rv),"folder lock not released successfully");
 
@@ -377,7 +383,8 @@ nsPop3Sink::ReleaseFolderLock()
   bool haveSemaphore;
   nsCOMPtr <nsISupports> supports = do_QueryInterface(static_cast<nsIPop3Sink*>(this));
   result = m_folder->TestSemaphore(supports, &haveSemaphore);
-  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug, ("ReleaseFolderLock haveSemaphore = %s", haveSemaphore ? "TRUE" : "FALSE"));
+  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+          (POP3LOG("ReleaseFolderLock haveSemaphore = %s"), haveSemaphore ? "TRUE" : "FALSE"));
 
   if(NS_SUCCEEDED(result) && haveSemaphore)
     result = m_folder->ReleaseSemaphore(supports);
@@ -404,7 +411,8 @@ nsPop3Sink::AbortMailDelivery(nsIPop3Protocol *protocol)
   we have truncated the inbox, so berkeley mailbox and msf file are in sync*/
   if (m_newMailParser)
     m_newMailParser->UpdateDBFolderInfo();
-  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug, ("Calling ReleaseFolderLock from AbortMailDelivery"));
+  MOZ_LOG(POP3LOGMODULE, mozilla::LogLevel::Debug,
+          (POP3LOG("Calling ReleaseFolderLock from AbortMailDelivery")));
 
   nsresult rv = ReleaseFolderLock();
   NS_ASSERTION(NS_SUCCEEDED(rv),"folder lock not released successfully");
@@ -519,18 +527,7 @@ nsPop3Sink::IncorporateBegin(const char* uidlString,
     m_newMailParser = nullptr;
     rv = NS_OK;
   }
-  else
-  {
-    if (m_downloadingToTempFile)
-    {
-      // Tell the parser to use the offset that will be in the dest folder,
-      // not the temp folder, so that the msg hdr will start off with
-      // the correct mdb oid
-      int64_t fileSize;
-      path->GetFileSize(&fileSize);
-      m_newMailParser->SetEnvelopePos((uint32_t) fileSize);
-    }
-  }
+
     if (closure)
         *closure = (void*) this;
 
@@ -695,7 +692,7 @@ nsresult nsPop3Sink::HandleTempDownloadFailed(nsIMsgWindow *msgWindow)
   bundle->FormatStringFromName(
     MOZ_UTF16("pop3TmpDownloadError"),
     params, 2, getter_Copies(confirmString));
-  nsCOMPtr<nsIDOMWindow> parentWindow;
+  nsCOMPtr<mozIDOMWindowProxy> parentWindow;
   nsCOMPtr<nsIPromptService> promptService = do_GetService(NS_PROMPTSERVICE_CONTRACTID);
   nsCOMPtr<nsIDocShell> docShell;
   if (msgWindow)
