@@ -168,7 +168,7 @@ STDMETHODIMP CMapiImp::Login(unsigned long aUIArg, LOGIN_PW_TYPE aLogin, LOGIN_P
 
     nsMAPIConfiguration *pConfig = nsMAPIConfiguration::GetMAPIConfiguration();
     if (pConfig != nullptr)
-        nResult = pConfig->RegisterSession(aUIArg, aLogin, aPassWord,
+        nResult = pConfig->RegisterSession(aUIArg, wwc(aLogin), wwc(aPassWord),
                                            (aFlags & MAPI_FORCE_DOWNLOAD), bNewSession,
                                            &nSession_Id, id_key.get());
     switch (nResult)
@@ -845,7 +845,7 @@ MsgMapiListContext::DeleteMessage(nsMsgKey key)
 #if 0 
   else if ( m_folder->GetIMAPFolderInfoMail() )
   {
-    nsAutoTArray<nsMsgKey, 1> messageKeys;
+    AutoTArray<nsMsgKey, 1> messageKeys;
     messageKeys.AppendElement(key);
 
     (m_folder->GetIMAPFolderInfoMail())->DeleteSpecifiedMessages(pane, messageKeys, nsMsgKey_None);
