@@ -1,6 +1,8 @@
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
 
+requestLongerTimeout(2);
+
 // Bug 453440 - Test the timespan-based logic of the sanitizer code
 var now_mSec = Date.now();
 var now_uSec = now_mSec * 1000;
@@ -40,16 +42,12 @@ function promiseDownloadRemoved(list) {
   return deferred.promise;
 }
 
-function test() {
-  waitForExplicitFinish();
-
-  Task.spawn(function() {
-    yield setupDownloads();
-    yield setupFormHistory();
-    yield setupHistory();
-    yield onHistoryReady();
-  }).then(null, ex => ok(false, ex)).then(finish);
-}
+add_task(function* test() {
+  yield setupDownloads();
+  yield setupFormHistory();
+  yield setupHistory();
+  yield onHistoryReady();
+});
 
 function countEntries(name, message, check) {
   let deferred = Promise.defer();
@@ -59,7 +57,7 @@ function countEntries(name, message, check) {
     obj.fieldname = name;
 
   let count;
-  FormHistory.count(obj, { handleResult: function (result) count = result,
+  FormHistory.count(obj, { handleResult: result => count = result,
                            handleError: function (error) {
                              do_throw("Error occurred searching form history: " + error);
                              deferred.reject(error)
@@ -75,7 +73,7 @@ function countEntries(name, message, check) {
   return deferred.promise;
 }
 
-function onHistoryReady() {
+function* onHistoryReady() {
   var hoursSinceMidnight = new Date().getHours();
   var minutesSinceMidnight = hoursSinceMidnight * 60 + new Date().getMinutes();
 
@@ -98,13 +96,14 @@ function onHistoryReady() {
 
   let publicList = yield Downloads.getList(Downloads.PUBLIC);
   let downloadPromise = promiseDownloadRemoved(publicList);
+  let formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear 10 minutes ago
   s.range = [now_uSec - 10*60*1000000, now_uSec];
-  s.sanitize();
+  yield s.sanitize();
   s.range = null;
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://10minutes.com"))),
@@ -155,12 +154,13 @@ function onHistoryReady() {
     ok((yield downloadExists(publicList, "fakefile-today")), "'Today' download should still be present");
 
   downloadPromise = promiseDownloadRemoved(publicList);
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear 1 hour
   Sanitizer.prefs.setIntPref("timeSpan", 1);
-  s.sanitize();
+  yield s.sanitize();
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://1hour.com"))),
@@ -204,13 +204,14 @@ function onHistoryReady() {
     ok((yield downloadExists(publicList, "fakefile-today")), "'Today' download should still be present");
   
   downloadPromise = promiseDownloadRemoved(publicList);
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear 1 hour 10 minutes
   s.range = [now_uSec - 70*60*1000000, now_uSec];
-  s.sanitize();
+  yield s.sanitize();
   s.range = null;
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://1hour10minutes.com"))),
@@ -249,12 +250,13 @@ function onHistoryReady() {
     ok((yield downloadExists(publicList, "fakefile-today")), "'Today' download should still be present");
 
   downloadPromise = promiseDownloadRemoved(publicList);
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear 2 hours
   Sanitizer.prefs.setIntPref("timeSpan", 2);
-  s.sanitize();
+  yield s.sanitize();
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://2hour.com"))),
@@ -287,15 +289,16 @@ function onHistoryReady() {
   ok((yield downloadExists(publicList, "fakefile-4-hour-10-minutes")), "4 hour 10 minute download should still be present");
   if (hoursSinceMidnight > 2)
     ok((yield downloadExists(publicList, "fakefile-today")), "'Today' download should still be present");
-  
+
   downloadPromise = promiseDownloadRemoved(publicList);
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear 2 hours 10 minutes
   s.range = [now_uSec - 130*60*1000000, now_uSec];
-  s.sanitize();
+  yield s.sanitize();
   s.range = null;
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://2hour10minutes.com"))),
@@ -326,12 +329,13 @@ function onHistoryReady() {
     ok((yield downloadExists(publicList, "fakefile-today")), "'Today' download should still be present");
 
   downloadPromise = promiseDownloadRemoved(publicList);
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear 4 hours
   Sanitizer.prefs.setIntPref("timeSpan", 3);
-  s.sanitize();
+  yield s.sanitize();
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://4hour.com"))),
@@ -358,13 +362,14 @@ function onHistoryReady() {
     ok((yield downloadExists(publicList, "fakefile-today")), "'Today' download should still be present");
 
   downloadPromise = promiseDownloadRemoved(publicList);
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear 4 hours 10 minutes
   s.range = [now_uSec - 250*60*1000000, now_uSec];
-  s.sanitize();
+  yield s.sanitize();
   s.range = null;
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://4hour10minutes.com"))),
@@ -393,12 +398,13 @@ function onHistoryReady() {
   } else {
     downloadPromise = Promise.resolve();
   }
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Clear Today
   Sanitizer.prefs.setIntPref("timeSpan", 4);
-  s.sanitize();
+  yield s.sanitize();
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   // Be careful.  If we add our objectss just before midnight, and sanitize
@@ -421,12 +427,13 @@ function onHistoryReady() {
   ok((yield downloadExists(publicList, "fakefile-old")), "Year old download should still be present");
 
   downloadPromise = promiseDownloadRemoved(publicList);
+  formHistoryPromise = promiseFormHistoryRemoved();
 
   // Choose everything
   Sanitizer.prefs.setIntPref("timeSpan", 0);
-  s.sanitize();
+  yield s.sanitize();
 
-  yield promiseFormHistoryRemoved();
+  yield formHistoryPromise;
   yield downloadPromise;
 
   ok(!(yield promiseIsURIVisited(makeURI("http://before-today.com"))),
@@ -470,11 +477,10 @@ function setupHistory() {
   let lastYear = new Date();
   lastYear.setFullYear(lastYear.getFullYear() - 1);
   addPlace(makeURI("http://before-today.com/"), "Before Today", lastYear.getTime() * 1000);
-
   PlacesUtils.asyncHistory.updatePlaces(places, {
-    handleError: function () ok(false, "Unexpected error in adding visit."),
-    handleResult: function () { },
-    handleCompletion: function () deferred.resolve()
+    handleError: () => ok(false, "Unexpected error in adding visit."),
+    handleResult: () => { },
+    handleCompletion: () => deferred.resolve()
   });
 
   return deferred.promise;
@@ -486,7 +492,7 @@ function setupFormHistory() {
     let deferred = Promise.defer();
 
     let results = [];
-    FormHistory.search(terms, params, { handleResult: function (result) results.push(result),
+    FormHistory.search(terms, params, { handleResult: result => results.push(result),
                                         handleError: function (error) {
                                           do_throw("Error occurred searching form history: " + error);
                                           deferred.reject(error);
