@@ -8,6 +8,7 @@ var Cc = Components.classes;
 var Ci = Components.interfaces;
 var Cu = Components.utils;
 
+Cu.import("resource://gre/modules/AppConstants.jsm");
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource://gre/modules/FileUtils.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
@@ -103,7 +104,7 @@ Bookmarks.prototype = {
         // In Safari, it is possible (though quite cumbersome) to move
         // bookmarks to the bookmarks root, which is the parent folder of
         // all bookmarks "collections".  That is somewhat in parallel with
-        // both the places root and the unfiled-bookmarks root. 
+        // both the places root and the unfiled-bookmarks root.
         // Because the former is only an implementation detail in our UI,
         // the unfiled root seems to be the best choice.
         folderGuid = PlacesUtils.bookmarks.unfiledGuid;
@@ -297,7 +298,7 @@ MainPreferencesPropertyList.prototype = {
   _readSync: function MPPL__readSync() {
     if ("_dict" in this)
       return this._dict;
-  
+
     let inputStream = Cc["@mozilla.org/network/file-input-stream;1"].
                       createInstance(Ci.nsIFileInputStream);
     inputStream.init(this._file, -1, -1, 0);
@@ -325,7 +326,7 @@ Preferences.prototype = {
 
         this._dict = aDict;
 
-        let invert = function(webkitVal) !webkitVal;
+        let invert = webkitVal => !webkitVal;
         this._set("AutoFillPasswords", "signon.rememberSignons");
         this._set("OpenNewTabsInFront", "browser.tabs.loadInBackground", invert);
         this._set("WebKitJavaScriptCanOpenWindowsAutomatically",
@@ -342,19 +343,23 @@ Preferences.prototype = {
         // Allowed                          TRUE      1
         // Allowed, originating site only   --        3
         this._set("WebKitDisplayImagesKey", "permissions.default.image",
-                  function(webkitVal) webkitVal ? 1 : 2);
+                  webkitVal => webkitVal ? 1 : 2);
 
-#ifdef XP_WIN
-        // Cookie-accept policy.
-        // For the OS X version, see WebFoundationCookieBehavior.
-        // Setting                    Safari          Firefox
-        // Always Accept              0               0
-        // Accept from Originating    2               1
-        // Never Accept               1               2
-        this._set("WebKitCookieStorageAcceptPolicy",
-          "network.cookie.cookieBehavior",
-          function(webkitVal) webkitVal == 0 ? 0 : webkitVal == 1 ? 2 : 1);
-#endif
+        if (AppConstants.platform == "win") {
+          // Cookie-accept policy.
+          // For the OS X version, see WebFoundationCookieBehavior.
+          // Setting                    Safari          Firefox
+          // Always Accept              0               0
+          // Accept from Originating    2               1
+          // Never Accept               1               2
+          let firefoxVal = 0;
+          if (webkitVal != 0) {
+            firefoxVal = webkitVal == 1 ? 2 : 1;
+          }
+          this._set("WebKitCookieStorageAcceptPolicy",
+            "network.cookie.cookieBehavior",
+            firefoxVal);
+        }
 
         this._migrateFontSettings();
         yield this._migrateDownloadsFolder();
@@ -533,10 +538,10 @@ SearchStrings.prototype = {
         if (aDict.has("RecentSearchStrings")) {
           let recentSearchStrings = aDict.get("RecentSearchStrings");
           if (recentSearchStrings && recentSearchStrings.length > 0) {
-            let changes = [{op: "add",
+            let changes = recentSearchStrings.map((searchString) => (
+                           {op: "add",
                             fieldname: "searchbar-history",
-                            value: searchString}
-                           for (searchString of recentSearchStrings)];
+                            value: searchString}));
             FormHistory.update(changes);
           }
         }
@@ -544,7 +549,6 @@ SearchStrings.prototype = {
   }
 };
 
-#ifdef XP_MACOSX
 // On OS X, the cookie-accept policy preference is stored in a separate
 // property list.
 // For the Windows version, check Preferences.migrate.
@@ -566,15 +570,18 @@ WebFoundationCookieBehavior.prototype = {
           // Accept from Originating    current page    1
           // Never Accept               never           2
           let acceptCookies = aDict.get("NSHTTPAcceptCookies");
-          let cookieValue = acceptCookies == "never" ? 2 :
-                            acceptCookies == "current page" ? 1 : 0;
+          let cookieValue = 0;
+          if (acceptCookies == "never") {
+            cookieValue = 2;
+          } else if (acceptCookies == "current page") {
+            cookieValue = 1;
+          }
           Services.prefs.setIntPref("network.cookie.cookieBehavior",
                                     cookieValue);
         }
       }.bind(this), aCallback));
   }
 };
-#endif
 
 function SafariProfileMigrator() {
 }
@@ -582,12 +589,12 @@ function SafariProfileMigrator() {
 SafariProfileMigrator.prototype = Object.create(MigratorPrototype);
 
 SafariProfileMigrator.prototype.getResources = function SM_getResources() {
-  let profileDir =
-#ifdef XP_MACOSX
-    FileUtils.getDir("ULibDir", ["Safari"], false);
-#else
-    FileUtils.getDir("AppData", ["Apple Computer", "Safari"], false);
-#endif
+  let profileDir;
+  if (AppConstants.platform == "macosx") {
+    profileDir = FileUtils.getDir("ULibDir", ["Safari"], false);
+  } else {
+    profileDir = FileUtils.getDir("AppData", ["Apple Computer", "Safari"], false);
+  }
   if (!profileDir.exists())
     return null;
 
@@ -601,20 +608,20 @@ SafariProfileMigrator.prototype.getResources = function SM_getResources() {
 
   pushProfileFileResource("History.plist", History);
   pushProfileFileResource("Bookmarks.plist", Bookmarks);
-  
+
   // The Reading List feature was introduced at the same time in Windows and
   // Mac versions of Safari.  Not surprisingly, they are stored in the same
   // format in both versions.  Surpsingly, only on Windows there is a
-  // separate property list for it.  This isn't #ifdefed out on mac, because
+  // separate property list for it.  This code is used on mac too, because
   // Apple may fix this at some point.
   pushProfileFileResource("ReadingList.plist", Bookmarks);
 
-  let prefsDir = 
-#ifdef XP_MACOSX
-    FileUtils.getDir("UsrPrfs", [], false);
-#else
-    FileUtils.getDir("AppData", ["Apple Computer", "Preferences"], false);
-#endif
+  let prefsDir;
+  if (AppConstants.platform == "macosx") {
+    prefsDir = FileUtils.getDir("UsrPrfs", [], false);
+  } else {
+    prefsDir = FileUtils.getDir("AppData", ["Apple Computer", "Preferences"], false);
+  }
 
   let prefs = this.mainPreferencesPropertyList;
   if (prefs) {
@@ -622,13 +629,13 @@ SafariProfileMigrator.prototype.getResources = function SM_getResources() {
     resources.push(new SearchStrings(prefs));
   }
 
-#ifdef XP_MACOSX
-  // On OS X, the cookie-accept policy preference is stored in a separate
-  // property list.
-  let wfFile = FileUtils.getFile("UsrPrfs", ["com.apple.WebFoundation.plist"]);
-  if (wfFile.exists())
-    resources.push(new WebFoundationCookieBehavior(wfFile));
-#endif
+  if (AppConstants.platform == "macosx") {
+    // On OS X, the cookie-accept policy preference is stored in a separate
+    // property list.
+    let wfFile = FileUtils.getFile("UsrPrfs", ["com.apple.WebFoundation.plist"]);
+    if (wfFile.exists())
+      resources.push(new WebFoundationCookieBehavior(wfFile));
+  }
 
   return resources;
 };
@@ -636,12 +643,12 @@ SafariProfileMigrator.prototype.getResources = function SM_getResources() {
 Object.defineProperty(SafariProfileMigrator.prototype, "mainPreferencesPropertyList", {
   get: function get_mainPreferencesPropertyList() {
     if (this._mainPreferencesPropertyList === undefined) {
-      let file = 
-#ifdef XP_MACOSX
-        FileUtils.getDir("UsrPrfs", [], false);
-#else
-        FileUtils.getDir("AppData", ["Apple Computer", "Preferences"], false);
-#endif
+      let file;
+      if (AppConstants.platform == "macosx") {
+        file = FileUtils.getDir("UsrPrfs", [], false);
+      } else {
+        file = FileUtils.getDir("AppData", ["Apple Computer", "Preferences"], false);
+      }
       if (file.exists()) {
         file.append("com.apple.Safari.plist");
         if (file.exists()) {
