@@ -400,7 +400,7 @@ function promiseAllButPrimaryWindowClosed() {
     }
   }
 
-  return Promise.all(windows.map(promiseWindowClosed));
+  return Promise.all(windows.map(BrowserTestUtils.closeWindow));
 }
 
 // Forget all closed windows.
@@ -438,7 +438,8 @@ function whenNewWindowLoaded(aOptions, aCallback) {
 
     win.addEventListener("load", function onLoad() {
       win.removeEventListener("load", onLoad);
-      resolve(promiseBrowserLoaded(win.gBrowser.selectedBrowser));
+      let browser = win.gBrowser.selectedBrowser;
+      promiseBrowserLoaded(browser).then(resolve);
     });
   });
 
@@ -448,34 +449,6 @@ function promiseNewWindowLoaded(aOptions) {
   return new Promise(resolve => whenNewWindowLoaded(aOptions, resolve));
 }
 
-/**
- * Chrome windows aren't closed synchronously. Provide a helper method to close
- * a window and wait until we received the "domwindowclosed" notification for it.
- */
-function promiseWindowClosed(win) {
-  let promise = new Promise(resolve => {
-    Services.obs.addObserver(function obs(subject, topic) {
-      if (subject == win) {
-        Services.obs.removeObserver(obs, topic);
-        resolve();
-      }
-    }, "domwindowclosed", false);
-  });
-
-  win.close();
-  return promise;
-}
-
-function runInContent(browser, func, arg, callback = null) {
-  let deferred = Promise.defer();
-
-  let mm = browser.messageManager;
-  mm.sendAsyncMessage("ss-test:run", {code: func.toSource()}, {arg: arg});
-  mm.addMessageListener("ss-test:runFinished", ({data}) => deferred.resolve(data));
-
-  return deferred.promise;
-}
-
 /**
  * This waits for the browser-delayed-startup-finished notification of a given
  * window. It indicates that the windows has loaded completely and is ready to
@@ -538,112 +511,6 @@ function promiseRemoveTab(tab) {
   return BrowserTestUtils.removeTab(tab);
 }
 
-/**
- * Returns a Promise that resolves once a remote <xul:browser> has experienced
- * a crash. Also does the job of cleaning up the minidump of the crash.
- *
- * @param browser
- *        The <xul:browser> that will crash
- * @return Promise
- */
-function crashBrowser(browser) {
-  /**
-   * Returns the directory where crash dumps are stored.
-   *
-   * @return nsIFile
-   */
-  function getMinidumpDirectory() {
-    let dir = Services.dirsvc.get('ProfD', Ci.nsIFile);
-    dir.append("minidumps");
-    return dir;
-  }
-
-  /**
-   * Removes a file from a directory. This is a no-op if the file does not
-   * exist.
-   *
-   * @param directory
-   *        The nsIFile representing the directory to remove from.
-   * @param filename
-   *        A string for the file to remove from the directory.
-   */
-  function removeFile(directory, filename) {
-    let file = directory.clone();
-    file.append(filename);
-    if (file.exists()) {
-      file.remove(false);
-    }
-  }
-
-  // This frame script is injected into the remote browser, and used to
-  // intentionally crash the tab. We crash by using js-ctypes and dereferencing
-  // a bad pointer. The crash should happen immediately upon loading this
-  // frame script.
-  let frame_script = () => {
-    const Cu = Components.utils;
-    Cu.import("resource://gre/modules/ctypes.jsm");
-
-    let dies = function() {
-      privateNoteIntentionalCrash();
-      let zero = new ctypes.intptr_t(8);
-      let badptr = ctypes.cast(zero, ctypes.PointerType(ctypes.int32_t));
-      badptr.contents
-    };
-
-    dump("Et tu, Brute?");
-    dies();
-  }
-
-  let crashCleanupPromise = new Promise((resolve, reject) => {
-    let observer = (subject, topic, data) => {
-      is(topic, 'ipc:content-shutdown', 'Received correct observer topic.');
-      ok(subject instanceof Ci.nsIPropertyBag2,
-         'Subject implements nsIPropertyBag2.');
-      // we might see this called as the process terminates due to previous tests.
-      // We are only looking for "abnormal" exits...
-      if (!subject.hasKey("abnormal")) {
-        info("This is a normal termination and isn't the one we are looking for...");
-        return;
-      }
-
-      let dumpID;
-      if ('nsICrashReporter' in Ci) {
-        dumpID = subject.getPropertyAsAString('dumpID');
-        ok(dumpID, "dumpID is present and not an empty string");
-      }
-
-      if (dumpID) {
-        let minidumpDirectory = getMinidumpDirectory();
-        removeFile(minidumpDirectory, dumpID + '.dmp');
-        removeFile(minidumpDirectory, dumpID + '.extra');
-      }
-
-      Services.obs.removeObserver(observer, 'ipc:content-shutdown');
-      info("Crash cleaned up");
-      resolve();
-    };
-
-    Services.obs.addObserver(observer, 'ipc:content-shutdown');
-  });
-
-  let aboutTabCrashedLoadPromise = new Promise((resolve, reject) => {
-    browser.addEventListener("AboutTabCrashedLoad", function onCrash() {
-      browser.removeEventListener("AboutTabCrashedLoad", onCrash, false);
-      info("about:tabcrashed loaded");
-      resolve();
-    }, false, true);
-  });
-
-  // This frame script will crash the remote browser as soon as it is
-  // evaluated.
-  let mm = browser.messageManager;
-  mm.loadFrameScript("data:,(" + frame_script.toString() + ")();", false);
-  return Promise.all([crashCleanupPromise, aboutTabCrashedLoadPromise]).then(() => {
-    let tab = gBrowser.getTabForBrowser(browser);
-    is(tab.getAttribute("crashed"), "true", "Tab should be marked as crashed");
-  });
-}
-
 // Write DOMSessionStorage data to the given browser.
 function modifySessionStorage(browser, data, options = {}) {
   return ContentTask.spawn(browser, [data, options], function* ([data, options]) {
@@ -656,13 +523,13 @@ function modifySessionStorage(browser, data, options = {}) {
     let storage = frame.sessionStorage;
 
     return new Promise(resolve => {
-      addEventListener("MozStorageChanged", function onStorageChanged(event) {
+      addEventListener("MozSessionStorageChanged", function onStorageChanged(event) {
         if (event.storageArea == storage) {
           keys.delete(event.key);
         }
 
         if (keys.size == 0) {
-          removeEventListener("MozStorageChanged", onStorageChanged, true);
+          removeEventListener("MozSessionStorageChanged", onStorageChanged, true);
           resolve();
         }
       }, true);
@@ -673,3 +540,23 @@ function modifySessionStorage(browser, data, options = {}) {
     });
   });
 }
+
+function pushPrefs(...aPrefs) {
+  return new Promise(resolve => {
+    SpecialPowers.pushPrefEnv({"set": aPrefs}, resolve);
+  });
+}
+
+function popPrefs() {
+  return new Promise(resolve => {
+    SpecialPowers.popPrefEnv(resolve);
+  });
+}
+
+function* checkScroll(tab, expected, msg) {
+  let browser = tab.linkedBrowser;
+  yield TabStateFlusher.flush(browser);
+
+  let scroll = JSON.parse(ss.getTabState(tab)).scroll || null;
+  is(JSON.stringify(scroll), JSON.stringify(expected), msg);
+}
