@@ -50,6 +50,9 @@
 #include <mbstring.h>
 #include <shlwapi.h>
 
+#include <lm.h>
+#undef ACCESS_READ
+
 #ifndef MAX_BUF
 #define MAX_BUF 4096
 #endif
@@ -86,38 +89,6 @@ OpenKeyForReading(HKEY aKeyRoot, const nsAString& aKeyName, HKEY* aKey)
   return NS_OK;
 }
 
-static bool
-GetPrefString(const nsCString& aPrefName, nsAString& aValue)
-{
-  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
-  if (!prefs) {
-    return false;
-  }
-
-  nsAutoCString prefCStr;
-  nsresult rv = prefs->GetCharPref(aPrefName.get(),
-                                   getter_Copies(prefCStr));
-  if (NS_FAILED(rv)) {
-    return false;
-  }
-  CopyUTF8toUTF16(prefCStr, aValue);
-
-  return true;
-}
-
-static bool
-SetPrefString(const nsCString& aPrefName, const nsString& aValue)
-{
-  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID));
-  if (!prefs) {
-    return false;
-  }
-
-  nsresult rv = prefs->SetCharPref(aPrefName.get(),
-                                   NS_ConvertUTF16toUTF8(aValue).get());
-  return NS_SUCCEEDED(rv);
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 // Default Browser Registry Settings
 //
@@ -355,7 +326,7 @@ nsWindowsShellService::ShortcutMaintenance()
 }
 
 static bool
-IsAARDefault(const nsRefPtr<IApplicationAssociationRegistration>& pAAR,
+IsAARDefault(const RefPtr<IApplicationAssociationRegistration>& pAAR,
              LPCWSTR aClassName)
 {
   // Make sure the Prog ID matches what we have
@@ -376,147 +347,9 @@ IsAARDefault(const nsRefPtr<IApplicationAssociationRegistration>& pAAR,
 }
 
 static void
-GetUserChoiceKeyName(LPCWSTR aClassName, bool aIsProtocol,
-                     nsAString& aKeyName)
+IsDefaultBrowserWin8(bool aCheckAllTypes, bool* aIsDefaultBrowser)
 {
-  aKeyName.AssignLiteral(aIsProtocol
-    ? "Software\\Microsoft\\Windows\\Shell\\Associations\\UrlAssociations\\"
-    : "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\");
-  aKeyName.Append(aClassName);
-  aKeyName.AppendLiteral("\\UserChoice");
-}
-
-static void
-GetHashPrefName(LPCWSTR aClassName, nsACString& aPrefName)
-{
-  aPrefName.AssignLiteral("browser.shell.associationHash.");
-  aPrefName.Append(NS_ConvertUTF16toUTF8(*aClassName == L'.' ? aClassName + 1
-                                                             : aClassName));
-}
-
-static bool
-SaveWin8RegistryHash(const nsRefPtr<IApplicationAssociationRegistration>& pAAR,
-                     LPCWSTR aClassName)
-{
-  bool isProtocol = *aClassName != L'.';
-  bool isDefault = IsAARDefault(pAAR, aClassName);
-  // We can save the value only if Firefox is the default.
-  if (!isDefault) {
-    return isDefault;
-  }
-
-  nsAutoString keyName;
-  GetUserChoiceKeyName(aClassName, isProtocol, keyName);
-
-  nsCOMPtr<nsIWindowsRegKey> regKey =
-    do_CreateInstance("@mozilla.org/windows-registry-key;1");
-  if (!regKey) {
-    return isDefault;
-  }
-
-  nsresult rv = regKey->Open(nsIWindowsRegKey::ROOT_KEY_CURRENT_USER,
-                             keyName, nsIWindowsRegKey::ACCESS_READ);
-  if (NS_FAILED(rv)) {
-    return isDefault;
-  }
-
-  nsAutoString hash;
-  rv = regKey->ReadStringValue(NS_LITERAL_STRING("Hash"), hash);
-  if (NS_FAILED(rv)) {
-    return isDefault;
-  }
-
-  nsAutoCString prefName;
-  GetHashPrefName(aClassName, prefName);
-  SetPrefString(prefName, hash);
-
-  return isDefault;
-}
-
-static bool
-RestoreWin8RegistryHash(const nsRefPtr<IApplicationAssociationRegistration>& pAAR,
-                        LPCWSTR aClassName)
-{
-  nsAutoCString prefName;
-  GetHashPrefName(aClassName, prefName);
-  nsAutoString hash;
-  if (!GetPrefString(prefName, hash)) {
-    return false;
-  }
-
-  bool isProtocol = *aClassName != L'.';
-  nsString progId = isProtocol ? NS_LITERAL_STRING("FirefoxURL")
-                               : NS_LITERAL_STRING("FirefoxHTML");
-
-  nsAutoString keyName;
-  GetUserChoiceKeyName(aClassName, isProtocol, keyName);
-
-  nsCOMPtr<nsIWindowsRegKey> regKey =
-    do_CreateInstance("@mozilla.org/windows-registry-key;1");
-  if (!regKey) {
-    return false;
-  }
-
-  nsresult rv = regKey->Open(nsIWindowsRegKey::ROOT_KEY_CURRENT_USER,
-                             keyName, nsIWindowsRegKey::ACCESS_READ);
-  if (NS_SUCCEEDED(rv)) {
-    nsAutoString currValue;
-    if (NS_SUCCEEDED(regKey->ReadStringValue(NS_LITERAL_STRING("Hash"),
-                                             currValue)) &&
-        currValue.Equals(hash) &&
-        NS_SUCCEEDED(regKey->ReadStringValue(NS_LITERAL_STRING("ProgId"),
-                                             currValue)) &&
-        currValue.Equals(progId)) {
-      // The value is already set.
-      return true;
-    }
-    // We need to close this explicitly because nsIWindowsRegKey::SetKey
-    // does not close the old key.
-    regKey->Close();
-  }
-
-  // We have to use the registry function directly because
-  // nsIWindowsRegKey::Create will only return NS_ERROR_FAILURE
-  // on failure.
-  HKEY theKey;
-  DWORD res = ::RegOpenKeyExW(HKEY_CURRENT_USER, keyName.get(), 0,
-                              KEY_READ | KEY_SET_VALUE, &theKey);
-  if (REG_FAILED(res)) {
-    if (res != ERROR_ACCESS_DENIED && res != ERROR_FILE_NOT_FOUND) {
-      return false;
-    }
-    if (res == ERROR_ACCESS_DENIED) {
-      res = ::RegDeleteKeyW(HKEY_CURRENT_USER, keyName.get());
-      if (REG_FAILED(res)) {
-        return false;
-      }
-    }
-    res = ::RegCreateKeyExW(HKEY_CURRENT_USER, keyName.get(), 0,
-                            nullptr, 0, KEY_READ | KEY_SET_VALUE,
-                            nullptr, &theKey, nullptr);
-    if (REG_FAILED(res)) {
-      return false;
-    }
-  }
-  regKey->SetKey(theKey);
-
-  rv = regKey->WriteStringValue(NS_LITERAL_STRING("Hash"), hash);
-  if (NS_FAILED(rv)) {
-    return false;
-  }
-
-  rv = regKey->WriteStringValue(NS_LITERAL_STRING("ProgId"), progId);
-  if (NS_FAILED(rv)) {
-    return false;
-  }
-
-  return IsAARDefault(pAAR, aClassName);
-}
-
-static void
-SaveWin8RegistryHashes(bool aCheckAllTypes, bool* aIsDefaultBrowser)
-{
-  nsRefPtr<IApplicationAssociationRegistration> pAAR;
+  RefPtr<IApplicationAssociationRegistration> pAAR;
   HRESULT hr = CoCreateInstance(CLSID_ApplicationAssociationRegistration,
                                 nullptr,
                                 CLSCTX_INPROC,
@@ -526,48 +359,14 @@ SaveWin8RegistryHashes(bool aCheckAllTypes, bool* aIsDefaultBrowser)
     return;
   }
 
-  bool res = SaveWin8RegistryHash(pAAR, L"http");
+  bool res = IsAARDefault(pAAR, L"http");
   if (*aIsDefaultBrowser) {
     *aIsDefaultBrowser = res;
   }
-  SaveWin8RegistryHash(pAAR, L"https");
-  SaveWin8RegistryHash(pAAR, L"ftp");
-  res = SaveWin8RegistryHash(pAAR, L".html");
+  res = IsAARDefault(pAAR, L".html");
   if (*aIsDefaultBrowser && aCheckAllTypes) {
     *aIsDefaultBrowser = res;
   }
-  SaveWin8RegistryHash(pAAR, L".htm");
-  SaveWin8RegistryHash(pAAR, L".shtml");
-  SaveWin8RegistryHash(pAAR, L".xhtml");
-  SaveWin8RegistryHash(pAAR, L".xht");
-}
-
-static bool
-RestoreWin8RegistryHashes(bool aClaimAllTypes)
-{
-  nsRefPtr<IApplicationAssociationRegistration> pAAR;
-  HRESULT hr = CoCreateInstance(CLSID_ApplicationAssociationRegistration,
-                                nullptr,
-                                CLSCTX_INPROC,
-                                IID_IApplicationAssociationRegistration,
-                                getter_AddRefs(pAAR));
-  if (FAILED(hr)) {
-    return false;
-  }
-
-  bool res = RestoreWin8RegistryHash(pAAR, L"http");
-  res = RestoreWin8RegistryHash(pAAR, L"https") && res;
-  RestoreWin8RegistryHash(pAAR, L"ftp");
-  bool res2 = RestoreWin8RegistryHash(pAAR, L".html");
-  res2 = RestoreWin8RegistryHash(pAAR, L".htm") && res2;
-  if (aClaimAllTypes) {
-    res = res && res2;
-  }
-  RestoreWin8RegistryHash(pAAR, L".shtml");
-  RestoreWin8RegistryHash(pAAR, L".xhtml");
-  RestoreWin8RegistryHash(pAAR, L".xht");
-
-  return res;
 }
 
 /*
@@ -580,7 +379,7 @@ bool
 nsWindowsShellService::IsDefaultBrowserVista(bool aCheckAllTypes,
                                              bool* aIsDefaultBrowser)
 {
-  nsRefPtr<IApplicationAssociationRegistration> pAAR;
+  RefPtr<IApplicationAssociationRegistration> pAAR;
   HRESULT hr = CoCreateInstance(CLSID_ApplicationAssociationRegistration,
                                 nullptr,
                                 CLSCTX_INPROC,
@@ -608,12 +407,6 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
                                         bool aForAllTypes,
                                         bool* aIsDefaultBrowser)
 {
-  // If this is the first browser window, maintain internal state that we've
-  // checked this session (so that subsequent window opens don't show the
-  // default browser dialog).
-  if (aStartupCheck)
-    mCheckedThisSession = true;
-
   // Assume we're the default unless one of the several checks below tell us
   // otherwise.
   *aIsDefaultBrowser = true;
@@ -701,7 +494,7 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
   if (*aIsDefaultBrowser) {
     IsDefaultBrowserVista(aForAllTypes, aIsDefaultBrowser);
     if (IsWin8OrLater()) {
-      SaveWin8RegistryHashes(aForAllTypes, aIsDefaultBrowser);
+      IsDefaultBrowserWin8(aForAllTypes, aIsDefaultBrowser);
     }
   }
 
@@ -807,13 +600,6 @@ nsWindowsShellService::IsDefaultBrowser(bool aStartupCheck,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWindowsShellService::GetCanSetDesktopBackground(bool* aResult)
-{
-  *aResult = true;
-  return NS_OK;
-}
-
 static nsresult
 DynSHOpenWithDialog(HWND hwndParent, const OPENASINFO *poainfo)
 {
@@ -859,9 +645,96 @@ nsWindowsShellService::LaunchControlPanelDefaultsSelectionUI()
   return SUCCEEDED(hr) ? NS_OK : NS_ERROR_FAILURE;
 }
 
+nsresult
+nsWindowsShellService::LaunchControlPanelDefaultPrograms()
+{
+  // Default Programs is a Vista+ feature
+  if (!IsVistaOrLater()) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // Build the path control.exe path safely
+  WCHAR controlEXEPath[MAX_PATH + 1] = { '\0' };
+  if (!GetSystemDirectoryW(controlEXEPath, MAX_PATH)) {
+    return NS_ERROR_FAILURE;
+  }
+  LPCWSTR controlEXE = L"control.exe";
+  if (wcslen(controlEXEPath) + wcslen(controlEXE) >= MAX_PATH) {
+    return NS_ERROR_FAILURE;
+  }
+  if (!PathAppendW(controlEXEPath, controlEXE)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  WCHAR params[] = L"control.exe /name Microsoft.DefaultPrograms /page pageDefaultProgram";
+  STARTUPINFOW si = {sizeof(si), 0};
+  si.dwFlags = STARTF_USESHOWWINDOW;
+  si.wShowWindow = SW_SHOWDEFAULT;
+  PROCESS_INFORMATION pi = {0};
+  if (!CreateProcessW(controlEXEPath, params, nullptr, nullptr, FALSE,
+                      0, nullptr, nullptr, &si, &pi)) {
+    return NS_ERROR_FAILURE;
+  }
+  CloseHandle(pi.hProcess);
+  CloseHandle(pi.hThread);
+
+  return NS_OK;
+}
+
+static bool
+IsWindowsLogonConnected()
+{
+  WCHAR userName[UNLEN + 1];
+  DWORD size = ArrayLength(userName);
+  if (!GetUserNameW(userName, &size)) {
+    return false;
+  }
+
+  LPUSER_INFO_24 info;
+  if (NetUserGetInfo(nullptr, userName, 24, (LPBYTE *)&info)
+      != NERR_Success) {
+    return false;
+  }
+  bool connected = info->usri24_internet_identity;
+  NetApiBufferFree(info);
+
+  return connected;
+}
+
+static bool
+SettingsAppBelievesConnected()
+{
+  nsresult rv;
+  nsCOMPtr<nsIWindowsRegKey> regKey =
+    do_CreateInstance("@mozilla.org/windows-registry-key;1", &rv);
+  if (NS_FAILED(rv)) {
+    return false;
+  }
+
+  rv = regKey->Open(nsIWindowsRegKey::ROOT_KEY_CURRENT_USER,
+                    NS_LITERAL_STRING("SOFTWARE\\Microsoft\\Windows\\Shell\\Associations"),
+                    nsIWindowsRegKey::ACCESS_READ);
+  if (NS_FAILED(rv)) {
+    return false;
+  }
+
+  uint32_t value;
+  rv = regKey->ReadIntValue(NS_LITERAL_STRING("IsConnectedAtLogon"), &value);
+  if (NS_FAILED(rv)) {
+    return false;
+  }
+
+  return !!value;
+}
+
 nsresult
 nsWindowsShellService::LaunchModernSettingsDialogDefaultApps()
 {
+  if (!IsWindowsLogonConnected() && SettingsAppBelievesConnected()) {
+    // Use the classic Control Panel to work around a bug of Windows 10.
+    return LaunchControlPanelDefaultPrograms();
+  }
+
   IApplicationActivationManager* pActivator;
   HRESULT hr = CoCreateInstance(CLSID_ApplicationActivationManager,
                                 nullptr,
@@ -875,6 +748,15 @@ nsWindowsShellService::LaunchModernSettingsDialogDefaultApps()
            L"windows.immersivecontrolpanel_cw5n1h2txyewy"
            L"!microsoft.windows.immersivecontrolpanel",
            L"page=SettingsPageAppsDefaults", AO_NONE, &pid);
+    if (SUCCEEDED(hr)) {
+      // Do not check error because we could at least open
+      // the "Default apps" setting.
+      pActivator->ActivateApplication(
+             L"windows.immersivecontrolpanel_cw5n1h2txyewy"
+             L"!microsoft.windows.immersivecontrolpanel",
+             L"page=SettingsPageAppsDefaults"
+             L"&target=SystemSettings_DefaultApps_Browser", AO_NONE, &pid);
+    }
     pActivator->Release();
     return SUCCEEDED(hr) ? NS_OK : NS_ERROR_FAILURE;
   }
@@ -937,8 +819,7 @@ nsWindowsShellService::SetDefaultBrowser(bool aClaimAllTypes, bool aForAllUsers)
   }
 
   nsresult rv = LaunchHelper(appHelperPath);
-  if (NS_SUCCEEDED(rv) && IsWin8OrLater() &&
-      !RestoreWin8RegistryHashes(aClaimAllTypes)) {
+  if (NS_SUCCEEDED(rv) && IsWin8OrLater()) {
     if (aClaimAllTypes) {
       if (IsWin10OrLater()) {
         rv = LaunchModernSettingsDialogDefaultApps();
@@ -968,8 +849,6 @@ nsWindowsShellService::SetDefaultBrowser(bool aClaimAllTypes, bool aForAllUsers)
       if (NS_FAILED(rv)) {
         rv = LaunchControlPanelDefaultsSelectionUI();
       }
-      bool isDefault;
-      SaveWin8RegistryHashes(aClaimAllTypes, &isDefault);
     }
   }
 
@@ -984,92 +863,6 @@ nsWindowsShellService::SetDefaultBrowser(bool aClaimAllTypes, bool aForAllUsers)
   return rv;
 }
 
-NS_IMETHODIMP
-nsWindowsShellService::GetShouldSkipCheckDefaultBrowser(bool* aResult)
-{
-  NS_ENSURE_ARG_POINTER(aResult);
-
-  nsresult rv;
-  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  rv = prefs->GetBoolPref(PREF_SKIPDEFAULTBROWSERCHECK, aResult);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  if (*aResult) {
-    // Only skip the default browser check once. The next attempt in
-    // a new session should proceed.
-    return prefs->SetBoolPref(PREF_SKIPDEFAULTBROWSERCHECK, false);
-  }
-
-  int32_t defaultBrowserCheckCount;
-  rv = prefs->GetIntPref(PREF_DEFAULTBROWSERCHECKCOUNT,
-                         &defaultBrowserCheckCount);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  if (defaultBrowserCheckCount < 4) {
-    *aResult = false;
-    return prefs->SetIntPref(PREF_DEFAULTBROWSERCHECKCOUNT,
-                             defaultBrowserCheckCount + 1);
-  }
-
-  // Disable the default browser check after three attempts.
-  // Don't modify PREF_CHECKDEFAULTBROWSER since that is a
-  // user-initiated action and it shouldn't get re-enabled
-  // if it has been user disabled.
-  *aResult = true;
-  return rv;
-}
-
-NS_IMETHODIMP
-nsWindowsShellService::GetShouldCheckDefaultBrowser(bool* aResult)
-{
-  NS_ENSURE_ARG_POINTER(aResult);
-
-  // If we've already checked, the browser has been started and this is a
-  // new window open, and we don't want to check again.
-  if (mCheckedThisSession) {
-    *aResult = false;
-    return NS_OK;
-  }
-
-  nsresult rv;
-#ifndef RELEASE_BUILD
-  bool skipDefaultBrowserCheck;
-  rv = GetShouldSkipCheckDefaultBrowser(&skipDefaultBrowserCheck);
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-  if (skipDefaultBrowserCheck) {
-    *aResult = false;
-    return rv;
-  }
-#endif
-
-  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  return prefs->GetBoolPref(PREF_CHECKDEFAULTBROWSER, aResult);
-}
-
-NS_IMETHODIMP
-nsWindowsShellService::SetShouldCheckDefaultBrowser(bool aShouldCheck)
-{
-  nsresult rv;
-  nsCOMPtr<nsIPrefBranch> prefs(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  return prefs->SetBoolPref(PREF_CHECKDEFAULTBROWSER, aShouldCheck);
-}
-
 static nsresult
 WriteBitmap(nsIFile* aFile, imgIContainer* aImage)
 {
@@ -1411,8 +1204,7 @@ nsWindowsShellService::SetDesktopBackgroundColor(uint32_t aColor)
   return regKey->Close();
 }
 
-nsWindowsShellService::nsWindowsShellService() : 
-  mCheckedThisSession(false) 
+nsWindowsShellService::nsWindowsShellService()
 {
 }
 
