@@ -17,7 +17,6 @@ namespace dom {
 enum StructuredCloneTags {
   SCTAG_BASE = JS_SCTAG_USER_MIN,
 
-  // These tags are used only for main thread structured clone.
   SCTAG_DOM_BLOB,
 
   // This tag is obsolete and exists only for backwards compatibility with
@@ -46,6 +45,15 @@ enum StructuredCloneTags {
 
   SCTAG_DOM_RTC_CERTIFICATE,
 
+  SCTAG_DOM_FORMDATA,
+
+  // This tag is for OffscreenCanvas.
+  SCTAG_DOM_CANVAS,
+
+  SCTAG_DOM_EXPANDED_PRINCIPAL,
+
+  SCTAG_DOM_DIRECTORY,
+
   SCTAG_DOM_MAX
 };
 
