@@ -5,21 +5,26 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "nsHostObjectProtocolHandler.h"
-#include "nsHostObjectURI.h"
-#include "nsError.h"
-#include "nsClassHashtable.h"
-#include "nsNetUtil.h"
-#include "nsIPrincipal.h"
+
 #include "DOMMediaStream.h"
-#include "mozilla/dom/MediaSource.h"
-#include "nsIMemoryReporter.h"
+#include "mozilla/dom/Exceptions.h"
 #include "mozilla/dom/File.h"
-#include "mozilla/Preferences.h"
+#include "mozilla/dom/MediaSource.h"
 #include "mozilla/LoadInfo.h"
+#include "mozilla/ModuleUtils.h"
+#include "mozilla/Preferences.h"
+#include "nsClassHashtable.h"
+#include "nsContentUtils.h"
+#include "nsError.h"
+#include "nsHostObjectURI.h"
+#include "nsIMemoryReporter.h"
+#include "nsIPrincipal.h"
+#include "nsIUUIDGenerator.h"
+#include "nsNetUtil.h"
 
 using mozilla::dom::BlobImpl;
 using mozilla::ErrorResult;
-using mozilla::LoadInfo;
+using mozilla::net::LoadInfo;
 
 // -----------------------------------------------------------------------
 // Hash table
@@ -64,15 +69,125 @@ class BlobURLsReporter final : public nsIMemoryReporter
   NS_IMETHOD CollectReports(nsIHandleReportCallback* aCallback,
                             nsISupports* aData, bool aAnonymize) override
   {
-    EnumArg env;
-    env.mCallback = aCallback;
-    env.mData = aData;
-    env.mAnonymize = aAnonymize;
-
-    if (gDataTable) {
-      gDataTable->EnumerateRead(CountCallback, &env);
-      gDataTable->EnumerateRead(ReportCallback, &env);
+    if (!gDataTable) {
+      return NS_OK;
     }
+
+    nsDataHashtable<nsPtrHashKey<nsIDOMBlob>, uint32_t> refCounts;
+
+    // Determine number of URLs per blob, to handle the case where it's > 1.
+    for (auto iter = gDataTable->Iter(); !iter.Done(); iter.Next()) {
+      nsCOMPtr<nsIDOMBlob> blob =
+        do_QueryInterface(iter.UserData()->mObject);
+      if (blob) {
+        refCounts.Put(blob, refCounts.Get(blob) + 1);
+      }
+    }
+
+    for (auto iter = gDataTable->Iter(); !iter.Done(); iter.Next()) {
+      nsCStringHashKey::KeyType key = iter.Key();
+      DataInfo* info = iter.UserData();
+
+      nsCOMPtr<nsIDOMBlob> tmp = do_QueryInterface(info->mObject);
+      RefPtr<mozilla::dom::Blob> blob =
+        static_cast<mozilla::dom::Blob*>(tmp.get());
+
+      if (blob) {
+        NS_NAMED_LITERAL_CSTRING(desc,
+          "A blob URL allocated with URL.createObjectURL; the referenced "
+          "blob cannot be freed until all URLs for it have been explicitly "
+          "invalidated with URL.revokeObjectURL.");
+        nsAutoCString path, url, owner, specialDesc;
+        nsCOMPtr<nsIURI> principalURI;
+        uint64_t size = 0;
+        uint32_t refCount = 1;
+        DebugOnly<bool> blobWasCounted;
+
+        blobWasCounted = refCounts.Get(blob, &refCount);
+        MOZ_ASSERT(blobWasCounted);
+        MOZ_ASSERT(refCount > 0);
+
+        bool isMemoryFile = blob->IsMemoryFile();
+
+        if (isMemoryFile) {
+          ErrorResult rv;
+          size = blob->GetSize(rv);
+          if (NS_WARN_IF(rv.Failed())) {
+            rv.SuppressException();
+            size = 0;
+          }
+        }
+
+        path = isMemoryFile ? "memory-blob-urls/" : "file-blob-urls/";
+        BuildPath(path, key, info, aAnonymize);
+
+        if (refCount > 1) {
+          nsAutoCString addrStr;
+
+          addrStr = "0x";
+          addrStr.AppendInt((uint64_t)(nsIDOMBlob*)blob, 16);
+
+          path += " ";
+          path.AppendInt(refCount);
+          path += "@";
+          path += addrStr;
+
+          specialDesc = desc;
+          specialDesc += "\n\nNOTE: This blob (address ";
+          specialDesc += addrStr;
+          specialDesc += ") has ";
+          specialDesc.AppendInt(refCount);
+          specialDesc += " URLs.";
+          if (isMemoryFile) {
+            specialDesc += " Its size is divided ";
+            specialDesc += refCount > 2 ? "among" : "between";
+            specialDesc += " them in this report.";
+          }
+        }
+
+        const nsACString& descString = specialDesc.IsEmpty()
+            ? static_cast<const nsACString&>(desc)
+            : static_cast<const nsACString&>(specialDesc);
+        if (isMemoryFile) {
+          aCallback->Callback(EmptyCString(),
+              path,
+              KIND_OTHER,
+              UNITS_BYTES,
+              size / refCount,
+              descString,
+              aData);
+        } else {
+          aCallback->Callback(EmptyCString(),
+              path,
+              KIND_OTHER,
+              UNITS_COUNT,
+              1,
+              descString,
+              aData);
+        }
+      } else {
+        // Just report the path for the DOMMediaStream or MediaSource.
+        nsCOMPtr<mozilla::dom::MediaSource>
+          ms(do_QueryInterface(info->mObject));
+        nsAutoCString path;
+        path = ms ? "media-source-urls/" : "dom-media-stream-urls/";
+        BuildPath(path, key, info, aAnonymize);
+
+        NS_NAMED_LITERAL_CSTRING(desc,
+          "An object URL allocated with URL.createObjectURL; the referenced "
+          "data cannot be freed until all URLs for it have been explicitly "
+          "invalidated with URL.revokeObjectURL.");
+
+        aCallback->Callback(EmptyCString(),
+            path,
+            KIND_OTHER,
+            UNITS_COUNT,
+            1,
+            desc,
+            aData);
+      }
+    }
+
     return NS_OK;
   }
 
@@ -88,11 +203,7 @@ class BlobURLsReporter final : public nsIMemoryReporter
       return;
     }
 
-    nsresult rv;
-    nsIXPConnect* xpc = nsContentUtils::XPConnect();
-    nsCOMPtr<nsIStackFrame> frame;
-    rv = xpc->GetCurrentJSStack(getter_AddRefs(frame));
-    NS_ENSURE_SUCCESS_VOID(rv);
+    nsCOMPtr<nsIStackFrame> frame = dom::GetCurrentJSStack(maxFrames);
 
     nsAutoCString origin;
     nsCOMPtr<nsIURI> principalURI;
@@ -101,12 +212,17 @@ class BlobURLsReporter final : public nsIMemoryReporter
       principalURI->GetPrePath(origin);
     }
 
-    for (uint32_t i = 0; i < maxFrames && frame; ++i) {
+    // If we got a frame, we better have a current JSContext.  This is cheating
+    // a bit; ideally we'd have our caller pass in a JSContext, or have
+    // GetCurrentJSStack() hand out the JSContext it found.
+    JSContext* cx = frame ? nsContentUtils::GetCurrentJSContext() : nullptr;
+
+    for (uint32_t i = 0; frame; ++i) {
       nsString fileNameUTF16;
       int32_t lineNumber = 0;
 
-      frame->GetFilename(fileNameUTF16);
-      frame->GetLineNumber(&lineNumber);
+      frame->GetFilename(cx, fileNameUTF16);
+      frame->GetLineNumber(cx, &lineNumber);
 
       if (!fileNameUTF16.IsEmpty()) {
         NS_ConvertUTF16toUTF8 fileName(fileNameUTF16);
@@ -133,36 +249,16 @@ class BlobURLsReporter final : public nsIMemoryReporter
         stack += ")/";
       }
 
-      rv = frame->GetCaller(getter_AddRefs(frame));
+      nsCOMPtr<nsIStackFrame> caller;
+      nsresult rv = frame->GetCaller(cx, getter_AddRefs(caller));
       NS_ENSURE_SUCCESS_VOID(rv);
+      caller.swap(frame);
     }
   }
 
  private:
   ~BlobURLsReporter() {}
 
-  struct EnumArg {
-    nsIHandleReportCallback* mCallback;
-    nsISupports* mData;
-    bool mAnonymize;
-    nsDataHashtable<nsPtrHashKey<nsIDOMBlob>, uint32_t> mRefCounts;
-  };
-
-  // Determine number of URLs per blob, to handle the case where it's > 1.
-  static PLDHashOperator CountCallback(nsCStringHashKey::KeyType aKey,
-                                       DataInfo* aInfo,
-                                       void* aUserArg)
-  {
-    EnumArg* envp = static_cast<EnumArg*>(aUserArg);
-    nsCOMPtr<nsIDOMBlob> blob;
-
-    blob = do_QueryInterface(aInfo->mObject);
-    if (blob) {
-      envp->mRefCounts.Put(blob, envp->mRefCounts.Get(blob) + 1);
-    }
-    return PL_DHASH_NEXT;
-  }
-
   static void BuildPath(nsAutoCString& path,
                         nsCStringHashKey::KeyType aKey,
                         DataInfo* aInfo,
@@ -199,112 +295,6 @@ class BlobURLsReporter final : public nsIMemoryReporter
       path += url;
     }
   }
-
-  static PLDHashOperator ReportCallback(nsCStringHashKey::KeyType aKey,
-                                        DataInfo* aInfo,
-                                        void* aUserArg)
-  {
-    EnumArg* envp = static_cast<EnumArg*>(aUserArg);
-    nsCOMPtr<nsIDOMBlob> tmp = do_QueryInterface(aInfo->mObject);
-    nsRefPtr<mozilla::dom::Blob> blob = static_cast<mozilla::dom::Blob*>(tmp.get());
-
-    if (blob) {
-      NS_NAMED_LITERAL_CSTRING
-        (desc, "A blob URL allocated with URL.createObjectURL; the referenced "
-         "blob cannot be freed until all URLs for it have been explicitly "
-         "invalidated with URL.revokeObjectURL.");
-      nsAutoCString path, url, owner, specialDesc;
-      nsCOMPtr<nsIURI> principalURI;
-      uint64_t size = 0;
-      uint32_t refCount = 1;
-      DebugOnly<bool> blobWasCounted;
-
-      blobWasCounted = envp->mRefCounts.Get(blob, &refCount);
-      MOZ_ASSERT(blobWasCounted);
-      MOZ_ASSERT(refCount > 0);
-
-      bool isMemoryFile = blob->IsMemoryFile();
-
-      if (isMemoryFile) {
-        ErrorResult rv;
-        size = blob->GetSize(rv);
-        if (NS_WARN_IF(rv.Failed())) {
-          rv.SuppressException();
-          size = 0;
-        }
-      }
-
-      path = isMemoryFile ? "memory-blob-urls/" : "file-blob-urls/";
-      BuildPath(path, aKey, aInfo, envp->mAnonymize);
-
-      if (refCount > 1) {
-        nsAutoCString addrStr;
-
-        addrStr = "0x";
-        addrStr.AppendInt((uint64_t)(nsIDOMBlob*)blob, 16);
-
-        path += " ";
-        path.AppendInt(refCount);
-        path += "@";
-        path += addrStr;
-
-        specialDesc = desc;
-        specialDesc += "\n\nNOTE: This blob (address ";
-        specialDesc += addrStr;
-        specialDesc += ") has ";
-        specialDesc.AppendInt(refCount);
-        specialDesc += " URLs.";
-        if (isMemoryFile) {
-          specialDesc += " Its size is divided ";
-          specialDesc += refCount > 2 ? "among" : "between";
-          specialDesc += " them in this report.";
-        }
-      }
-
-      const nsACString& descString = specialDesc.IsEmpty()
-          ? static_cast<const nsACString&>(desc)
-          : static_cast<const nsACString&>(specialDesc);
-      if (isMemoryFile) {
-        envp->mCallback->Callback(EmptyCString(),
-            path,
-            KIND_OTHER,
-            UNITS_BYTES,
-            size / refCount,
-            descString,
-            envp->mData);
-      }
-      else {
-        envp->mCallback->Callback(EmptyCString(),
-            path,
-            KIND_OTHER,
-            UNITS_COUNT,
-            1,
-            descString,
-            envp->mData);
-      }
-    } else {
-      // Just report the path for the DOMMediaStream or MediaSource.
-      nsCOMPtr<mozilla::dom::MediaSource> ms(do_QueryInterface(aInfo->mObject));
-      nsAutoCString path;
-      path = ms ? "media-source-urls/" : "dom-media-stream-urls/";
-      BuildPath(path, aKey, aInfo, envp->mAnonymize);
-
-      NS_NAMED_LITERAL_CSTRING
-        (desc, "An object URL allocated with URL.createObjectURL; the referenced "
-               "data cannot be freed until all URLs for it have been explicitly "
-               "invalidated with URL.revokeObjectURL.");
-
-      envp->mCallback->Callback(EmptyCString(),
-          path,
-          KIND_OTHER,
-          UNITS_COUNT,
-          1,
-          desc,
-          envp->mData);
-    }
-
-    return PL_DHASH_NEXT;
-  }
 };
 
 NS_IMPL_ISUPPORTS(BlobURLsReporter, nsIMemoryReporter)
@@ -417,16 +407,26 @@ GetDataInfo(const nsACString& aUri)
   }
 
   DataInfo* res;
-  nsCString uriIgnoringRef;
-  int32_t hashPos = aUri.FindChar('#');
-  if (hashPos < 0) {
-    uriIgnoringRef = aUri;
+
+  // Let's remove any fragment and query from this URI.
+  int32_t hasFragmentPos = aUri.FindChar('#');
+  int32_t hasQueryPos = aUri.FindChar('?');
+
+  int32_t pos = -1;
+  if (hasFragmentPos >= 0 && hasQueryPos >= 0) {
+    pos = std::min(hasFragmentPos, hasQueryPos);
+  } else if (hasFragmentPos >= 0) {
+    pos = hasFragmentPos;
+  } else {
+    pos = hasQueryPos;
   }
-  else {
-    uriIgnoringRef = StringHead(aUri, hashPos);
+
+  if (pos < 0) {
+    gDataTable->Get(aUri, &res);
+  } else {
+    gDataTable->Get(StringHead(aUri, pos), &res);
   }
-  gDataTable->Get(uriIgnoringRef, &res);
-  
+
   return res;
 }
 
@@ -510,7 +510,7 @@ nsHostObjectProtocolHandler::NewURI(const nsACString& aSpec,
 
   DataInfo* info = GetDataInfo(aSpec);
 
-  nsRefPtr<nsHostObjectURI> uri =
+  RefPtr<nsHostObjectURI> uri =
     new nsHostObjectURI(info ? info->mPrincipal.get() : nullptr);
 
   rv = uri->SetSpec(aSpec);
@@ -559,11 +559,14 @@ nsHostObjectProtocolHandler::NewChannel2(nsIURI* uri,
     return rv.StealNSResult();
   }
 
+  nsAutoString contentType;
+  blob->GetType(contentType);
+
   nsCOMPtr<nsIChannel> channel;
   rv = NS_NewInputStreamChannelInternal(getter_AddRefs(channel),
                                         uri,
                                         stream,
-                                        EmptyCString(), // aContentType
+                                        NS_ConvertUTF16toUTF8(contentType),
                                         EmptyCString(), // aContentCharset
                                         aLoadInfo);
   if (NS_WARN_IF(rv.Failed())) {
@@ -669,7 +672,7 @@ NS_GetBlobForBlobURISpec(const nsACString& aSpec, BlobImpl** aBlob)
 nsresult
 NS_GetStreamForBlobURI(nsIURI* aURI, nsIInputStream** aStream)
 {
-  nsRefPtr<BlobImpl> blobImpl;
+  RefPtr<BlobImpl> blobImpl;
   ErrorResult rv;
   rv = NS_GetBlobForBlobURI(aURI, getter_AddRefs(blobImpl));
   if (NS_WARN_IF(rv.Failed())) {
@@ -704,7 +707,7 @@ nsFontTableProtocolHandler::NewURI(const nsACString& aSpec,
                                    nsIURI *aBaseURI,
                                    nsIURI **aResult)
 {
-  nsRefPtr<nsIURI> uri;
+  RefPtr<nsIURI> uri;
 
   // Either you got here via a ref or a fonttable: uri
   if (aSpec.Length() && aSpec.CharAt(0) == '#') {
@@ -717,7 +720,7 @@ nsFontTableProtocolHandler::NewURI(const nsACString& aSpec,
     // fonttable: scheme.
     // If aSpec is a relative URI -other- than a bare #ref,
     // this will leave uri empty, and we'll return a failure code below.
-    uri = new nsSimpleURI();
+    uri = new mozilla::net::nsSimpleURI();
     uri->SetSpec(aSpec);
   }
 
@@ -746,3 +749,53 @@ NS_GetSourceForMediaSourceURI(nsIURI* aURI, mozilla::dom::MediaSource** aSource)
   source.forget(aSource);
   return NS_OK;
 }
+
+#define NS_BLOBPROTOCOLHANDLER_CID \
+{ 0xb43964aa, 0xa078, 0x44b2, \
+  { 0xb0, 0x6b, 0xfd, 0x4d, 0x1b, 0x17, 0x2e, 0x66 } }
+
+#define NS_MEDIASTREAMPROTOCOLHANDLER_CID \
+{ 0x27d1fa24, 0x2b73, 0x4db3, \
+  { 0xab, 0x48, 0xb9, 0x83, 0x83, 0x40, 0xe0, 0x81 } }
+
+#define NS_MEDIASOURCEPROTOCOLHANDLER_CID \
+{ 0x12ef31fc, 0xa8fb, 0x4661, \
+  { 0x9a, 0x63, 0xfb, 0x61, 0x04,0x5d, 0xb8, 0x61 } }
+
+#define NS_FONTTABLEPROTOCOLHANDLER_CID \
+{ 0x3fc8f04e, 0xd719, 0x43ca, \
+  { 0x9a, 0xd0, 0x18, 0xee, 0x32, 0x02, 0x11, 0xf2 } }
+
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsBlobProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsMediaStreamProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsMediaSourceProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(nsFontTableProtocolHandler)
+
+NS_DEFINE_NAMED_CID(NS_BLOBPROTOCOLHANDLER_CID);
+NS_DEFINE_NAMED_CID(NS_MEDIASTREAMPROTOCOLHANDLER_CID);
+NS_DEFINE_NAMED_CID(NS_MEDIASOURCEPROTOCOLHANDLER_CID);
+NS_DEFINE_NAMED_CID(NS_FONTTABLEPROTOCOLHANDLER_CID);
+
+static const mozilla::Module::CIDEntry kHostObjectProtocolHandlerCIDs[] = {
+  { &kNS_BLOBPROTOCOLHANDLER_CID, false, nullptr, nsBlobProtocolHandlerConstructor },
+  { &kNS_MEDIASTREAMPROTOCOLHANDLER_CID, false, nullptr, nsMediaStreamProtocolHandlerConstructor },
+  { &kNS_MEDIASOURCEPROTOCOLHANDLER_CID, false, nullptr, nsMediaSourceProtocolHandlerConstructor },
+  { &kNS_FONTTABLEPROTOCOLHANDLER_CID, false, nullptr, nsFontTableProtocolHandlerConstructor },
+  { nullptr }
+};
+
+static const mozilla::Module::ContractIDEntry kHostObjectProtocolHandlerContracts[] = {
+  { NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX BLOBURI_SCHEME, &kNS_BLOBPROTOCOLHANDLER_CID },
+  { NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX MEDIASTREAMURI_SCHEME, &kNS_MEDIASTREAMPROTOCOLHANDLER_CID },
+  { NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX MEDIASOURCEURI_SCHEME, &kNS_MEDIASOURCEPROTOCOLHANDLER_CID },
+  { NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX FONTTABLEURI_SCHEME, &kNS_FONTTABLEPROTOCOLHANDLER_CID },
+  { nullptr }
+};
+
+static const mozilla::Module kHostObjectProtocolHandlerModule = {
+  mozilla::Module::kVersion,
+  kHostObjectProtocolHandlerCIDs,
+  kHostObjectProtocolHandlerContracts
+};
+
+NSMODULE_DEFN(HostObjectProtocolHandler) = &kHostObjectProtocolHandlerModule;
