@@ -41,8 +41,6 @@
 #undef Status
 #endif
 
-class AsyncVerifyRedirectCallbackForwarder;
-class nsFormData;
 class nsIJARChannel;
 class nsILoadGroup;
 class nsIUnicodeDecoder;
@@ -53,6 +51,7 @@ namespace mozilla {
 namespace dom {
 class Blob;
 class BlobSet;
+class FormData;
 } // namespace dom
 
 // A helper for building up an ArrayBuffer object's data
@@ -142,7 +141,7 @@ public:
   IMPL_EVENT_HANDLER(load)
   IMPL_EVENT_HANDLER(timeout)
   IMPL_EVENT_HANDLER(loadend)
-  
+
   virtual void DisconnectFromOwner() override;
 };
 
@@ -208,7 +207,6 @@ public:
   // The WebIDL constructors.
   static already_AddRefed<nsXMLHttpRequest>
   Constructor(const mozilla::dom::GlobalObject& aGlobal,
-              JSContext* aCx,
               const mozilla::dom::MozXMLHttpRequestParameters& aParams,
               ErrorResult& aRv)
   {
@@ -220,7 +218,7 @@ public:
       return nullptr;
     }
 
-    nsRefPtr<nsXMLHttpRequest> req = new nsXMLHttpRequest();
+    RefPtr<nsXMLHttpRequest> req = new nsXMLHttpRequest();
     req->Construct(principal->GetPrincipal(), global);
     req->InitParameters(aParams.mMozAnon, aParams.mMozSystem);
     return req.forget();
@@ -228,18 +226,17 @@ public:
 
   static already_AddRefed<nsXMLHttpRequest>
   Constructor(const mozilla::dom::GlobalObject& aGlobal,
-              JSContext* aCx,
               const nsAString& ignored,
               ErrorResult& aRv)
   {
     // Pretend like someone passed null, so we can pick up the default values
     mozilla::dom::MozXMLHttpRequestParameters params;
-    if (!params.Init(aCx, JS::NullHandleValue)) {
+    if (!params.Init(aGlobal.Context(), JS::NullHandleValue)) {
       aRv.Throw(NS_ERROR_UNEXPECTED);
       return nullptr;
     }
 
-    return Constructor(aGlobal, aCx, params, aRv);
+    return Constructor(aGlobal, params, aRv);
   }
 
   void Construct(nsIPrincipal* aPrincipal,
@@ -248,7 +245,7 @@ public:
                  nsILoadGroup* aLoadGroup = nullptr)
   {
     MOZ_ASSERT(aPrincipal);
-    MOZ_ASSERT_IF(nsCOMPtr<nsPIDOMWindow> win = do_QueryInterface(
+    MOZ_ASSERT_IF(nsCOMPtr<nsPIDOMWindowInner> win = do_QueryInterface(
       aGlobalObject), win->IsInnerWindow());
     mPrincipal = aPrincipal;
     BindToOwner(aGlobalObject);
@@ -364,7 +361,7 @@ private:
     {
       mValue.mString = &aString;
     }
-    explicit RequestBody(nsFormData& aFormData) : mType(FormData)
+    explicit RequestBody(mozilla::dom::FormData& aFormData) : mType(FormData)
     {
       mValue.mFormData = &aFormData;
     }
@@ -389,7 +386,7 @@ private:
       mozilla::dom::Blob* mBlob;
       nsIDocument* mDocument;
       const nsAString* mString;
-      nsFormData* mFormData;
+      mozilla::dom::FormData* mFormData;
       nsIInputStream* mStream;
     };
 
@@ -426,7 +423,8 @@ private:
     return Send(Nullable<RequestBody>(aBody));
   }
 
-  bool IsDeniedCrossSiteRequest();
+  bool IsCrossSiteCORSRequest();
+  bool IsDeniedCrossSiteCORSRequest();
 
   // Tell our channel what network interface ID we were told to use.
   // If it's an HTTP channel and we were told to use a non-default
@@ -467,7 +465,8 @@ public:
       aRv = Send(RequestBody(aString));
     }
   }
-  void Send(JSContext* /*aCx*/, nsFormData& aFormData, ErrorResult& aRv)
+  void Send(JSContext* /*aCx*/, mozilla::dom::FormData& aFormData,
+            ErrorResult& aRv)
   {
     aRv = Send(RequestBody(aFormData));
   }
@@ -575,7 +574,7 @@ public:
 
   nsresult init(nsIPrincipal* principal,
                 nsIScriptContext* scriptContext,
-                nsPIDOMWindow* globalObject,
+                nsPIDOMWindowInner* globalObject,
                 nsIURI* baseURI);
 
   void SetRequestObserver(nsIRequestObserver* aObserver);
@@ -605,7 +604,7 @@ protected:
                 uint32_t count,
                 uint32_t *writeCount);
   nsresult CreateResponseParsedJSON(JSContext* aCx);
-  void CreatePartialBlob();
+  void CreatePartialBlob(ErrorResult& aRv);
   bool CreateDOMBlob(nsIRequest *request);
   // Change the state of the object with this. The broadcast argument
   // determines if the onreadystatechange listener should be called.
@@ -620,18 +619,9 @@ protected:
 
   void ChangeStateToDone();
 
-  /**
-   * Check if aChannel is ok for a cross-site request by making sure no
-   * inappropriate headers are set, and no username/password is set.
-   *
-   * Also updates the XML_HTTP_REQUEST_USE_XSITE_AC bit.
-   */
-  nsresult CheckChannelForCrossSiteRequest(nsIChannel* aChannel);
-
   void StartProgressEventTimer();
 
-  friend class AsyncVerifyRedirectCallbackForwarder;
-  void OnRedirectVerifyCallback(nsresult result);
+  nsresult OnRedirectVerifyCallback(nsresult result);
 
   nsresult Open(const nsACString& method, const nsACString& url, bool async,
                 const mozilla::dom::Optional<nsAString>& user,
@@ -643,7 +633,6 @@ protected:
   nsCOMPtr<nsIPrincipal> mPrincipal;
   nsCOMPtr<nsIChannel> mChannel;
   nsCOMPtr<nsIDocument> mResponseXML;
-  nsCOMPtr<nsIChannel> mCORSPreflightChannel;
   nsTArray<nsCString> mCORSUnsafeHeaders;
 
   nsCOMPtr<nsIStreamListener> mXMLParserStreamListener;
@@ -708,10 +697,10 @@ protected:
 
   // It is either a cached blob-response from the last call to GetResponse,
   // but is also explicitly set in OnStopRequest.
-  nsRefPtr<mozilla::dom::Blob> mResponseBlob;
+  RefPtr<mozilla::dom::Blob> mResponseBlob;
   // Non-null only when we are able to get a os-file representation of the
   // response, i.e. when loading from a file.
-  nsRefPtr<mozilla::dom::Blob> mDOMBlob;
+  RefPtr<mozilla::dom::Blob> mDOMBlob;
   // We stream data to mBlobSet when response type is "blob" or "moz-blob"
   // and mDOMBlob is null.
   nsAutoPtr<mozilla::dom::BlobSet> mBlobSet;
@@ -738,7 +727,7 @@ protected:
 
   uint32_t mState;
 
-  nsRefPtr<nsXMLHttpRequestUpload> mUpload;
+  RefPtr<nsXMLHttpRequestUpload> mUpload;
   int64_t mUploadTransferred;
   int64_t mUploadTotal;
   bool mUploadLengthComputable;
@@ -805,6 +794,8 @@ protected:
 
   void ResetResponse();
 
+  bool ShouldBlockAuthPrompt();
+
   struct RequestHeader
   {
     nsCString header;
@@ -841,6 +832,7 @@ private:
 // XMLHttpRequest via XPCOM stuff.
 class nsXMLHttpRequestXPCOMifier final : public nsIStreamListener,
                                          public nsIChannelEventSink,
+                                         public nsIAsyncVerifyRedirectCallback,
                                          public nsIProgressEventSink,
                                          public nsIInterfaceRequestor,
                                          public nsITimerCallback
@@ -865,13 +857,14 @@ public:
   NS_FORWARD_NSISTREAMLISTENER(mXHR->)
   NS_FORWARD_NSIREQUESTOBSERVER(mXHR->)
   NS_FORWARD_NSICHANNELEVENTSINK(mXHR->)
+  NS_FORWARD_NSIASYNCVERIFYREDIRECTCALLBACK(mXHR->)
   NS_FORWARD_NSIPROGRESSEVENTSINK(mXHR->)
   NS_FORWARD_NSITIMERCALLBACK(mXHR->)
 
   NS_DECL_NSIINTERFACEREQUESTOR
 
 private:
-  nsRefPtr<nsXMLHttpRequest> mXHR;
+  RefPtr<nsXMLHttpRequest> mXHR;
 };
 
 class nsXHRParseEndListener : public nsIDOMEventListener
