@@ -5,7 +5,7 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "BluetoothDaemonAvrcpInterface.h"
-#include "BluetoothDaemonSetupInterface.h"
+#include "mozilla/UniquePtr.h"
 #include "mozilla/unused.h"
 
 BEGIN_BLUETOOTH_NAMESPACE
@@ -16,8 +16,6 @@ using namespace mozilla::ipc;
 // AVRCP module
 //
 
-const int BluetoothDaemonAvrcpModule::MAX_NUM_CLIENTS = 1;
-
 BluetoothAvrcpNotificationHandler*
   BluetoothDaemonAvrcpModule::sNotificationHandler;
 
@@ -28,31 +26,23 @@ BluetoothDaemonAvrcpModule::SetNotificationHandler(
   sNotificationHandler = aNotificationHandler;
 }
 
-nsresult
-BluetoothDaemonAvrcpModule::Send(DaemonSocketPDU* aPDU,
-                                 BluetoothAvrcpResultHandler* aRes)
-{
-  if (aRes) {
-    aRes->AddRef(); // Keep reference for response
-  }
-  return Send(aPDU, static_cast<void*>(aRes));
-}
-
 void
 BluetoothDaemonAvrcpModule::HandleSvc(const DaemonSocketPDUHeader& aHeader,
-                                      DaemonSocketPDU& aPDU, void* aUserData)
+                                      DaemonSocketPDU& aPDU,
+                                      DaemonSocketResultHandler* aRes)
 {
   static void (BluetoothDaemonAvrcpModule::* const HandleOp[])(
-    const DaemonSocketPDUHeader&, DaemonSocketPDU&, void*) = {
-    INIT_ARRAY_AT(0, &BluetoothDaemonAvrcpModule::HandleRsp),
-    INIT_ARRAY_AT(1, &BluetoothDaemonAvrcpModule::HandleNtf),
+    const DaemonSocketPDUHeader&, DaemonSocketPDU&,
+    DaemonSocketResultHandler*) = {
+    [0] = &BluetoothDaemonAvrcpModule::HandleRsp,
+    [1] = &BluetoothDaemonAvrcpModule::HandleNtf
   };
 
   MOZ_ASSERT(!NS_IsMainThread());
 
   unsigned int isNtf = !!(aHeader.mOpcode & 0x80);
 
-  (this->*(HandleOp[isNtf]))(aHeader, aPDU, aUserData);
+  (this->*(HandleOp[isNtf]))(aHeader, aPDU, aRes);
 }
 
 // Commands
@@ -65,21 +55,21 @@ BluetoothDaemonAvrcpModule::GetPlayStatusRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAY_STATUS_RSP,
-                           1 + // Play status
-                           4 + // Duration
-                           4)); // Position
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_GET_PLAY_STATUS_RSP,
+                                1 + // Play status
+                                4 + // Duration
+                                4); // Position
 
   nsresult rv = PackPDU(aPlayStatus, aSongLen, aSongPos, *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -90,10 +80,10 @@ BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_LIST_PLAYER_APP_ATTR_RSP,
-                           1 + // # Attributes
-                           aNumAttr)); // Player attributes
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_LIST_PLAYER_APP_ATTR_RSP,
+                                1 + // # Attributes
+                                aNumAttr); // Player attributes
 
   nsresult rv = PackPDU(
     PackConversion<int, uint8_t>(aNumAttr),
@@ -101,11 +91,11 @@ BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -115,21 +105,21 @@ BluetoothDaemonAvrcpModule::ListPlayerAppValueRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_LIST_PLAYER_APP_VALUE_RSP,
-                           1 + // # Values
-                           aNumVal)); // Player values
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_LIST_PLAYER_APP_VALUE_RSP,
+                                1 + // # Values
+                                aNumVal); // Player values
 
   nsresult rv = PackPDU(PackConversion<int, uint8_t>(aNumVal),
                         PackArray<uint8_t>(aPVals, aNumVal), *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -140,21 +130,22 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_RSP,
-                           1 + // # Pairs
-                           2 * aNumAttrs)); // Attribute-value pairs
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_RSP,
+                                1 + // # Pairs
+                                2 * aNumAttrs); // Attribute-value pairs
+
   nsresult rv = PackPDU(
     aNumAttrs,
     BluetoothAvrcpAttributeValuePairs(aIds, aValues, aNumAttrs), *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -165,20 +156,21 @@ BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP,
-                           0)); // Dynamically allocated
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP,
+                                0); // Dynamically allocated
+
   nsresult rv = PackPDU(
     PackConversion<int, uint8_t>(aNumAttr),
     BluetoothAvrcpAttributeTextPairs(aIds, aTexts, aNumAttr), *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -189,20 +181,21 @@ BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP,
-                           0)); // Dynamically allocated
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP,
+                                0); // Dynamically allocated
+
   nsresult rv = PackPDU(
     PackConversion<int, uint8_t>(aNumVal),
     BluetoothAvrcpAttributeTextPairs(aIds, aTexts, aNumVal), *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -213,20 +206,21 @@ BluetoothDaemonAvrcpModule::GetElementAttrRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_GET_ELEMENT_ATTR_RSP,
-                           0)); // Dynamically allocated
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_GET_ELEMENT_ATTR_RSP,
+                                0); // Dynamically allocated
+
   nsresult rv = PackPDU(
     aNumAttr,
     PackArray<BluetoothAvrcpElementAttribute>(aAttr, aNumAttr), *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -236,19 +230,19 @@ BluetoothDaemonAvrcpModule::SetPlayerAppValueRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_SET_PLAYER_APP_VALUE_RSP,
-                           1)); // Status code
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_SET_PLAYER_APP_VALUE_RSP,
+                                1); // Status code
 
   nsresult rv = PackPDU(aRspStatus, *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -260,12 +254,12 @@ BluetoothDaemonAvrcpModule::RegisterNotificationRspCmd(
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_REGISTER_NOTIFICATION_RSP,
-                           1 + // Event
-                           1 + // Type
-                           1 + // Data length
-                           256)); // Maximum data length
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_REGISTER_NOTIFICATION_RSP,
+                                1 + // Event
+                                1 + // Type
+                                1 + // Data length
+                                256); // Maximum data length
 
   BluetoothAvrcpEventParamPair data(aEvent, aParam);
   nsresult rv = PackPDU(aEvent, aType, static_cast<uint8_t>(data.GetLength()),
@@ -273,11 +267,11 @@ BluetoothDaemonAvrcpModule::RegisterNotificationRspCmd(
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -287,19 +281,19 @@ BluetoothDaemonAvrcpModule::SetVolumeCmd(uint8_t aVolume,
 {
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsAutoPtr<DaemonSocketPDU> pdu(
-    new DaemonSocketPDU(SERVICE_ID, OPCODE_SET_VOLUME,
-                           1)); // Volume
+  UniquePtr<DaemonSocketPDU> pdu =
+    MakeUnique<DaemonSocketPDU>(SERVICE_ID, OPCODE_SET_VOLUME,
+                                1); // Volume
 
   nsresult rv = PackPDU(aVolume, *pdu);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  rv = Send(pdu, aRes);
+  rv = Send(pdu.get(), aRes);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  unused << pdu.forget();
+  Unused << pdu.release();
   return NS_OK;
 }
 
@@ -418,34 +412,34 @@ BluetoothDaemonAvrcpModule::SetVolumeRsp(
 void
 BluetoothDaemonAvrcpModule::HandleRsp(
   const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU,
-  void* aUserData)
+  DaemonSocketResultHandler* aRes)
 {
   static void (BluetoothDaemonAvrcpModule::* const HandleRsp[])(
     const DaemonSocketPDUHeader&,
     DaemonSocketPDU&,
     BluetoothAvrcpResultHandler*) = {
-    INIT_ARRAY_AT(OPCODE_ERROR,
-      &BluetoothDaemonAvrcpModule::ErrorRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAY_STATUS_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayStatusRspRsp),
-    INIT_ARRAY_AT(OPCODE_LIST_PLAYER_APP_ATTR_RSP,
-      &BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspRsp),
-    INIT_ARRAY_AT(OPCODE_LIST_PLAYER_APP_VALUE_RSP,
-      &BluetoothDaemonAvrcpModule::ListPlayerAppValueRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAYER_APP_VALUE_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayerAppValueRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP,
-      &BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspRsp),
-    INIT_ARRAY_AT(OPCODE_GET_ELEMENT_ATTR_RSP,
-      &BluetoothDaemonAvrcpModule::GetElementAttrRspRsp),
-    INIT_ARRAY_AT(OPCODE_SET_PLAYER_APP_VALUE_RSP,
-      &BluetoothDaemonAvrcpModule::SetPlayerAppValueRspRsp),
-    INIT_ARRAY_AT(OPCODE_REGISTER_NOTIFICATION_RSP,
-      &BluetoothDaemonAvrcpModule::RegisterNotificationRspRsp),
-    INIT_ARRAY_AT(OPCODE_SET_VOLUME,
-      &BluetoothDaemonAvrcpModule::SetVolumeRsp)
+    [OPCODE_ERROR] =
+      &BluetoothDaemonAvrcpModule::ErrorRsp,
+    [OPCODE_GET_PLAY_STATUS_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayStatusRspRsp,
+    [OPCODE_LIST_PLAYER_APP_ATTR_RSP] =
+      &BluetoothDaemonAvrcpModule::ListPlayerAppAttrRspRsp,
+    [OPCODE_LIST_PLAYER_APP_VALUE_RSP] =
+      &BluetoothDaemonAvrcpModule::ListPlayerAppValueRspRsp,
+    [OPCODE_GET_PLAYER_APP_VALUE_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayerAppValueRspRsp,
+    [OPCODE_GET_PLAYER_APP_ATTR_TEXT_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayerAppAttrTextRspRsp,
+    [OPCODE_GET_PLAYER_APP_VALUE_TEXT_RSP] =
+      &BluetoothDaemonAvrcpModule::GetPlayerAppValueTextRspRsp,
+    [OPCODE_GET_ELEMENT_ATTR_RSP]=
+      &BluetoothDaemonAvrcpModule::GetElementAttrRspRsp,
+    [OPCODE_SET_PLAYER_APP_VALUE_RSP] =
+      &BluetoothDaemonAvrcpModule::SetPlayerAppValueRspRsp,
+    [OPCODE_REGISTER_NOTIFICATION_RSP] =
+      &BluetoothDaemonAvrcpModule::RegisterNotificationRspRsp,
+    [OPCODE_SET_VOLUME] =
+      &BluetoothDaemonAvrcpModule::SetVolumeRsp
   };
 
   MOZ_ASSERT(!NS_IsMainThread()); // I/O thread
@@ -455,9 +449,8 @@ BluetoothDaemonAvrcpModule::HandleRsp(
     return;
   }
 
-  nsRefPtr<BluetoothAvrcpResultHandler> res =
-    already_AddRefed<BluetoothAvrcpResultHandler>(
-      static_cast<BluetoothAvrcpResultHandler*>(aUserData));
+  RefPtr<BluetoothAvrcpResultHandler> res =
+    static_cast<BluetoothAvrcpResultHandler*>(aRes);
 
   if (!res) {
     return; // Return early if no result handler has been set for response
@@ -493,13 +486,12 @@ public:
   { }
 
   nsresult
-  operator () (nsString& aArg1, unsigned long& aArg2) const
+  operator () (BluetoothAddress& aArg1, unsigned long& aArg2) const
   {
     DaemonSocketPDU& pdu = GetPDU();
 
     /* Read address */
-    nsresult rv = UnpackPDU(
-      pdu, UnpackConversion<BluetoothAddress, nsAString>(aArg1));
+    nsresult rv = UnpackPDU(pdu, aArg1);
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -507,7 +499,7 @@ public:
     /* Read feature */
     rv = UnpackPDU(
       pdu,
-      UnpackConversion<BluetoothAvrcpRemoteFeature, unsigned long>(aArg2));
+      UnpackConversion<BluetoothAvrcpRemoteFeatureBits, unsigned long>(aArg2));
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -563,7 +555,7 @@ public:
 
   nsresult
   operator () (uint8_t& aArg1,
-               nsAutoArrayPtr<BluetoothAvrcpPlayerAttribute>& aArg2) const
+               UniquePtr<BluetoothAvrcpPlayerAttribute[]>& aArg2) const
   {
     DaemonSocketPDU& pdu = GetPDU();
 
@@ -604,7 +596,7 @@ public:
 
   nsresult
   operator () (uint8_t& aArg1,
-               nsAutoArrayPtr<BluetoothAvrcpPlayerAttribute>& aArg2) const
+               UniquePtr<BluetoothAvrcpPlayerAttribute[]>& aArg2) const
   {
     DaemonSocketPDU& pdu = GetPDU();
 
@@ -645,7 +637,7 @@ public:
 
   nsresult
   operator () (uint8_t& aArg1, uint8_t& aArg2,
-               nsAutoArrayPtr<uint8_t>& aArg3) const
+               UniquePtr<uint8_t[]>& aArg3) const
   {
     DaemonSocketPDU& pdu = GetPDU();
 
@@ -700,7 +692,7 @@ public:
 
   nsresult
   operator () (uint8_t& aArg1,
-               nsAutoArrayPtr<BluetoothAvrcpMediaAttribute>& aArg2) const
+               UniquePtr<BluetoothAvrcpMediaAttribute[]>& aArg2) const
   {
     DaemonSocketPDU& pdu = GetPDU();
 
@@ -749,73 +741,46 @@ BluetoothDaemonAvrcpModule::VolumeChangeNtf(
     UnpackPDUInitOp(aPDU));
 }
 
-// Init operator class for PassthroughCmdNotification
-class BluetoothDaemonAvrcpModule::PassthroughCmdInitOp final
-  : private PDUInitOp
-{
-public:
-  PassthroughCmdInitOp(DaemonSocketPDU& aPDU)
-    : PDUInitOp(aPDU)
-  { }
-
-  nsresult
-  operator () (int& aArg1, int& aArg2) const
-  {
-    DaemonSocketPDU& pdu = GetPDU();
-
-    nsresult rv = UnpackPDU(pdu, UnpackConversion<uint8_t, int>(aArg1));
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-    rv = UnpackPDU(pdu, UnpackConversion<uint8_t, int>(aArg2));
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
-    WarnAboutTrailingData();
-    return NS_OK;
-  }
-};
-
 void
 BluetoothDaemonAvrcpModule::PassthroughCmdNtf(
   const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU)
 {
   PassthroughCmdNotification::Dispatch(
     &BluetoothAvrcpNotificationHandler::PassthroughCmdNotification,
-    PassthroughCmdInitOp(aPDU));
+    UnpackPDUInitOp(aPDU));
 }
 #endif
 
 void
 BluetoothDaemonAvrcpModule::HandleNtf(
   const DaemonSocketPDUHeader& aHeader, DaemonSocketPDU& aPDU,
-  void* aUserData)
+  DaemonSocketResultHandler* aRes)
 {
   static void (BluetoothDaemonAvrcpModule::* const HandleNtf[])(
     const DaemonSocketPDUHeader&, DaemonSocketPDU&) = {
 #if ANDROID_VERSION >= 19
-    INIT_ARRAY_AT(0, &BluetoothDaemonAvrcpModule::RemoteFeatureNtf),
-    INIT_ARRAY_AT(1, &BluetoothDaemonAvrcpModule::GetPlayStatusNtf),
-    INIT_ARRAY_AT(2, &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf),
-    INIT_ARRAY_AT(3, &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf),
-    INIT_ARRAY_AT(4, &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf),
-    INIT_ARRAY_AT(5, &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf),
-    INIT_ARRAY_AT(6, &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf),
-    INIT_ARRAY_AT(7, &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf),
-    INIT_ARRAY_AT(8, &BluetoothDaemonAvrcpModule::GetElementAttrNtf),
-    INIT_ARRAY_AT(9, &BluetoothDaemonAvrcpModule::RegisterNotificationNtf),
-    INIT_ARRAY_AT(10, &BluetoothDaemonAvrcpModule::VolumeChangeNtf),
-    INIT_ARRAY_AT(11, &BluetoothDaemonAvrcpModule::PassthroughCmdNtf)
+    [0] = &BluetoothDaemonAvrcpModule::RemoteFeatureNtf,
+    [1] = &BluetoothDaemonAvrcpModule::GetPlayStatusNtf,
+    [2] = &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf,
+    [3] = &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf,
+    [4] = &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf,
+    [5] = &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf,
+    [6] = &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf,
+    [7] = &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf,
+    [8] = &BluetoothDaemonAvrcpModule::GetElementAttrNtf,
+    [9] = &BluetoothDaemonAvrcpModule::RegisterNotificationNtf,
+    [10] = &BluetoothDaemonAvrcpModule::VolumeChangeNtf,
+    [11] = &BluetoothDaemonAvrcpModule::PassthroughCmdNtf
 #else
-    INIT_ARRAY_AT(0, &BluetoothDaemonAvrcpModule::GetPlayStatusNtf),
-    INIT_ARRAY_AT(1, &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf),
-    INIT_ARRAY_AT(2, &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf),
-    INIT_ARRAY_AT(3, &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf),
-    INIT_ARRAY_AT(4, &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf),
-    INIT_ARRAY_AT(5, &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf),
-    INIT_ARRAY_AT(6, &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf),
-    INIT_ARRAY_AT(7, &BluetoothDaemonAvrcpModule::GetElementAttrNtf),
-    INIT_ARRAY_AT(8, &BluetoothDaemonAvrcpModule::RegisterNotificationNtf)
+    [0] = &BluetoothDaemonAvrcpModule::GetPlayStatusNtf,
+    [1] = &BluetoothDaemonAvrcpModule::ListPlayerAppAttrNtf,
+    [2] = &BluetoothDaemonAvrcpModule::ListPlayerAppValuesNtf,
+    [3] = &BluetoothDaemonAvrcpModule::GetPlayerAppValueNtf,
+    [4] = &BluetoothDaemonAvrcpModule::GetPlayerAppAttrsTextNtf,
+    [5] = &BluetoothDaemonAvrcpModule::GetPlayerAppValuesTextNtf,
+    [6] = &BluetoothDaemonAvrcpModule::SetPlayerAppValueNtf,
+    [7] = &BluetoothDaemonAvrcpModule::GetElementAttrNtf,
+    [8] = &BluetoothDaemonAvrcpModule::RegisterNotificationNtf
 #endif
   };
 
@@ -843,115 +808,13 @@ BluetoothDaemonAvrcpInterface::BluetoothDaemonAvrcpInterface(
 BluetoothDaemonAvrcpInterface::~BluetoothDaemonAvrcpInterface()
 { }
 
-class BluetoothDaemonAvrcpInterface::InitResultHandler final
-  : public BluetoothSetupResultHandler
-{
-public:
-  InitResultHandler(BluetoothAvrcpResultHandler* aRes)
-    : mRes(aRes)
-  {
-    MOZ_ASSERT(mRes);
-  }
-
-  void OnError(BluetoothStatus aStatus) override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    mRes->OnError(aStatus);
-  }
-
-  void RegisterModule() override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    mRes->Init();
-  }
-
-private:
-  nsRefPtr<BluetoothAvrcpResultHandler> mRes;
-};
-
 void
-BluetoothDaemonAvrcpInterface::Init(
-  BluetoothAvrcpNotificationHandler* aNotificationHandler,
-  BluetoothAvrcpResultHandler* aRes)
+BluetoothDaemonAvrcpInterface::SetNotificationHandler(
+  BluetoothAvrcpNotificationHandler* aNotificationHandler)
 {
   MOZ_ASSERT(mModule);
 
-  // Set notification handler _before_ registering the module. It could
-  // happen that we receive notifications, before the result handler runs.
   mModule->SetNotificationHandler(aNotificationHandler);
-
-  InitResultHandler* res;
-
-  if (aRes) {
-    res = new InitResultHandler(aRes);
-  } else {
-    // We don't need a result handler if the caller is not interested.
-    res = nullptr;
-  }
-
-  nsresult rv = mModule->RegisterModule(
-    BluetoothDaemonAvrcpModule::SERVICE_ID,
-    BluetoothDaemonAvrcpModule::MAX_NUM_CLIENTS, 0x00, res);
-
-  if (NS_FAILED(rv) && aRes) {
-    DispatchError(aRes, rv);
-  }
-}
-
-class BluetoothDaemonAvrcpInterface::CleanupResultHandler final
-  : public BluetoothSetupResultHandler
-{
-public:
-  CleanupResultHandler(BluetoothDaemonAvrcpModule* aModule,
-                       BluetoothAvrcpResultHandler* aRes)
-    : mModule(aModule)
-    , mRes(aRes)
-  {
-    MOZ_ASSERT(mModule);
-  }
-
-  void OnError(BluetoothStatus aStatus) override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    if (mRes) {
-      mRes->OnError(aStatus);
-    }
-  }
-
-  void UnregisterModule() override
-  {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    // Clear notification handler _after_ module has been
-    // unregistered. While unregistering the module, we might
-    // still receive notifications.
-    mModule->SetNotificationHandler(nullptr);
-
-    if (mRes) {
-      mRes->Cleanup();
-    }
-  }
-
-private:
-  BluetoothDaemonAvrcpModule* mModule;
-  nsRefPtr<BluetoothAvrcpResultHandler> mRes;
-};
-
-void
-BluetoothDaemonAvrcpInterface::Cleanup(
-  BluetoothAvrcpResultHandler* aRes)
-{
-  MOZ_ASSERT(mModule);
-
-  nsresult rv = mModule->UnregisterModule(
-    BluetoothDaemonAvrcpModule::SERVICE_ID,
-    new CleanupResultHandler(mModule, aRes));
-  if (NS_FAILED(rv)) {
-    DispatchError(aRes, rv);
-  }
 }
 
 void
