@@ -13,7 +13,7 @@
 #include "BluetoothService.h"
 #include "BluetoothSocket.h"
 #include "BluetoothUtils.h"
-#include "BluetoothUuid.h"
+#include "BluetoothUuidHelper.h"
 
 #include "jsapi.h"
 #include "mozilla/dom/bluetooth/BluetoothTypes.h"
@@ -82,28 +82,13 @@ namespace {
   static const char kHfpCrlf[] = "\xd\xa";
 
   // UUID of Handsfree Audio Gateway
-  static const BluetoothUuid kHandsfreeAG = {
-    {
-      0x00, 0x00, 0x11, 0x1F, 0x00, 0x00, 0x10, 0x00,
-      0x80, 0x00, 0x00, 0x80, 0x5F, 0x9B, 0x34, 0xFB
-    }
-  };
+  static const BluetoothUuid kHandsfreeAG(HANDSFREE_AG);
 
   // UUID of Headset Audio Gateway
-  static const BluetoothUuid kHeadsetAG = {
-    {
-      0x00, 0x00, 0x11, 0x12, 0x00, 0x00, 0x10, 0x00,
-      0x80, 0x00, 0x00, 0x80, 0x5F, 0x9B, 0x34, 0xFB
-    }
-  };
+  static const BluetoothUuid kHeadsetAG(HEADSET_AG);
 
   // Unknown service UUID (for SCO socket)
-  static const BluetoothUuid kUnknownService = {
-    {
-      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00,
-      0x80, 0x00, 0x00, 0x80, 0x5F, 0x9B, 0x34, 0xFB
-    }
-  };
+  static const BluetoothUuid kUnknownService(BluetoothServiceClass::UNKNOWN);
 
 #ifdef MOZ_B2G_RIL
   // Sending ringtone related
@@ -257,10 +242,10 @@ BluetoothHfpManager::Notify(const hal::BatteryInformation& aBatteryInfo)
 }
 
 #ifdef MOZ_B2G_RIL
-class BluetoothHfpManager::RespondToBLDNTask : public Task
+class BluetoothHfpManager::RespondToBLDNTask : public Runnable
 {
 private:
-  void Run() override
+  NS_IMETHOD Run() override
   {
     MOZ_ASSERT(sBluetoothHfpManager);
 
@@ -268,10 +253,11 @@ private:
       sBluetoothHfpManager->mDialingRequestProcessed = true;
       sBluetoothHfpManager->SendLine("ERROR");
     }
+    return NS_OK;
   }
 };
 
-class BluetoothHfpManager::SendRingIndicatorTask : public Task
+class BluetoothHfpManager::SendRingIndicatorTask : public Runnable
 {
 public:
   SendRingIndicatorTask(const nsAString& aNumber, int aType)
@@ -281,18 +267,18 @@ public:
     MOZ_ASSERT(NS_IsMainThread());
   }
 
-  void Run() override
+  NS_IMETHOD Run() override
   {
     MOZ_ASSERT(NS_IsMainThread());
 
     // Stop sending RING indicator
     if (sStopSendingRingFlag) {
-      return;
+      return NS_OK;
     }
 
     if (!sBluetoothHfpManager) {
       BT_WARNING("BluetoothHfpManager no longer exists, cannot send ring!");
-      return;
+      return NS_OK;
     }
 
     nsAutoCString ringMsg("RING");
@@ -306,10 +292,10 @@ public:
       sBluetoothHfpManager->SendLine(clipMsg.get());
     }
 
-    MessageLoop::current()->
-      PostDelayedTask(FROM_HERE,
-                      new SendRingIndicatorTask(mNumber, mType),
-                      sRingInterval);
+    MessageLoop::current()->PostDelayedTask(
+      MakeAndAddRef<SendRingIndicatorTask>(mNumber, mType), sRingInterval);
+
+    return NS_OK;
   }
 
 private:
@@ -318,14 +304,16 @@ private:
 };
 #endif // MOZ_B2G_RIL
 
-class BluetoothHfpManager::CloseScoTask : public Task
+class BluetoothHfpManager::CloseScoTask : public Runnable
 {
 private:
-  void Run() override
+  NS_IMETHOD Run() override
   {
     MOZ_ASSERT(sBluetoothHfpManager);
 
     sBluetoothHfpManager->DisconnectSco();
+
+    return NS_OK;
   }
 };
 
@@ -462,7 +450,7 @@ BluetoothHfpManager::Init()
   Notify(batteryInfo);
 
 #ifdef MOZ_B2G_RIL
-  mListener = new BluetoothRilListener();
+  mListener = MakeUnique<BluetoothRilListener>();
   if (!mListener->Listen(true)) {
     BT_WARNING("Failed to start listening RIL");
     return false;
@@ -477,7 +465,7 @@ BluetoothHfpManager::Init()
   nsresult rv = settings->CreateLock(nullptr, getter_AddRefs(settingsLock));
   NS_ENSURE_SUCCESS(rv, false);
 
-  nsRefPtr<GetVolumeTask> callback = new GetVolumeTask();
+  RefPtr<GetVolumeTask> callback = new GetVolumeTask();
   rv = settingsLock->Get(AUDIO_VOLUME_BT_SCO_ID, callback);
   NS_ENSURE_SUCCESS(rv, false);
 
@@ -540,8 +528,11 @@ BluetoothHfpManager::NotifyConnectionStatusChanged(const nsAString& aType)
   nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
   NS_ENSURE_TRUE_VOID(obs);
 
+  nsAutoString deviceAddressStr;
+  AddressToString(mDeviceAddress, deviceAddressStr);
+
   if (NS_FAILED(obs->NotifyObservers(this, NS_ConvertUTF16toUTF8(aType).get(),
-                                     mDeviceAddress.get()))) {
+                                     deviceAddressStr.get()))) {
     BT_WARNING("Failed to notify observsers!");
   }
 
@@ -590,7 +581,8 @@ BluetoothHfpManager::HandleVolumeChanged(nsISupports* aSubject)
   //  {"key":"volumeup", "value":10}
   //  {"key":"volumedown", "value":2}
 
-  RootedDictionary<dom::SettingChangeNotification> setting(nsContentUtils::RootingCx());
+  RootedDictionary<dom::SettingChangeNotification>
+    setting(nsContentUtils::RootingCx());
 
   if (!WrappedJSToDictionary(aSubject, setting)) {
     return;
@@ -744,7 +736,7 @@ BluetoothHfpManager::ParseAtCommand(const nsACString& aAtCommand,
 // Virtual function of class SocketConsumer
 void
 BluetoothHfpManager::ReceiveSocketData(BluetoothSocket* aSocket,
-                                       nsAutoPtr<UnixSocketBuffer>& aMessage)
+                                       UniquePtr<UnixSocketBuffer>& aMessage)
 {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aSocket);
@@ -843,7 +835,8 @@ BluetoothHfpManager::ReceiveSocketData(BluetoothSocket* aSocket,
     if (!atCommandValues[0].EqualsLiteral("3") ||
         !atCommandValues[1].EqualsLiteral("0")) {
       if (mCMEE) {
-        SendCommand(RESPONSE_CME_ERROR, BluetoothCmeError::OPERATION_NOT_SUPPORTED);
+        SendCommand(RESPONSE_CME_ERROR,
+                    BluetoothCmeError::OPERATION_NOT_SUPPORTED);
       } else {
         SendLine("ERROR");
       }
@@ -995,7 +988,7 @@ BluetoothHfpManager::ReceiveSocketData(BluetoothSocket* aSocket,
     }
 
     MessageLoop::current()->
-      PostDelayedTask(FROM_HERE, new RespondToBLDNTask(),
+      PostDelayedTask(MakeAndAddRef<RespondToBLDNTask>(),
                       sWaitingForDialingInterval);
 
     // Don't send response 'OK' here because we'll respond later in either
@@ -1120,7 +1113,7 @@ respond_with_ok:
 }
 
 void
-BluetoothHfpManager::Connect(const nsAString& aDeviceAddress,
+BluetoothHfpManager::Connect(const BluetoothAddress& aDeviceAddress,
                              BluetoothProfileController* aController)
 {
   MOZ_ASSERT(NS_IsMainThread());
@@ -1141,8 +1134,7 @@ BluetoothHfpManager::Connect(const nsAString& aDeviceAddress,
     return;
   }
 
-  nsString uuid;
-  BluetoothUuidHelper::GetString(BluetoothServiceClass::HANDSFREE, uuid);
+  const BluetoothUuid uuid(HANDSFREE);
 
   if (NS_FAILED(bs->GetServiceChannel(aDeviceAddress, uuid, this))) {
     aController->NotifyCompletion(NS_LITERAL_STRING(ERR_NO_AVAILABLE_RESOURCE));
@@ -1225,7 +1217,8 @@ BluetoothHfpManager::Disconnect(BluetoothProfileController* aController)
 
   if (!mSocket) {
     if (aController) {
-      aController->NotifyCompletion(NS_LITERAL_STRING(ERR_ALREADY_DISCONNECTED));
+      aController->NotifyCompletion(
+        NS_LITERAL_STRING(ERR_ALREADY_DISCONNECTED));
     }
     return;
   }
@@ -1500,15 +1493,17 @@ BluetoothHfpManager::HandleCallStateChanged(uint32_t aCallIndex,
   }
   mCurrentCallArray[aCallIndex].mNumber = aNumber;
 
-  nsRefPtr<nsRunnable> sendRingTask;
+  RefPtr<Runnable> sendRingTask;
   nsString address;
 
   switch (aCallState) {
     case nsITelephonyService::CALL_STATE_HELD:
       switch (prevCallState) {
         case nsITelephonyService::CALL_STATE_CONNECTED: {
-          uint32_t numActive = GetNumberOfCalls(nsITelephonyService::CALL_STATE_CONNECTED);
-          uint32_t numHeld = GetNumberOfCalls(nsITelephonyService::CALL_STATE_HELD);
+          uint32_t numActive =
+            GetNumberOfCalls(nsITelephonyService::CALL_STATE_CONNECTED);
+          uint32_t numHeld =
+            GetNumberOfCalls(nsITelephonyService::CALL_STATE_HELD);
           uint32_t numConCalls = GetNumberOfConCalls();
 
           /**
@@ -1530,20 +1525,24 @@ BluetoothHfpManager::HandleCallStateChanged(uint32_t aCallIndex,
           if (!aIsConference) {
             if (numActive + numHeld == 1) {
               // A single active call is put on hold.
-              sCINDItems[CINDType::CALLHELD].value = CallHeldState::ONHOLD_NOACTIVE;
+              sCINDItems[CINDType::CALLHELD].value =
+                CallHeldState::ONHOLD_NOACTIVE;
             } else {
               // An active call is placed on hold or active/held calls swapped.
-              sCINDItems[CINDType::CALLHELD].value = CallHeldState::ONHOLD_ACTIVE;
+              sCINDItems[CINDType::CALLHELD].value =
+                CallHeldState::ONHOLD_ACTIVE;
             }
             SendCommand(RESPONSE_CIEV, CINDType::CALLHELD);
           } else if (GetNumberOfConCalls(nsITelephonyService::CALL_STATE_HELD)
                      == numConCalls) {
             if (numActive + numHeld == numConCalls) {
               // An active conference call is put on hold.
-              sCINDItems[CINDType::CALLHELD].value = CallHeldState::ONHOLD_NOACTIVE;
+              sCINDItems[CINDType::CALLHELD].value =
+                CallHeldState::ONHOLD_NOACTIVE;
             } else {
               // Active calls are placed on hold or active/held calls swapped.
-              sCINDItems[CINDType::CALLHELD].value = CallHeldState::ONHOLD_ACTIVE;
+              sCINDItems[CINDType::CALLHELD].value =
+                CallHeldState::ONHOLD_ACTIVE;
             }
             SendCommand(RESPONSE_CIEV, CINDType::CALLHELD);
           }
@@ -1580,9 +1579,8 @@ BluetoothHfpManager::HandleCallStateChanged(uint32_t aCallIndex,
         }
 
         MessageLoop::current()->PostDelayedTask(
-          FROM_HERE,
-          new SendRingIndicatorTask(number,
-                                    mCurrentCallArray[aCallIndex].mType),
+          MakeAndAddRef<SendRingIndicatorTask>(
+            number, mCurrentCallArray[aCallIndex].mType),
           sRingInterval);
       }
       break;
@@ -1690,8 +1688,7 @@ BluetoothHfpManager::HandleCallStateChanged(uint32_t aCallIndex,
           DisconnectSco();
         } else {
           // Close Sco later since Dialer is still playing busy tone via HF.
-          MessageLoop::current()->PostDelayedTask(FROM_HERE,
-                                                  new CloseScoTask(),
+          MessageLoop::current()->PostDelayedTask(MakeAndAddRef<CloseScoTask>(),
                                                   sBusyToneInterval);
         }
 
@@ -1872,7 +1869,7 @@ BluetoothHfpManager::OnSocketDisconnect(BluetoothSocket* aSocket)
 }
 
 void
-BluetoothHfpManager::OnUpdateSdpRecords(const nsAString& aDeviceAddress)
+BluetoothHfpManager::OnUpdateSdpRecords(const BluetoothAddress& aDeviceAddress)
 {
   // UpdateSdpRecord() is not called so this callback function should not
   // be invoked.
@@ -1880,12 +1877,13 @@ BluetoothHfpManager::OnUpdateSdpRecords(const nsAString& aDeviceAddress)
 }
 
 void
-BluetoothHfpManager::OnGetServiceChannel(const nsAString& aDeviceAddress,
-                                         const nsAString& aServiceUuid,
-                                         int aChannel)
+BluetoothHfpManager::OnGetServiceChannel(
+  const BluetoothAddress& aDeviceAddress,
+  const BluetoothUuid& aServiceUuid,
+  int aChannel)
 {
   MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(!aDeviceAddress.IsEmpty());
+  MOZ_ASSERT(!aDeviceAddress.IsCleared());
 
   BluetoothService* bs = BluetoothService::Get();
   NS_ENSURE_TRUE_VOID(bs);
@@ -1893,13 +1891,12 @@ BluetoothHfpManager::OnGetServiceChannel(const nsAString& aDeviceAddress,
   if (aChannel < 0) {
     // If we can't find Handsfree server channel number on the remote device,
     // try to create HSP connection instead.
-    nsString hspUuid;
-    BluetoothUuidHelper::GetString(BluetoothServiceClass::HEADSET, hspUuid);
+    const BluetoothUuid uuid(HEADSET);
 
-    if (aServiceUuid.Equals(hspUuid)) {
+    if (aServiceUuid == uuid) {
       OnConnect(NS_LITERAL_STRING(ERR_SERVICE_CHANNEL_NOT_FOUND));
     } else if (NS_FAILED(bs->GetServiceChannel(aDeviceAddress,
-                                               hspUuid, this))) {
+                                               uuid, this))) {
       OnConnect(NS_LITERAL_STRING(ERR_NO_AVAILABLE_RESOURCE));
     } else {
       mIsHsp = true;
@@ -1969,7 +1966,7 @@ BluetoothHfpManager::IsConnected()
 }
 
 void
-BluetoothHfpManager::GetAddress(nsAString& aDeviceAddress)
+BluetoothHfpManager::GetAddress(BluetoothAddress& aDeviceAddress)
 {
   return mSocket->GetAddress(aDeviceAddress);
 }
@@ -2082,7 +2079,7 @@ BluetoothHfpManager::OnConnect(const nsAString& aErrorStr)
    */
   NS_ENSURE_TRUE_VOID(mController);
 
-  nsRefPtr<BluetoothProfileController> controller = mController.forget();
+  RefPtr<BluetoothProfileController> controller = mController.forget();
   controller->NotifyCompletion(aErrorStr);
 }
 
@@ -2101,9 +2098,18 @@ BluetoothHfpManager::OnDisconnect(const nsAString& aErrorStr)
    */
   NS_ENSURE_TRUE_VOID(mController);
 
-  nsRefPtr<BluetoothProfileController> controller = mController.forget();
+  RefPtr<BluetoothProfileController> controller = mController.forget();
   controller->NotifyCompletion(aErrorStr);
 }
 
+bool
+BluetoothHfpManager::IsNrecEnabled()
+{
+  // Add this function and return default value to avoid build break
+  // since NREC function isn't developed in bluez yet.
+  // Please see Bug 825149 for more information.
+  return HFP_NREC_STARTED;
+}
+
 NS_IMPL_ISUPPORTS(BluetoothHfpManager, nsIObserver)
 
