@@ -54,16 +54,9 @@ public:
     nsresult rv = BodyCreateDir(aDBDir);
     if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
 
-    {
-      mozStorageTransaction trans(aConn, false,
-                                  mozIStorageConnection::TRANSACTION_IMMEDIATE);
-
-      rv = db::CreateSchema(aConn);
-      if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
-
-      rv = trans.Commit();
-      if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
-    }
+    // executes in its own transaction
+    rv = db::CreateOrMigrateSchema(aConn);
+    if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
 
     // If the Context marker file exists, then the last session was
     // not cleanly shutdown.  In these cases sqlite will ensure that
@@ -81,12 +74,12 @@ public:
                                   mozIStorageConnection::TRANSACTION_IMMEDIATE);
 
       // Clean up orphaned Cache objects
-      nsAutoTArray<CacheId, 8> orphanedCacheIdList;
+      AutoTArray<CacheId, 8> orphanedCacheIdList;
       nsresult rv = db::FindOrphanedCacheIds(aConn, orphanedCacheIdList);
       if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
 
       for (uint32_t i = 0; i < orphanedCacheIdList.Length(); ++i) {
-        nsAutoTArray<nsID, 16> deletedBodyIdList;
+        AutoTArray<nsID, 16> deletedBodyIdList;
         rv = db::DeleteCacheId(aConn, orphanedCacheIdList[i], deletedBodyIdList);
         if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
 
@@ -95,7 +88,7 @@ public:
       }
 
       // Clean up orphaned body objects
-      nsAutoTArray<nsID, 64> knownBodyIdList;
+      AutoTArray<nsID, 64> knownBodyIdList;
       rv = db::GetKnownBodyIds(aConn, knownBodyIdList);
 
       rv = BodyDeleteOrphanedFiles(aDBDir, knownBodyIdList);
@@ -145,7 +138,7 @@ public:
     }
 
     rv = BodyDeleteFiles(dbDir, mDeletedBodyIdList);
-    unused << NS_WARN_IF(NS_FAILED(rv));
+    Unused << NS_WARN_IF(NS_FAILED(rv));
 
     aResolver->Resolve(rv);
   }
@@ -154,12 +147,12 @@ private:
   nsTArray<nsID> mDeletedBodyIdList;
 };
 
-bool IsHeadRequest(CacheRequest aRequest, CacheQueryParams aParams)
+bool IsHeadRequest(const CacheRequest& aRequest, const CacheQueryParams& aParams)
 {
   return !aParams.ignoreMethod() && aRequest.method().LowerCaseEqualsLiteral("head");
 }
 
-bool IsHeadRequest(CacheRequestOrVoid aRequest, CacheQueryParams aParams)
+bool IsHeadRequest(const CacheRequestOrVoid& aRequest, const CacheQueryParams& aParams)
 {
   if (aRequest.type() == CacheRequestOrVoid::TCacheRequest) {
     return !aParams.ignoreMethod() &&
@@ -189,7 +182,7 @@ public:
     nsresult rv = MaybeCreateInstance();
     if (NS_WARN_IF(NS_FAILED(rv))) { return rv; }
 
-    nsRefPtr<Manager> ref = Get(aManagerId);
+    RefPtr<Manager> ref = Get(aManagerId);
     if (!ref) {
       // TODO: replace this with a thread pool (bug 1119864)
       nsCOMPtr<nsIThread> ioThread;
@@ -201,7 +194,7 @@ public:
       // There may be an old manager for this origin in the process of
       // cleaning up.  We need to tell the new manager about this so
       // that it won't actually start until the old manager is done.
-      nsRefPtr<Manager> oldManager = Get(aManagerId, Closing);
+      RefPtr<Manager> oldManager = Get(aManagerId, Closing);
       ref->Init(oldManager);
 
       MOZ_ASSERT(!sFactory->mManagerList.Contains(ref));
@@ -226,7 +219,7 @@ public:
     // chains to an old Manager we want it to be the most recent one.
     ManagerList::BackwardIterator iter(sFactory->mManagerList);
     while (iter.HasMore()) {
-      nsRefPtr<Manager> manager = iter.GetNext();
+      RefPtr<Manager> manager = iter.GetNext();
       if (aState == manager->GetState() && *manager->mManagerId == *aManagerId) {
         return manager.forget();
       }
@@ -249,55 +242,61 @@ public:
   }
 
   static void
-  StartAbortOnMainThread(const nsACString& aOrigin)
+  Abort(const nsACString& aOrigin)
   {
-    MOZ_ASSERT(NS_IsMainThread());
+    mozilla::ipc::AssertIsOnBackgroundThread();
 
-    // Lock for sBackgroundThread.
-    StaticMutexAutoLock lock(sMutex);
-
-    if (!sBackgroundThread) {
+    if (!sFactory) {
       return;
     }
 
-    // Guaranteed to succeed because we should get abort only before the
-    // background thread is destroyed.
-    nsCOMPtr<nsIRunnable> runnable = new AbortRunnable(aOrigin);
-    nsresult rv = sBackgroundThread->Dispatch(runnable,
-                                              nsIThread::DISPATCH_NORMAL);
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(rv));
+    MOZ_ASSERT(!sFactory->mManagerList.IsEmpty());
+
+    {
+      ManagerList::ForwardIterator iter(sFactory->mManagerList);
+      while (iter.HasMore()) {
+        RefPtr<Manager> manager = iter.GetNext();
+        if (aOrigin.IsVoid() ||
+            manager->mManagerId->QuotaOrigin() == aOrigin) {
+          manager->Abort();
+        }
+      }
+    }
   }
 
   static void
-  StartShutdownAllOnMainThread()
+  ShutdownAll()
   {
-    MOZ_ASSERT(NS_IsMainThread());
+    mozilla::ipc::AssertIsOnBackgroundThread();
 
-    // Lock for sFactoryShutdown and sBackgroundThread.
-    StaticMutexAutoLock lock(sMutex);
-
-    sFactoryShutdown = true;
-
-    if (!sBackgroundThread) {
+    if (!sFactory) {
       return;
     }
 
-    // Guaranteed to succeed because we should be shutdown before the
-    // background thread is destroyed.
-    nsCOMPtr<nsIRunnable> runnable = new ShutdownAllRunnable();
-    nsresult rv = sBackgroundThread->Dispatch(runnable,
-                                              nsIThread::DISPATCH_NORMAL);
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(rv));
+    MOZ_ASSERT(!sFactory->mManagerList.IsEmpty());
+
+    {
+      // Note that we are synchronously calling shutdown code here.  If any
+      // of the shutdown code synchronously decides to delete the Factory
+      // we need to delay that delete until the end of this method.
+      AutoRestore<bool> restore(sFactory->mInSyncShutdown);
+      sFactory->mInSyncShutdown = true;
+
+      ManagerList::ForwardIterator iter(sFactory->mManagerList);
+      while (iter.HasMore()) {
+        RefPtr<Manager> manager = iter.GetNext();
+        manager->Shutdown();
+      }
+    }
+
+    MaybeDestroyInstance();
   }
 
   static bool
-  IsShutdownAllCompleteOnMainThread()
+  IsShutdownAllComplete()
   {
-    MOZ_ASSERT(NS_IsMainThread());
-    StaticMutexAutoLock lock(sMutex);
-    // Infer whether we have shutdown using the sBackgroundThread value.  We
-    // guarantee this is nullptr when sFactory is destroyed.
-    return sFactoryShutdown && !sBackgroundThread;
+    mozilla::ipc::AssertIsOnBackgroundThread();
+    return !sFactory;
   }
 
 private:
@@ -329,12 +328,6 @@ private:
         if (sFactoryShutdown) {
           return NS_ERROR_ILLEGAL_DURING_SHUTDOWN;
         }
-
-        // Cannot use ClearOnShutdown() because we're on the background thread.
-        // This is automatically cleared when Factory::Remove() calls
-        // MaybeDestroyInstance().
-        MOZ_ASSERT(!sBackgroundThread);
-        sBackgroundThread = NS_GetCurrentThread();
       }
 
       // We cannot use ClearOnShutdown() here because we're not on the main
@@ -366,134 +359,21 @@ private:
       return;
     }
 
-    // Be clear about what we are locking.  sFactory is bg thread only, so
-    // we don't need to lock it here.  Just protect sBackgroundThread.
-    {
-      StaticMutexAutoLock lock(sMutex);
-      MOZ_ASSERT(sBackgroundThread);
-      sBackgroundThread = nullptr;
-    }
-
     sFactory = nullptr;
   }
 
-  static void
-  AbortOnBackgroundThread(const nsACString& aOrigin)
-  {
-    mozilla::ipc::AssertIsOnBackgroundThread();
-
-    // The factory was destroyed between when abort started on main thread and
-    // when we could start abort on the worker thread.  Just declare abort
-    // complete.
-    if (!sFactory) {
-#ifdef DEBUG
-      StaticMutexAutoLock lock(sMutex);
-      MOZ_ASSERT(!sBackgroundThread);
-#endif
-      return;
-    }
-
-    MOZ_ASSERT(!sFactory->mManagerList.IsEmpty());
-
-    {
-      ManagerList::ForwardIterator iter(sFactory->mManagerList);
-      while (iter.HasMore()) {
-        nsRefPtr<Manager> manager = iter.GetNext();
-        if (aOrigin.IsVoid() ||
-            manager->mManagerId->ExtendedOrigin() == aOrigin) {
-          manager->Abort();
-        }
-      }
-    }
-  }
-
-  static void
-  ShutdownAllOnBackgroundThread()
-  {
-    mozilla::ipc::AssertIsOnBackgroundThread();
-
-    // The factory shutdown between when shutdown started on main thread and
-    // when we could start shutdown on the worker thread.  Just declare
-    // shutdown complete.  The sFactoryShutdown flag prevents the factory
-    // from racing to restart here.
-    if (!sFactory) {
-#ifdef DEBUG
-      StaticMutexAutoLock lock(sMutex);
-      MOZ_ASSERT(!sBackgroundThread);
-#endif
-      return;
-    }
-
-    MOZ_ASSERT(!sFactory->mManagerList.IsEmpty());
-
-    {
-      // Note that we are synchronously calling shutdown code here.  If any
-      // of the shutdown code synchronously decides to delete the Factory
-      // we need to delay that delete until the end of this method.
-      AutoRestore<bool> restore(sFactory->mInSyncShutdown);
-      sFactory->mInSyncShutdown = true;
-
-      ManagerList::ForwardIterator iter(sFactory->mManagerList);
-      while (iter.HasMore()) {
-        nsRefPtr<Manager> manager = iter.GetNext();
-        manager->Shutdown();
-      }
-    }
-
-    MaybeDestroyInstance();
-  }
-
-  class AbortRunnable final : public nsRunnable
-  {
-  public:
-    explicit AbortRunnable(const nsACString& aOrigin)
-      : mOrigin(aOrigin)
-    { }
-
-    NS_IMETHOD
-    Run() override
-    {
-      mozilla::ipc::AssertIsOnBackgroundThread();
-      AbortOnBackgroundThread(mOrigin);
-      return NS_OK;
-    }
-  private:
-    ~AbortRunnable() { }
-
-    const nsCString mOrigin;
-  };
-
-  class ShutdownAllRunnable final : public nsRunnable
-  {
-  public:
-    NS_IMETHOD
-    Run() override
-    {
-      mozilla::ipc::AssertIsOnBackgroundThread();
-      ShutdownAllOnBackgroundThread();
-      return NS_OK;
-    }
-  private:
-    ~ShutdownAllRunnable() { }
-  };
-
   // Singleton created on demand and deleted when last Manager is cleared
   // in Remove().
   // PBackground thread only.
   static StaticAutoPtr<Factory> sFactory;
 
-  // protects following static attributes
+  // protects following static attribute
   static StaticMutex sMutex;
 
   // Indicate if shutdown has occurred to block re-creation of sFactory.
   // Must hold sMutex to access.
   static bool sFactoryShutdown;
 
-  // Background thread owning all Manager objects.  Only set while sFactory is
-  // set.
-  // Must hold sMutex to access.
-  static StaticRefPtr<nsIThread> sBackgroundThread;
-
   // Weak references as we don't want to keep Manager objects alive forever.
   // When a Manager is destroyed it calls Factory::Remove() to clear itself.
   // PBackground thread only.
@@ -515,9 +395,6 @@ StaticMutex Manager::Factory::sMutex;
 // static
 bool Manager::Factory::sFactoryShutdown = false;
 
-// static
-StaticRefPtr<nsIThread> Manager::Factory::sBackgroundThread;
-
 // ----------------------------------------------------------------------------
 
 // Abstract class to help implement the various Actions.  The vast majority
@@ -549,7 +426,7 @@ protected:
     mManager = nullptr;
   }
 
-  nsRefPtr<Manager> mManager;
+  RefPtr<Manager> mManager;
   const ListenerId mListenerId;
 };
 
@@ -592,7 +469,7 @@ public:
   }
 
 private:
-  nsRefPtr<Manager> mManager;
+  RefPtr<Manager> mManager;
   const CacheId mCacheId;
   nsTArray<nsID> mDeletedBodyIdList;
 };
@@ -657,7 +534,7 @@ public:
 private:
   const CacheId mCacheId;
   const CacheMatchArgs mArgs;
-  nsRefPtr<StreamList> mStreamList;
+  RefPtr<StreamList> mStreamList;
   bool mFoundResponse;
   SavedResponse mResponse;
 };
@@ -720,7 +597,7 @@ public:
 private:
   const CacheId mCacheId;
   const CacheMatchAllArgs mArgs;
-  nsRefPtr<StreamList> mStreamList;
+  RefPtr<StreamList> mStreamList;
   nsTArray<SavedResponse> mSavedResponses;
 };
 
@@ -904,7 +781,7 @@ private:
     }
 
     rv = trans.Commit();
-    unused << NS_WARN_IF(NS_FAILED(rv));
+    Unused << NS_WARN_IF(NS_FAILED(rv));
 
     DoResolve(rv);
   }
@@ -1035,10 +912,10 @@ private:
     // May be on any thread, including STS event target.  Non-owning runnable
     // here since we are guaranteed the Action will survive until
     // CompleteOnInitiatingThread is called.
-    nsCOMPtr<nsIRunnable> runnable = NS_NewNonOwningRunnableMethodWithArgs<nsresult>(
+    nsCOMPtr<nsIRunnable> runnable = NewNonOwningRunnableMethod<nsresult>(
       this, &CachePutAllAction::OnAsyncCopyComplete, aRv);
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(
-      mTargetThread->Dispatch(runnable, nsIThread::DISPATCH_NORMAL)));
+    MOZ_ALWAYS_SUCCEEDS(
+      mTargetThread->Dispatch(runnable, nsIThread::DISPATCH_NORMAL));
   }
 
   void
@@ -1068,13 +945,13 @@ private:
     mTargetThread = nullptr;
 
     // Make sure to de-ref the resolver per the Action API contract.
-    nsRefPtr<Action::Resolver> resolver;
+    RefPtr<Action::Resolver> resolver;
     mResolver.swap(resolver);
     resolver->Resolve(aRv);
   }
 
   // initiating thread only
-  nsRefPtr<Manager> mManager;
+  RefPtr<Manager> mManager;
   const ListenerId mListenerId;
 
   // Set on initiating thread, read on target thread.  State machine guarantees
@@ -1084,7 +961,7 @@ private:
   uint32_t mExpectedAsyncCopyCompletions;
 
   // target thread only
-  nsRefPtr<Resolver> mResolver;
+  RefPtr<Resolver> mResolver;
   nsCOMPtr<nsIFile> mDBDir;
   nsCOMPtr<mozIStorageConnection> mConn;
   nsCOMPtr<nsIThread> mTargetThread;
@@ -1211,7 +1088,7 @@ public:
 private:
   const CacheId mCacheId;
   const CacheKeysArgs mArgs;
-  nsRefPtr<StreamList> mStreamList;
+  RefPtr<StreamList> mStreamList;
   nsTArray<SavedRequest> mSavedRequests;
 };
 
@@ -1273,7 +1150,7 @@ public:
 private:
   const Namespace mNamespace;
   const StorageMatchArgs mArgs;
-  nsRefPtr<StreamList> mStreamList;
+  RefPtr<StreamList> mStreamList;
   bool mFoundResponse;
   SavedResponse mSavedResponse;
 };
@@ -1416,13 +1293,13 @@ public:
       if (!mManager->SetCacheIdOrphanedIfRefed(mCacheId)) {
 
         // no outstanding references, delete immediately
-        nsRefPtr<Context> context = mManager->mContext;
+        RefPtr<Context> context = mManager->mContext;
 
         if (context->IsCanceled()) {
           context->NoteOrphanedData();
         } else {
           context->CancelForCacheId(mCacheId);
-          nsRefPtr<Action> action =
+          RefPtr<Action> action =
             new DeleteOrphanedCacheAction(mManager, mCacheId);
           context->Dispatch(action);
         }
@@ -1496,7 +1373,7 @@ Manager::Listener::OnOpComplete(ErrorResult&& aRv, const CacheOpResult& aResult,
                                 const SavedResponse& aSavedResponse,
                                 StreamList* aStreamList)
 {
-  nsAutoTArray<SavedResponse, 1> responseList;
+  AutoTArray<SavedResponse, 1> responseList;
   responseList.AppendElement(aSavedResponse);
   OnOpComplete(Move(aRv), aResult, INVALID_CACHE_ID, responseList,
                nsTArray<SavedRequest>(), aStreamList);
@@ -1538,13 +1415,13 @@ Manager::Get(ManagerId* aManagerId)
 
 // static
 void
-Manager::ShutdownAllOnMainThread()
+Manager::ShutdownAll()
 {
-  MOZ_ASSERT(NS_IsMainThread());
+  mozilla::ipc::AssertIsOnBackgroundThread();
 
-  Factory::StartShutdownAllOnMainThread();
+  Factory::ShutdownAll();
 
-  while (!Factory::IsShutdownAllCompleteOnMainThread()) {
+  while (!Factory::IsShutdownAllComplete()) {
     if (!NS_ProcessNextEvent()) {
       NS_WARNING("Something bad happened!");
       break;
@@ -1554,11 +1431,11 @@ Manager::ShutdownAllOnMainThread()
 
 // static
 void
-Manager::AbortOnMainThread(const nsACString& aOrigin)
+Manager::Abort(const nsACString& aOrigin)
 {
-  MOZ_ASSERT(NS_IsMainThread());
+  mozilla::ipc::AssertIsOnBackgroundThread();
 
-  Factory::StartAbortOnMainThread(aOrigin);
+  Factory::Abort(aOrigin);
 }
 
 void
@@ -1653,7 +1530,7 @@ Manager::ReleaseCacheId(CacheId aCacheId)
       if (mCacheIdRefs[i].mCount == 0) {
         bool orphaned = mCacheIdRefs[i].mOrphaned;
         mCacheIdRefs.RemoveElementAt(i);
-        nsRefPtr<Context> context = mContext;
+        RefPtr<Context> context = mContext;
         // If the context is already gone, then orphan flag should have been
         // set in RemoveContext().
         if (orphaned && context) {
@@ -1661,7 +1538,7 @@ Manager::ReleaseCacheId(CacheId aCacheId)
             context->NoteOrphanedData();
           } else {
             context->CancelForCacheId(aCacheId);
-            nsRefPtr<Action> action = new DeleteOrphanedCacheAction(this,
+            RefPtr<Action> action = new DeleteOrphanedCacheAction(this,
                                                                     aCacheId);
             context->Dispatch(action);
           }
@@ -1702,14 +1579,14 @@ Manager::ReleaseBodyId(const nsID& aBodyId)
       if (mBodyIdRefs[i].mCount < 1) {
         bool orphaned = mBodyIdRefs[i].mOrphaned;
         mBodyIdRefs.RemoveElementAt(i);
-        nsRefPtr<Context> context = mContext;
+        RefPtr<Context> context = mContext;
         // If the context is already gone, then orphan flag should have been
         // set in RemoveContext().
         if (orphaned && context) {
           if (context->IsCanceled()) {
             context->NoteOrphanedData();
           } else {
-            nsRefPtr<Action> action = new DeleteOrphanedBodyAction(aBodyId);
+            RefPtr<Action> action = new DeleteOrphanedBodyAction(aBodyId);
             context->Dispatch(action);
           }
         }
@@ -1724,7 +1601,7 @@ Manager::ReleaseBodyId(const nsID& aBodyId)
 already_AddRefed<ManagerId>
 Manager::GetManagerId() const
 {
-  nsRefPtr<ManagerId> ref = mManagerId;
+  RefPtr<ManagerId> ref = mManagerId;
   return ref.forget();
 }
 
@@ -1757,13 +1634,13 @@ Manager::ExecuteCacheOp(Listener* aListener, CacheId aCacheId,
     return;
   }
 
-  nsRefPtr<Context> context = mContext;
+  RefPtr<Context> context = mContext;
   MOZ_ASSERT(!context->IsCanceled());
 
-  nsRefPtr<StreamList> streamList = new StreamList(this, context);
+  RefPtr<StreamList> streamList = new StreamList(this, context);
   ListenerId listenerId = SaveListener(aListener);
 
-  nsRefPtr<Action> action;
+  RefPtr<Action> action;
   switch(aOpArgs.type()) {
     case CacheOpArgs::TCacheMatchArgs:
       action = new CacheMatchAction(this, listenerId, aCacheId,
@@ -1801,13 +1678,13 @@ Manager::ExecuteStorageOp(Listener* aListener, Namespace aNamespace,
     return;
   }
 
-  nsRefPtr<Context> context = mContext;
+  RefPtr<Context> context = mContext;
   MOZ_ASSERT(!context->IsCanceled());
 
-  nsRefPtr<StreamList> streamList = new StreamList(this, context);
+  RefPtr<StreamList> streamList = new StreamList(this, context);
   ListenerId listenerId = SaveListener(aListener);
 
-  nsRefPtr<Action> action;
+  RefPtr<Action> action;
   switch(aOpArgs.type()) {
     case CacheOpArgs::TStorageMatchArgs:
       action = new StorageMatchAction(this, listenerId, aNamespace,
@@ -1850,12 +1727,12 @@ Manager::ExecutePutAll(Listener* aListener, CacheId aCacheId,
     return;
   }
 
-  nsRefPtr<Context> context = mContext;
+  RefPtr<Context> context = mContext;
   MOZ_ASSERT(!context->IsCanceled());
 
   ListenerId listenerId = SaveListener(aListener);
 
-  nsRefPtr<Action> action = new CachePutAllAction(this, listenerId, aCacheId,
+  RefPtr<Action> action = new CachePutAllAction(this, listenerId, aCacheId,
                                                   aPutList, aRequestStreamList,
                                                   aResponseStreamList);
 
@@ -1884,9 +1761,7 @@ Manager::~Manager()
 
   // Don't spin the event loop in the destructor waiting for the thread to
   // shutdown.  Defer this to the main thread, instead.
-  nsCOMPtr<nsIRunnable> runnable =
-    NS_NewRunnableMethod(ioThread, &nsIThread::Shutdown);
-  MOZ_ALWAYS_TRUE(NS_SUCCEEDED(NS_DispatchToMainThread(runnable)));
+  MOZ_ALWAYS_SUCCEEDS(NS_DispatchToMainThread(NewRunnableMethod(ioThread, &nsIThread::Shutdown)));
 }
 
 void
@@ -1894,7 +1769,7 @@ Manager::Init(Manager* aOldManager)
 {
   NS_ASSERT_OWNINGTHREAD(Manager);
 
-  nsRefPtr<Context> oldContext;
+  RefPtr<Context> oldContext;
   if (aOldManager) {
     oldContext = aOldManager->mContext;
   }
@@ -1902,8 +1777,8 @@ Manager::Init(Manager* aOldManager)
   // Create the context immediately.  Since there can at most be one Context
   // per Manager now, this lets us cleanly call Factory::Remove() once the
   // Context goes away.
-  nsRefPtr<Action> setupAction = new SetupAction();
-  nsRefPtr<Context> ref = Context::Create(this, mIOThread, setupAction,
+  RefPtr<Action> setupAction = new SetupAction();
+  RefPtr<Context> ref = Context::Create(this, mIOThread, setupAction,
                                           oldContext);
   mContext = ref;
 }
@@ -1930,7 +1805,7 @@ Manager::Shutdown()
   // If there is a context, then cancel and only note that we are done after
   // its cleaned up.
   if (mContext) {
-    nsRefPtr<Context> context = mContext;
+    RefPtr<Context> context = mContext;
     context->CancelAll();
     return;
   }
@@ -1948,7 +1823,7 @@ Manager::Abort()
   NoteClosing();
 
   // Cancel and only note that we are done after the context is cleaned up.
-  nsRefPtr<Context> context = mContext;
+  RefPtr<Context> context = mContext;
   context->CancelAll();
 }
 
@@ -2025,7 +1900,7 @@ Manager::NoteOrphanedBodyIdList(const nsTArray<nsID>& aDeletedBodyIdList)
 {
   NS_ASSERT_OWNINGTHREAD(Manager);
 
-  nsAutoTArray<nsID, 64> deleteNowList;
+  AutoTArray<nsID, 64> deleteNowList;
   deleteNowList.SetCapacity(aDeletedBodyIdList.Length());
 
   for (uint32_t i = 0; i < aDeletedBodyIdList.Length(); ++i) {
@@ -2035,9 +1910,9 @@ Manager::NoteOrphanedBodyIdList(const nsTArray<nsID>& aDeletedBodyIdList)
   }
 
   // TODO: note that we need to check these bodies for staleness on startup (bug 1110446)
-  nsRefPtr<Context> context = mContext;
+  RefPtr<Context> context = mContext;
   if (!deleteNowList.IsEmpty() && context && !context->IsCanceled()) {
-    nsRefPtr<Action> action = new DeleteOrphanedBodyAction(deleteNowList);
+    RefPtr<Action> action = new DeleteOrphanedBodyAction(deleteNowList);
     context->Dispatch(action);
   }
 }
@@ -2052,7 +1927,7 @@ Manager::MaybeAllowContextToClose()
   // Cache state information to complete before doing this.  Once we allow
   // the Context to close we may not reliably get notified of storage
   // invalidation.
-  nsRefPtr<Context> context = mContext;
+  RefPtr<Context> context = mContext;
   if (context && mListeners.IsEmpty()
               && mCacheIdRefs.IsEmpty()
               && mBodyIdRefs.IsEmpty()) {
