@@ -37,6 +37,7 @@
 #include "mozilla/dom/CameraFacesDetectedEventBinding.h"
 #include "mozilla/dom/CameraStateChangeEvent.h"
 #include "mozilla/dom/CameraClosedEvent.h"
+#include "mozilla/dom/VideoStreamTrack.h"
 #include "mozilla/dom/BlobEvent.h"
 #include "DOMCameraDetectedFace.h"
 #include "mozilla/dom/BindingUtils.h"
@@ -46,6 +47,43 @@ using namespace mozilla;
 using namespace mozilla::dom;
 using namespace mozilla::ipc;
 
+class mozilla::TrackCreatedListener : public MediaStreamListener
+{
+public:
+  explicit TrackCreatedListener(nsDOMCameraControl* aCameraControl)
+    : mCameraControl(aCameraControl) {}
+
+  void Forget() { mCameraControl = nullptr; }
+
+  void DoNotifyTrackCreated(TrackID aTrackID)
+  {
+    MOZ_ASSERT(NS_IsMainThread());
+
+    if (!mCameraControl) {
+      return;
+    }
+
+    mCameraControl->TrackCreated(aTrackID);
+  }
+
+  void NotifyQueuedTrackChanges(MediaStreamGraph* aGraph, TrackID aID,
+                                StreamTime aTrackOffset, uint32_t aTrackEvents,
+                                const MediaSegment& aQueuedMedia,
+                                MediaStream* aInputStream,
+                                TrackID aInputTrackID) override
+  {
+    if (aTrackEvents & TRACK_EVENT_CREATED) {
+      aGraph->DispatchToMainThreadAfterStreamStateUpdate(NewRunnableMethod<TrackID>(
+          this, &TrackCreatedListener::DoNotifyTrackCreated, aID));
+    }
+  }
+
+protected:
+  ~TrackCreatedListener() {}
+
+  nsDOMCameraControl* mCameraControl;
+};
+
 #ifdef MOZ_WIDGET_GONK
 StaticRefPtr<ICameraControl> nsDOMCameraControl::sCachedCameraControl;
 /* static */ nsresult nsDOMCameraControl::sCachedCameraControlStartResult = NS_OK;
@@ -116,7 +154,7 @@ protected:
   }
 
 protected:
-  nsRefPtr<nsDOMCameraControl> mDOMCameraControl;
+  RefPtr<nsDOMCameraControl> mDOMCameraControl;
   bool mState;
 };
 
@@ -131,44 +169,6 @@ StartRecordingHelper::HandleEvent(nsIDOMEvent* aEvent)
 
 NS_IMPL_ISUPPORTS(mozilla::StartRecordingHelper, nsIDOMEventListener)
 
-class mozilla::RecorderPosterHelper : public nsIDOMEventListener
-{
-public:
-  NS_DECL_ISUPPORTS
-  NS_DECL_NSIDOMEVENTLISTENER
-
-  explicit RecorderPosterHelper(nsDOMCameraControl* aDOMCameraControl)
-    : mDOMCameraControl(aDOMCameraControl)
-    , mState(CameraControlListener::kPosterFailed)
-  {
-    MOZ_COUNT_CTOR(RecorderPosterHelper);
-  }
-
-protected:
-  virtual ~RecorderPosterHelper()
-  {
-    MOZ_COUNT_DTOR(RecorderPosterHelper);
-    mDOMCameraControl->OnRecorderStateChange(mState, 0, 0);
-  }
-
-protected:
-  nsRefPtr<nsDOMCameraControl> mDOMCameraControl;
-  CameraControlListener::RecorderState mState;
-};
-
-NS_IMETHODIMP
-RecorderPosterHelper::HandleEvent(nsIDOMEvent* aEvent)
-{
-  nsString eventType;
-  aEvent->GetType(eventType);
-  if (eventType.EqualsLiteral("success")) {
-    mState = CameraControlListener::kPosterCreated;
-  }
-  return NS_OK;
-}
-
-NS_IMPL_ISUPPORTS(mozilla::RecorderPosterHelper, nsIDOMEventListener)
-
 nsDOMCameraControl::DOMCameraConfiguration::DOMCameraConfiguration()
   : CameraConfiguration()
   , mMaxFocusAreas(0)
@@ -202,7 +202,7 @@ nsDOMCameraControl::PreinitCameraHardware()
 {
   // Assume a default, minimal configuration. This should initialize the
   // hardware, but won't (can't) start the preview.
-  nsRefPtr<ICameraControl> cameraControl = ICameraControl::Create(kDefaultCameraId);
+  RefPtr<ICameraControl> cameraControl = ICameraControl::Create(kDefaultCameraId);
   if (NS_WARN_IF(!cameraControl)) {
     return;
   }
@@ -247,8 +247,8 @@ nsDOMCameraControl::DiscardCachedCameraInstance(nsITimer* aTimer, void* aClosure
 nsDOMCameraControl::nsDOMCameraControl(uint32_t aCameraId,
                                        const CameraConfiguration& aInitialConfig,
                                        Promise* aPromise,
-                                       nsPIDOMWindow* aWindow)
-  : DOMMediaStream()
+                                       nsPIDOMWindowInner* aWindow)
+  : DOMMediaStream(aWindow, nullptr)
   , mCameraControl(nullptr)
   , mAudioChannelAgent(nullptr)
   , mGetCameraPromise(aPromise)
@@ -260,10 +260,11 @@ nsDOMCameraControl::nsDOMCameraControl(uint32_t aCameraId,
 {
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   mInput = new CameraPreviewMediaStream(this);
+  mOwnedStream = mInput;
 
   BindToOwner(aWindow);
 
-  nsRefPtr<DOMCameraConfiguration> initialConfig =
+  RefPtr<DOMCameraConfiguration> initialConfig =
     new DOMCameraConfiguration(aInitialConfig);
 
   // Create and initialize the underlying camera.
@@ -317,12 +318,17 @@ nsDOMCameraControl::nsDOMCameraControl(uint32_t aCameraId,
 #endif
   mCurrentConfiguration = initialConfig.forget();
 
-  // Attach our DOM-facing media stream to our viewfinder stream.
-  InitStreamCommon(mInput);
-  MOZ_ASSERT(mWindow, "Shouldn't be created with a null window!");
-  if (mWindow->GetExtantDoc()) {
-    CombineWithPrincipal(mWindow->GetExtantDoc()->NodePrincipal());
-  }
+  // Register a TrackCreatedListener directly on CameraPreviewMediaStream
+  // so we can know the TrackID of the video track.
+  mTrackCreatedListener = new TrackCreatedListener(this);
+  mInput->AddListener(mTrackCreatedListener);
+
+  // Register the playback listener directly on the camera input stream.
+  // We want as low latency as possible for the camera, thus avoiding
+  // MediaStreamGraph altogether. Don't do the regular InitStreamCommon()
+  // to avoid initializing the Owned and Playback streams. This is OK since
+  // we are not user/DOM facing anyway.
+  CreateAndAddPlaybackStreamListener(mInput);
 
   // Register a listener for camera events.
   mListener = new DOMCameraControlListener(this, mInput);
@@ -362,6 +368,15 @@ nsDOMCameraControl::~nsDOMCameraControl()
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   /*invoke DOMMediaStream destroy*/
   Destroy();
+
+  if (mInput) {
+    mInput->Destroy();
+    mInput = nullptr;
+  }
+  if (mTrackCreatedListener) {
+    mTrackCreatedListener->Forget();
+    mTrackCreatedListener = nullptr;
+  }
 }
 
 JSObject*
@@ -497,6 +512,32 @@ nsDOMCameraControl::Get(uint32_t aKey, nsTArray<CameraRegion>& aValue)
   return NS_OK;
 }
 
+MediaStream*
+nsDOMCameraControl::GetCameraStream() const
+{
+  return mInput;
+}
+
+void
+nsDOMCameraControl::TrackCreated(TrackID aTrackID) {
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_RELEASE_ASSERT(mWindow, "Shouldn't have been created with a null window!");
+  nsIPrincipal* principal = mWindow->GetExtantDoc()
+                          ? mWindow->GetExtantDoc()->NodePrincipal()
+                          : nullptr;
+
+  // This track is not connected through a port.
+  MediaInputPort* inputPort = nullptr;
+  dom::VideoStreamTrack* track =
+    new dom::VideoStreamTrack(this, aTrackID, aTrackID,
+                              new BasicUnstoppableTrackSource(principal));
+  RefPtr<TrackPort> port =
+    new TrackPort(inputPort, track,
+                  TrackPort::InputPortOwnership::OWNED);
+  mTracks.AppendElement(port.forget());
+  NotifyTrackAdded(track);
+}
+
 #define THROW_IF_NO_CAMERACONTROL(...)                                          \
   do {                                                                          \
     if (!mCameraControl) {                                                      \
@@ -774,7 +815,7 @@ nsDOMCameraControl::Capabilities()
     return nullptr;
   }
 
-  nsRefPtr<CameraCapabilities> caps = mCapabilities;
+  RefPtr<CameraCapabilities> caps = mCapabilities;
   if (!caps) {
     caps = new CameraCapabilities(mWindow, mCameraControl);
     mCapabilities = caps;
@@ -792,23 +833,16 @@ nsDOMCameraControl::StartRecording(const CameraStartRecordingOptions& aOptions,
 {
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
 
-  nsRefPtr<Promise> promise = CreatePromise(aRv);
+  RefPtr<Promise> promise = CreatePromise(aRv);
   if (aRv.Failed()) {
     return nullptr;
   }
 
-  // Must supply both the poster path and storage area or neither
-  if (aOptions.mPosterFilepath.IsEmpty() ==
-      static_cast<bool>(aOptions.mPosterStorageArea.get())) {
-    promise->MaybeReject(NS_ERROR_ILLEGAL_VALUE);
-    return promise.forget();
-  }
-
   // If we are trying to start recording, already recording or are still
   // waiting for a poster to be created/fail, we need to wait
   if (mStartRecordingPromise || mRecording ||
       mRecordingStoppedDeferred ||
-      !mOptions.mPosterFilepath.IsEmpty()) {
+      mOptions.mCreatePoster) {
     promise->MaybeReject(NS_ERROR_IN_PROGRESS);
     return promise.forget();
   }
@@ -819,7 +853,7 @@ nsDOMCameraControl::StartRecording(const CameraStartRecordingOptions& aOptions,
   }
 
   mDSFileDescriptor = new DeviceStorageFileDescriptor();
-  nsRefPtr<DOMRequest> request = aStorageArea.CreateFileDescriptor(aFilename,
+  RefPtr<DOMRequest> request = aStorageArea.CreateFileDescriptor(aFilename,
                                                                    mDSFileDescriptor.get(),
                                                                    aRv);
   if (aRv.Failed()) {
@@ -851,8 +885,7 @@ nsDOMCameraControl::OnCreatedFileDescriptor(bool aSucceeded)
     // Race condition where StopRecording comes in before we issue
     // the start recording request to Gonk
     rv = NS_ERROR_ABORT;
-    mOptions.mPosterFilepath.Truncate();
-    mOptions.mPosterStorageArea = nullptr;
+    mOptions.mCreatePoster = false;
   } else if (aSucceeded && mDSFileDescriptor->mFileDescriptor.IsValid()) {
     ICameraControl::StartRecordingOptions o;
 
@@ -860,7 +893,7 @@ nsDOMCameraControl::OnCreatedFileDescriptor(bool aSucceeded)
     o.maxFileSizeBytes = mOptions.mMaxFileSizeBytes;
     o.maxVideoLengthMs = mOptions.mMaxVideoLengthMs;
     o.autoEnableLowLightTorch = mOptions.mAutoEnableLowLightTorch;
-    o.createPoster = !mOptions.mPosterFilepath.IsEmpty();
+    o.createPoster = mOptions.mCreatePoster;
     rv = mCameraControl->StartRecording(mDSFileDescriptor.get(), &o);
     if (NS_SUCCEEDED(rv)) {
       return;
@@ -873,7 +906,7 @@ nsDOMCameraControl::OnCreatedFileDescriptor(bool aSucceeded)
     // An error occured. We need to manually close the file associated with the
     // FileDescriptor, and we shouldn't do this on the main thread, so we
     // use a little helper.
-    nsRefPtr<CloseFileRunnable> closer =
+    RefPtr<CloseFileRunnable> closer =
       new CloseFileRunnable(mDSFileDescriptor->mFileDescriptor);
     closer->Dispatch();
   }
@@ -890,6 +923,24 @@ nsDOMCameraControl::StopRecording(ErrorResult& aRv)
   aRv = mCameraControl->StopRecording();
 }
 
+void
+nsDOMCameraControl::PauseRecording(ErrorResult& aRv)
+{
+  DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
+  THROW_IF_NO_CAMERACONTROL();
+
+  aRv = mCameraControl->PauseRecording();
+}
+
+void
+nsDOMCameraControl::ResumeRecording(ErrorResult& aRv)
+{
+  DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
+  THROW_IF_NO_CAMERACONTROL();
+
+  aRv = mCameraControl->ResumeRecording();
+}
+
 void
 nsDOMCameraControl::ResumePreview(ErrorResult& aRv)
 {
@@ -905,7 +956,7 @@ nsDOMCameraControl::SetConfiguration(const CameraConfiguration& aConfiguration,
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   THROW_IF_NO_CAMERACONTROL(nullptr);
 
-  nsRefPtr<Promise> promise = CreatePromise(aRv);
+  RefPtr<Promise> promise = CreatePromise(aRv);
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -945,7 +996,7 @@ nsDOMCameraControl::AutoFocus(ErrorResult& aRv)
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   THROW_IF_NO_CAMERACONTROL(nullptr);
 
-  nsRefPtr<Promise> promise = mAutoFocusPromise.forget();
+  RefPtr<Promise> promise = mAutoFocusPromise.forget();
   if (promise) {
     // There is already a call to AutoFocus() in progress, cancel it and
     // invoke the error callback (if one was passed in).
@@ -991,7 +1042,7 @@ nsDOMCameraControl::TakePicture(const CameraPictureOptions& aOptions,
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   THROW_IF_NO_CAMERACONTROL(nullptr);
 
-  nsRefPtr<Promise> promise = CreatePromise(aRv);
+  RefPtr<Promise> promise = CreatePromise(aRv);
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -1042,7 +1093,7 @@ nsDOMCameraControl::ReleaseHardware(ErrorResult& aRv)
 {
   DOM_CAMERA_LOGI("%s:%d : this=%p\n", __func__, __LINE__, this);
 
-  nsRefPtr<Promise> promise = CreatePromise(aRv);
+  RefPtr<Promise> promise = CreatePromise(aRv);
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -1101,7 +1152,7 @@ nsDOMCameraControl::ReleaseAudioChannelAgent()
 {
 #ifdef MOZ_B2G
   if (mAudioChannelAgent) {
-    mAudioChannelAgent->NotifyStoppedPlaying(nsIAudioChannelAgent::AUDIO_AGENT_DONT_NOTIFY);
+    mAudioChannelAgent->NotifyStoppedPlaying();
     mAudioChannelAgent = nullptr;
   }
 #endif
@@ -1135,10 +1186,8 @@ nsDOMCameraControl::NotifyRecordingStatusChange(const nsString& aMsg)
     // Camera app will stop recording when it falls to the background, so no callback is necessary.
     mAudioChannelAgent->Init(mWindow, (int32_t)AudioChannel::Content, nullptr);
     // Video recording doesn't output any sound, so it's not necessary to check canPlay.
-    float volume = 0.0;
-    bool muted = true;
-    rv = mAudioChannelAgent->NotifyStartedPlaying(nsIAudioChannelAgent::AUDIO_AGENT_DONT_NOTIFY,
-                                                  &volume, &muted);
+    AudioPlaybackConfig config;
+    rv = mAudioChannelAgent->NotifyStartedPlaying(&config, AudioChannelService::AudibleState::eAudible);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
@@ -1159,9 +1208,9 @@ nsDOMCameraControl::CreatePromise(ErrorResult& aRv)
 }
 
 void
-nsDOMCameraControl::AbortPromise(nsRefPtr<Promise>& aPromise)
+nsDOMCameraControl::AbortPromise(RefPtr<Promise>& aPromise)
 {
-  nsRefPtr<Promise> promise = aPromise.forget();
+  RefPtr<Promise> promise = aPromise.forget();
   if (promise) {
     promise->MaybeReject(NS_ERROR_NOT_AVAILABLE);
   }
@@ -1198,7 +1247,7 @@ nsDOMCameraControl::DispatchStateEvent(const nsString& aType, const nsString& aS
   CameraStateChangeEventInit eventInit;
   eventInit.mNewState = aState;
 
-  nsRefPtr<CameraStateChangeEvent> event =
+  RefPtr<CameraStateChangeEvent> event =
     CameraStateChangeEvent::Constructor(this, aType, eventInit);
 
   DispatchTrustedEvent(event);
@@ -1209,7 +1258,7 @@ nsDOMCameraControl::OnGetCameraComplete()
 {
   // The hardware is open, so we can return a camera to JS, even if
   // the preview hasn't started yet.
-  nsRefPtr<Promise> promise = mGetCameraPromise.forget();
+  RefPtr<Promise> promise = mGetCameraPromise.forget();
   if (promise) {
     CameraGetPromiseData data;
     data.mCamera = this;
@@ -1226,8 +1275,6 @@ nsDOMCameraControl::OnHardwareStateChange(CameraControlListener::HardwareState a
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   MOZ_ASSERT(NS_IsMainThread());
 
-  ErrorResult ignored;
-
   switch (aState) {
     case CameraControlListener::kHardwareOpen:
       DOM_CAMERA_LOGI("DOM OnHardwareStateChange: open\n");
@@ -1242,7 +1289,7 @@ nsDOMCameraControl::OnHardwareStateChange(CameraControlListener::HardwareState a
     case CameraControlListener::kHardwareClosed:
       DOM_CAMERA_LOGI("DOM OnHardwareStateChange: closed\n");
       if (!mSetInitialConfig) {
-        nsRefPtr<Promise> promise = mReleasePromise.forget();
+        RefPtr<Promise> promise = mReleasePromise.forget();
         if (promise) {
           promise->MaybeResolve(JS::UndefinedHandleValue);
         }
@@ -1268,7 +1315,7 @@ nsDOMCameraControl::OnHardwareStateChange(CameraControlListener::HardwareState a
             break;
         }
 
-        nsRefPtr<CameraClosedEvent> event =
+        RefPtr<CameraClosedEvent> event =
           CameraClosedEvent::Constructor(this,
                                          NS_LITERAL_STRING("close"),
                                          eventInit);
@@ -1328,31 +1375,23 @@ nsDOMCameraControl::OnPoster(BlobImpl* aPoster)
 {
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(!mOptions.mPosterFilepath.IsEmpty());
+  MOZ_ASSERT(mOptions.mCreatePoster);
 
-  // Destructor will trigger an error notification if any step fails
-  nsRefPtr<RecorderPosterHelper> listener = new RecorderPosterHelper(this);
-  if (NS_WARN_IF(!aPoster)) {
-    return;
-  }
-
-  nsRefPtr<Blob> blob = Blob::Create(GetParentObject(), aPoster);
+  RefPtr<Blob> blob = Blob::Create(GetParentObject(), aPoster);
   if (NS_WARN_IF(!blob)) {
+    OnRecorderStateChange(CameraControlListener::kPosterFailed, 0, 0);
     return;
   }
 
-  if (NS_WARN_IF(!mOptions.mPosterStorageArea)) {
-    return;
-  }
+  BlobEventInit eventInit;
+  eventInit.mData = blob;
 
-  ErrorResult rv;
-  nsRefPtr<DOMRequest> request =
-    mOptions.mPosterStorageArea->AddNamed(blob, mOptions.mPosterFilepath, rv);
-  if (NS_WARN_IF(rv.Failed())) {
-    return;
-  }
+  RefPtr<BlobEvent> event = BlobEvent::Constructor(this,
+                                                     NS_LITERAL_STRING("poster"),
+                                                     eventInit);
 
-  RegisterStorageRequestEvents(request, listener);
+  DispatchTrustedEvent(event);
+  OnRecorderStateChange(CameraControlListener::kPosterCreated, 0, 0);
 }
 
 void
@@ -1363,13 +1402,12 @@ nsDOMCameraControl::OnRecorderStateChange(CameraControlListener::RecorderState a
   DOM_CAMERA_LOGT("%s:%d : this=%p, state=%u\n", __func__, __LINE__, this, aState);
   MOZ_ASSERT(NS_IsMainThread());
 
-  ErrorResult ignored;
   nsString state;
 
   switch (aState) {
     case CameraControlListener::kRecorderStarted:
       {
-        nsRefPtr<Promise> promise = mStartRecordingPromise.forget();
+        RefPtr<Promise> promise = mStartRecordingPromise.forget();
         if (promise) {
           promise->MaybeResolve(JS::UndefinedHandleValue);
         }
@@ -1379,7 +1417,7 @@ nsDOMCameraControl::OnRecorderStateChange(CameraControlListener::RecorderState a
       break;
 
     case CameraControlListener::kRecorderStopped:
-      if (!mOptions.mPosterFilepath.IsEmpty()) {
+      if (mOptions.mCreatePoster) {
         mRecordingStoppedDeferred = true;
         return;
       }
@@ -1390,14 +1428,20 @@ nsDOMCameraControl::OnRecorderStateChange(CameraControlListener::RecorderState a
 
     case CameraControlListener::kPosterCreated:
       state = NS_LITERAL_STRING("PosterCreated");
-      mOptions.mPosterFilepath.Truncate();
-      mOptions.mPosterStorageArea = nullptr;
+      mOptions.mCreatePoster = false;
       break;
 
     case CameraControlListener::kPosterFailed:
       state = NS_LITERAL_STRING("PosterFailed");
-      mOptions.mPosterFilepath.Truncate();
-      mOptions.mPosterStorageArea = nullptr;
+      mOptions.mCreatePoster = false;
+      break;
+
+    case CameraControlListener::kRecorderPaused:
+      state = NS_LITERAL_STRING("Paused");
+      break;
+
+    case CameraControlListener::kRecorderResumed:
+      state = NS_LITERAL_STRING("Resumed");
       break;
 
 #ifdef MOZ_B2G_CAMERA
@@ -1433,7 +1477,7 @@ nsDOMCameraControl::OnRecorderStateChange(CameraControlListener::RecorderState a
 
   DispatchStateEvent(NS_LITERAL_STRING("recorderstatechange"), state);
 
-  if (mRecordingStoppedDeferred && mOptions.mPosterFilepath.IsEmpty()) {
+  if (mRecordingStoppedDeferred && !mOptions.mCreatePoster) {
     mRecordingStoppedDeferred = false;
     OnRecorderStateChange(CameraControlListener::kRecorderStopped, 0, 0);
   }
@@ -1469,7 +1513,7 @@ nsDOMCameraControl::OnConfigurationChange(DOMCameraConfiguration* aConfiguration
     return;
   }
 
-  nsRefPtr<Promise> promise = mSetConfigurationPromise.forget();
+  RefPtr<Promise> promise = mSetConfigurationPromise.forget();
   if (promise) {
     promise->MaybeResolve(*aConfiguration);
   }
@@ -1484,7 +1528,7 @@ nsDOMCameraControl::OnConfigurationChange(DOMCameraConfiguration* aConfiguration
                                        mCurrentConfiguration->mPictureSize.mWidth,
                                        mCurrentConfiguration->mPictureSize.mHeight);
 
-  nsRefPtr<CameraConfigurationEvent> event =
+  RefPtr<CameraConfigurationEvent> event =
     CameraConfigurationEvent::Constructor(this,
                                           NS_LITERAL_STRING("configurationchanged"),
                                           eventInit);
@@ -1498,7 +1542,7 @@ nsDOMCameraControl::OnAutoFocusComplete(bool aAutoFocusSucceeded)
   DOM_CAMERA_LOGT("%s:%d : this=%p\n", __func__, __LINE__, this);
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsRefPtr<Promise> promise = mAutoFocusPromise.forget();
+  RefPtr<Promise> promise = mAutoFocusPromise.forget();
   if (promise) {
     promise->MaybeResolve(aAutoFocusSucceeded);
   }
@@ -1540,7 +1584,7 @@ nsDOMCameraControl::OnFacesDetected(const nsTArray<ICameraControl::Face>& aFaces
   CameraFacesDetectedEventInit eventInit;
   eventInit.mFaces.SetValue(faces);
 
-  nsRefPtr<CameraFacesDetectedEvent> event =
+  RefPtr<CameraFacesDetectedEvent> event =
     CameraFacesDetectedEvent::Constructor(this,
                                           NS_LITERAL_STRING("facesdetected"),
                                           eventInit);
@@ -1555,17 +1599,17 @@ nsDOMCameraControl::OnTakePictureComplete(nsIDOMBlob* aPicture)
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(aPicture);
 
-  nsRefPtr<Promise> promise = mTakePicturePromise.forget();
+  RefPtr<Promise> promise = mTakePicturePromise.forget();
   if (promise) {
     nsCOMPtr<nsIDOMBlob> picture = aPicture;
     promise->MaybeResolve(picture);
   }
 
-  nsRefPtr<Blob> blob = static_cast<Blob*>(aPicture);
+  RefPtr<Blob> blob = static_cast<Blob*>(aPicture);
   BlobEventInit eventInit;
   eventInit.mData = blob;
 
-  nsRefPtr<BlobEvent> event = BlobEvent::Constructor(this,
+  RefPtr<BlobEvent> event = BlobEvent::Constructor(this,
                                                      NS_LITERAL_STRING("picture"),
                                                      eventInit);
 
@@ -1579,7 +1623,7 @@ nsDOMCameraControl::OnUserError(CameraControlListener::UserContext aContext, nsr
     this, aContext, aError);
   MOZ_ASSERT(NS_IsMainThread());
 
-  nsRefPtr<Promise> promise;
+  RefPtr<Promise> promise;
 
   switch (aContext) {
     case CameraControlListener::kInStartCamera:
@@ -1656,6 +1700,18 @@ nsDOMCameraControl::OnUserError(CameraControlListener::UserContext aContext, nsr
       NS_WARNING("Failed to stop recording");
       return;
 
+    case CameraControlListener::kInPauseRecording:
+      // This method doesn't have any callbacks, so all we can do is log the
+      // failure. This only happens after the hardware has been released.
+      NS_WARNING("Failed to pause recording");
+      return;
+
+    case CameraControlListener::kInResumeRecording:
+      // This method doesn't have any callbacks, so all we can do is log the
+      // failure. This only happens after the hardware has been released.
+      NS_WARNING("Failed to resume recording");
+      return;
+
     case CameraControlListener::kInStartPreview:
       // This method doesn't have any callbacks, so all we can do is log the
       // failure. This only happens after the hardware has been released.
