@@ -225,8 +225,33 @@ HTMLSelectElement::InsertOptionsIntoList(nsIContent* aOptions,
                                          int32_t aDepth,
                                          bool aNotify)
 {
+  MOZ_ASSERT(aDepth == 0 || aDepth == 1);
   int32_t insertIndex = aListIndex;
-  InsertOptionsIntoListRecurse(aOptions, &insertIndex, aDepth);
+
+  HTMLOptionElement* optElement = HTMLOptionElement::FromContent(aOptions);
+  if (optElement) {
+    mOptions->InsertOptionAt(optElement, insertIndex);
+    insertIndex++;
+  } else if (aDepth == 0) {
+    // If it's at the top level, then we just found out there are non-options
+    // at the top level, which will throw off the insert count
+    mNonOptionChildren++;
+
+    // Deal with optgroups
+    if (aOptions->IsHTMLElement(nsGkAtoms::optgroup)) {
+      mOptGroupCount++;
+
+      for (nsIContent* child = aOptions->GetFirstChild();
+           child;
+           child = child->GetNextSibling()) {
+        optElement = HTMLOptionElement::FromContent(child);
+        if (optElement) {
+          mOptions->InsertOptionAt(optElement, insertIndex);
+          insertIndex++;
+        }
+      }
+    }
+  } // else ignore even if optgroup; we want to ignore nested optgroups.
 
   // Deal with the selected list
   if (insertIndex - aListIndex) {
@@ -256,7 +281,7 @@ HTMLSelectElement::InsertOptionsIntoList(nsIContent* aOptions,
         selectFrame->AddOption(i);
       }
 
-      nsRefPtr<HTMLOptionElement> option = Item(i);
+      RefPtr<HTMLOptionElement> option = Item(i);
       if (option && option->Selected()) {
         // Clear all other options
         if (!HasAttr(kNameSpaceID_None, nsGkAtoms::multiple)) {
@@ -281,10 +306,40 @@ HTMLSelectElement::RemoveOptionsFromList(nsIContent* aOptions,
                                          int32_t aDepth,
                                          bool aNotify)
 {
+  MOZ_ASSERT(aDepth == 0 || aDepth == 1);
   int32_t numRemoved = 0;
-  nsresult rv = RemoveOptionsFromListRecurse(aOptions, aListIndex, &numRemoved,
-                                             aDepth);
-  NS_ENSURE_SUCCESS(rv, rv);
+
+  HTMLOptionElement* optElement = HTMLOptionElement::FromContent(aOptions);
+  if (optElement) {
+    if (mOptions->ItemAsOption(aListIndex) != optElement) {
+      NS_ERROR("wrong option at index");
+      return NS_ERROR_UNEXPECTED;
+    }
+    mOptions->RemoveOptionAt(aListIndex);
+    numRemoved++;
+  } else if (aDepth == 0) {
+    // Yay, one less artifact at the top level.
+    mNonOptionChildren--;
+
+    // Recurse down deeper for options
+    if (mOptGroupCount && aOptions->IsHTMLElement(nsGkAtoms::optgroup)) {
+      mOptGroupCount--;
+
+      for (nsIContent* child = aOptions->GetFirstChild();
+          child;
+          child = child->GetNextSibling()) {
+        optElement = HTMLOptionElement::FromContent(child);
+        if (optElement) {
+          if (mOptions->ItemAsOption(aListIndex) != optElement) {
+            NS_ERROR("wrong option at index");
+            return NS_ERROR_UNEXPECTED;
+          }
+          mOptions->RemoveOptionAt(aListIndex);
+          numRemoved++;
+        }
+      }
+    }
+  } // else don't check for an optgroup; we want to ignore nested optgroups
 
   if (numRemoved) {
     // Tell the widget we removed the options
@@ -324,91 +379,6 @@ HTMLSelectElement::RemoveOptionsFromList(nsIContent* aOptions,
   return NS_OK;
 }
 
-// If the document is such that recursing over these options gets us
-// deeper than four levels, there is something terribly wrong with the
-// world.
-void
-HTMLSelectElement::InsertOptionsIntoListRecurse(nsIContent* aOptions,
-                                                int32_t* aInsertIndex,
-                                                int32_t aDepth)
-{
-  // We *assume* here that someone's brain has not gone horribly
-  // wrong by putting <option> inside of <option>.  I'm sorry, I'm
-  // just not going to look for an option inside of an option.
-  // Sue me.
-
-  HTMLOptionElement* optElement = HTMLOptionElement::FromContent(aOptions);
-  if (optElement) {
-    mOptions->InsertOptionAt(optElement, *aInsertIndex);
-    (*aInsertIndex)++;
-    return;
-  }
-
-  // If it's at the top level, then we just found out there are non-options
-  // at the top level, which will throw off the insert count
-  if (aDepth == 0) {
-    mNonOptionChildren++;
-  }
-
-  // Recurse down into optgroups
-  if (aOptions->IsHTMLElement(nsGkAtoms::optgroup)) {
-    mOptGroupCount++;
-
-    for (nsIContent* child = aOptions->GetFirstChild();
-         child;
-         child = child->GetNextSibling()) {
-      InsertOptionsIntoListRecurse(child, aInsertIndex, aDepth + 1);
-    }
-  }
-}
-
-// If the document is such that recursing over these options gets us deeper than
-// four levels, there is something terribly wrong with the world.
-nsresult
-HTMLSelectElement::RemoveOptionsFromListRecurse(nsIContent* aOptions,
-                                                int32_t aRemoveIndex,
-                                                int32_t* aNumRemoved,
-                                                int32_t aDepth)
-{
-  // We *assume* here that someone's brain has not gone horribly
-  // wrong by putting <option> inside of <option>.  I'm sorry, I'm
-  // just not going to look for an option inside of an option.
-  // Sue me.
-
-  nsCOMPtr<nsIDOMHTMLOptionElement> optElement(do_QueryInterface(aOptions));
-  if (optElement) {
-    if (mOptions->ItemAsOption(aRemoveIndex) != optElement) {
-      NS_ERROR("wrong option at index");
-      return NS_ERROR_UNEXPECTED;
-    }
-    mOptions->RemoveOptionAt(aRemoveIndex);
-    (*aNumRemoved)++;
-    return NS_OK;
-  }
-
-  // Yay, one less artifact at the top level.
-  if (aDepth == 0) {
-    mNonOptionChildren--;
-  }
-
-  // Recurse down deeper for options
-  if (mOptGroupCount && aOptions->IsHTMLElement(nsGkAtoms::optgroup)) {
-    mOptGroupCount--;
-
-    for (nsIContent* child = aOptions->GetFirstChild();
-         child;
-         child = child->GetNextSibling()) {
-      nsresult rv = RemoveOptionsFromListRecurse(child,
-                                                 aRemoveIndex,
-                                                 aNumRemoved,
-                                                 aDepth + 1);
-      NS_ENSURE_SUCCESS(rv, rv);
-    }
-  }
-
-  return NS_OK;
-}
-
 // XXXldb Doing the processing before the content nodes have been added
 // to the document (as the name of this function seems to require, and
 // as the callers do), is highly unusual.  Passing around unparented
@@ -420,10 +390,10 @@ HTMLSelectElement::WillAddOptions(nsIContent* aOptions,
                                   int32_t aContentIndex,
                                   bool aNotify)
 {
-  int32_t level = GetContentDepth(aParent);
-  if (level == -1) {
-    return NS_ERROR_FAILURE;
+  if (this != aParent && this != aParent->GetParent()) {
+    return NS_OK;
   }
+  int32_t level = aParent == this ? 0 : 1;
 
   // Get the index where the options will be inserted
   int32_t ind = -1;
@@ -462,11 +432,10 @@ HTMLSelectElement::WillRemoveOptions(nsIContent* aParent,
                                      int32_t aContentIndex,
                                      bool aNotify)
 {
-  int32_t level = GetContentDepth(aParent);
-  NS_ASSERTION(level >= 0, "getting notified by unexpected content");
-  if (level == -1) {
-    return NS_ERROR_FAILURE;
+  if (this != aParent && this != aParent->GetParent()) {
+    return NS_OK;
   }
+  int32_t level = this == aParent ? 0 : 1;
 
   // Get the index where the options will be removed
   nsIContent* currentKid = aParent->GetChildAt(aContentIndex);
@@ -489,24 +458,6 @@ HTMLSelectElement::WillRemoveOptions(nsIContent* aParent,
   return NS_OK;
 }
 
-int32_t
-HTMLSelectElement::GetContentDepth(nsIContent* aContent)
-{
-  nsIContent* content = aContent;
-
-  int32_t retval = 0;
-  while (content != this) {
-    retval++;
-    content = content->GetParent();
-    if (!content) {
-      retval = -1;
-      break;
-    }
-  }
-
-  return retval;
-}
-
 int32_t
 HTMLSelectElement::GetOptionIndexAt(nsIContent* aOptions)
 {
@@ -528,9 +479,7 @@ HTMLSelectElement::GetOptionIndexAfter(nsIContent* aOptions)
   //   in the parent.
   // - If it's not there, search for the first option after the parent.
   if (aOptions == this) {
-    uint32_t len;
-    GetLength(&len);
-    return len;
+    return Length();
   }
 
   int32_t retval = -1;
@@ -558,8 +507,6 @@ HTMLSelectElement::GetFirstOptionIndex(nsIContent* aOptions)
   HTMLOptionElement* optElement = HTMLOptionElement::FromContent(aOptions);
   if (optElement) {
     GetOptionIndex(optElement, 0, true, &listIndex);
-    // If you nested stuff under the option, you're just plain
-    // screwed.  *I'm* not going to aid and abet your evil deed.
     return listIndex;
   }
 
@@ -630,7 +577,7 @@ HTMLSelectElement::Add(nsGenericHTMLElement& aElement,
 
   // Just in case we're not the parent, get the parent of the reference
   // element
-  nsINode* parent = aBefore->Element::GetParentNode();
+  nsCOMPtr<nsINode> parent = aBefore->Element::GetParentNode();
   if (!parent || !nsContentUtils::ContentIsDescendantOf(parent, this)) {
     // NOT_FOUND_ERR: Raised if before is not a descendant of the SELECT
     // element.
@@ -747,7 +694,7 @@ HTMLSelectElement::SetLength(uint32_t aLength, ErrorResult& aRv)
 
   if (curlen > aLength) { // Remove extra options
     for (uint32_t i = curlen; i > aLength; --i) {
-      MOZ_ALWAYS_TRUE(NS_SUCCEEDED(Remove(i - 1)));
+      MOZ_ALWAYS_SUCCEEDS(Remove(i - 1));
     }
   } else if (aLength > curlen) {
     if (aLength > MAX_DYNAMIC_SELECT_LENGTH) {
@@ -755,14 +702,14 @@ HTMLSelectElement::SetLength(uint32_t aLength, ErrorResult& aRv)
       return;
     }
 
-    nsRefPtr<mozilla::dom::NodeInfo> nodeInfo;
+    RefPtr<mozilla::dom::NodeInfo> nodeInfo;
 
     nsContentUtils::NameChanged(mNodeInfo, nsGkAtoms::option,
                                 getter_AddRefs(nodeInfo));
 
     nsCOMPtr<nsINode> node = NS_NewHTMLOptionElement(nodeInfo.forget());
 
-    nsRefPtr<nsTextNode> text = new nsTextNode(mNodeInfo->NodeInfoManager());
+    RefPtr<nsTextNode> text = new nsTextNode(mNodeInfo->NodeInfoManager());
 
     aRv = node->AppendChildTo(text, false);
     if (aRv.Failed()) {
@@ -885,7 +832,7 @@ HTMLSelectElement::OnOptionSelected(nsISelectControlFrame* aSelectFrame,
 
   if (aChangeOptionState) {
     // Tell the option to get its bad self selected
-    nsRefPtr<HTMLOptionElement> option = Item(static_cast<uint32_t>(aIndex));
+    RefPtr<HTMLOptionElement> option = Item(static_cast<uint32_t>(aIndex));
     if (option) {
       option->SetSelectedInternal(aSelected, aNotify);
     }
@@ -1009,7 +956,7 @@ HTMLSelectElement::SetOptionsSelectedByIndex(int32_t aStartIndex,
       for (uint32_t optIndex = AssertedCast<uint32_t>(aStartIndex);
            optIndex <= AssertedCast<uint32_t>(aEndIndex);
            optIndex++) {
-        nsRefPtr<HTMLOptionElement> option = Item(optIndex);
+        RefPtr<HTMLOptionElement> option = Item(optIndex);
 
         // Ignore disabled options.
         if (!(aOptionsMask & SET_DISABLED)) {
@@ -1114,7 +1061,7 @@ NS_IMETHODIMP
 HTMLSelectElement::IsOptionDisabled(int32_t aIndex, bool* aIsDisabled)
 {
   *aIsDisabled = false;
-  nsRefPtr<HTMLOptionElement> option = Item(aIndex);
+  RefPtr<HTMLOptionElement> option = Item(aIndex);
   NS_ENSURE_TRUE(option, NS_ERROR_FAILURE);
 
   *aIsDisabled = IsOptionDisabled(option);
@@ -1140,7 +1087,7 @@ HTMLSelectElement::IsOptionDisabled(HTMLOptionElement* aOption)
         return false;
       }
 
-      nsRefPtr<HTMLOptGroupElement> optGroupElement =
+      RefPtr<HTMLOptGroupElement> optGroupElement =
         HTMLOptGroupElement::FromContent(node);
 
       if (!optGroupElement) {
@@ -1175,7 +1122,7 @@ HTMLSelectElement::GetValue(DOMString& aValue)
     return;
   }
 
-  nsRefPtr<HTMLOptionElement> option =
+  RefPtr<HTMLOptionElement> option =
     Item(static_cast<uint32_t>(selectedIndex));
 
   if (!option) {
@@ -1192,7 +1139,7 @@ HTMLSelectElement::SetValue(const nsAString& aValue)
   uint32_t length = Length();
 
   for (uint32_t i = 0; i < length; i++) {
-    nsRefPtr<HTMLOptionElement> option = Item(i);
+    RefPtr<HTMLOptionElement> option = Item(i);
     if (!option) {
       continue;
     }
@@ -1201,9 +1148,11 @@ HTMLSelectElement::SetValue(const nsAString& aValue)
     option->GetValue(optionVal);
     if (optionVal.Equals(aValue)) {
       SetSelectedIndexInternal(int32_t(i), true);
-      break;
+      return NS_OK;
     }
   }
+  // No matching option was found.
+  SetSelectedIndexInternal(-1, true);
   return NS_OK;
 }
 
@@ -1464,7 +1413,7 @@ HTMLSelectElement::GetAttributeChangeHint(const nsIAtom* aAttribute,
       nsGenericHTMLFormElementWithState::GetAttributeChangeHint(aAttribute, aModType);
   if (aAttribute == nsGkAtoms::multiple ||
       aAttribute == nsGkAtoms::size) {
-    NS_UpdateHint(retval, NS_STYLE_HINT_FRAMECHANGE);
+    retval |= NS_STYLE_HINT_FRAMECHANGE;
   }
   return retval;
 }
@@ -1487,7 +1436,7 @@ HTMLSelectElement::GetAttributeMappingFunction() const
 }
 
 bool
-HTMLSelectElement::IsDisabledForEvents(uint32_t aMessage)
+HTMLSelectElement::IsDisabledForEvents(EventMessage aMessage)
 {
   nsIFormControlFrame* formControlFrame = GetFormControlFrame(false);
   nsIFrame* formFrame = nullptr;
@@ -1501,7 +1450,7 @@ nsresult
 HTMLSelectElement::PreHandleEvent(EventChainPreVisitor& aVisitor)
 {
   aVisitor.mCanHandle = false;
-  if (IsDisabledForEvents(aVisitor.mEvent->message)) {
+  if (IsDisabledForEvents(aVisitor.mEvent->mMessage)) {
     return NS_OK;
   }
 
@@ -1511,7 +1460,7 @@ HTMLSelectElement::PreHandleEvent(EventChainPreVisitor& aVisitor)
 nsresult
 HTMLSelectElement::PostHandleEvent(EventChainPostVisitor& aVisitor)
 {
-  if (aVisitor.mEvent->message == NS_FOCUS_CONTENT) {
+  if (aVisitor.mEvent->mMessage == eFocus) {
     // If the invalid UI is shown, we should show it while focused and
     // update the invalid/valid UI.
     mCanShowInvalidUI = !IsValid() && ShouldShowValidityUI();
@@ -1522,7 +1471,7 @@ HTMLSelectElement::PostHandleEvent(EventChainPostVisitor& aVisitor)
 
     // We don't have to update NS_EVENT_STATE_MOZ_UI_INVALID nor
     // NS_EVENT_STATE_MOZ_UI_VALID given that the states should not change.
-  } else if (aVisitor.mEvent->message == NS_BLUR_CONTENT) {
+  } else if (aVisitor.mEvent->mMessage == eBlur) {
     mCanShowInvalidUI = true;
     mCanShowValidUI = true;
 
@@ -1581,7 +1530,7 @@ HTMLSelectElement::IntrinsicState() const
 NS_IMETHODIMP
 HTMLSelectElement::SaveState()
 {
-  nsRefPtr<SelectState> state = new SelectState();
+  RefPtr<SelectState> state = new SelectState();
 
   uint32_t len = Length();
 
@@ -1668,7 +1617,7 @@ HTMLSelectElement::Reset()
   uint32_t numOptions = Length();
 
   for (uint32_t i = 0; i < numOptions; i++) {
-    nsRefPtr<HTMLOptionElement> option = Item(i);
+    RefPtr<HTMLOptionElement> option = Item(i);
     if (option) {
       //
       // Reset the option to its default value
@@ -1748,7 +1697,7 @@ HTMLSelectElement::SubmitNamesValues(nsFormSubmission* aFormSubmission)
     }
 
     nsString value;
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(option->GetValue(value)));
+    MOZ_ALWAYS_SUCCEEDS(option->GetValue(value));
 
     if (keyGenProcessor) {
       nsString tmp(value);
@@ -1785,16 +1734,23 @@ HTMLSelectElement::DispatchContentReset()
 }
 
 static void
-AddOptionsRecurse(nsIContent* aRoot, HTMLOptionsCollection* aArray)
+AddOptions(nsIContent* aRoot, HTMLOptionsCollection* aArray)
 {
-  for (nsIContent* cur = aRoot->GetFirstChild();
-       cur;
-       cur = cur->GetNextSibling()) {
-    HTMLOptionElement* opt = HTMLOptionElement::FromContent(cur);
+  for (nsIContent* child = aRoot->GetFirstChild();
+       child;
+       child = child->GetNextSibling()) {
+    HTMLOptionElement* opt = HTMLOptionElement::FromContent(child);
     if (opt) {
       aArray->AppendOption(opt);
-    } else if (cur->IsHTMLElement(nsGkAtoms::optgroup)) {
-      AddOptionsRecurse(cur, aArray);
+    } else if (child->IsHTMLElement(nsGkAtoms::optgroup)) {
+      for (nsIContent* grandchild = child->GetFirstChild();
+           grandchild;
+           grandchild = grandchild->GetNextSibling()) {
+        opt = HTMLOptionElement::FromContent(grandchild);
+        if (opt) {
+          aArray->AppendOption(opt);
+        }
+      }
     }
   }
 }
@@ -1803,7 +1759,7 @@ void
 HTMLSelectElement::RebuildOptionsArray(bool aNotify)
 {
   mOptions->Clear();
-  AddOptionsRecurse(this, mOptions);
+  AddOptions(this, mOptions);
   FindSelectedIndex(0, aNotify);
 }
 
@@ -1817,7 +1773,7 @@ HTMLSelectElement::IsValueMissing()
   uint32_t length = Length();
 
   for (uint32_t i = 0; i < length; ++i) {
-    nsRefPtr<HTMLOptionElement> option = Item(i);
+    RefPtr<HTMLOptionElement> option = Item(i);
     if (!option->Selected()) {
       continue;
     }
@@ -1827,7 +1783,7 @@ HTMLSelectElement::IsValueMissing()
     }
 
     nsAutoString value;
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(option->GetValue(value)));
+    MOZ_ALWAYS_SUCCEEDS(option->GetValue(value));
     if (!value.IsEmpty()) {
       return false;
     }
@@ -1863,28 +1819,29 @@ HTMLSelectElement::GetValidationMessage(nsAString& aValidationMessage,
 
 #ifdef DEBUG
 
-static void
-VerifyOptionsRecurse(nsIContent* aRoot, int32_t& aIndex,
-                     HTMLOptionsCollection* aArray)
-{
-  for (nsIContent* cur = aRoot->GetFirstChild();
-       cur;
-       cur = cur->GetNextSibling()) {
-    nsCOMPtr<nsIDOMHTMLOptionElement> opt = do_QueryInterface(cur);
-    if (opt) {
-      NS_ASSERTION(opt == aArray->ItemAsOption(aIndex++),
-                   "Options collection broken");
-    } else if (cur->IsHTMLElement(nsGkAtoms::optgroup)) {
-      VerifyOptionsRecurse(cur, aIndex, aArray);
-    }
-  }
-}
-
 void
 HTMLSelectElement::VerifyOptionsArray()
 {
-  int32_t aIndex = 0;
-  VerifyOptionsRecurse(this, aIndex, mOptions);
+  int32_t index = 0;
+  for (nsIContent* child = nsINode::GetFirstChild();
+       child;
+       child = child->GetNextSibling()) {
+    HTMLOptionElement* opt = HTMLOptionElement::FromContent(child);
+    if (opt) {
+      NS_ASSERTION(opt == mOptions->ItemAsOption(index++),
+                   "Options collection broken");
+    } else if (child->IsHTMLElement(nsGkAtoms::optgroup)) {
+      for (nsIContent* grandchild = child->GetFirstChild();
+           grandchild;
+           grandchild = grandchild->GetNextSibling()) {
+        opt = HTMLOptionElement::FromContent(grandchild);
+        if (opt) {
+          NS_ASSERTION(opt == mOptions->ItemAsOption(index++),
+                       "Options collection broken");
+        }
+      }
+    }
+  }
 }
 
 #endif
@@ -1928,6 +1885,28 @@ HTMLSelectElement::UpdateSelectedOptions()
   }
 }
 
+bool
+HTMLSelectElement::OpenInParentProcess()
+{
+  nsIFormControlFrame* formControlFrame = GetFormControlFrame(false);
+  nsIComboboxControlFrame* comboFrame = do_QueryFrame(formControlFrame);
+  if (comboFrame) {
+    return comboFrame->IsOpenInParentProcess();
+  }
+
+  return false;
+}
+
+void
+HTMLSelectElement::SetOpenInParentProcess(bool aVal)
+{
+  nsIFormControlFrame* formControlFrame = GetFormControlFrame(false);
+  nsIComboboxControlFrame* comboFrame = do_QueryFrame(formControlFrame);
+  if (comboFrame) {
+    comboFrame->SetOpenInParentProcess(aVal);
+  }
+}
+
 JSObject*
 HTMLSelectElement::WrapNode(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
 {
