@@ -7,7 +7,6 @@
 #include "IDBDatabase.h"
 
 #include "FileInfo.h"
-#include "FileManager.h"
 #include "IDBEvents.h"
 #include "IDBFactory.h"
 #include "IDBIndex.h"
@@ -40,8 +39,6 @@
 #include "mozilla/ipc/InputStreamParams.h"
 #include "mozilla/ipc/InputStreamUtils.h"
 #include "nsCOMPtr.h"
-#include "nsContentUtils.h"
-#include "nsIConsoleService.h"
 #include "nsIDocument.h"
 #include "nsIObserver.h"
 #include "nsIObserverService.h"
@@ -50,6 +47,7 @@
 #include "nsThreadUtils.h"
 #include "ProfilerHelpers.h"
 #include "ReportInternalError.h"
+#include "ScriptErrorHelper.h"
 #include "nsQueryObject.h"
 
 // Include this last to avoid path problems on Windows.
@@ -57,7 +55,6 @@
 
 namespace mozilla {
 namespace dom {
-namespace indexedDB {
 
 using namespace mozilla::dom::quota;
 using namespace mozilla::ipc;
@@ -70,7 +67,7 @@ const char kMemoryPressureObserverTopic[] = "memory-pressure";
 const char kWindowObserverTopic[] = "inner-window-destroyed";
 
 class CancelableRunnableWrapper final
-  : public nsICancelableRunnable
+  : public CancelableRunnable
 {
   nsCOMPtr<nsIRunnable> mRunnable;
 
@@ -82,61 +79,12 @@ public:
     MOZ_ASSERT(aRunnable);
   }
 
-  NS_DECL_ISUPPORTS
-
 private:
   ~CancelableRunnableWrapper()
   { }
 
   NS_DECL_NSIRUNNABLE
-  NS_DECL_NSICANCELABLERUNNABLE
-};
-
-// XXX This should either be ported to PBackground or removed someday.
-class CreateFileHelper final
-  : public nsRunnable
-{
-  nsRefPtr<IDBDatabase> mDatabase;
-  nsRefPtr<IDBRequest> mRequest;
-  nsRefPtr<FileInfo> mFileInfo;
-
-  const nsString mName;
-  const nsString mType;
-  const nsString mDatabaseName;
-  const nsCString mOrigin;
-
-  const PersistenceType mPersistenceType;
-
-  nsresult mResultCode;
-
-public:
-  static nsresult
-  CreateAndDispatch(IDBDatabase* aDatabase,
-                    IDBRequest* aRequest,
-                    const nsAString& aName,
-                    const nsAString& aType);
-
-  NS_DECL_ISUPPORTS_INHERITED
-
-private:
-  CreateFileHelper(IDBDatabase* aDatabase,
-                   IDBRequest* aRequest,
-                   const nsAString& aName,
-                   const nsAString& aType,
-                   const nsACString& aOrigin);
-
-  ~CreateFileHelper()
-  {
-    MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  }
-
-  nsresult
-  DoDatabaseWork();
-
-  void
-  DoMainThreadWork(nsresult aResultCode);
-
-  NS_DECL_NSIRUNNABLE
+  nsresult Cancel() override;
 };
 
 class DatabaseFile final
@@ -169,7 +117,7 @@ private:
     mDatabase->AssertIsOnOwningThread();
 
     if (aWhy != Deletion) {
-      nsRefPtr<IDBDatabase> database = mDatabase;
+      RefPtr<IDBDatabase> database = mDatabase;
       database->NoteFinishedFileActor(this);
     }
 
@@ -181,46 +129,6 @@ private:
 
 } // namespace
 
-class IDBDatabase::LogWarningRunnable final
-  : public nsRunnable
-{
-  nsCString mMessageName;
-  nsString mFilename;
-  uint32_t mLineNumber;
-  uint64_t mInnerWindowID;
-  bool mIsChrome;
-
-public:
-  LogWarningRunnable(const char* aMessageName,
-                     const nsAString& aFilename,
-                     uint32_t aLineNumber,
-                     bool aIsChrome,
-                     uint64_t aInnerWindowID)
-    : mMessageName(aMessageName)
-    , mFilename(aFilename)
-    , mLineNumber(aLineNumber)
-    , mInnerWindowID(aInnerWindowID)
-    , mIsChrome(aIsChrome)
-  {
-    MOZ_ASSERT(!NS_IsMainThread());
-  }
-
-  static void
-  LogWarning(const char* aMessageName,
-             const nsAString& aFilename,
-             uint32_t aLineNumber,
-             bool aIsChrome,
-             uint64_t aInnerWindowID);
-
-  NS_DECL_ISUPPORTS_INHERITED
-
-private:
-  ~LogWarningRunnable()
-  { }
-
-  NS_DECL_NSIRUNNABLE
-};
-
 class IDBDatabase::Observer final
   : public nsIObserver
 {
@@ -256,18 +164,20 @@ private:
   NS_DECL_NSIOBSERVER
 };
 
-IDBDatabase::IDBDatabase(IDBWrapperCache* aOwnerCache,
+IDBDatabase::IDBDatabase(IDBOpenDBRequest* aRequest,
                          IDBFactory* aFactory,
                          BackgroundDatabaseChild* aActor,
                          DatabaseSpec* aSpec)
-  : IDBWrapperCache(aOwnerCache)
+  : IDBWrapperCache(aRequest)
   , mFactory(aFactory)
   , mSpec(aSpec)
   , mBackgroundActor(aActor)
+  , mFileHandleDisabled(aRequest->IsFileHandleDisabled())
   , mClosed(false)
   , mInvalidated(false)
+  , mQuotaExceeded(false)
 {
-  MOZ_ASSERT(aOwnerCache);
+  MOZ_ASSERT(aRequest);
   MOZ_ASSERT(aFactory);
   aFactory->AssertIsOnOwningThread();
   MOZ_ASSERT(aActor);
@@ -282,29 +192,27 @@ IDBDatabase::~IDBDatabase()
 
 // static
 already_AddRefed<IDBDatabase>
-IDBDatabase::Create(IDBWrapperCache* aOwnerCache,
+IDBDatabase::Create(IDBOpenDBRequest* aRequest,
                     IDBFactory* aFactory,
                     BackgroundDatabaseChild* aActor,
                     DatabaseSpec* aSpec)
 {
-  MOZ_ASSERT(aOwnerCache);
+  MOZ_ASSERT(aRequest);
   MOZ_ASSERT(aFactory);
   aFactory->AssertIsOnOwningThread();
   MOZ_ASSERT(aActor);
   MOZ_ASSERT(aSpec);
 
-  nsRefPtr<IDBDatabase> db =
-    new IDBDatabase(aOwnerCache, aFactory, aActor, aSpec);
+  RefPtr<IDBDatabase> db =
+    new IDBDatabase(aRequest, aFactory, aActor, aSpec);
 
-  db->SetScriptOwner(aOwnerCache->GetScriptOwner());
+  db->SetScriptOwner(aRequest->GetScriptOwner());
 
   if (NS_IsMainThread()) {
-    if (nsPIDOMWindow* window = aFactory->GetParentObject()) {
-      MOZ_ASSERT(window->IsInnerWindow());
-
+    if (nsPIDOMWindowInner* window = aFactory->GetParentObject()) {
       uint64_t windowId = window->WindowID();
 
-      nsRefPtr<Observer> observer = new Observer(db, windowId);
+      RefPtr<Observer> observer = new Observer(db, windowId);
 
       nsCOMPtr<nsIObserverService> obsSvc = GetObserverService();
       MOZ_ASSERT(obsSvc);
@@ -342,6 +250,13 @@ IDBDatabase::AssertIsOnOwningThread() const
   mFactory->AssertIsOnOwningThread();
 }
 
+PRThread*
+IDBDatabase::OwningThread() const
+{
+  MOZ_ASSERT(mFactory);
+  return mFactory->OwningThread();
+}
+
 #endif // DEBUG
 
 void
@@ -363,8 +278,8 @@ IDBDatabase::CloseInternal()
         obsSvc->RemoveObserver(mObserver, kCycleCollectionObserverTopic);
         obsSvc->RemoveObserver(mObserver, kMemoryPressureObserverTopic);
 
-        MOZ_ALWAYS_TRUE(NS_SUCCEEDED(
-          obsSvc->RemoveObserver(mObserver, kWindowObserverTopic)));
+        MOZ_ALWAYS_SUCCEEDS(
+          obsSvc->RemoveObserver(mObserver, kWindowObserverTopic));
       }
 
       mObserver = nullptr;
@@ -433,14 +348,14 @@ IDBDatabase::RefreshSpec(bool aMayDelete)
   AssertIsOnOwningThread();
 
   for (auto iter = mTransactions.Iter(); !iter.Done(); iter.Next()) {
-    nsRefPtr<IDBTransaction> transaction = iter.Get()->GetKey();
+    RefPtr<IDBTransaction> transaction = iter.Get()->GetKey();
     MOZ_ASSERT(transaction);
     transaction->AssertIsOnOwningThread();
     transaction->RefreshSpec(aMayDelete);
   }
 }
 
-nsPIDOMWindow*
+nsPIDOMWindowInner*
 IDBDatabase::GetParentObject() const
 {
   return mFactory->GetParentObject();
@@ -472,7 +387,7 @@ IDBDatabase::ObjectStoreNames() const
 
   const nsTArray<ObjectStoreSpec>& objectStores = mSpec->objectStores();
 
-  nsRefPtr<DOMStringList> list = new DOMStringList();
+  RefPtr<DOMStringList> list = new DOMStringList();
 
   if (!objectStores.IsEmpty()) {
     nsTArray<nsString>& listNames = list->StringArray();
@@ -489,7 +404,7 @@ IDBDatabase::ObjectStoreNames() const
 already_AddRefed<nsIDocument>
 IDBDatabase::GetOwnerDocument() const
 {
-  if (nsPIDOMWindow* window = GetOwner()) {
+  if (nsPIDOMWindowInner* window = GetOwner()) {
     nsCOMPtr<nsIDocument> doc = window->GetExtantDoc();
     return doc.forget();
   }
@@ -555,7 +470,7 @@ IDBDatabase::CreateObjectStore(
     RefreshSpec(/* aMayDelete */ false);
   }
 
-  nsRefPtr<IDBObjectStore> objectStore =
+  RefPtr<IDBObjectStore> objectStore =
     transaction->CreateObjectStore(*newSpec);
   MOZ_ASSERT(objectStore);
 
@@ -640,42 +555,28 @@ IDBDatabase::DeleteObjectStore(const nsAString& aName, ErrorResult& aRv)
 }
 
 already_AddRefed<IDBTransaction>
-IDBDatabase::Transaction(const StringOrStringSequence& aStoreNames,
+IDBDatabase::Transaction(JSContext* aCx,
+                         const StringOrStringSequence& aStoreNames,
                          IDBTransactionMode aMode,
                          ErrorResult& aRv)
 {
   AssertIsOnOwningThread();
 
-  aRv.MightThrowJSException();
-
-  if (aMode == IDBTransactionMode::Readwriteflush &&
+  if ((aMode == IDBTransactionMode::Readwriteflush ||
+       aMode == IDBTransactionMode::Cleanup) &&
       !IndexedDatabaseManager::ExperimentalFeaturesEnabled()) {
     // Pretend that this mode doesn't exist. We don't have a way to annotate
     // certain enum values as depending on preferences so we just duplicate the
     // normal exception generation here.
-    ThreadsafeAutoJSContext cx;
-
-    // Disable any automatic error reporting that might be set up so that we
-    // can grab the exception object.
-    AutoForceSetExceptionOnContext forceExn(cx);
-
-    MOZ_ALWAYS_FALSE(
-      ThrowErrorMessage(cx,
-                        MSG_INVALID_ENUM_VALUE,
-                        "Argument 2 of IDBDatabase.transaction",
-                        "readwriteflush",
-                        "IDBTransactionMode"));
-    MOZ_ASSERT(JS_IsExceptionPending(cx));
-
-    JS::Rooted<JS::Value> exception(cx);
-    MOZ_ALWAYS_TRUE(JS_GetPendingException(cx, &exception));
-
-    aRv.ThrowJSException(cx, exception);
+    aRv.ThrowTypeError<MSG_INVALID_ENUM_VALUE>(
+      NS_LITERAL_STRING("Argument 2 of IDBDatabase.transaction"),
+      NS_LITERAL_STRING("readwriteflush"),
+      NS_LITERAL_STRING("IDBTransactionMode"));
     return nullptr;
   }
 
-  nsRefPtr<IDBTransaction> transaction;
-  aRv = Transaction(aStoreNames, aMode, getter_AddRefs(transaction));
+  RefPtr<IDBTransaction> transaction;
+  aRv = Transaction(aCx, aStoreNames, aMode, getter_AddRefs(transaction));
   if (NS_WARN_IF(aRv.Failed())) {
     return nullptr;
   }
@@ -684,13 +585,15 @@ IDBDatabase::Transaction(const StringOrStringSequence& aStoreNames,
 }
 
 nsresult
-IDBDatabase::Transaction(const StringOrStringSequence& aStoreNames,
+IDBDatabase::Transaction(JSContext* aCx,
+                         const StringOrStringSequence& aStoreNames,
                          IDBTransactionMode aMode,
                          IDBTransaction** aTransaction)
 {
   AssertIsOnOwningThread();
 
-  if (NS_WARN_IF(aMode == IDBTransactionMode::Readwriteflush &&
+  if (NS_WARN_IF((aMode == IDBTransactionMode::Readwriteflush ||
+                  aMode == IDBTransactionMode::Cleanup) &&
                  !IndexedDatabaseManager::ExperimentalFeaturesEnabled())) {
     IDB_REPORT_INTERNAL_ERR();
     return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
@@ -705,7 +608,7 @@ IDBDatabase::Transaction(const StringOrStringSequence& aStoreNames,
     return NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR;
   }
 
-  nsAutoTArray<nsString, 1> stackSequence;
+  AutoTArray<nsString, 1> stackSequence;
 
   if (aStoreNames.IsString()) {
     stackSequence.AppendElement(aStoreNames.GetAsString());
@@ -763,20 +666,29 @@ IDBDatabase::Transaction(const StringOrStringSequence& aStoreNames,
       mode = IDBTransaction::READ_ONLY;
       break;
     case IDBTransactionMode::Readwrite:
-      mode = IDBTransaction::READ_WRITE;
+      if (mQuotaExceeded) {
+        mode = IDBTransaction::CLEANUP;
+        mQuotaExceeded = false;
+      } else {
+        mode = IDBTransaction::READ_WRITE;
+      }
       break;
     case IDBTransactionMode::Readwriteflush:
       mode = IDBTransaction::READ_WRITE_FLUSH;
       break;
+    case IDBTransactionMode::Cleanup:
+      mode = IDBTransaction::CLEANUP;
+      mQuotaExceeded = false;
+      break;
     case IDBTransactionMode::Versionchange:
-      return NS_ERROR_DOM_INVALID_ACCESS_ERR;
+      return NS_ERROR_DOM_TYPE_ERR;
 
     default:
       MOZ_CRASH("Unknown mode!");
   }
 
-  nsRefPtr<IDBTransaction> transaction =
-    IDBTransaction::Create(this, sortedStoreNames, mode);
+  RefPtr<IDBTransaction> transaction =
+    IDBTransaction::Create(aCx, this, sortedStoreNames, mode);
   if (NS_WARN_IF(!transaction)) {
     IDB_REPORT_INTERNAL_ERR();
     return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
@@ -800,6 +712,10 @@ IDBDatabase::Transaction(const StringOrStringSequence& aStoreNames,
 
   transaction->SetBackgroundActor(actor);
 
+  if (mode == IDBTransaction::CLEANUP) {
+    ExpireFileActors(/* aExpireAll */ true);
+  }
+
   transaction.forget(aTransaction);
   return NS_OK;
 }
@@ -814,15 +730,12 @@ IDBDatabase::Storage() const
 }
 
 already_AddRefed<IDBRequest>
-IDBDatabase::CreateMutableFile(const nsAString& aName,
+IDBDatabase::CreateMutableFile(JSContext* aCx,
+                               const nsAString& aName,
                                const Optional<nsAString>& aType,
                                ErrorResult& aRv)
 {
-  if (!IndexedDatabaseManager::IsMainProcess() || !NS_IsMainThread()) {
-    IDB_WARNING("Not supported!");
-    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
-    return nullptr;
-  }
+  AssertIsOnOwningThread();
 
   if (QuotaManager::IsShuttingDown()) {
     IDB_REPORT_INTERNAL_ERR();
@@ -830,24 +743,33 @@ IDBDatabase::CreateMutableFile(const nsAString& aName,
     return nullptr;
   }
 
-  if (mClosed) {
+  if (mClosed || mFileHandleDisabled) {
     aRv.Throw(NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR);
     return nullptr;
   }
 
-  nsRefPtr<IDBRequest> request = IDBRequest::Create(this, nullptr);
-
   nsString type;
   if (aType.WasPassed()) {
     type = aType.Value();
   }
 
-  mFactory->IncrementParentLoggingRequestSerialNumber();
+  CreateFileParams params(nsString(aName), type);
 
-  aRv = CreateFileHelper::CreateAndDispatch(this, request, aName, type);
-  if (NS_WARN_IF(aRv.Failed())) {
-    return nullptr;
-  }
+  RefPtr<IDBRequest> request = IDBRequest::Create(aCx, this, nullptr);
+  MOZ_ASSERT(request);
+
+  BackgroundDatabaseRequestChild* actor =
+    new BackgroundDatabaseRequestChild(this, request);
+
+  IDB_LOG_MARK("IndexedDB %s: Child  Request[%llu]: "
+                 "database(%s).createMutableFile(%s)",
+               "IndexedDB %s: C R[%llu]: IDBDatabase.createMutableFile()",
+               IDB_LOG_ID_STRING(),
+               request->LoggingSerialNumber(),
+               IDB_LOG_STRINGIFY(this),
+               NS_ConvertUTF16toUTF8(aName).get());
+
+  mBackgroundActor->SendPBackgroundIDBDatabaseRequestConstructor(actor, params);
 
   return request.forget();
 }
@@ -881,8 +803,8 @@ IDBDatabase::AbortTransactions(bool aShouldWarn)
 
   class MOZ_STACK_CLASS Helper final
   {
-    typedef nsAutoTArray<nsRefPtr<IDBTransaction>, 20> StrongTransactionArray;
-    typedef nsAutoTArray<IDBTransaction*, 20> WeakTransactionArray;
+    typedef AutoTArray<RefPtr<IDBTransaction>, 20> StrongTransactionArray;
+    typedef AutoTArray<IDBTransaction*, 20> WeakTransactionArray;
 
   public:
     static void
@@ -927,7 +849,7 @@ IDBDatabase::AbortTransactions(bool aShouldWarn)
       // additional strong references here.
       WeakTransactionArray transactionsThatNeedWarning;
 
-      for (nsRefPtr<IDBTransaction>& transaction : transactionsToAbort) {
+      for (RefPtr<IDBTransaction>& transaction : transactionsToAbort) {
         MOZ_ASSERT(transaction);
         MOZ_ASSERT(!transaction->IsDone());
 
@@ -940,6 +862,7 @@ IDBDatabase::AbortTransactions(bool aShouldWarn)
             // We warn for any transactions that could have written data.
             case IDBTransaction::READ_WRITE:
             case IDBTransaction::READ_WRITE_FLUSH:
+            case IDBTransaction::CLEANUP:
             case IDBTransaction::VERSION_CHANGE:
               transactionsThatNeedWarning.AppendElement(transaction);
               break;
@@ -959,10 +882,10 @@ IDBDatabase::AbortTransactions(bool aShouldWarn)
         MOZ_ASSERT(transaction);
 
         nsString filename;
-        uint32_t lineNo;
-        transaction->GetCallerLocation(filename, &lineNo);
+        uint32_t lineNo, column;
+        transaction->GetCallerLocation(filename, &lineNo, &column);
 
-        aDatabase->LogWarning(kWarningMessage, filename, lineNo);
+        aDatabase->LogWarning(kWarningMessage, filename, lineNo, column);
       }
     }
   };
@@ -1056,27 +979,15 @@ IDBDatabase::NoteFinishedFileActor(PBackgroundIDBDatabaseFileChild* aFileActor)
   AssertIsOnOwningThread();
   MOZ_ASSERT(aFileActor);
 
-  class MOZ_STACK_CLASS Helper final
-  {
-  public:
-    static PLDHashOperator
-    Remove(nsISupports* aKey,
-           PBackgroundIDBDatabaseFileChild*& aValue,
-           void* aClosure)
-    {
-      MOZ_ASSERT(aKey);
-      MOZ_ASSERT(aValue);
-      MOZ_ASSERT(aClosure);
+  for (auto iter = mFileActors.Iter(); !iter.Done(); iter.Next()) {
+    MOZ_ASSERT(iter.Key());
+    PBackgroundIDBDatabaseFileChild* actor = iter.Data();
+    MOZ_ASSERT(actor);
 
-      if (aValue == static_cast<PBackgroundIDBDatabaseFileChild*>(aClosure)) {
-        return PL_DHASH_REMOVE;
-      }
-
-      return PL_DHASH_NEXT;
+    if (actor == aFileActor) {
+      iter.Remove();
     }
-  };
-
-  mFileActors.Enumerate(&Helper::Remove, aFileActor);
+  }
 }
 
 void
@@ -1088,7 +999,7 @@ IDBDatabase::NoteReceivedBlob(Blob* aBlob)
 
 #ifdef DEBUG
   {
-    nsRefPtr<BlobImpl> blobImpl = aBlob->Impl();
+    RefPtr<BlobImpl> blobImpl = aBlob->Impl();
     MOZ_ASSERT(blobImpl);
 
     nsCOMPtr<nsIRemoteBlob> remoteBlob = do_QueryObject(blobImpl);
@@ -1125,9 +1036,9 @@ IDBDatabase::DelayedMaybeExpireFileActors()
   }
 
   nsCOMPtr<nsIRunnable> runnable =
-    NS_NewRunnableMethodWithArg<bool>(this,
-                                      &IDBDatabase::ExpireFileActors,
-                                      /* aExpireAll */ false);
+    NewRunnableMethod<bool>(this,
+                            &IDBDatabase::ExpireFileActors,
+                            /* aExpireAll */ false);
   MOZ_ASSERT(runnable);
 
   if (!NS_IsMainThread()) {
@@ -1136,7 +1047,7 @@ IDBDatabase::DelayedMaybeExpireFileActors()
     cancelable.swap(runnable);
   }
 
-  MOZ_ALWAYS_TRUE(NS_SUCCEEDED(NS_DispatchToCurrentThread(runnable)));
+  MOZ_ALWAYS_SUCCEEDS(NS_DispatchToCurrentThread(runnable));
 }
 
 nsresult
@@ -1145,8 +1056,7 @@ IDBDatabase::GetQuotaInfo(nsACString& aOrigin,
 {
   using mozilla::dom::quota::QuotaManager;
 
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(NS_IsMainThread(), "This can't work off the main thread!");
 
   if (aPersistenceType) {
     *aPersistenceType = mSpec->metadata().persistenceType();
@@ -1161,7 +1071,7 @@ IDBDatabase::GetQuotaInfo(nsACString& aOrigin,
       MOZ_CRASH("Is this needed?!");
 
     case PrincipalInfo::TSystemPrincipalInfo:
-      QuotaManager::GetInfoForChrome(nullptr, &aOrigin, nullptr);
+      QuotaManager::GetInfoForChrome(nullptr, nullptr, &aOrigin, nullptr);
       return NS_OK;
 
     case PrincipalInfo::TContentPrincipalInfo: {
@@ -1173,6 +1083,7 @@ IDBDatabase::GetQuotaInfo(nsACString& aOrigin,
       }
 
       rv = QuotaManager::GetInfoFromPrincipal(principal,
+                                              nullptr,
                                               nullptr,
                                               &aOrigin,
                                               nullptr);
@@ -1195,23 +1106,16 @@ IDBDatabase::ExpireFileActors(bool aExpireAll)
 {
   AssertIsOnOwningThread();
 
-  class MOZ_STACK_CLASS Helper final
-  {
-  public:
-    static PLDHashOperator
-    MaybeExpireFileActors(nsISupports* aKey,
-                          PBackgroundIDBDatabaseFileChild*& aValue,
-                          void* aClosure)
-    {
-      MOZ_ASSERT(aKey);
-      MOZ_ASSERT(aValue);
-      MOZ_ASSERT(aClosure);
+  if (mBackgroundActor && mFileActors.Count()) {
+    for (auto iter = mFileActors.Iter(); !iter.Done(); iter.Next()) {
+      nsISupports* key = iter.Key();
+      PBackgroundIDBDatabaseFileChild* actor = iter.Data();
+      MOZ_ASSERT(key);
+      MOZ_ASSERT(actor);
 
-      const bool expiringAll = *static_cast<bool*>(aClosure);
-
-      bool shouldExpire = expiringAll;
+      bool shouldExpire = aExpireAll;
       if (!shouldExpire) {
-        nsCOMPtr<nsIWeakReference> weakRef = do_QueryInterface(aKey);
+        nsCOMPtr<nsIWeakReference> weakRef = do_QueryInterface(key);
         MOZ_ASSERT(weakRef);
 
         nsCOMPtr<nsISupports> referent = do_QueryReferent(weakRef);
@@ -1219,19 +1123,13 @@ IDBDatabase::ExpireFileActors(bool aExpireAll)
       }
 
       if (shouldExpire) {
-        PBackgroundIDBDatabaseFileChild::Send__delete__(aValue);
+        PBackgroundIDBDatabaseFileChild::Send__delete__(actor);
 
-        if (!expiringAll) {
-          return PL_DHASH_REMOVE;
+        if (!aExpireAll) {
+          iter.Remove();
         }
       }
-
-      return PL_DHASH_NEXT;
     }
-  };
-
-  if (mBackgroundActor && mFileActors.Count()) {
-    mFileActors.Enumerate(&Helper::MaybeExpireFileActors, &aExpireAll);
     if (aExpireAll) {
       mFileActors.Clear();
     }
@@ -1262,9 +1160,9 @@ IDBDatabase::ExpireFileActors(bool aExpireAll)
 void
 IDBDatabase::NoteLiveMutableFile(IDBMutableFile* aMutableFile)
 {
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
+  AssertIsOnOwningThread();
   MOZ_ASSERT(aMutableFile);
+  aMutableFile->AssertIsOnOwningThread();
   MOZ_ASSERT(!mLiveMutableFiles.Contains(aMutableFile));
 
   mLiveMutableFiles.AppendElement(aMutableFile);
@@ -1273,45 +1171,22 @@ IDBDatabase::NoteLiveMutableFile(IDBMutableFile* aMutableFile)
 void
 IDBDatabase::NoteFinishedMutableFile(IDBMutableFile* aMutableFile)
 {
-  // This should always happen in the main process but occasionally it is called
-  // after the IndexedDatabaseManager has already shut down.
-  //   MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
+  AssertIsOnOwningThread();
   MOZ_ASSERT(aMutableFile);
+  aMutableFile->AssertIsOnOwningThread();
 
-  // It's ok if this is called more than once, so don't assert that aMutableFile
-  // is in the list already.
+  // It's ok if this is called after we cleared the array, so don't assert that
+  // aMutableFile is in the list.
 
   mLiveMutableFiles.RemoveElement(aMutableFile);
 }
 
-void
-IDBDatabase::OnNewFileHandle()
-{
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(mBackgroundActor);
-
-  mBackgroundActor->SendNewFileHandle();
-}
-
-void
-IDBDatabase::OnFileHandleFinished()
-{
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(mBackgroundActor);
-
-  mBackgroundActor->SendFileHandleFinished();
-}
-
 void
 IDBDatabase::InvalidateMutableFiles()
 {
-  if (!mLiveMutableFiles.IsEmpty()) {
-    MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-    MOZ_ASSERT(NS_IsMainThread());
+  AssertIsOnOwningThread();
 
+  if (!mLiveMutableFiles.IsEmpty()) {
     for (uint32_t count = mLiveMutableFiles.Length(), index = 0;
          index < count;
          index++) {
@@ -1337,26 +1212,19 @@ IDBDatabase::Invalidate()
 void
 IDBDatabase::LogWarning(const char* aMessageName,
                         const nsAString& aFilename,
-                        uint32_t aLineNumber)
+                        uint32_t aLineNumber,
+                        uint32_t aColumnNumber)
 {
   AssertIsOnOwningThread();
   MOZ_ASSERT(aMessageName);
 
-  if (NS_IsMainThread()) {
-    LogWarningRunnable::LogWarning(aMessageName,
-                                   aFilename,
-                                   aLineNumber,
-                                   mFactory->IsChrome(),
-                                   mFactory->InnerWindowID());
-  } else {
-    nsRefPtr<LogWarningRunnable> runnable =
-      new LogWarningRunnable(aMessageName,
-                             aFilename,
-                             aLineNumber,
-                             mFactory->IsChrome(),
-                             mFactory->InnerWindowID());
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(NS_DispatchToMainThread(runnable)));
-  }
+  ScriptErrorHelper::DumpLocalizedMessage(nsDependentCString(aMessageName),
+                                          aFilename,
+                                          aLineNumber,
+                                          aColumnNumber,
+                                          nsIScriptError::warningFlag,
+                                          mFactory->IsChrome(),
+                                          mFactory->InnerWindowID());
 }
 
 NS_IMPL_ADDREF_INHERITED(IDBDatabase, IDBWrapperCache)
@@ -1413,8 +1281,6 @@ IDBDatabase::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
   return IDBDatabaseBinding::Wrap(aCx, this, aGivenProto);
 }
 
-NS_IMPL_ISUPPORTS(CancelableRunnableWrapper, nsIRunnable, nsICancelableRunnable)
-
 NS_IMETHODIMP
 CancelableRunnableWrapper::Run()
 {
@@ -1428,7 +1294,7 @@ CancelableRunnableWrapper::Run()
   return NS_OK;
 }
 
-NS_IMETHODIMP
+nsresult
 CancelableRunnableWrapper::Cancel()
 {
   if (mRunnable) {
@@ -1439,266 +1305,6 @@ CancelableRunnableWrapper::Cancel()
   return NS_ERROR_UNEXPECTED;
 }
 
-CreateFileHelper::CreateFileHelper(IDBDatabase* aDatabase,
-                                   IDBRequest* aRequest,
-                                   const nsAString& aName,
-                                   const nsAString& aType,
-                                   const nsACString& aOrigin)
-  : mDatabase(aDatabase)
-  , mRequest(aRequest)
-  , mName(aName)
-  , mType(aType)
-  , mDatabaseName(aDatabase->Name())
-  , mOrigin(aOrigin)
-  , mPersistenceType(aDatabase->Spec()->metadata().persistenceType())
-  , mResultCode(NS_OK)
-{
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(aDatabase);
-  MOZ_ASSERT(aRequest);
-  MOZ_ASSERT(mPersistenceType != PERSISTENCE_TYPE_INVALID);
-}
-
-// static
-nsresult
-CreateFileHelper::CreateAndDispatch(IDBDatabase* aDatabase,
-                                    IDBRequest* aRequest,
-                                    const nsAString& aName,
-                                    const nsAString& aType)
-{
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(aDatabase);
-  aDatabase->AssertIsOnOwningThread();
-  MOZ_ASSERT(aDatabase->Factory());
-  MOZ_ASSERT(aRequest);
-  MOZ_ASSERT(!QuotaManager::IsShuttingDown());
-
-  nsCString origin;
-  nsresult rv = aDatabase->GetQuotaInfo(origin, nullptr);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  MOZ_ASSERT(!origin.IsEmpty());
-
-  nsRefPtr<CreateFileHelper> helper =
-    new CreateFileHelper(aDatabase, aRequest, aName, aType, origin);
-
-  QuotaManager* quotaManager = QuotaManager::Get();
-  MOZ_ASSERT(quotaManager);
-
-  nsCOMPtr<nsIEventTarget> ioThread = quotaManager->IOThread();
-  MOZ_ASSERT(ioThread);
-
-  if (NS_WARN_IF(NS_FAILED(ioThread->Dispatch(helper, NS_DISPATCH_NORMAL)))) {
-    IDB_REPORT_INTERNAL_ERR();
-    return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
-  }
-
-  return NS_OK;
-}
-
-nsresult
-CreateFileHelper::DoDatabaseWork()
-{
-  AssertIsOnIOThread();
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(!mFileInfo);
-
-  PROFILER_LABEL("IndexedDB",
-                 "CreateFileHelper::DoDatabaseWork",
-                 js::ProfileEntry::Category::STORAGE);
-
-  if (IndexedDatabaseManager::InLowDiskSpaceMode()) {
-    NS_WARNING("Refusing to create file because disk space is low!");
-    return NS_ERROR_DOM_INDEXEDDB_QUOTA_ERR;
-  }
-
-  IndexedDatabaseManager* idbManager = IndexedDatabaseManager::Get();
-  MOZ_ASSERT(idbManager);
-
-  nsRefPtr<FileManager> fileManager =
-    idbManager->GetFileManager(mPersistenceType, mOrigin, mDatabaseName);
-  MOZ_ASSERT(fileManager);
-
-  nsRefPtr<FileInfo> fileInfo = fileManager->GetNewFileInfo();
-  if (NS_WARN_IF(!fileInfo)) {
-    IDB_REPORT_INTERNAL_ERR();
-    return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
-  }
-
-  const int64_t fileId = fileInfo->Id();
-
-  nsCOMPtr<nsIFile> journalDirectory = fileManager->EnsureJournalDirectory();
-  if (NS_WARN_IF(!journalDirectory)) {
-    IDB_REPORT_INTERNAL_ERR();
-    return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
-  }
-
-  nsCOMPtr<nsIFile> journalFile =
-    fileManager->GetFileForId(journalDirectory, fileId);
-  if (NS_WARN_IF(!journalFile)) {
-    IDB_REPORT_INTERNAL_ERR();
-    return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
-  }
-
-  nsresult rv = journalFile->Create(nsIFile::NORMAL_FILE_TYPE, 0644);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIFile> fileDirectory = fileManager->GetDirectory();
-  if (NS_WARN_IF(!fileDirectory)) {
-    IDB_REPORT_INTERNAL_ERR();
-    return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
-  }
-
-  nsCOMPtr<nsIFile> file = fileManager->GetFileForId(fileDirectory, fileId);
-  if (NS_WARN_IF(!file)) {
-    IDB_REPORT_INTERNAL_ERR();
-    return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
-  }
-
-  rv = file->Create(nsIFile::NORMAL_FILE_TYPE, 0644);
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    return rv;
-  }
-
-  mFileInfo.swap(fileInfo);
-  return NS_OK;
-}
-
-void
-CreateFileHelper::DoMainThreadWork(nsresult aResultCode)
-{
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-  MOZ_ASSERT(NS_IsMainThread());
-
-  if (mDatabase->IsInvalidated()) {
-    IDB_REPORT_INTERNAL_ERR();
-    aResultCode = NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
-  }
-
-  nsRefPtr<IDBMutableFile> mutableFile;
-  if (NS_SUCCEEDED(aResultCode)) {
-    mutableFile =
-      IDBMutableFile::Create(mDatabase, mName, mType, mFileInfo.forget());
-    MOZ_ASSERT(mutableFile);
-  }
-
-  DispatchMutableFileResult(mRequest, aResultCode, mutableFile);
-}
-
-NS_IMPL_ISUPPORTS_INHERITED0(CreateFileHelper, nsRunnable)
-
-NS_IMETHODIMP
-CreateFileHelper::Run()
-{
-  MOZ_ASSERT(IndexedDatabaseManager::IsMainProcess());
-
-  if (NS_IsMainThread()) {
-    DoMainThreadWork(mResultCode);
-
-    mDatabase = nullptr;
-    mRequest = nullptr;
-    mFileInfo = nullptr;
-
-    return NS_OK;
-  }
-
-  AssertIsOnIOThread();
-  MOZ_ASSERT(NS_SUCCEEDED(mResultCode));
-
-  nsresult rv = DoDatabaseWork();
-  if (NS_WARN_IF(NS_FAILED(rv))) {
-    mResultCode = rv;
-  }
-
-  MOZ_ALWAYS_TRUE(NS_SUCCEEDED(NS_DispatchToMainThread(this)));
-
-  return NS_OK;
-}
-
-
-// static
-void
-IDBDatabase::
-LogWarningRunnable::LogWarning(const char* aMessageName,
-                               const nsAString& aFilename,
-                               uint32_t aLineNumber,
-                               bool aIsChrome,
-                               uint64_t aInnerWindowID)
-{
-  MOZ_ASSERT(NS_IsMainThread());
-  MOZ_ASSERT(aMessageName);
-
-  nsXPIDLString localizedMessage;
-  if (NS_WARN_IF(NS_FAILED(
-    nsContentUtils::GetLocalizedString(nsContentUtils::eDOM_PROPERTIES,
-                                       aMessageName,
-                                       localizedMessage)))) {
-    return;
-  }
-
-  nsAutoCString category;
-  if (aIsChrome) {
-    category.AssignLiteral("chrome ");
-  } else {
-    category.AssignLiteral("content ");
-  }
-  category.AppendLiteral("javascript");
-
-  nsCOMPtr<nsIConsoleService> consoleService =
-    do_GetService(NS_CONSOLESERVICE_CONTRACTID);
-  MOZ_ASSERT(consoleService);
-
-  nsCOMPtr<nsIScriptError> scriptError =
-    do_CreateInstance(NS_SCRIPTERROR_CONTRACTID);
-  MOZ_ASSERT(consoleService);
-
-  if (aInnerWindowID) {
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(
-      scriptError->InitWithWindowID(localizedMessage,
-                                    aFilename,
-                                    /* aSourceLine */ EmptyString(),
-                                    aLineNumber,
-                                    /* aColumnNumber */ 0,
-                                    nsIScriptError::warningFlag,
-                                    category,
-                                    aInnerWindowID)));
-  } else {
-    MOZ_ALWAYS_TRUE(NS_SUCCEEDED(
-      scriptError->Init(localizedMessage,
-                        aFilename,
-                        /* aSourceLine */ EmptyString(),
-                        aLineNumber,
-                        /* aColumnNumber */ 0,
-                        nsIScriptError::warningFlag,
-                        category.get())));
-  }
-
-  MOZ_ALWAYS_TRUE(NS_SUCCEEDED(consoleService->LogMessage(scriptError)));
-}
-
-NS_IMPL_ISUPPORTS_INHERITED0(IDBDatabase::LogWarningRunnable, nsRunnable)
-
-NS_IMETHODIMP
-IDBDatabase::
-LogWarningRunnable::Run()
-{
-  MOZ_ASSERT(NS_IsMainThread());
-
-  LogWarning(mMessageName.get(),
-             mFilename,
-             mLineNumber,
-             mIsChrome,
-             mInnerWindowID);
-
-  return NS_OK;
-}
-
 NS_IMPL_ISUPPORTS(IDBDatabase::Observer, nsIObserver)
 
 NS_IMETHODIMP
@@ -1716,10 +1322,10 @@ Observer::Observe(nsISupports* aSubject,
       MOZ_ASSERT(supportsInt);
 
       uint64_t windowId;
-      MOZ_ALWAYS_TRUE(NS_SUCCEEDED(supportsInt->GetData(&windowId)));
+      MOZ_ALWAYS_SUCCEEDS(supportsInt->GetData(&windowId));
 
       if (windowId == mWindowId) {
-        nsRefPtr<IDBDatabase> database = mWeakDatabase;
+        RefPtr<IDBDatabase> database = mWeakDatabase;
         mWeakDatabase = nullptr;
 
         database->InvalidateInternal();
@@ -1732,7 +1338,7 @@ Observer::Observe(nsISupports* aSubject,
   if (!strcmp(aTopic, kCycleCollectionObserverTopic) ||
       !strcmp(aTopic, kMemoryPressureObserverTopic)) {
     if (mWeakDatabase) {
-      nsRefPtr<IDBDatabase> database = mWeakDatabase;
+      RefPtr<IDBDatabase> database = mWeakDatabase;
 
       database->ExpireFileActors(/* aExpireAll */ false);
     }
@@ -1744,6 +1350,85 @@ Observer::Observe(nsISupports* aSubject,
   return NS_OK;
 }
 
-} // namespace indexedDB
+nsresult
+IDBDatabase::RenameObjectStore(int64_t aObjectStoreId, const nsAString& aName)
+{
+  MOZ_ASSERT(mSpec);
+
+  nsTArray<ObjectStoreSpec>& objectStores = mSpec->objectStores();
+
+  ObjectStoreSpec* foundObjectStoreSpec = nullptr;
+  // Find the matched object store spec and check if 'aName' is already used by
+  // another object store.
+  for (uint32_t objCount = objectStores.Length(), objIndex = 0;
+       objIndex < objCount;
+       objIndex++) {
+    const ObjectStoreSpec& objSpec = objectStores[objIndex];
+    if (objSpec.metadata().id() == aObjectStoreId) {
+      MOZ_ASSERT(!foundObjectStoreSpec);
+      foundObjectStoreSpec = &objectStores[objIndex];
+      continue;
+    }
+    if (aName == objSpec.metadata().name()) {
+      return NS_ERROR_DOM_INDEXEDDB_CONSTRAINT_ERR;
+    }
+  }
+
+  MOZ_ASSERT(foundObjectStoreSpec);
+
+  // Update the name of the matched object store.
+  foundObjectStoreSpec->metadata().name() = nsString(aName);
+
+  return NS_OK;
+}
+
+nsresult
+IDBDatabase::RenameIndex(int64_t aObjectStoreId,
+                         int64_t aIndexId,
+                         const nsAString& aName)
+{
+  MOZ_ASSERT(mSpec);
+
+  nsTArray<ObjectStoreSpec>& objectStores = mSpec->objectStores();
+
+  ObjectStoreSpec* foundObjectStoreSpec = nullptr;
+  // Find the matched index metadata and check if 'aName' is already used by
+  // another index.
+  for (uint32_t objCount = objectStores.Length(), objIndex = 0;
+       objIndex < objCount;
+       objIndex++) {
+    const ObjectStoreSpec& objSpec = objectStores[objIndex];
+    if (objSpec.metadata().id() == aObjectStoreId) {
+      foundObjectStoreSpec = &objectStores[objIndex];
+      break;
+    }
+  }
+
+  MOZ_ASSERT(foundObjectStoreSpec);
+
+  nsTArray<IndexMetadata>& indexes = foundObjectStoreSpec->indexes();
+  IndexMetadata* foundIndexMetadata = nullptr;
+  for (uint32_t idxCount = indexes.Length(), idxIndex = 0;
+       idxIndex < idxCount;
+       idxIndex++) {
+    const IndexMetadata& metadata = indexes[idxIndex];
+    if (metadata.id() == aIndexId) {
+      MOZ_ASSERT(!foundIndexMetadata);
+      foundIndexMetadata = &indexes[idxIndex];
+      continue;
+    }
+    if (aName == metadata.name()) {
+      return NS_ERROR_DOM_INDEXEDDB_CONSTRAINT_ERR;
+    }
+  }
+
+  MOZ_ASSERT(foundIndexMetadata);
+
+  // Update the name of the matched object store.
+  foundIndexMetadata->name() = nsString(aName);
+
+  return NS_OK;
+}
+
 } // namespace dom
 } // namespace mozilla
