@@ -20,9 +20,10 @@
 #include "IndexedDatabaseInlines.h"
 #include "IndexedDatabaseManager.h"
 #include "js/Class.h"
+#include "js/Date.h"
 #include "js/StructuredClone.h"
 #include "KeyPath.h"
-#include "mozilla/Endian.h"
+#include "mozilla/EndianUtils.h"
 #include "mozilla/ErrorResult.h"
 #include "mozilla/Move.h"
 #include "mozilla/dom/BindingUtils.h"
@@ -33,6 +34,7 @@
 #include "mozilla/dom/IDBMutableFileBinding.h"
 #include "mozilla/dom/BlobBinding.h"
 #include "mozilla/dom/IDBObjectStoreBinding.h"
+#include "mozilla/dom/StructuredCloneHolder.h"
 #include "mozilla/dom/StructuredCloneTags.h"
 #include "mozilla/dom/indexedDB/PBackgroundIDBSharedTypes.h"
 #include "mozilla/dom/ipc/BlobChild.h"
@@ -52,7 +54,6 @@
 
 namespace mozilla {
 namespace dom {
-namespace indexedDB {
 
 using namespace mozilla::dom::quota;
 using namespace mozilla::dom::workers;
@@ -60,20 +61,21 @@ using namespace mozilla::ipc;
 
 struct IDBObjectStore::StructuredCloneWriteInfo
 {
-  struct BlobOrFileInfo
+  struct BlobOrMutableFile
   {
-    nsRefPtr<Blob> mBlob;
-    nsRefPtr<FileInfo> mFileInfo;
+    RefPtr<Blob> mBlob;
+    RefPtr<IDBMutableFile> mMutableFile;
 
     bool
-    operator==(const BlobOrFileInfo& aOther) const
+    operator==(const BlobOrMutableFile& aOther) const
     {
-      return this->mBlob == aOther.mBlob && this->mFileInfo == aOther.mFileInfo;
+      return this->mBlob == aOther.mBlob &&
+             this->mMutableFile == aOther.mMutableFile;
     }
   };
 
   JSAutoStructuredCloneBuffer mCloneBuffer;
-  nsTArray<BlobOrFileInfo> mBlobOrFileInfos;
+  nsTArray<BlobOrMutableFile> mBlobOrMutableFiles;
   IDBDatabase* mDatabase;
   uint64_t mOffsetToKeyProp;
 
@@ -95,7 +97,7 @@ struct IDBObjectStore::StructuredCloneWriteInfo
 
     MOZ_COUNT_CTOR(StructuredCloneWriteInfo);
 
-    mBlobOrFileInfos.SwapElements(aCloneWriteInfo.mBlobOrFileInfos);
+    mBlobOrMutableFiles.SwapElements(aCloneWriteInfo.mBlobOrMutableFiles);
     aCloneWriteInfo.mOffsetToKeyProp = 0;
   }
 
@@ -109,7 +111,7 @@ struct IDBObjectStore::StructuredCloneWriteInfo
   {
     return this->mCloneBuffer.nbytes() == aOther.mCloneBuffer.nbytes() &&
            this->mCloneBuffer.data() == aOther.mCloneBuffer.data() &&
-           this->mBlobOrFileInfos == aOther.mBlobOrFileInfos &&
+           this->mBlobOrMutableFiles == aOther.mBlobOrMutableFiles &&
            this->mDatabase == aOther.mDatabase &&
            this->mOffsetToKeyProp == aOther.mOffsetToKeyProp;
   }
@@ -128,7 +130,7 @@ struct IDBObjectStore::StructuredCloneWriteInfo
       }
     }
 
-    mBlobOrFileInfos.Clear();
+    mBlobOrMutableFiles.Clear();
 
     mOffsetToKeyProp = aOther.offsetToKeyProp();
     return true;
@@ -195,15 +197,15 @@ struct MOZ_STACK_CLASS GetAddInfoClosure final
 };
 
 already_AddRefed<IDBRequest>
-GenerateRequest(IDBObjectStore* aObjectStore)
+GenerateRequest(JSContext* aCx, IDBObjectStore* aObjectStore)
 {
   MOZ_ASSERT(aObjectStore);
   aObjectStore->AssertIsOnOwningThread();
 
   IDBTransaction* transaction = aObjectStore->Transaction();
 
-  nsRefPtr<IDBRequest> request =
-    IDBRequest::Create(aObjectStore, transaction->Database(), transaction);
+  RefPtr<IDBRequest> request =
+    IDBRequest::Create(aCx, aObjectStore, transaction->Database(), transaction);
   MOZ_ASSERT(request);
 
   return request.forget();
@@ -233,6 +235,10 @@ StructuredCloneWriteCallback(JSContext* aCx,
 
   IDBMutableFile* mutableFile;
   if (NS_SUCCEEDED(UNWRAP_OBJECT(IDBMutableFile, aObj, mutableFile))) {
+    if (cloneWriteInfo->mDatabase->IsFileHandleDisabled()) {
+      return false;
+    }
+
     IDBDatabase* database = mutableFile->Database();
     MOZ_ASSERT(database);
 
@@ -265,15 +271,12 @@ StructuredCloneWriteCallback(JSContext* aCx,
       }
     }
 
-    nsRefPtr<FileInfo> fileInfo = mutableFile->GetFileInfo();
-    MOZ_ASSERT(fileInfo);
-
-    if (cloneWriteInfo->mBlobOrFileInfos.Length() > size_t(UINT32_MAX)) {
+    if (cloneWriteInfo->mBlobOrMutableFiles.Length() > size_t(UINT32_MAX)) {
       MOZ_ASSERT(false, "Fix the structured clone data to use a bigger type!");
       return false;
     }
 
-    const uint32_t index = uint32_t(cloneWriteInfo->mBlobOrFileInfos.Length());
+    const uint32_t index = cloneWriteInfo->mBlobOrMutableFiles.Length();
 
     NS_ConvertUTF16toUTF8 convType(mutableFile->Type());
     uint32_t convTypeLength =
@@ -291,10 +294,10 @@ StructuredCloneWriteCallback(JSContext* aCx,
       return false;
     }
 
-    IDBObjectStore::StructuredCloneWriteInfo::BlobOrFileInfo*
-      newBlobOrFileInfo =
-        cloneWriteInfo->mBlobOrFileInfos.AppendElement();
-    newBlobOrFileInfo->mFileInfo.swap(fileInfo);
+    IDBObjectStore::StructuredCloneWriteInfo::BlobOrMutableFile*
+      newBlobOrMutableFile =
+        cloneWriteInfo->mBlobOrMutableFiles.AppendElement();
+    newBlobOrMutableFile->mMutableFile = mutableFile;
 
     return true;
   }
@@ -315,14 +318,13 @@ StructuredCloneWriteCallback(JSContext* aCx,
       uint32_t convTypeLength =
         NativeEndian::swapToLittleEndian(convType.Length());
 
-      if (cloneWriteInfo->mBlobOrFileInfos.Length() > size_t(UINT32_MAX)) {
+      if (cloneWriteInfo->mBlobOrMutableFiles.Length() > size_t(UINT32_MAX)) {
         MOZ_ASSERT(false,
                    "Fix the structured clone data to use a bigger type!");
         return false;
       }
 
-      const uint32_t index =
-        uint32_t(cloneWriteInfo->mBlobOrFileInfos.Length());
+      const uint32_t index = cloneWriteInfo->mBlobOrMutableFiles.Length();
 
       if (!JS_WriteUint32Pair(aWriter,
                               blob->IsFile() ? SCTAG_DOM_FILE : SCTAG_DOM_BLOB,
@@ -333,7 +335,7 @@ StructuredCloneWriteCallback(JSContext* aCx,
         return false;
       }
 
-      nsRefPtr<File> file = blob->ToFile();
+      RefPtr<File> file = blob->ToFile();
       if (file) {
         ErrorResult rv;
         int64_t lastModifiedDate = file->GetLastModified(rv);
@@ -348,30 +350,23 @@ StructuredCloneWriteCallback(JSContext* aCx,
         uint32_t convNameLength =
           NativeEndian::swapToLittleEndian(convName.Length());
 
-        if (!JS_WriteBytes(aWriter, &lastModifiedDate, sizeof(lastModifiedDate)) || 
+        if (!JS_WriteBytes(aWriter, &lastModifiedDate, sizeof(lastModifiedDate)) ||
             !JS_WriteBytes(aWriter, &convNameLength, sizeof(convNameLength)) ||
             !JS_WriteBytes(aWriter, convName.get(), convName.Length())) {
           return false;
         }
       }
 
-      IDBObjectStore::StructuredCloneWriteInfo::BlobOrFileInfo*
-        newBlobOrFileInfo =
-          cloneWriteInfo->mBlobOrFileInfos.AppendElement();
-      newBlobOrFileInfo->mBlob = blob;
+      IDBObjectStore::StructuredCloneWriteInfo::BlobOrMutableFile*
+        newBlobOrMutableFile =
+          cloneWriteInfo->mBlobOrMutableFiles.AppendElement();
+      newBlobOrMutableFile->mBlob = blob;
 
       return true;
     }
   }
 
-  // Try using the runtime callbacks
-  const JSStructuredCloneCallbacks* runtimeCallbacks =
-    js::GetContextStructuredCloneCallbacks(aCx);
-  if (runtimeCallbacks) {
-    return runtimeCallbacks->write(aCx, aWriter, aObj, nullptr);
-  }
-
-  return false;
+  return StructuredCloneHolder::WriteFullySerializableObjects(aCx, aWriter, aObj);
 }
 
 nsresult
@@ -429,6 +424,16 @@ ActorFromRemoteBlobImpl(BlobImpl* aImpl)
   return nullptr;
 }
 
+bool
+ResolveMysteryMutableFile(IDBMutableFile* aMutableFile,
+                          const nsString& aName,
+                          const nsString& aType)
+{
+  MOZ_ASSERT(aMutableFile);
+  aMutableFile->SetLazyData(aName, aType);
+  return true;
+}
+
 bool
 ResolveMysteryFile(BlobImpl* aImpl,
                    const nsString& aName,
@@ -439,8 +444,7 @@ ResolveMysteryFile(BlobImpl* aImpl,
   BlobChild* actor = ActorFromRemoteBlobImpl(aImpl);
   if (actor) {
     return actor->SetMysteryBlobInfo(aName, aContentType,
-                                     aSize, aLastModifiedDate,
-                                     BlobDirState::eUnknownIfDir);
+                                     aSize, aLastModifiedDate);
   }
   return true;
 }
@@ -572,31 +576,29 @@ class ValueDeserializationHelper
 public:
   static bool
   CreateAndWrapMutableFile(JSContext* aCx,
-                           IDBDatabase* aDatabase,
                            StructuredCloneFile& aFile,
                            const MutableFileData& aData,
                            JS::MutableHandle<JSObject*> aResult)
   {
-    MOZ_ASSERT(aDatabase);
-    MOZ_ASSERT(aFile.mFileInfo);
+    MOZ_ASSERT(aCx);
+    MOZ_ASSERT(aFile.mMutable);
 
-    if (!IndexedDatabaseManager::IsMainProcess() || !NS_IsMainThread()) {
+    if (!aFile.mMutableFile || !NS_IsMainThread()) {
       return false;
     }
 
-    nsRefPtr<IDBMutableFile> mutableFile =
-      IDBMutableFile::Create(aDatabase,
-                             aData.name,
-                             aData.type,
-                             aFile.mFileInfo.forget());
-    MOZ_ASSERT(mutableFile);
-
-    JS::Rooted<JSObject*> result(aCx, mutableFile->WrapObject(aCx, nullptr));
-    if (NS_WARN_IF(!result)) {
+    if (NS_WARN_IF(!ResolveMysteryMutableFile(aFile.mMutableFile,
+                                              aData.name,
+                                              aData.type))) {
       return false;
     }
 
-    aResult.set(result);
+    JS::Rooted<JS::Value> wrappedMutableFile(aCx);
+    if (!ToJSValue(aCx, aFile.mMutableFile, &wrappedMutableFile)) {
+      return false;
+    }
+
+    aResult.set(&wrappedMutableFile.toObject());
     return true;
   }
 
@@ -607,9 +609,11 @@ public:
                           const BlobOrFileData& aData,
                           JS::MutableHandle<JSObject*> aResult)
   {
+    MOZ_ASSERT(aCx);
     MOZ_ASSERT(aData.tag == SCTAG_DOM_FILE ||
                aData.tag == SCTAG_DOM_FILE_WITHOUT_LASTMODIFIEDDATE ||
                aData.tag == SCTAG_DOM_BLOB);
+    MOZ_ASSERT(!aFile.mMutable);
     MOZ_ASSERT(aFile.mBlob);
 
     // It can happen that this IDB is chrome code, so there is no parent, but
@@ -660,7 +664,7 @@ public:
     }
 
     MOZ_ASSERT(aFile.mBlob->IsFile());
-    nsRefPtr<File> file = aFile.mBlob->ToFile();
+    RefPtr<File> file = aFile.mBlob->ToFile();
     MOZ_ASSERT(file);
 
     JS::Rooted<JS::Value> wrappedFile(aCx);
@@ -678,13 +682,10 @@ class IndexDeserializationHelper
 public:
   static bool
   CreateAndWrapMutableFile(JSContext* aCx,
-                           IDBDatabase* aDatabase,
                            StructuredCloneFile& aFile,
                            const MutableFileData& aData,
                            JS::MutableHandle<JSObject*> aResult)
   {
-    MOZ_ASSERT(!aDatabase);
-
     // MutableFile can't be used in index creation, so just make a dummy object.
     JS::Rooted<JSObject*> obj(aCx, JS_NewPlainObject(aCx));
     if (NS_WARN_IF(!obj)) {
@@ -749,8 +750,8 @@ public:
       return false;
     }
 
-    JS::Rooted<JSObject*> date(aCx,
-      JS_NewDateObjectMsec(aCx, aData.lastModifiedDate));
+    JS::ClippedTime time = JS::TimeClip(aData.lastModifiedDate);
+    JS::Rooted<JSObject*> date(aCx, JS::NewDateObject(aCx, time));
     if (NS_WARN_IF(!date)) {
       return false;
     }
@@ -768,6 +769,64 @@ public:
   }
 };
 
+// We don't need to upgrade database on B2G. See the comment in ActorsParent.cpp,
+// UpgradeSchemaFrom18_0To19_0()
+#if !defined(MOZ_B2G)
+
+class UpgradeDeserializationHelper
+{
+public:
+  static bool
+  CreateAndWrapMutableFile(JSContext* aCx,
+                           StructuredCloneFile& aFile,
+                           const MutableFileData& aData,
+                           JS::MutableHandle<JSObject*> aResult)
+  {
+    MOZ_ASSERT(aCx);
+    MOZ_ASSERT(!aFile.mMutable);
+
+    aFile.mMutable = true;
+
+    // Just make a dummy object. The file_ids upgrade function is only
+    // interested in the |mMutable| flag.
+    JS::Rooted<JSObject*> obj(aCx, JS_NewPlainObject(aCx));
+
+    if (NS_WARN_IF(!obj)) {
+      return false;
+    }
+
+    aResult.set(obj);
+    return true;
+  }
+
+  static bool
+  CreateAndWrapBlobOrFile(JSContext* aCx,
+                          IDBDatabase* aDatabase,
+                          StructuredCloneFile& aFile,
+                          const BlobOrFileData& aData,
+                          JS::MutableHandle<JSObject*> aResult)
+  {
+    MOZ_ASSERT(aCx);
+    MOZ_ASSERT(!aFile.mMutable);
+    MOZ_ASSERT(aData.tag == SCTAG_DOM_FILE ||
+               aData.tag == SCTAG_DOM_FILE_WITHOUT_LASTMODIFIEDDATE ||
+               aData.tag == SCTAG_DOM_BLOB);
+
+    // Just make a dummy object. The file_ids upgrade function is only interested
+    // in the |mMutable| flag.
+    JS::Rooted<JSObject*> obj(aCx, JS_NewPlainObject(aCx));
+
+    if (NS_WARN_IF(!obj)) {
+      return false;
+    }
+
+    aResult.set(obj);
+    return true;
+  }
+};
+
+#endif // MOZ_B2G
+
 template <class Traits>
 JSObject*
 CommonStructuredCloneReadCallback(JSContext* aCx,
@@ -807,7 +866,6 @@ CommonStructuredCloneReadCallback(JSContext* aCx,
       }
 
       if (NS_WARN_IF(!Traits::CreateAndWrapMutableFile(aCx,
-                                                       cloneReadInfo->mDatabase,
                                                        file,
                                                        data,
                                                        &result))) {
@@ -833,14 +891,8 @@ CommonStructuredCloneReadCallback(JSContext* aCx,
     return result;
   }
 
-  const JSStructuredCloneCallbacks* runtimeCallbacks =
-    js::GetContextStructuredCloneCallbacks(aCx);
-
-  if (runtimeCallbacks) {
-    return runtimeCallbacks->read(aCx, aReader, aTag, aData, nullptr);
-  }
-
-  return nullptr;
+  return StructuredCloneHolder::ReadFullySerializableObjects(aCx, aReader,
+                                                             aTag);
 }
 
 // static
@@ -890,7 +942,7 @@ IDBObjectStore::Create(IDBTransaction* aTransaction,
   MOZ_ASSERT(aTransaction);
   aTransaction->AssertIsOnOwningThread();
 
-  nsRefPtr<IDBObjectStore> objectStore =
+  RefPtr<IDBObjectStore> objectStore =
     new IDBObjectStore(aTransaction, &aSpec);
 
   return objectStore.forget();
@@ -903,12 +955,17 @@ IDBObjectStore::AppendIndexUpdateInfo(
                                     const KeyPath& aKeyPath,
                                     bool aUnique,
                                     bool aMultiEntry,
+                                    const nsCString& aLocale,
                                     JSContext* aCx,
                                     JS::Handle<JS::Value> aVal,
                                     nsTArray<IndexUpdateInfo>& aUpdateInfoArray)
 {
   nsresult rv;
 
+#ifdef ENABLE_INTL_API
+  const bool localeAware = !aLocale.IsEmpty();
+#endif
+
   if (!aMultiEntry) {
     Key key;
     rv = aKeyPath.ExtractKey(aCx, aVal, key);
@@ -925,6 +982,14 @@ IDBObjectStore::AppendIndexUpdateInfo(
     IndexUpdateInfo* updateInfo = aUpdateInfoArray.AppendElement();
     updateInfo->indexId() = aIndexID;
     updateInfo->value() = key;
+#ifdef ENABLE_INTL_API
+    if (localeAware) {
+      rv = key.ToLocaleBasedKey(updateInfo->localizedValue(), aLocale);
+      if (NS_WARN_IF(NS_FAILED(rv))) {
+        return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
+      }
+    }
+#endif
 
     return NS_OK;
   }
@@ -934,7 +999,12 @@ IDBObjectStore::AppendIndexUpdateInfo(
     return NS_OK;
   }
 
-  if (JS_IsArrayObject(aCx, val)) {
+  bool isArray;
+  if (!JS_IsArrayObject(aCx, val, &isArray)) {
+    IDB_REPORT_INTERNAL_ERR();
+    return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
+  }
+  if (isArray) {
     JS::Rooted<JSObject*> array(aCx, &val.toObject());
     uint32_t arrayLength;
     if (NS_WARN_IF(!JS_GetArrayLength(aCx, array, &arrayLength))) {
@@ -959,6 +1029,14 @@ IDBObjectStore::AppendIndexUpdateInfo(
       IndexUpdateInfo* updateInfo = aUpdateInfoArray.AppendElement();
       updateInfo->indexId() = aIndexID;
       updateInfo->value() = value;
+#ifdef ENABLE_INTL_API
+      if (localeAware) {
+        rv = value.ToLocaleBasedKey(updateInfo->localizedValue(), aLocale);
+        if (NS_WARN_IF(NS_FAILED(rv))) {
+          return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
+        }
+      }
+#endif
     }
   }
   else {
@@ -972,6 +1050,14 @@ IDBObjectStore::AppendIndexUpdateInfo(
     IndexUpdateInfo* updateInfo = aUpdateInfoArray.AppendElement();
     updateInfo->indexId() = aIndexID;
     updateInfo->value() = value;
+#ifdef ENABLE_INTL_API
+    if (localeAware) {
+      rv = value.ToLocaleBasedKey(updateInfo->localizedValue(), aLocale);
+      if (NS_WARN_IF(NS_FAILED(rv))) {
+        return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
+      }
+    }
+#endif
   }
 
   return NS_OK;
@@ -1021,6 +1107,8 @@ IDBObjectStore::DeserializeValue(JSContext* aCx,
     nullptr
   };
 
+  // FIXME: Consider to use StructuredCloneHolder here and in other
+  //        deserializing methods.
   if (!JS_ReadStructuredClone(aCx, data, dataLen, JS_STRUCTURED_CLONE_VERSION,
                               aValue, &callbacks, &aCloneReadInfo)) {
     return false;
@@ -1067,6 +1155,51 @@ IDBObjectStore::DeserializeIndexValue(JSContext* aCx,
   return true;
 }
 
+#if !defined(MOZ_B2G)
+
+// static
+bool
+IDBObjectStore::DeserializeUpgradeValue(JSContext* aCx,
+                                        StructuredCloneReadInfo& aCloneReadInfo,
+                                        JS::MutableHandle<JS::Value> aValue)
+{
+  MOZ_ASSERT(!NS_IsMainThread());
+  MOZ_ASSERT(aCx);
+
+  if (aCloneReadInfo.mData.IsEmpty()) {
+    aValue.setUndefined();
+    return true;
+  }
+
+  size_t dataLen = aCloneReadInfo.mData.Length();
+
+  uint64_t* data =
+    const_cast<uint64_t*>(reinterpret_cast<uint64_t*>(
+      aCloneReadInfo.mData.Elements()));
+
+  MOZ_ASSERT(!(dataLen % sizeof(*data)));
+
+  JSAutoRequest ar(aCx);
+
+  static JSStructuredCloneCallbacks callbacks = {
+    CommonStructuredCloneReadCallback<UpgradeDeserializationHelper>,
+    nullptr,
+    nullptr,
+    nullptr,
+    nullptr,
+    nullptr
+  };
+
+  if (!JS_ReadStructuredClone(aCx, data, dataLen, JS_STRUCTURED_CLONE_VERSION,
+                              aValue, &callbacks, &aCloneReadInfo)) {
+    return false;
+  }
+
+  return true;
+}
+
+#endif // MOZ_B2G
+
 #ifdef DEBUG
 
 void
@@ -1125,8 +1258,9 @@ IDBObjectStore::GetAddInfo(JSContext* aCx,
     const IndexMetadata& metadata = indexes[idxIndex];
 
     rv = AppendIndexUpdateInfo(metadata.id(), metadata.keyPath(),
-                               metadata.unique(), metadata.multiEntry(), aCx,
-                               aValue, aUpdateInfoArray);
+                               metadata.unique(), metadata.multiEntry(),
+                               metadata.locale(), aCx, aValue,
+                               aUpdateInfoArray);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return rv;
     }
@@ -1161,7 +1295,8 @@ IDBObjectStore::AddOrPut(JSContext* aCx,
   MOZ_ASSERT(aCx);
   MOZ_ASSERT_IF(aFromCursor, aOverwrite);
 
-  if (mDeletedSpec) {
+  if (mTransaction->GetMode() == IDBTransaction::CLEANUP ||
+      mDeletedSpec) {
     aRv.Throw(NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR);
     return nullptr;
   }
@@ -1206,17 +1341,15 @@ IDBObjectStore::AddOrPut(JSContext* aCx,
   commonParams.key() = key;
   commonParams.indexUpdateInfos().SwapElements(updateInfo);
 
-  // Convert any blobs or fileIds into DatabaseFileOrMutableFileId.
-  nsTArray<StructuredCloneWriteInfo::BlobOrFileInfo>& blobOrFileInfos =
-    cloneWriteInfo.mBlobOrFileInfos;
+  // Convert any blobs or mutable files into DatabaseOrMutableFile.
+  nsTArray<StructuredCloneWriteInfo::BlobOrMutableFile>& blobOrMutableFiles =
+    cloneWriteInfo.mBlobOrMutableFiles;
 
-  FallibleTArray<nsRefPtr<FileInfo>> fileInfosToKeepAlive;
+  if (!blobOrMutableFiles.IsEmpty()) {
+    const uint32_t count = blobOrMutableFiles.Length();
 
-  if (!blobOrFileInfos.IsEmpty()) {
-    const uint32_t count = blobOrFileInfos.Length();
-
-    FallibleTArray<DatabaseFileOrMutableFileId> fileActorOrMutableFileIds;
-    if (NS_WARN_IF(!fileActorOrMutableFileIds.SetCapacity(count, fallible))) {
+    FallibleTArray<DatabaseOrMutableFile> fileOrMutableFileActors;
+    if (NS_WARN_IF(!fileOrMutableFileActors.SetCapacity(count, fallible))) {
       aRv = NS_ERROR_OUT_OF_MEMORY;
       return nullptr;
     }
@@ -1224,41 +1357,37 @@ IDBObjectStore::AddOrPut(JSContext* aCx,
     IDBDatabase* database = mTransaction->Database();
 
     for (uint32_t index = 0; index < count; index++) {
-      StructuredCloneWriteInfo::BlobOrFileInfo& blobOrFileInfo =
-        blobOrFileInfos[index];
-      MOZ_ASSERT((blobOrFileInfo.mBlob && !blobOrFileInfo.mFileInfo) ||
-                 (!blobOrFileInfo.mBlob && blobOrFileInfo.mFileInfo));
+      StructuredCloneWriteInfo::BlobOrMutableFile& blobOrMutableFile =
+        blobOrMutableFiles[index];
+      MOZ_ASSERT((blobOrMutableFile.mBlob && !blobOrMutableFile.mMutableFile) ||
+                 (!blobOrMutableFile.mBlob && blobOrMutableFile.mMutableFile));
 
-      if (blobOrFileInfo.mBlob) {
+      if (blobOrMutableFile.mBlob) {
         PBackgroundIDBDatabaseFileChild* fileActor =
-          database->GetOrCreateFileActorForBlob(blobOrFileInfo.mBlob);
+          database->GetOrCreateFileActorForBlob(blobOrMutableFile.mBlob);
         if (NS_WARN_IF(!fileActor)) {
           IDB_REPORT_INTERNAL_ERR();
           aRv = NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
           return nullptr;
         }
 
-        MOZ_ALWAYS_TRUE(fileActorOrMutableFileIds.AppendElement(fileActor,
-                                                                fallible));
+        MOZ_ALWAYS_TRUE(fileOrMutableFileActors.AppendElement(fileActor,
+                                                              fallible));
       } else {
-        const int64_t fileId = blobOrFileInfo.mFileInfo->Id();
-        MOZ_ASSERT(fileId > 0);
-
-        MOZ_ALWAYS_TRUE(fileActorOrMutableFileIds.AppendElement(fileId,
-                                                                fallible));
-
-        nsRefPtr<FileInfo>* newFileInfo =
-          fileInfosToKeepAlive.AppendElement(fallible);
-        if (NS_WARN_IF(!newFileInfo)) {
-          aRv = NS_ERROR_OUT_OF_MEMORY;
+        PBackgroundMutableFileChild* mutableFileActor =
+          blobOrMutableFile.mMutableFile->GetBackgroundActor();
+        if (NS_WARN_IF(!mutableFileActor)) {
+          IDB_REPORT_INTERNAL_ERR();
+          aRv = NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
           return nullptr;
         }
 
-        newFileInfo->swap(blobOrFileInfo.mFileInfo);
+        MOZ_ALWAYS_TRUE(fileOrMutableFileActors.AppendElement(mutableFileActor,
+                                                              fallible));
       }
     }
 
-    commonParams.files().SwapElements(fileActorOrMutableFileIds);
+    commonParams.files().SwapElements(fileOrMutableFileActors);
   }
 
   RequestParams params;
@@ -1268,7 +1397,7 @@ IDBObjectStore::AddOrPut(JSContext* aCx,
     params = ObjectStoreAddParams(commonParams);
   }
 
-  nsRefPtr<IDBRequest> request = GenerateRequest(this);
+  RefPtr<IDBRequest> request = GenerateRequest(aCx, this);
   MOZ_ASSERT(request);
 
   if (!aFromCursor) {
@@ -1297,16 +1426,7 @@ IDBObjectStore::AddOrPut(JSContext* aCx,
     }
   }
 
-  BackgroundRequestChild* actor = mTransaction->StartRequest(request, params);
-  MOZ_ASSERT(actor);
-
-  if (!fileInfosToKeepAlive.IsEmpty()) {
-    nsTArray<nsRefPtr<FileInfo>> fileInfos;
-    fileInfosToKeepAlive.SwapElements(fileInfos);
-
-    actor->HoldFileInfosUntilComplete(fileInfos);
-    MOZ_ASSERT(fileInfos.IsEmpty());
-  }
+  mTransaction->StartRequest(request, params);
 
   return request.forget();
 }
@@ -1330,7 +1450,7 @@ IDBObjectStore::GetAllInternal(bool aKeysOnly,
     return nullptr;
   }
 
-  nsRefPtr<IDBKeyRange> keyRange;
+  RefPtr<IDBKeyRange> keyRange;
   aRv = IDBKeyRange::FromJSVal(aCx, aKey, getter_AddRefs(keyRange));
   if (NS_WARN_IF(aRv.Failed())) {
     return nullptr;
@@ -1356,7 +1476,7 @@ IDBObjectStore::GetAllInternal(bool aKeysOnly,
     params = ObjectStoreGetAllParams(id, optionalKeyRange, limit);
   }
 
-  nsRefPtr<IDBRequest> request = GenerateRequest(this);
+  RefPtr<IDBRequest> request = GenerateRequest(aCx, this);
   MOZ_ASSERT(request);
 
   if (aKeysOnly) {
@@ -1393,7 +1513,7 @@ IDBObjectStore::GetAllInternal(bool aKeysOnly,
 }
 
 already_AddRefed<IDBRequest>
-IDBObjectStore::Clear(ErrorResult& aRv)
+IDBObjectStore::Clear(JSContext* aCx, ErrorResult& aRv)
 {
   AssertIsOnOwningThread();
 
@@ -1415,7 +1535,7 @@ IDBObjectStore::Clear(ErrorResult& aRv)
   ObjectStoreClearParams params;
   params.objectStoreId() = Id();
 
-  nsRefPtr<IDBRequest> request = GenerateRequest(this);
+  RefPtr<IDBRequest> request = GenerateRequest(aCx, this);
   MOZ_ASSERT(request);
 
   IDB_LOG_MARK("IndexedDB %s: Child  Transaction[%lld] Request[%llu]: "
@@ -1464,12 +1584,12 @@ IDBObjectStore::Index(const nsAString& aName, ErrorResult &aRv)
 
   const int64_t desiredId = metadata->id();
 
-  nsRefPtr<IDBIndex> index;
+  RefPtr<IDBIndex> index;
 
   for (uint32_t idxCount = mIndexes.Length(), idxIndex = 0;
        idxIndex < idxCount;
        idxIndex++) {
-    nsRefPtr<IDBIndex>& existingIndex = mIndexes[idxIndex];
+    RefPtr<IDBIndex>& existingIndex = mIndexes[idxIndex];
 
     if (existingIndex->Id() == desiredId) {
       index = existingIndex;
@@ -1491,7 +1611,7 @@ NS_IMPL_CYCLE_COLLECTION_CLASS(IDBObjectStore)
 
 NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(IDBObjectStore)
   NS_IMPL_CYCLE_COLLECTION_TRACE_PRESERVED_WRAPPER
-  NS_IMPL_CYCLE_COLLECTION_TRACE_JSVAL_MEMBER_CALLBACK(mCachedKeyPath)
+  NS_IMPL_CYCLE_COLLECTION_TRACE_JS_MEMBER_CALLBACK(mCachedKeyPath)
 NS_IMPL_CYCLE_COLLECTION_TRACE_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(IDBObjectStore)
@@ -1529,7 +1649,7 @@ IDBObjectStore::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
   return IDBObjectStoreBinding::Wrap(aCx, this, aGivenProto);
 }
 
-nsPIDOMWindow*
+nsPIDOMWindowInner*
 IDBObjectStore::GetParentObject() const
 {
   return mTransaction->GetParentObject();
@@ -1567,7 +1687,7 @@ IDBObjectStore::IndexNames()
 
   const nsTArray<IndexMetadata>& indexes = mSpec->indexes();
 
-  nsRefPtr<DOMStringList> list = new DOMStringList();
+  RefPtr<DOMStringList> list = new DOMStringList();
 
   if (!indexes.IsEmpty()) {
     nsTArray<nsString>& listNames = list->StringArray();
@@ -1598,7 +1718,7 @@ IDBObjectStore::Get(JSContext* aCx,
     return nullptr;
   }
 
-  nsRefPtr<IDBKeyRange> keyRange;
+  RefPtr<IDBKeyRange> keyRange;
   aRv = IDBKeyRange::FromJSVal(aCx, aKey, getter_AddRefs(keyRange));
   if (aRv.Failed()) {
     return nullptr;
@@ -1614,7 +1734,7 @@ IDBObjectStore::Get(JSContext* aCx,
   params.objectStoreId() = Id();
   keyRange->ToSerialized(params.keyRange());
 
-  nsRefPtr<IDBRequest> request = GenerateRequest(this);
+  RefPtr<IDBRequest> request = GenerateRequest(aCx, this);
   MOZ_ASSERT(request);
 
   IDB_LOG_MARK("IndexedDB %s: Child  Transaction[%lld] Request[%llu]: "
@@ -1656,7 +1776,7 @@ IDBObjectStore::DeleteInternal(JSContext* aCx,
     return nullptr;
   }
 
-  nsRefPtr<IDBKeyRange> keyRange;
+  RefPtr<IDBKeyRange> keyRange;
   aRv = IDBKeyRange::FromJSVal(aCx, aKey, getter_AddRefs(keyRange));
   if (NS_WARN_IF((aRv.Failed()))) {
     return nullptr;
@@ -1672,7 +1792,7 @@ IDBObjectStore::DeleteInternal(JSContext* aCx,
   params.objectStoreId() = Id();
   keyRange->ToSerialized(params.keyRange());
 
-  nsRefPtr<IDBRequest> request = GenerateRequest(this);
+  RefPtr<IDBRequest> request = GenerateRequest(aCx, this);
   MOZ_ASSERT(request);
 
   if (!aFromCursor) {
@@ -1695,50 +1815,12 @@ IDBObjectStore::DeleteInternal(JSContext* aCx,
 
 already_AddRefed<IDBIndex>
 IDBObjectStore::CreateIndex(const nsAString& aName,
-                            const nsAString& aKeyPath,
+                            const StringOrStringSequence& aKeyPath,
                             const IDBIndexParameters& aOptionalParameters,
                             ErrorResult& aRv)
 {
   AssertIsOnOwningThread();
 
-  KeyPath keyPath(0);
-  if (NS_FAILED(KeyPath::Parse(aKeyPath, &keyPath)) ||
-      !keyPath.IsValid()) {
-    aRv.Throw(NS_ERROR_DOM_SYNTAX_ERR);
-    return nullptr;
-  }
-
-  return CreateIndexInternal(aName, keyPath, aOptionalParameters, aRv);
-}
-
-already_AddRefed<IDBIndex>
-IDBObjectStore::CreateIndex(const nsAString& aName,
-                            const Sequence<nsString >& aKeyPath,
-                            const IDBIndexParameters& aOptionalParameters,
-                            ErrorResult& aRv)
-{
-  AssertIsOnOwningThread();
-
-  KeyPath keyPath(0);
-  if (aKeyPath.IsEmpty() ||
-      NS_FAILED(KeyPath::Parse(aKeyPath, &keyPath)) ||
-      !keyPath.IsValid()) {
-    aRv.Throw(NS_ERROR_DOM_SYNTAX_ERR);
-    return nullptr;
-  }
-
-  return CreateIndexInternal(aName, keyPath, aOptionalParameters, aRv);
-}
-
-already_AddRefed<IDBIndex>
-IDBObjectStore::CreateIndexInternal(
-                                  const nsAString& aName,
-                                  const KeyPath& aKeyPath,
-                                  const IDBIndexParameters& aOptionalParameters,
-                                  ErrorResult& aRv)
-{
-  AssertIsOnOwningThread();
-
   if (mTransaction->GetMode() != IDBTransaction::VERSION_CHANGE ||
       mDeletedSpec) {
     aRv.Throw(NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR);
@@ -1763,7 +1845,24 @@ IDBObjectStore::CreateIndexInternal(
     }
   }
 
-  if (aOptionalParameters.mMultiEntry && aKeyPath.IsArray()) {
+  KeyPath keyPath(0);
+  if (aKeyPath.IsString()) {
+    if (NS_FAILED(KeyPath::Parse(aKeyPath.GetAsString(), &keyPath)) ||
+        !keyPath.IsValid()) {
+      aRv.Throw(NS_ERROR_DOM_SYNTAX_ERR);
+      return nullptr;
+    }
+  } else {
+    MOZ_ASSERT(aKeyPath.IsStringSequence());
+    if (aKeyPath.GetAsStringSequence().IsEmpty() ||
+        NS_FAILED(KeyPath::Parse(aKeyPath.GetAsStringSequence(), &keyPath)) ||
+        !keyPath.IsValid()) {
+      aRv.Throw(NS_ERROR_DOM_SYNTAX_ERR);
+      return nullptr;
+    }
+  }
+
+  if (aOptionalParameters.mMultiEntry && keyPath.IsArray()) {
     aRv.Throw(NS_ERROR_DOM_INVALID_ACCESS_ERR);
     return nullptr;
   }
@@ -1779,10 +1878,24 @@ IDBObjectStore::CreateIndexInternal(
   const IndexMetadata* oldMetadataElements =
     indexes.IsEmpty() ? nullptr : indexes.Elements();
 
+  // With this setup we only validate the passed in locale name by the time we
+  // get to encoding Keys. Maybe we should do it here right away and error out.
+
+  // Valid locale names are always ASCII as per BCP-47.
+  nsCString locale = NS_LossyConvertUTF16toASCII(aOptionalParameters.mLocale);
+  bool autoLocale = locale.EqualsASCII("auto");
+#ifdef ENABLE_INTL_API
+  if (autoLocale) {
+    locale = IndexedDatabaseManager::GetLocale();
+  }
+#endif
+
   IndexMetadata* metadata = indexes.AppendElement(
-    IndexMetadata(transaction->NextIndexId(), nsString(aName), aKeyPath,
+    IndexMetadata(transaction->NextIndexId(), nsString(aName), keyPath,
+                  locale,
                   aOptionalParameters.mUnique,
-                  aOptionalParameters.mMultiEntry));
+                  aOptionalParameters.mMultiEntry,
+                  autoLocale));
 
   if (oldMetadataElements &&
       oldMetadataElements != indexes.Elements()) {
@@ -1794,7 +1907,7 @@ IDBObjectStore::CreateIndexInternal(
 
   transaction->CreateIndex(this, *metadata);
 
-  nsRefPtr<IDBIndex> index = IDBIndex::Create(this, *metadata);
+  RefPtr<IDBIndex> index = IDBIndex::Create(this, *metadata);
   MOZ_ASSERT(index);
 
   mIndexes.AppendElement(index);
@@ -1853,7 +1966,7 @@ IDBObjectStore::DeleteIndex(const nsAString& aName, ErrorResult& aRv)
       for (uint32_t indexCount = mIndexes.Length(), indexIndex = 0;
            indexIndex < indexCount;
            indexIndex++) {
-        nsRefPtr<IDBIndex>& index = mIndexes[indexIndex];
+        RefPtr<IDBIndex>& index = mIndexes[indexIndex];
 
         if (index->Id() == foundId) {
           index->NoteDeletion();
@@ -1910,7 +2023,7 @@ IDBObjectStore::Count(JSContext* aCx,
     return nullptr;
   }
 
-  nsRefPtr<IDBKeyRange> keyRange;
+  RefPtr<IDBKeyRange> keyRange;
   aRv = IDBKeyRange::FromJSVal(aCx, aKey, getter_AddRefs(keyRange));
   if (aRv.Failed()) {
     return nullptr;
@@ -1927,7 +2040,7 @@ IDBObjectStore::Count(JSContext* aCx,
     params.optionalKeyRange() = void_t();
   }
 
-  nsRefPtr<IDBRequest> request = GenerateRequest(this);
+  RefPtr<IDBRequest> request = GenerateRequest(aCx, this);
   MOZ_ASSERT(request);
 
   IDB_LOG_MARK("IndexedDB %s: Child  Transaction[%lld] Request[%llu]: "
@@ -1954,6 +2067,7 @@ IDBObjectStore::OpenCursorInternal(bool aKeysOnly,
                                    ErrorResult& aRv)
 {
   AssertIsOnOwningThread();
+  MOZ_ASSERT(aCx);
 
   if (mDeletedSpec) {
     aRv.Throw(NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR);
@@ -1965,7 +2079,7 @@ IDBObjectStore::OpenCursorInternal(bool aKeysOnly,
     return nullptr;
   }
 
-  nsRefPtr<IDBKeyRange> keyRange;
+  RefPtr<IDBKeyRange> keyRange;
   aRv = IDBKeyRange::FromJSVal(aCx, aRange, getter_AddRefs(keyRange));
   if (NS_WARN_IF(aRv.Failed())) {
     return nullptr;
@@ -2003,7 +2117,7 @@ IDBObjectStore::OpenCursorInternal(bool aKeysOnly,
     params = Move(openParams);
   }
 
-  nsRefPtr<IDBRequest> request = GenerateRequest(this);
+  RefPtr<IDBRequest> request = GenerateRequest(aCx, this);
   MOZ_ASSERT(request);
 
   if (aKeysOnly) {
@@ -2130,6 +2244,59 @@ IDBObjectStore::Name() const
   return mSpec->metadata().name();
 }
 
+void
+IDBObjectStore::SetName(const nsAString& aName, ErrorResult& aRv)
+{
+  AssertIsOnOwningThread();
+
+  if (mTransaction->GetMode() != IDBTransaction::VERSION_CHANGE ||
+      mDeletedSpec) {
+    aRv.Throw(NS_ERROR_DOM_INVALID_STATE_ERR);
+    return;
+  }
+
+  IDBTransaction* transaction = IDBTransaction::GetCurrent();
+  if (!transaction || transaction != mTransaction) {
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_TRANSACTION_INACTIVE_ERR);
+    return;
+  }
+
+  MOZ_ASSERT(transaction->IsOpen());
+
+  if (aName == mSpec->metadata().name()) {
+    return;
+  }
+
+  // Cache logging string of this object store before renaming.
+  const LoggingString loggingOldObjectStore(this);
+
+  nsresult rv =
+    transaction->Database()->RenameObjectStore(mSpec->metadata().id(),
+                                               aName);
+
+  if (NS_FAILED(rv)) {
+    aRv.Throw(rv);
+    return;
+  }
+
+  // Don't do this in the macro because we always need to increment the serial
+  // number to keep in sync with the parent.
+  const uint64_t requestSerialNumber = IDBRequest::NextSerialNumber();
+
+  IDB_LOG_MARK("IndexedDB %s: Child  Transaction[%lld] Request[%llu]: "
+                 "database(%s).transaction(%s).objectStore(%s).rename(%s)",
+               "IndexedDB %s: C T[%lld] R[%llu]: IDBObjectStore.rename()",
+               IDB_LOG_ID_STRING(),
+               mTransaction->LoggingSerialNumber(),
+               requestSerialNumber,
+               IDB_LOG_STRINGIFY(mTransaction->Database()),
+               IDB_LOG_STRINGIFY(mTransaction),
+               loggingOldObjectStore.get(),
+               IDB_LOG_STRINGIFY(this));
+
+  transaction->RenameObjectStore(mSpec->metadata().id(), aName);
+}
+
 bool
 IDBObjectStore::AutoIncrement() const
 {
@@ -2139,7 +2306,7 @@ IDBObjectStore::AutoIncrement() const
   return mSpec->metadata().autoIncrement();
 }
 
-const KeyPath&
+const indexedDB::KeyPath&
 IDBObjectStore::GetKeyPath() const
 {
   AssertIsOnOwningThread();
@@ -2157,6 +2324,5 @@ IDBObjectStore::HasValidKeyPath() const
   return GetKeyPath().IsValid();
 }
 
-} // namespace indexedDB
 } // namespace dom
 } // namespace mozilla
