@@ -46,13 +46,31 @@ AudioData::EnsureAudioBuffer()
 size_t
 AudioData::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
 {
-  size_t size = aMallocSizeOf(this) + aMallocSizeOf(mAudioData);
+  size_t size =
+    aMallocSizeOf(this) + mAudioData.SizeOfExcludingThis(aMallocSizeOf);
   if (mAudioBuffer) {
     size += mAudioBuffer->SizeOfIncludingThis(aMallocSizeOf);
   }
   return size;
 }
 
+bool
+AudioData::IsAudible() const
+{
+  if (!mAudioData) {
+    return false;
+  }
+
+  for (uint32_t frame = 0; frame < mFrames; ++frame) {
+    for (uint32_t channel = 0; channel < mChannels; ++channel) {
+      if (mAudioData[frame * mChannels + channel] != 0) {
+        return true;
+      }
+    }
+  }
+  return false;
+}
+
 /* static */
 already_AddRefed<AudioData>
 AudioData::TransferAndUpdateTimestampAndDuration(AudioData* aOther,
@@ -60,16 +78,14 @@ AudioData::TransferAndUpdateTimestampAndDuration(AudioData* aOther,
                                                   int64_t aDuration)
 {
   NS_ENSURE_TRUE(aOther, nullptr);
-  nsRefPtr<AudioData> v = new AudioData(aOther->mOffset,
-                                        aTimestamp,
-                                        aDuration,
-                                        aOther->mFrames,
-                                        aOther->mAudioData,
-                                        aOther->mChannels,
-                                        aOther->mRate);
+  RefPtr<AudioData> v = new AudioData(aOther->mOffset,
+                                      aTimestamp,
+                                      aDuration,
+                                      aOther->mFrames,
+                                      Move(aOther->mAudioData),
+                                      aOther->mChannels,
+                                      aOther->mRate);
   v->mDiscontinuity = aOther->mDiscontinuity;
-  // Remove aOther's AudioData as it can't be shared across two targets.
-  aOther->mAudioData.forget();
 
   return v.forget();
 }
@@ -150,7 +166,7 @@ already_AddRefed<VideoData>
 VideoData::ShallowCopyUpdateDuration(const VideoData* aOther,
                                      int64_t aDuration)
 {
-  nsRefPtr<VideoData> v = new VideoData(aOther->mOffset,
+  RefPtr<VideoData> v = new VideoData(aOther->mOffset,
                                         aOther->mTime,
                                         aDuration,
                                         aOther->mKeyframe,
@@ -168,7 +184,7 @@ VideoData::ShallowCopyUpdateTimestamp(const VideoData* aOther,
                                       int64_t aTimestamp)
 {
   NS_ENSURE_TRUE(aOther, nullptr);
-  nsRefPtr<VideoData> v = new VideoData(aOther->mOffset,
+  RefPtr<VideoData> v = new VideoData(aOther->mOffset,
                                         aTimestamp,
                                         aOther->GetEndTime() - aTimestamp,
                                         aOther->mKeyframe,
@@ -187,7 +203,7 @@ VideoData::ShallowCopyUpdateTimestampAndDuration(const VideoData* aOther,
                                                  int64_t aDuration)
 {
   NS_ENSURE_TRUE(aOther, nullptr);
-  nsRefPtr<VideoData> v = new VideoData(aOther->mOffset,
+  RefPtr<VideoData> v = new VideoData(aOther->mOffset,
                                         aTimestamp,
                                         aDuration,
                                         aOther->mKeyframe,
@@ -200,14 +216,14 @@ VideoData::ShallowCopyUpdateTimestampAndDuration(const VideoData* aOther,
 }
 
 /* static */
-void VideoData::SetVideoDataToImage(PlanarYCbCrImage* aVideoImage,
+bool VideoData::SetVideoDataToImage(PlanarYCbCrImage* aVideoImage,
                                     const VideoInfo& aInfo,
                                     const YCbCrBuffer &aBuffer,
                                     const IntRect& aPicture,
                                     bool aCopyData)
 {
   if (!aVideoImage) {
-    return;
+    return false;
   }
   const YCbCrBuffer::Plane &Y = aBuffer.mPlanes[0];
   const YCbCrBuffer::Plane &Cb = aBuffer.mPlanes[1];
@@ -231,9 +247,9 @@ void VideoData::SetVideoDataToImage(PlanarYCbCrImage* aVideoImage,
 
   aVideoImage->SetDelayedConversion(true);
   if (aCopyData) {
-    aVideoImage->SetData(data);
+    return aVideoImage->CopyData(data);
   } else {
-    aVideoImage->SetDataNoCopy(data);
+    return aVideoImage->AdoptData(data);
   }
 }
 
@@ -253,7 +269,7 @@ VideoData::Create(const VideoInfo& aInfo,
   if (!aImage && !aContainer) {
     // Create a dummy VideoData with no image. This gives us something to
     // send to media streams if necessary.
-    nsRefPtr<VideoData> v(new VideoData(aOffset,
+    RefPtr<VideoData> v(new VideoData(aOffset,
                                         aTime,
                                         aDuration,
                                         aKeyframe,
@@ -296,7 +312,7 @@ VideoData::Create(const VideoInfo& aInfo,
     return nullptr;
   }
 
-  nsRefPtr<VideoData> v(new VideoData(aOffset,
+  RefPtr<VideoData> v(new VideoData(aOffset,
                                       aTime,
                                       aDuration,
                                       aKeyframe,
@@ -314,11 +330,11 @@ VideoData::Create(const VideoInfo& aInfo,
     // format.
 #ifdef MOZ_WIDGET_GONK
     if (IsYV12Format(Y, Cb, Cr) && !IsInEmulator()) {
-      v->mImage = aContainer->CreateImage(ImageFormat::GRALLOC_PLANAR_YCBCR);
+      v->mImage = new layers::GrallocImage();
     }
 #endif
     if (!v->mImage) {
-      v->mImage = aContainer->CreateImage(ImageFormat::PLANAR_YCBCR);
+      v->mImage = aContainer->CreatePlanarYCbCrImage();
     }
   } else {
     v->mImage = aImage;
@@ -330,26 +346,27 @@ VideoData::Create(const VideoInfo& aInfo,
   NS_ASSERTION(v->mImage->GetFormat() == ImageFormat::PLANAR_YCBCR ||
                v->mImage->GetFormat() == ImageFormat::GRALLOC_PLANAR_YCBCR,
                "Wrong format?");
-  PlanarYCbCrImage* videoImage = static_cast<PlanarYCbCrImage*>(v->mImage.get());
+  PlanarYCbCrImage* videoImage = v->mImage->AsPlanarYCbCrImage();
+  MOZ_ASSERT(videoImage);
 
-  if (!aImage) {
-    VideoData::SetVideoDataToImage(videoImage, aInfo, aBuffer, aPicture,
-                                   true /* aCopyData */);
-  } else {
-    VideoData::SetVideoDataToImage(videoImage, aInfo, aBuffer, aPicture,
-                                   false /* aCopyData */);
+  bool shouldCopyData = (aImage == nullptr);
+  if (!VideoData::SetVideoDataToImage(videoImage, aInfo, aBuffer, aPicture,
+                                      shouldCopyData)) {
+    return nullptr;
   }
 
 #ifdef MOZ_WIDGET_GONK
   if (!videoImage->IsValid() && !aImage && IsYV12Format(Y, Cb, Cr)) {
     // Failed to allocate gralloc. Try fallback.
-    v->mImage = aContainer->CreateImage(ImageFormat::PLANAR_YCBCR);
+    v->mImage = aContainer->CreatePlanarYCbCrImage();
     if (!v->mImage) {
       return nullptr;
     }
-    videoImage = static_cast<PlanarYCbCrImage*>(v->mImage.get());
-    VideoData::SetVideoDataToImage(videoImage, aInfo, aBuffer, aPicture,
-                                   true /* aCopyData */);
+    videoImage = v->mImage->AsPlanarYCbCrImage();
+    if(!VideoData::SetVideoDataToImage(videoImage, aInfo, aBuffer, aPicture,
+                                       true /* aCopyData */)) {
+      return nullptr;
+    }
   }
 #endif
   return v.forget();
@@ -394,12 +411,12 @@ VideoData::CreateFromImage(const VideoInfo& aInfo,
                            int64_t aOffset,
                            int64_t aTime,
                            int64_t aDuration,
-                           const nsRefPtr<Image>& aImage,
+                           const RefPtr<Image>& aImage,
                            bool aKeyframe,
                            int64_t aTimecode,
                            const IntRect& aPicture)
 {
-  nsRefPtr<VideoData> v(new VideoData(aOffset,
+  RefPtr<VideoData> v(new VideoData(aOffset,
                                       aTime,
                                       aDuration,
                                       aKeyframe,
@@ -426,7 +443,7 @@ VideoData::Create(const VideoInfo& aInfo,
   if (!aContainer) {
     // Create a dummy VideoData with no image. This gives us something to
     // send to media streams if necessary.
-    nsRefPtr<VideoData> v(new VideoData(aOffset,
+    RefPtr<VideoData> v(new VideoData(aOffset,
                                         aTime,
                                         aDuration,
                                         aKeyframe,
@@ -454,7 +471,7 @@ VideoData::Create(const VideoInfo& aInfo,
     return nullptr;
   }
 
-  nsRefPtr<VideoData> v(new VideoData(aOffset,
+  RefPtr<VideoData> v(new VideoData(aOffset,
                                       aTime,
                                       aDuration,
                                       aKeyframe,
@@ -462,60 +479,31 @@ VideoData::Create(const VideoInfo& aInfo,
                                       aInfo.mDisplay,
                                       0));
 
-  v->mImage = aContainer->CreateImage(ImageFormat::GRALLOC_PLANAR_YCBCR);
-  if (!v->mImage) {
-    return nullptr;
-  }
-  NS_ASSERTION(v->mImage->GetFormat() == ImageFormat::GRALLOC_PLANAR_YCBCR,
-               "Wrong format?");
-  typedef mozilla::layers::GrallocImage GrallocImage;
-  GrallocImage* videoImage = static_cast<GrallocImage*>(v->mImage.get());
-  GrallocImage::GrallocData data;
-
-  data.mPicSize = aPicture.Size();
-  data.mGraphicBuffer = aBuffer;
-
-  videoImage->SetData(data);
+  RefPtr<layers::GrallocImage> image = new layers::GrallocImage();
+  image->AdoptData(aBuffer, aPicture.Size());
+  v->mImage = image;
 
   return v.forget();
 }
 #endif  // MOZ_OMX_DECODER
 
-// Alignment value - 1. 0 means that data isn't aligned.
-// For 32-bytes aligned, use 31U.
-#define RAW_DATA_ALIGNMENT 31U
-
 MediaRawData::MediaRawData()
   : MediaData(RAW_DATA, 0)
   , mCrypto(mCryptoInternal)
-  , mData(nullptr)
-  , mSize(0)
-  , mBuffer(nullptr)
-  , mCapacity(0)
 {
 }
 
 MediaRawData::MediaRawData(const uint8_t* aData, size_t aSize)
   : MediaData(RAW_DATA, 0)
   , mCrypto(mCryptoInternal)
-  , mData(nullptr)
-  , mSize(0)
-  , mBuffer(nullptr)
-  , mCapacity(0)
+  , mBuffer(aData, aSize)
 {
-  if (!EnsureCapacity(aSize)) {
-    return;
-  }
-
-  // We ensure sufficient capacity above so this shouldn't fail.
-  memcpy(mData, aData, aSize);
-  mSize = aSize;
 }
 
 already_AddRefed<MediaRawData>
 MediaRawData::Clone() const
 {
-  nsRefPtr<MediaRawData> s = new MediaRawData;
+  RefPtr<MediaRawData> s = new MediaRawData;
   s->mTimecode = mTimecode;
   s->mTime = mTime;
   s->mDuration = mDuration;
@@ -524,47 +512,12 @@ MediaRawData::Clone() const
   s->mExtraData = mExtraData;
   s->mCryptoInternal = mCryptoInternal;
   s->mTrackInfo = mTrackInfo;
-  if (mSize) {
-    if (!s->EnsureCapacity(mSize)) {
-      return nullptr;
-    }
-
-    memcpy(s->mData, mData, mSize);
-    s->mSize = mSize;
+  if (!s->mBuffer.Append(mBuffer.Data(), mBuffer.Length())) {
+    return nullptr;
   }
   return s.forget();
 }
 
-// EnsureCapacity ensures that the buffer is big enough to hold
-// aSize. It doesn't set the mSize. It's up to the caller to adjust it.
-bool
-MediaRawData::EnsureCapacity(size_t aSize)
-{
-  const size_t sizeNeeded = aSize + RAW_DATA_ALIGNMENT * 2;
-
-  if (mData && mCapacity >= sizeNeeded) {
-    return true;
-  }
-  nsAutoArrayPtr<uint8_t> newBuffer;
-  newBuffer = new (fallible) uint8_t[sizeNeeded];
-  if (!newBuffer) {
-    return false;
-  }
-
-  // Find alignment address.
-  const uintptr_t alignmask = RAW_DATA_ALIGNMENT;
-  uint8_t* newData = reinterpret_cast<uint8_t*>(
-    (reinterpret_cast<uintptr_t>(newBuffer.get()) + alignmask) & ~alignmask);
-  MOZ_ASSERT(uintptr_t(newData) % (RAW_DATA_ALIGNMENT+1) == 0);
-  memcpy(newData, mData, mSize);
-
-  mBuffer = newBuffer.forget();
-  mCapacity = sizeNeeded;
-  mData = newData;
-
-  return true;
-}
-
 MediaRawData::~MediaRawData()
 {
 }
@@ -573,7 +526,7 @@ size_t
 MediaRawData::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
 {
   size_t size = aMallocSizeOf(this);
-  size += aMallocSizeOf(mBuffer.get());
+  size += mBuffer.SizeOfExcludingThis(aMallocSizeOf);
   return size;
 }
 
@@ -589,69 +542,34 @@ MediaRawDataWriter::MediaRawDataWriter(MediaRawData* aMediaRawData)
 {
 }
 
-bool
-MediaRawDataWriter::EnsureSize(size_t aSize)
-{
-  if (aSize <= mTarget->mSize) {
-    return true;
-  }
-  if (!mTarget->EnsureCapacity(aSize)) {
-    return false;
-  }
-  return true;
-}
-
 bool
 MediaRawDataWriter::SetSize(size_t aSize)
 {
-  if (aSize > mTarget->mSize && !EnsureSize(aSize)) {
-    return false;
-  }
-
-  mTarget->mSize = aSize;
-  return true;
+  return mTarget->mBuffer.SetLength(aSize);
 }
 
 bool
 MediaRawDataWriter::Prepend(const uint8_t* aData, size_t aSize)
 {
-  if (!EnsureSize(aSize + mTarget->mSize)) {
-    return false;
-  }
-
-  // Shift the data to the right by aSize to leave room for the new data.
-  memmove(mTarget->mData + aSize, mTarget->mData, mTarget->mSize);
-  memcpy(mTarget->mData, aData, aSize);
-
-  mTarget->mSize += aSize;
-  return true;
+  return mTarget->mBuffer.Prepend(aData, aSize);
 }
 
 bool
 MediaRawDataWriter::Replace(const uint8_t* aData, size_t aSize)
 {
-  // If aSize is smaller than our current size, we leave the buffer as is,
-  // only adjusting the reported size.
-  if (!EnsureSize(aSize)) {
-    return false;
-  }
-
-  memcpy(mTarget->mData, aData, aSize);
-  mTarget->mSize = aSize;
-  return true;
+  return mTarget->mBuffer.Replace(aData, aSize);
 }
 
 void
 MediaRawDataWriter::Clear()
 {
-  mTarget->mSize = 0;
-  mTarget->mData = nullptr;
+  mTarget->mBuffer.Clear();
 }
 
 uint8_t*
 MediaRawDataWriter::Data()
 {
-  return mTarget->mData;
+  return mTarget->mBuffer.Data();
 }
 
 size_t
