@@ -137,11 +137,11 @@ var checkAllTrackStats = pc => {
 // renegotiation test.
 var commandsPeerConnectionInitial = [
   function PC_SETUP_SIGNALING_CLIENT(test) {
-    if (test.steeplechase) {
+    if (test.testOptions.steeplechase) {
       test.setupSignalingClient();
       test.registerSignalingCallback("ice_candidate", function (message) {
         var pc = test.pcRemote ? test.pcRemote : test.pcLocal;
-        pc.storeOrAddIceCandidate(new mozRTCIceCandidate(message.ice_candidate));
+        pc.storeOrAddIceCandidate(new RTCIceCandidate(message.ice_candidate));
       });
       test.registerSignalingCallback("end_of_trickle_ice", function (message) {
         test.signalingMessagesFinished();
@@ -165,12 +165,12 @@ var commandsPeerConnectionInitial = [
     test.pcRemote.logSignalingState();
   },
 
-  function PC_LOCAL_SETUP_ADDSTREAM_HANDLER(test) {
-    test.pcLocal.setupAddStreamEventHandler();
+  function PC_LOCAL_SETUP_TRACK_HANDLER(test) {
+    test.pcLocal.setupTrackEventHandler();
   },
 
-  function PC_REMOTE_SETUP_ADDSTREAM_HANDLER(test) {
-    test.pcRemote.setupAddStreamEventHandler();
+  function PC_REMOTE_SETUP_TRACK_HANDLER(test) {
+    test.pcRemote.setupTrackEventHandler();
   },
 
   function PC_LOCAL_CHECK_INITIAL_SIGNALINGSTATE(test) {
@@ -193,6 +193,15 @@ var commandsPeerConnectionInitial = [
        "Initial remote ICE connection state is 'new'");
   },
 
+  function PC_LOCAL_CHECK_INITIAL_CAN_TRICKLE_SYNC(test) {
+    is(test.pcLocal._pc.canTrickleIceCandidates, null,
+       "Local trickle status should start out unknown");
+  },
+
+  function PC_REMOTE_CHECK_INITIAL_CAN_TRICKLE_SYNC(test) {
+    is(test.pcRemote._pc.canTrickleIceCandidates, null,
+       "Remote trickle status should start out unknown");
+  },
 ];
 
 var commandsGetUserMedia = [
@@ -208,38 +217,28 @@ var commandsGetUserMedia = [
 var commandsPeerConnectionOfferAnswer = [
   function PC_LOCAL_SETUP_ICE_HANDLER(test) {
     test.pcLocal.setupIceCandidateHandler(test);
-    if (test.steeplechase) {
-      test.pcLocal.endOfTrickleIce.then(() => {
-        send_message({"type": "end_of_trickle_ice"});
-      });
-    }
   },
 
   function PC_REMOTE_SETUP_ICE_HANDLER(test) {
     test.pcRemote.setupIceCandidateHandler(test);
-    if (test.steeplechase) {
-      test.pcRemote.endOfTrickleIce.then(() => {
-        send_message({"type": "end_of_trickle_ice"});
-      });
-    }
   },
 
   function PC_LOCAL_STEEPLECHASE_SIGNAL_EXPECTED_LOCAL_TRACKS(test) {
-    if (test.steeplechase) {
+    if (test.testOptions.steeplechase) {
       send_message({"type": "local_expected_tracks",
                     "expected_tracks": test.pcLocal.expectedLocalTrackInfoById});
     }
   },
 
   function PC_REMOTE_STEEPLECHASE_SIGNAL_EXPECTED_LOCAL_TRACKS(test) {
-    if (test.steeplechase) {
+    if (test.testOptions.steeplechase) {
       send_message({"type": "remote_expected_tracks",
                     "expected_tracks": test.pcRemote.expectedLocalTrackInfoById});
     }
   },
 
   function PC_LOCAL_GET_EXPECTED_REMOTE_TRACKS(test) {
-    if (test.steeplechase) {
+    if (test.testOptions.steeplechase) {
       return test.getSignalingMessage("remote_expected_tracks").then(
           message => {
             test.pcLocal.expectedRemoteTrackInfoById = message.expected_tracks;
@@ -252,7 +251,7 @@ var commandsPeerConnectionOfferAnswer = [
   },
 
   function PC_REMOTE_GET_EXPECTED_REMOTE_TRACKS(test) {
-    if (test.steeplechase) {
+    if (test.testOptions.steeplechase) {
       return test.getSignalingMessage("local_expected_tracks").then(
           message => {
             test.pcRemote.expectedRemoteTrackInfoById = message.expected_tracks;
@@ -272,7 +271,7 @@ var commandsPeerConnectionOfferAnswer = [
   },
 
   function PC_LOCAL_STEEPLECHASE_SIGNAL_OFFER(test) {
-    if (test.steeplechase) {
+    if (test.testOptions.steeplechase) {
       send_message({"type": "offer",
                     "offer": test.originalOffer,
                     "offer_constraints": test.pcLocal.constraints,
@@ -292,7 +291,7 @@ var commandsPeerConnectionOfferAnswer = [
   },
 
   function PC_REMOTE_GET_OFFER(test) {
-    if (!test.steeplechase) {
+    if (!test.testOptions.steeplechase) {
       test._local_offer = test.originalOffer;
       test._offer_constraints = test.pcLocal.constraints;
       test._offer_options = test.pcLocal.offerOptions;
@@ -301,7 +300,7 @@ var commandsPeerConnectionOfferAnswer = [
     return test.getSignalingMessage("offer")
       .then(message => {
         ok("offer" in message, "Got an offer message");
-        test._local_offer = new mozRTCSessionDescription(message.offer);
+        test._local_offer = new RTCSessionDescription(message.offer);
         test._offer_constraints = message.offer_constraints;
         test._offer_options = message.offer_options;
       });
@@ -315,16 +314,23 @@ var commandsPeerConnectionOfferAnswer = [
       });
   },
 
+  function PC_REMOTE_CHECK_CAN_TRICKLE_SYNC(test) {
+    is(test.pcRemote._pc.canTrickleIceCandidates, true,
+       "Remote thinks that local can trickle");
+  },
+
   function PC_LOCAL_SANE_LOCAL_SDP(test) {
-    test.pcLocal.verifySdp(test._local_offer, "offer",
-                           test._offer_constraints, test._offer_options,
-                           true);
+    test.pcLocal.localRequiresTrickleIce =
+      sdputils.verifySdp(test._local_offer, "offer",
+                         test._offer_constraints, test._offer_options,
+                         test.testOptions);
   },
 
   function PC_REMOTE_SANE_REMOTE_SDP(test) {
-    test.pcRemote.verifySdp(test._local_offer, "offer",
-                            test._offer_constraints, test._offer_options,
-                            false);
+    test.pcRemote.remoteRequiresTrickleIce =
+      sdputils.verifySdp(test._local_offer, "offer",
+                         test._offer_constraints, test._offer_options,
+                         test.testOptions);
   },
 
   function PC_REMOTE_CREATE_ANSWER(test) {
@@ -332,7 +338,7 @@ var commandsPeerConnectionOfferAnswer = [
       .then(answer => {
         is(test.pcRemote.signalingState, HAVE_REMOTE_OFFER,
            "Remote createAnswer does not change signaling state");
-        if (test.steeplechase) {
+        if (test.testOptions.steeplechase) {
           send_message({"type": "answer",
                         "answer": test.originalAnswer,
                         "answer_constraints": test.pcRemote.constraints});
@@ -342,56 +348,17 @@ var commandsPeerConnectionOfferAnswer = [
       });
   },
 
-  function PC_REMOTE_CHECK_FOR_DUPLICATED_PORTS_IN_SDP(test) {
-    var re = /a=candidate.* (UDP|TCP) [\d]+ ([\d\.]+) ([\d]+) typ host/g;
-
-    var _sdpCandidatesIntoArray = sdp => {
-      var regexArray = [];
-      var resultArray = [];
-      while ((regexArray = re.exec(sdp)) !== null) {
-        info("regexArray: " + regexArray);
-        if ((regexArray[1] === "TCP") && (regexArray[3] === "9")) {
-          // As both sides can advertise TCP active connection on port 9 lets
-          // ignore them all together
-          info("Ignoring TCP candidate on port 9");
-          continue;
-        }
-        var triple = regexArray[1] + ":" + regexArray[2] + ":" + regexArray[3];
-        info("triple: " + triple);
-        if (resultArray.indexOf(triple) !== -1) {
-          dump("SDP: " + sdp.replace(/[\r]/g, '') + "\n");
-          ok(false, "This Transport:IP:Port " + triple + " appears twice in the SDP above!");
-        }
-        resultArray.push(triple);
-      }
-      return resultArray;
-    };
-
-    var offerTriples = _sdpCandidatesIntoArray(test._local_offer.sdp);
-    info("Offer ICE host candidates: " + JSON.stringify(offerTriples));
-
-    var answerTriples = _sdpCandidatesIntoArray(test.originalAnswer.sdp);
-    info("Answer ICE host candidates: " + JSON.stringify(answerTriples));
-
-    offerTriples.forEach(o => {
-      if (answerTriples.indexOf(o) !== -1) {
-        dump("SDP offer: " + test._local_offer.sdp.replace(/[\r]/g, '') + "\n");
-        dump("SDP answer: " + test.originalAnswer.sdp.replace(/[\r]/g, '') + "\n");
-        ok(false, "This IP:Port " + o + " appears in SDP offer and answer!");
-      }
-    });
-  },
-
   function PC_REMOTE_SET_LOCAL_DESCRIPTION(test) {
     return test.setLocalDescription(test.pcRemote, test.originalAnswer, STABLE)
       .then(() => {
         is(test.pcRemote.signalingState, STABLE,
            "signalingState after remote setLocalDescription is 'stable'");
-      });
+      })
+      .then(() => test.pcRemote.markRemoteTracksAsNegotiated());
   },
 
   function PC_LOCAL_GET_ANSWER(test) {
-    if (!test.steeplechase) {
+    if (!test.testOptions.steeplechase) {
       test._remote_answer = test.originalAnswer;
       test._answer_constraints = test.pcRemote.constraints;
       return Promise.resolve();
@@ -399,7 +366,7 @@ var commandsPeerConnectionOfferAnswer = [
 
     return test.getSignalingMessage("answer").then(message => {
       ok("answer" in message, "Got an answer message");
-      test._remote_answer = new mozRTCSessionDescription(message.answer);
+      test._remote_answer = new RTCSessionDescription(message.answer);
       test._answer_constraints = message.answer_constraints;
     });
   },
@@ -409,17 +376,26 @@ var commandsPeerConnectionOfferAnswer = [
       .then(() => {
         is(test.pcLocal.signalingState, STABLE,
            "signalingState after local setRemoteDescription is 'stable'");
-      });
+      })
+      .then(() => test.pcLocal.markRemoteTracksAsNegotiated());
   },
+
   function PC_REMOTE_SANE_LOCAL_SDP(test) {
-    test.pcRemote.verifySdp(test._remote_answer, "answer",
-                            test._offer_constraints, test._offer_options,
-                            true);
+    test.pcRemote.localRequiresTrickleIce =
+      sdputils.verifySdp(test._remote_answer, "answer",
+                         test._offer_constraints, test._offer_options,
+                         test.testOptions);
   },
   function PC_LOCAL_SANE_REMOTE_SDP(test) {
-    test.pcLocal.verifySdp(test._remote_answer, "answer",
-                           test._offer_constraints, test._offer_options,
-                           false);
+    test.pcLocal.remoteRequiresTrickleIce =
+      sdputils.verifySdp(test._remote_answer, "answer",
+                         test._offer_constraints, test._offer_options,
+                         test.testOptions);
+  },
+
+  function PC_LOCAL_CHECK_CAN_TRICKLE_SYNC(test) {
+    is(test.pcLocal._pc.canTrickleIceCandidates, true,
+       "Local thinks that remote can trickle");
   },
 
   function PC_LOCAL_WAIT_FOR_ICE_CONNECTED(test) {
@@ -438,14 +414,6 @@ var commandsPeerConnectionOfferAnswer = [
     return waitForAnIceCandidate(test.pcRemote);
   },
 
-  function PC_LOCAL_CHECK_MEDIA_TRACKS(test) {
-    return test.pcLocal.checkMediaTracks();
-  },
-
-  function PC_REMOTE_CHECK_MEDIA_TRACKS(test) {
-    return test.pcRemote.checkMediaTracks();
-  },
-
   function PC_LOCAL_WAIT_FOR_MEDIA_FLOW(test) {
     return test.pcLocal.waitForMediaFlow();
   },
@@ -456,43 +424,45 @@ var commandsPeerConnectionOfferAnswer = [
 
   function PC_LOCAL_CHECK_STATS(test) {
     return test.pcLocal.getStats().then(stats => {
-      test.pcLocal.checkStats(stats, test.steeplechase);
+      test.pcLocal.checkStats(stats, test.testOptions.steeplechase);
     });
   },
 
   function PC_REMOTE_CHECK_STATS(test) {
-    test.pcRemote.getStats().then(stats => {
-      test.pcRemote.checkStats(stats, test.steeplechase);
+    return test.pcRemote.getStats().then(stats => {
+      test.pcRemote.checkStats(stats, test.testOptions.steeplechase);
     });
   },
 
   function PC_LOCAL_CHECK_ICE_CONNECTION_TYPE(test) {
-    test.pcLocal.getStats().then(stats => {
-      test.pcLocal.checkStatsIceConnectionType(stats);
+    return test.pcLocal.getStats().then(stats => {
+      test.pcLocal.checkStatsIceConnectionType(stats,
+          test.testOptions.expectedLocalCandidateType);
     });
   },
 
   function PC_REMOTE_CHECK_ICE_CONNECTION_TYPE(test) {
-    test.pcRemote.getStats().then(stats => {
-      test.pcRemote.checkStatsIceConnectionType(stats);
+    return test.pcRemote.getStats().then(stats => {
+      test.pcRemote.checkStatsIceConnectionType(stats,
+          test.testOptions.expectedRemoteCandidateType);
     });
   },
 
   function PC_LOCAL_CHECK_ICE_CONNECTIONS(test) {
-    test.pcLocal.getStats().then(stats => {
+    return test.pcLocal.getStats().then(stats => {
       test.pcLocal.checkStatsIceConnections(stats,
                                             test._offer_constraints,
                                             test._offer_options,
-                                            test._remote_answer);
+                                            test.testOptions);
     });
   },
 
   function PC_REMOTE_CHECK_ICE_CONNECTIONS(test) {
-    test.pcRemote.getStats().then(stats => {
+    return test.pcRemote.getStats().then(stats => {
       test.pcRemote.checkStatsIceConnections(stats,
                                              test._offer_constraints,
                                              test._offer_options,
-                                             test.originalAnswer);
+                                             test.testOptions);
     });
   },
 
@@ -503,28 +473,56 @@ var commandsPeerConnectionOfferAnswer = [
     return test.pcRemote.checkMsids();
   },
 
-  function PC_LOCAL_CHECK_STATS(test) {
+  function PC_LOCAL_CHECK_TRACK_STATS(test) {
     return checkAllTrackStats(test.pcLocal);
   },
-  function PC_REMOTE_CHECK_STATS(test) {
+  function PC_REMOTE_CHECK_TRACK_STATS(test) {
     return checkAllTrackStats(test.pcRemote);
   },
-  function PC_LOCAL_WAIT_FOR_END_OF_TRICKLE(test) {
-    return test.pcLocal.endOfTrickleIce;
+  function PC_LOCAL_VERIFY_SDP_AFTER_END_OF_TRICKLE(test) {
+    if (test.pcLocal.endOfTrickleSdp) {
+      /* In case the endOfTrickleSdp promise is resolved already it will win the
+       * race because it gets evaluated first. But if endOfTrickleSdp is still
+       * pending the rejection will win the race. */
+      return Promise.race([
+        test.pcLocal.endOfTrickleSdp,
+        Promise.reject("No SDP")
+      ])
+      .then(sdp => sdputils.checkSdpAfterEndOfTrickle(sdp, test.testOptions, test.pcLocal.label),
+            () => info("pcLocal: Gathering is not complete yet, skipping post-gathering SDP check"));
+    }
   },
-  function PC_REMOTE_WAIT_FOR_END_OF_TRICKLE(test) {
-    return test.pcRemote.endOfTrickleIce;
+  function PC_REMOTE_VERIFY_SDP_AFTER_END_OF_TRICKLE(test) {
+    if (test.pcRemote.endOfTrickelSdp) {
+      /* In case the endOfTrickleSdp promise is resolved already it will win the
+       * race because it gets evaluated first. But if endOfTrickleSdp is still
+       * pending the rejection will win the race. */
+      return Promise.race([
+        test.pcRemote.endOfTrickleSdp,
+        Promise.reject("No SDP")
+      ])
+      .then(sdp => sdputils.checkSdpAfterEndOfTrickle(sdp, test.testOptions, test.pcRemote.label),
+            () => info("pcRemote: Gathering is not complete yet, skipping post-gathering SDP check"));
+    }
   }
 ];
 
+function PC_LOCAL_REMOVE_VP8_FROM_OFFER(test) {
+  isnot(test.originalOffer.sdp.search("H264/90000"), -1, "H.264 should be present in the SDP offer");
+  test.originalOffer.sdp = sdputils.removeVP8(test.originalOffer.sdp);
+  info("Updated H264 only offer: " + JSON.stringify(test.originalOffer));
+};
+
 function PC_LOCAL_REMOVE_BUNDLE_FROM_OFFER(test) {
-  test.originalOffer.sdp = test.originalOffer.sdp.replace(
-      /a=group:BUNDLE .*\r\n/g,
-      ""
-      );
+  test.originalOffer.sdp = sdputils.removeBundle(test.originalOffer.sdp);
   info("Updated no bundle offer: " + JSON.stringify(test.originalOffer));
 };
 
+function PC_LOCAL_REMOVE_RTCPMUX_FROM_OFFER(test) {
+  test.originalOffer.sdp = sdputils.removeRtcpMux(test.originalOffer.sdp);
+  info("Updated no RTCP-Mux offer: " + JSON.stringify(test.originalOffer));
+};
+
 var addRenegotiation = (chain, commands, checks) => {
   chain.append(commands);
   chain.append(commandsPeerConnectionOfferAnswer);
