@@ -13,7 +13,8 @@
 #include <stdint.h>
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/CheckedInt.h"
-#include "mozilla/Endian.h"
+#include "mozilla/EndianUtils.h"
+#include "mozilla/UniquePtr.h"
 #include <algorithm>
 
 using namespace mozilla::media;
@@ -23,7 +24,7 @@ namespace mozilla {
 // Un-comment to enable logging of seek bisections.
 //#define SEEK_LOGGING
 
-extern PRLogModuleInfo* gMediaDecoderLog;
+extern LazyLogModule gMediaDecoderLog;
 #define LOG(type, msg) MOZ_LOG(gMediaDecoderLog, type, msg)
 #ifdef SEEK_LOGGING
 #define SEEK_LOG(type, msg) MOZ_LOG(gMediaDecoderLog, type, msg)
@@ -79,6 +80,20 @@ namespace {
     return result;
   }
 
+  int32_t
+  ReadInt24LE(const char** aBuffer)
+  {
+    int32_t result = int32_t((uint8_t((*aBuffer)[2]) << 16) |
+                             (uint8_t((*aBuffer)[1]) << 8 ) |
+                             (uint8_t((*aBuffer)[0])));
+    if (((*aBuffer)[2] & 0x80) == 0x80) {
+      result = (result | 0xff000000);
+    }
+
+    *aBuffer += 3 * sizeof(char);
+    return result;
+  }
+
   uint16_t
   ReadUint16LE(const char** aBuffer)
   {
@@ -116,11 +131,6 @@ WaveReader::~WaveReader()
   MOZ_COUNT_DTOR(WaveReader);
 }
 
-nsresult WaveReader::Init(MediaDecoderReader* aCloneDonor)
-{
-  return NS_OK;
-}
-
 nsresult WaveReader::ReadMetadata(MediaInfo* aInfo,
                                   MetadataTags** aTags)
 {
@@ -150,15 +160,9 @@ nsresult WaveReader::ReadMetadata(MediaInfo* aInfo,
   return NS_OK;
 }
 
-bool
-WaveReader::IsMediaSeekable()
-{
-  // not used
-  return true;
-}
-
 template <typename T> T UnsignedByteToAudioSample(uint8_t aValue);
 template <typename T> T SignedShortToAudioSample(int16_t aValue);
+template <typename T> T Signed24bIntToAudioSample(int32_t aValue);
 
 template <> inline float
 UnsignedByteToAudioSample<float>(uint8_t aValue)
@@ -182,6 +186,18 @@ SignedShortToAudioSample<int16_t>(int16_t aValue)
   return aValue;
 }
 
+template <> inline float
+Signed24bIntToAudioSample<float>(int32_t aValue)
+{
+  return aValue / 8388608.0f;
+}
+
+template <> inline int16_t
+Signed24bIntToAudioSample<int16_t>(int32_t aValue)
+{
+  return aValue / 256;
+}
+
 bool WaveReader::DecodeAudioData()
 {
   MOZ_ASSERT(OnTaskQueue());
@@ -191,21 +207,27 @@ bool WaveReader::DecodeAudioData()
   int64_t remaining = len - pos;
   NS_ASSERTION(remaining >= 0, "Current wave position is greater than wave file length");
 
-  static const int64_t BLOCK_SIZE = 4096;
+  static const int64_t BLOCK_SIZE = 6144;
   int64_t readSize = std::min(BLOCK_SIZE, remaining);
   int64_t frames = readSize / mFrameSize;
 
+  MOZ_ASSERT(BLOCK_SIZE % 3 == 0);
+  MOZ_ASSERT(BLOCK_SIZE % 2 == 0);
+
   static_assert(uint64_t(BLOCK_SIZE) < UINT_MAX /
                 sizeof(AudioDataValue) / MAX_CHANNELS,
                 "bufferSize calculation could overflow.");
   const size_t bufferSize = static_cast<size_t>(frames * mChannels);
-  nsAutoArrayPtr<AudioDataValue> sampleBuffer(new AudioDataValue[bufferSize]);
+  AlignedAudioBuffer sampleBuffer(bufferSize);
+  if (!sampleBuffer) {
+    return false;
+  }
 
   static_assert(uint64_t(BLOCK_SIZE) < UINT_MAX / sizeof(char),
                 "BLOCK_SIZE too large for enumerator.");
-  nsAutoArrayPtr<char> dataBuffer(new char[static_cast<size_t>(readSize)]);
+  auto dataBuffer = MakeUnique<char[]>(static_cast<size_t>(readSize));
 
-  if (!ReadAll(dataBuffer, readSize)) {
+  if (!ReadAll(dataBuffer.get(), readSize)) {
     return false;
   }
 
@@ -220,6 +242,9 @@ bool WaveReader::DecodeAudioData()
       } else if (mSampleFormat == FORMAT_S16) {
         int16_t v =  ReadInt16LE(&d);
         *s++ = SignedShortToAudioSample<AudioDataValue>(v);
+      } else if (mSampleFormat == FORMAT_S24) {
+        int32_t v =  ReadInt24LE(&d);
+        *s++ = Signed24bIntToAudioSample<AudioDataValue>(v);
       }
     }
   }
@@ -234,7 +259,7 @@ bool WaveReader::DecodeAudioData()
                                  static_cast<int64_t>(posTime * USECS_PER_S),
                                  static_cast<int64_t>(readSizeTime * USECS_PER_S),
                                  static_cast<int32_t>(frames),
-                                 sampleBuffer.forget(),
+                                 Move(sampleBuffer),
                                  mChannels,
                                  mSampleRate));
 
@@ -249,19 +274,19 @@ bool WaveReader::DecodeVideoFrame(bool &aKeyframeSkip,
   return false;
 }
 
-nsRefPtr<MediaDecoderReader::SeekPromise>
-WaveReader::Seek(int64_t aTarget, int64_t aEndTime)
+RefPtr<MediaDecoderReader::SeekPromise>
+WaveReader::Seek(SeekTarget aTarget, int64_t aEndTime)
 {
   MOZ_ASSERT(OnTaskQueue());
-  LOG(LogLevel::Debug, ("%p About to seek to %lld", mDecoder, aTarget));
+  LOG(LogLevel::Debug, ("%p About to seek to %lld", mDecoder, aTarget.GetTime().ToMicroseconds()));
 
   if (NS_FAILED(ResetDecode())) {
     return SeekPromise::CreateAndReject(NS_ERROR_FAILURE, __func__);
   }
   double d = BytesToTime(GetDataLength());
   NS_ASSERTION(d < INT64_MAX / USECS_PER_S, "Duration overflow");
-  int64_t duration = static_cast<int64_t>(d * USECS_PER_S);
-  double seekTime = std::min(aTarget, duration) / static_cast<double>(USECS_PER_S);
+  media::TimeUnit duration = media::TimeUnit::FromSeconds(d);
+  double seekTime = std::min(aTarget.GetTime(), duration).ToSeconds();
   int64_t position = RoundDownToFrame(static_cast<int64_t>(TimeToBytes(seekTime)));
   NS_ASSERTION(INT64_MAX - mWavePCMOffset > position, "Integer overflow during wave seek");
   position += mWavePCMOffset;
@@ -269,7 +294,7 @@ WaveReader::Seek(int64_t aTarget, int64_t aEndTime)
   if (NS_FAILED(res)) {
     return SeekPromise::CreateAndReject(res, __func__);
   } else {
-    return SeekPromise::CreateAndResolve(aTarget, __func__);
+    return SeekPromise::CreateAndResolve(aTarget.GetTime(), __func__);
   }
 }
 
@@ -302,6 +327,8 @@ media::TimeIntervals WaveReader::GetBuffered()
 bool
 WaveReader::ReadAll(char* aBuf, int64_t aSize, int64_t* aBytesRead)
 {
+  MOZ_ASSERT(OnTaskQueue());
+
   if (aBytesRead) {
     *aBytesRead = 0;
   }
@@ -322,6 +349,8 @@ WaveReader::ReadAll(char* aBuf, int64_t aSize, int64_t* aBytesRead)
 bool
 WaveReader::LoadRIFFChunk()
 {
+  MOZ_ASSERT(OnTaskQueue());
+
   char riffHeader[RIFF_INITIAL_SIZE];
   const char* p = riffHeader;
 
@@ -353,6 +382,8 @@ WaveReader::LoadRIFFChunk()
 bool
 WaveReader::LoadFormatChunk(uint32_t aChunkSize)
 {
+  MOZ_ASSERT(OnTaskQueue());
+
   uint32_t rate, channels, frameSize, sampleFormat;
   char waveFormat[WAVE_FORMAT_CHUNK_SIZE];
   const char* p = waveFormat;
@@ -393,29 +424,10 @@ WaveReader::LoadFormatChunk(uint32_t aChunkSize)
   // considering the file invalid.  This code skips any extension of the
   // "format" chunk.
   if (aChunkSize > WAVE_FORMAT_CHUNK_SIZE) {
-    char extLength[2];
-    const char* p = extLength;
-
-    if (!ReadAll(extLength, sizeof(extLength))) {
-      return false;
-    }
-
-    static_assert(sizeof(uint16_t) <= sizeof(extLength),
-                  "Reads would overflow extLength buffer.");
-    uint16_t extra = ReadUint16LE(&p);
-    if (aChunkSize - (WAVE_FORMAT_CHUNK_SIZE + 2) != extra) {
-      NS_WARNING("Invalid extended format chunk size");
-      return false;
-    }
+    uint16_t extra = aChunkSize - WAVE_FORMAT_CHUNK_SIZE;
     extra += extra % 2;
-
-    if (extra > 0) {
-      static_assert(UINT16_MAX + (UINT16_MAX % 2) < UINT_MAX / sizeof(char),
-                    "chunkExtension array too large for iterator.");
-      nsAutoArrayPtr<char> chunkExtension(new char[extra]);
-      if (!ReadAll(chunkExtension.get(), extra)) {
-        return false;
-      }
+    if (NS_FAILED(mResource.Seek(nsISeekableStream::NS_SEEK_CUR, extra))) {
+      return false;
     }
   }
 
@@ -427,23 +439,25 @@ WaveReader::LoadFormatChunk(uint32_t aChunkSize)
   // but the channels check is intentionally limited to mono or stereo
   // when the media is intended for direct playback because that's what the
   // audio backend currently supports.
-  unsigned int actualFrameSize = (sampleFormat == 8 ? 1 : 2) * channels;
+  unsigned int actualFrameSize = sampleFormat * channels / 8;
   if (rate < 100 || rate > 96000 ||
       (((channels < 1 || channels > MAX_CHANNELS) ||
-       (frameSize != 1 && frameSize != 2 && frameSize != 4)) &&
+       (frameSize != 1 && frameSize != 2 && frameSize != 3 &&
+        frameSize != 4 && frameSize != 6)) &&
        !mIgnoreAudioOutputFormat) ||
-       (sampleFormat != 8 && sampleFormat != 16) ||
+       (sampleFormat != 8 && sampleFormat != 16 && sampleFormat != 24) ||
       frameSize != actualFrameSize) {
     NS_WARNING("Invalid WAVE metadata");
     return false;
   }
 
-  ReentrantMonitorAutoEnter monitor(mDecoder->GetReentrantMonitor());
   mSampleRate = rate;
   mChannels = channels;
   mFrameSize = frameSize;
   if (sampleFormat == 8) {
     mSampleFormat = FORMAT_U8;
+  } else if (sampleFormat == 24) {
+    mSampleFormat = FORMAT_S24;
   } else {
     mSampleFormat = FORMAT_S16;
   }
@@ -453,6 +467,8 @@ WaveReader::LoadFormatChunk(uint32_t aChunkSize)
 bool
 WaveReader::FindDataOffset(uint32_t aChunkSize)
 {
+  MOZ_ASSERT(OnTaskQueue());
+
   // RIFF chunks are always word (two byte) aligned.
   MOZ_ASSERT(mResource.Tell() % 2 == 0,
              "FindDataOffset called with unaligned resource");
@@ -463,7 +479,6 @@ WaveReader::FindDataOffset(uint32_t aChunkSize)
     return false;
   }
 
-  ReentrantMonitorAutoEnter monitor(mDecoder->GetReentrantMonitor());
   mWaveLength = aChunkSize;
   mWavePCMOffset = uint32_t(offset);
   return true;
@@ -472,6 +487,7 @@ WaveReader::FindDataOffset(uint32_t aChunkSize)
 double
 WaveReader::BytesToTime(int64_t aBytes) const
 {
+  MOZ_ASSERT(OnTaskQueue());
   MOZ_ASSERT(aBytes >= 0, "Must be >= 0");
   return float(aBytes) / mSampleRate / mFrameSize;
 }
@@ -479,6 +495,7 @@ WaveReader::BytesToTime(int64_t aBytes) const
 int64_t
 WaveReader::TimeToBytes(double aTime) const
 {
+  MOZ_ASSERT(OnTaskQueue());
   MOZ_ASSERT(aTime >= 0.0f, "Must be >= 0");
   return RoundDownToFrame(int64_t(aTime * mSampleRate * mFrameSize));
 }
@@ -486,6 +503,7 @@ WaveReader::TimeToBytes(double aTime) const
 int64_t
 WaveReader::RoundDownToFrame(int64_t aBytes) const
 {
+  MOZ_ASSERT(OnTaskQueue());
   MOZ_ASSERT(aBytes >= 0, "Must be >= 0");
   return aBytes - (aBytes % mFrameSize);
 }
@@ -493,6 +511,8 @@ WaveReader::RoundDownToFrame(int64_t aBytes) const
 int64_t
 WaveReader::GetDataLength()
 {
+  MOZ_ASSERT(OnTaskQueue());
+
   int64_t length = mWaveLength;
   // If the decoder has a valid content length, and it's shorter than the
   // expected length of the PCM data, calculate the playback duration from
@@ -511,33 +531,12 @@ WaveReader::GetPosition()
   return mResource.Tell();
 }
 
-bool
-WaveReader::GetNextChunk(uint32_t* aChunk, uint32_t* aChunkSize)
-{
-  MOZ_ASSERT(aChunk, "Must have aChunk");
-  MOZ_ASSERT(aChunkSize, "Must have aChunkSize");
-  MOZ_ASSERT(mResource.Tell() % 2 == 0,
-             "GetNextChunk called with unaligned resource");
-
-  char chunkHeader[CHUNK_HEADER_SIZE];
-  const char* p = chunkHeader;
-
-  if (!ReadAll(chunkHeader, sizeof(chunkHeader))) {
-    return false;
-  }
-
-  static_assert(sizeof(uint32_t) * 2 <= CHUNK_HEADER_SIZE,
-                "Reads would overflow chunkHeader buffer.");
-  *aChunk = ReadUint32BE(&p);
-  *aChunkSize = ReadUint32LE(&p);
-
-  return true;
-}
-
 bool
 WaveReader::LoadListChunk(uint32_t aChunkSize,
                           nsAutoPtr<dom::HTMLMediaElement::MetadataTags> &aTags)
 {
+  MOZ_ASSERT(OnTaskQueue());
+
   // List chunks are always word (two byte) aligned.
   MOZ_ASSERT(mResource.Tell() % 2 == 0,
              "LoadListChunk called with unaligned resource");
@@ -550,7 +549,7 @@ WaveReader::LoadListChunk(uint32_t aChunkSize,
     return false;
   }
 
-  nsAutoArrayPtr<char> chunk(new char[aChunkSize]);
+  auto chunk = MakeUnique<char[]>(aChunkSize);
   if (!ReadAll(chunk.get(), aChunkSize)) {
     return false;
   }
@@ -614,6 +613,8 @@ WaveReader::LoadListChunk(uint32_t aChunkSize,
 bool
 WaveReader::LoadAllChunks(nsAutoPtr<dom::HTMLMediaElement::MetadataTags> &aTags)
 {
+  MOZ_ASSERT(OnTaskQueue());
+
   // Chunks are always word (two byte) aligned.
   MOZ_ASSERT(mResource.Tell() % 2 == 0,
              "LoadAllChunks called with unaligned resource");
@@ -672,7 +673,7 @@ WaveReader::LoadAllChunks(nsAutoPtr<dom::HTMLMediaElement::MetadataTags> &aTags)
     static const int64_t MAX_CHUNK_SIZE = 1 << 16;
     static_assert(uint64_t(MAX_CHUNK_SIZE) < UINT_MAX / sizeof(char),
                   "MAX_CHUNK_SIZE too large for enumerator.");
-    nsAutoArrayPtr<char> chunk(new char[MAX_CHUNK_SIZE]);
+    auto chunk = MakeUnique<char[]>(MAX_CHUNK_SIZE);
     while (forward.value() > 0) {
       int64_t size = std::min(forward.value(), MAX_CHUNK_SIZE);
       if (!ReadAll(chunk.get(), size)) {
