@@ -8,6 +8,7 @@
 
 const {classes: Cc, interfaces: Ci, utils: Cu, results: Cr} = Components;
 
+Cu.importGlobalProperties(['Blob', 'FileReader']);
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/PhoneNumberUtils.jsm");
@@ -16,7 +17,7 @@ Cu.import("resource://gre/modules/Promise.jsm");
 const GONK_MMSSERVICE_CONTRACTID = "@mozilla.org/mms/gonkmmsservice;1";
 const GONK_MMSSERVICE_CID = Components.ID("{9b069b8c-8697-11e4-a406-474f5190272b}");
 
-let DEBUG = false;
+var DEBUG = false;
 function debug(s) {
   dump("-@- MmsService: " + s + "\n");
 };
@@ -35,7 +36,6 @@ const kSmsDeletedObserverTopic           = "sms-deleted";
 const NS_XPCOM_SHUTDOWN_OBSERVER_ID      = "xpcom-shutdown";
 const kNetworkConnStateChangedTopic      = "network-connection-state-changed";
 
-const kPrefRilRadioDisabled              = "ril.radio.disabled";
 const kPrefMmsDebuggingEnabled           = "mms.debugging.enabled";
 
 // HTTP status codes:
@@ -193,18 +193,12 @@ function getDefaultServiceId() {
 }
 
 /**
- * Return Radio disabled state.
+ * Return radio disabled state.
  */
-function getRadioDisabledState() {
-  let state;
-  try {
-    state = Services.prefs.getBoolPref(kPrefRilRadioDisabled);
-  } catch (e) {
-    if (DEBUG) debug("Getting preference 'ril.radio.disabled' fails.");
-    state = false;
-  }
-
-  return state;
+function isRadioOff(aServiceId) {
+    let connection = gMobileConnectionService.getItemByServiceId(aServiceId);
+    return connection.radioState
+      !== Ci.nsIMobileConnection.MOBILE_RADIO_STATE_ENABLED;
 }
 
 /**
@@ -421,7 +415,7 @@ MmsConnection.prototype = {
       this.pendingCallbacks.push(callback);
 
       let errorStatus;
-      if (getRadioDisabledState()) {
+      if (isRadioOff(this.serviceId)) {
         if (DEBUG) debug("Error! Radio is disabled when sending MMS.");
         errorStatus = _HTTP_STATUS_RADIO_DISABLED;
       } else if (this.getCardState() != Ci.nsIIcc.CARD_STATE_READY) {
@@ -992,7 +986,8 @@ function CancellableTransaction(cancellableId, serviceId) {
   this.isCancelled = false;
 }
 CancellableTransaction.prototype = {
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver,
+                                         Ci.nsIMobileConnectionListener]),
 
   // The timer for retrying sending or retrieving process.
   timer: null,
@@ -1009,8 +1004,9 @@ CancellableTransaction.prototype = {
     if (!this.isObserversAdded) {
       Services.obs.addObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, false);
       Services.obs.addObserver(this, kSmsDeletedObserverTopic, false);
-      Services.prefs.addObserver(kPrefRilRadioDisabled, this, false);
       Services.prefs.addObserver(kPrefDefaultServiceId, this, false);
+      gMobileConnectionService
+        .getItemByServiceId(this.serviceId).registerListener(this);
       this.isObserversAdded = true;
     }
 
@@ -1022,8 +1018,9 @@ CancellableTransaction.prototype = {
     if (this.isObserversAdded) {
       Services.obs.removeObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
       Services.obs.removeObserver(this, kSmsDeletedObserverTopic);
-      Services.prefs.removeObserver(kPrefRilRadioDisabled, this);
       Services.prefs.removeObserver(kPrefDefaultServiceId, this);
+      gMobileConnectionService
+        .getItemByServiceId(this.serviceId).unregisterListener(this);
       this.isObserversAdded = false;
     }
   },
@@ -1079,18 +1076,35 @@ CancellableTransaction.prototype = {
         break;
       }
       case NS_PREFBRANCH_PREFCHANGE_TOPIC_ID: {
-        if (data == kPrefRilRadioDisabled) {
-          if (getRadioDisabledState()) {
-            this.cancelRunning(_MMS_ERROR_RADIO_DISABLED);
-          }
-        } else if (data === kPrefDefaultServiceId &&
+        if (data === kPrefDefaultServiceId &&
                    this.serviceId != getDefaultServiceId()) {
           this.cancelRunning(_MMS_ERROR_SIM_CARD_CHANGED);
         }
         break;
       }
     }
-  }
+  },
+
+  // nsIMobileConnectionListener
+
+  notifyVoiceChanged: function() {},
+  notifyDataChanged: function() {},
+  notifyDataError: function(message) {},
+  notifyCFStateChanged: function(action, reason, number, timeSeconds, serviceClass) {},
+  notifyEmergencyCbModeChanged: function(active, timeoutMs) {},
+  notifyOtaStatusChanged: function(status) {},
+
+  notifyRadioStateChanged: function() {
+    if (isRadioOff(this.serviceId)) {
+      this.cancelRunning(_MMS_ERROR_RADIO_DISABLED);
+    }
+  },
+
+  notifyClirModeChanged: function(mode) {},
+  notifyLastKnownNetworkChanged: function() {},
+  notifyLastKnownHomeNetworkChanged: function() {},
+  notifyNetworkSelectionModeChanged: function() {},
+  notifyDeviceIdentitiesChanged: function() {}
 };
 
 /**
@@ -1294,7 +1308,7 @@ SendTransaction.prototype = Object.create(CancellableTransaction.prototype, {
 
       let numPartsToLoad = parts.length;
       parts.forEach((aPart) => {
-        if (!(aPart.content instanceof Ci.nsIDOMBlob)) {
+        if (!(aPart.content instanceof Blob)) {
           numPartsToLoad--;
           if (!numPartsToLoad) {
             callbackIfValid();
@@ -1302,8 +1316,7 @@ SendTransaction.prototype = Object.create(CancellableTransaction.prototype, {
           return;
         }
 
-        let fileReader = Cc["@mozilla.org/files/filereader;1"]
-                         .createInstance(Ci.nsIDOMFileReader);
+        let fileReader = new FileReader();
         fileReader.addEventListener("loadend", (aEvent) => {
           let arrayBuffer = aEvent.target.result;
           aPart.content = new Uint8Array(arrayBuffer);
@@ -1647,7 +1660,7 @@ MmsService.prototype = {
 
     savable.receivers = [];
     // We don't have Bcc in recevied MMS message.
-    for each (let type in ["cc", "to"]) {
+    for (let type of ["cc", "to"]) {
       if (intermediate.headers[type]) {
         if (intermediate.headers[type] instanceof Array) {
           for (let index in intermediate.headers[type]) {
@@ -1683,7 +1696,7 @@ MmsService.prototype = {
    *        A callback function that takes two arguments: one for X-Mms-Status,
    *        the other parsed MMS message.
    * @param aDomMessage
-   *        The nsIDOMMozMmsMessage object.
+   *        The nsIMmsMessage object.
    */
   retrieveMessage: function(aMmsConnection, aContentLocation, aCallback,
                             aDomMessage) {
@@ -1703,7 +1716,7 @@ MmsService.prototype = {
    * @param aName
    *        The system message name.
    * @param aDomMessage
-   *        The nsIDOMMozMmsMessage object.
+   *        The nsIMmsMessage object.
    */
   broadcastMmsSystemMessage: function(aName, aDomMessage) {
     if (DEBUG) debug("Broadcasting the MMS system message: " + aName);
@@ -1742,7 +1755,7 @@ MmsService.prototype = {
    * an MMS is sent.
    *
    * @params aDomMessage
-   *         The nsIDOMMozMmsMessage object.
+   *         The nsIMmsMessage object.
    */
   broadcastSentMessageEvent: function(aDomMessage) {
     // Broadcasting a 'sms-sent' system message to open apps.
@@ -1757,7 +1770,7 @@ MmsService.prototype = {
    * an MMS is failed to send.
    *
    * @params aDomMessage
-   *         The nsIDOMMozMmsMessage object.
+   *         The nsIMmsMessage object.
    */
   broadcastSentFailureMessageEvent: function(aDomMessage) {
     // Broadcasting a 'sms-sent' system message to open apps.
@@ -1772,7 +1785,7 @@ MmsService.prototype = {
    * an MMS is received.
    *
    * @params aDomMessage
-   *         The nsIDOMMozMmsMessage object.
+   *         The nsIMmsMessage object.
    */
   broadcastReceivedMessageEvent: function(aDomMessage) {
     // Broadcasting a 'sms-received' system message to open apps.
@@ -1822,8 +1835,13 @@ MmsService.prototype = {
                                        null,
                                        DELIVERY_STATUS_ERROR,
                                        null,
-                                       (rv, domMessage) =>
-                                         this.broadcastReceivedMessageEvent(domMessage));
+                                       (aRv, aDomMessage) => {
+        let mmsMessage = null;
+        try {
+          mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+        } catch (e) {}
+        this.broadcastReceivedMessageEvent(mmsMessage);
+      });
       return;
     }
 
@@ -1831,8 +1849,13 @@ MmsService.prototype = {
                                                      retrievedMessage,
                                                      savableMessage);
     gMobileMessageDatabaseService.saveReceivedMessage(savableMessage,
-                                                      (rv, domMessage) => {
-      let success = Components.isSuccessCode(rv);
+                                                      (aRv, aDomMessage) => {
+      let mmsMessage = null;
+      try {
+        mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+      } catch (e) {}
+
+      let success = Components.isSuccessCode(aRv);
 
       // Cite 6.2.1 "Transaction Flow" in OMA-TS-MMS_ENC-V1_3-20110913-A:
       // The M-NotifyResp.ind response PDU SHALL provide a message retrieval
@@ -1851,11 +1874,11 @@ MmsService.prototype = {
         // At this point we could send a message to content to notify the user
         // that storing an incoming MMS failed, most likely due to a full disk.
         // The end user has to retrieve the MMS again.
-        if (DEBUG) debug("Could not store MMS , error code " + rv);
+        if (DEBUG) debug("Could not store MMS , error code " + aRv);
         return;
       }
 
-      this.broadcastReceivedMessageEvent(domMessage);
+      this.broadcastReceivedMessageEvent(mmsMessage);
     });
   },
 
@@ -1968,12 +1991,17 @@ MmsService.prototype = {
 
       gMobileMessageDatabaseService
         .saveReceivedMessage(savableMessage,
-                             (aRv, aDomMessage) =>
-                               this.saveReceivedMessageCallback(mmsConnection,
-                                                                retrievalMode,
-                                                                savableMessage,
-                                                                aRv,
-                                                                aDomMessage));
+                             (aRv, aDomMessage) => {
+          let mmsMessage = null;
+          try {
+            mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+          } catch (e) {}
+          this.saveReceivedMessageCallback(mmsConnection,
+                                           retrievalMode,
+                                           savableMessage,
+                                           aRv,
+                                           mmsMessage);
+      });
     });
   },
 
@@ -2024,24 +2052,30 @@ MmsService.prototype = {
       .setMessageDeliveryStatusByEnvelopeId(envelopeId, address, deliveryStatus,
                                             (aRv, aDomMessage) => {
       if (DEBUG) debug("Marking the delivery status is done.");
+
+      let mmsMessage = null;
+      try {
+        mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+      } catch (e) {}
+
       // TODO bug 832140 handle !Components.isSuccessCode(aRv)
 
       let topic;
       if (mmsStatus === MMS.MMS_PDU_STATUS_RETRIEVED) {
         topic = kSmsDeliverySuccessObserverTopic;
         // Broadcasting a 'sms-delivery-success' system message to open apps.
-        this.broadcastMmsSystemMessage(topic, aDomMessage);
+        this.broadcastMmsSystemMessage(topic, mmsMessage);
       } else if (mmsStatus === MMS.MMS_PDU_STATUS_REJECTED) {
         topic = kSmsDeliveryErrorObserverTopic;
         // Broadcasting a 'sms-delivery-error' system message to open apps.
-        this.broadcastMmsSystemMessage(topic, aDomMessage);
+        this.broadcastMmsSystemMessage(topic, mmsMessage);
       } else {
         if (DEBUG) debug("Needn't fire event for this MMS status. Returning.");
         return;
       }
 
       // Notifying observers the delivery status is updated.
-      Services.obs.notifyObservers(aDomMessage, topic, null);
+      Services.obs.notifyObservers(mmsMessage, topic, null);
     });
   },
 
@@ -2073,6 +2107,11 @@ MmsService.prototype = {
     gMobileMessageDatabaseService
       .setMessageReadStatusByEnvelopeId(envelopeId, address, readStatus,
                                         (aRv, aDomMessage) => {
+      let mmsMessage = null;
+      try {
+        mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+      } catch (e) {}
+
       if (!Components.isSuccessCode(aRv)) {
         if (DEBUG) debug("Failed to update read status: " + aRv);
         return;
@@ -2084,13 +2123,13 @@ MmsService.prototype = {
         topic = kSmsReadSuccessObserverTopic;
 
         // Broadcasting a 'sms-read-success' system message to open apps.
-        this.broadcastMmsSystemMessage(topic, aDomMessage);
+        this.broadcastMmsSystemMessage(topic, mmsMessage);
       } else {
         topic = kSmsReadErrorObserverTopic;
       }
 
       // Notifying observers the read status is updated.
-      Services.obs.notifyObservers(aDomMessage, topic, null);
+      Services.obs.notifyObservers(mmsMessage, topic, null);
     });
   },
 
@@ -2305,21 +2344,27 @@ MmsService.prototype = {
                                        aEnvelopeId,
                                        (aRv, aDomMessage) => {
         if (DEBUG) debug("Marking the delivery state/staus is done. Notify sent or failed.");
+
+        let mmsMessage = null;
+        try {
+          mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+        } catch (e) {}
+
         // TODO bug 832140 handle !Components.isSuccessCode(aRv)
         if (!isSentSuccess) {
           if (DEBUG) debug("Sending MMS failed.");
-          aRequest.notifySendMessageFailed(aErrorCode, aDomMessage);
-          this.broadcastSentFailureMessageEvent(aDomMessage);
+          aRequest.notifySendMessageFailed(aErrorCode, mmsMessage);
+          this.broadcastSentFailureMessageEvent(mmsMessage);
           return;
         }
 
         if (DEBUG) debug("Sending MMS succeeded.");
 
         // Notifying observers the MMS message is sent.
-        this.broadcastSentMessageEvent(aDomMessage);
+        this.broadcastSentMessageEvent(mmsMessage);
 
         // Return the request after sending the MMS message successfully.
-        aRequest.notifyMessageSent(aDomMessage);
+        aRequest.notifyMessageSent(mmsMessage);
       });
     };
 
@@ -2331,29 +2376,34 @@ MmsService.prototype = {
     gMobileMessageDatabaseService
       .saveSendingMessage(savableMessage,
                           (aRv, aDomMessage) => {
+      let mmsMessage = null;
+      try {
+        mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+      } catch (e) {}
+
       if (!Components.isSuccessCode(aRv)) {
         if (DEBUG) debug("Error! Fail to save sending message! rv = " + aRv);
         aRequest.notifySendMessageFailed(
           gMobileMessageDatabaseService.translateCrErrorToMessageCallbackError(aRv),
-          aDomMessage);
-        this.broadcastSentFailureMessageEvent(aDomMessage);
+          mmsMessage);
+        this.broadcastSentFailureMessageEvent(mmsMessage);
         return;
       }
 
       if (DEBUG) debug("Saving sending message is done. Start to send.");
 
-      Services.obs.notifyObservers(aDomMessage, kSmsSendingObserverTopic, null);
+      Services.obs.notifyObservers(mmsMessage, kSmsSendingObserverTopic, null);
 
       if (errorCode !== Ci.nsIMobileMessageCallback.SUCCESS_NO_ERROR) {
         if (DEBUG) debug("Error! The params for sending MMS are invalid.");
-        sendTransactionCb(aDomMessage, errorCode, null);
+        sendTransactionCb(mmsMessage, errorCode, null);
         return;
       }
 
       // Check radio state in prior to default service Id.
-      if (getRadioDisabledState()) {
+      if (isRadioOff(aServiceId)) {
         if (DEBUG) debug("Error! Radio is disabled when sending MMS.");
-        sendTransactionCb(aDomMessage,
+        sendTransactionCb(mmsMessage,
                           Ci.nsIMobileMessageCallback.RADIO_DISABLED_ERROR,
                           null);
         return;
@@ -2364,7 +2414,7 @@ MmsService.prototype = {
       // SIM. Users have to manually swith the default SIM before sending.
       if (mmsConnection.serviceId != this.mmsDefaultServiceId) {
         if (DEBUG) debug("RIL service is not active to send MMS.");
-        sendTransactionCb(aDomMessage,
+        sendTransactionCb(mmsMessage,
                           Ci.nsIMobileMessageCallback.NON_ACTIVE_SIM_CARD_ERROR,
                           null);
         return;
@@ -2374,11 +2424,11 @@ MmsService.prototype = {
       let sendTransaction;
       try {
         sendTransaction =
-          new SendTransaction(mmsConnection, aDomMessage.id, savableMessage,
+          new SendTransaction(mmsConnection, mmsMessage.id, savableMessage,
                               savableMessage["deliveryStatusRequested"]);
       } catch (e) {
         if (DEBUG) debug("Exception: fail to create a SendTransaction instance.");
-        sendTransactionCb(aDomMessage,
+        sendTransactionCb(mmsMessage,
                           Ci.nsIMobileMessageCallback.INTERNAL_ERROR, null);
         return;
       }
@@ -2400,7 +2450,7 @@ MmsService.prototype = {
         }
         let envelopeId =
           aMsg && aMsg.headers && aMsg.headers["message-id"] || null;
-        sendTransactionCb(aDomMessage, errorCode, envelopeId);
+        sendTransactionCb(mmsMessage, errorCode, envelopeId);
       });
     });
   },
@@ -2409,6 +2459,11 @@ MmsService.prototype = {
     if (DEBUG) debug("Retrieving message with ID " + aMessageId);
     gMobileMessageDatabaseService
       .getMessageRecordById(aMessageId, (aRv, aMessageRecord, aDomMessage) => {
+      let mmsMessage = null;
+      try {
+        mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+      } catch (e) {}
+
       if (!Components.isSuccessCode(aRv)) {
         if (DEBUG) debug("Function getMessageRecordById() return error: " + aRv);
         aRequest.notifyGetMessageFailed(
@@ -2460,7 +2515,13 @@ MmsService.prototype = {
       // Hence, for manual retrieving, instead of checking radio state later
       // in MmsConnection.acquire(), We have to check radio state in prior to
       // iccId to return the error correctly.
-      if (getRadioDisabledState()) {
+      let numRadioInterfaces = gMobileConnectionService.numItems;
+      let isAllRadioOff = true;
+      for (let serviceId = 0; serviceId < numRadioInterfaces; serviceId++) {
+        isAllRadioOff &= isRadioOff(serviceId);
+      }
+
+      if (isAllRadioOff) {
         if (DEBUG) debug("Error! Radio is disabled when retrieving MMS.");
         aRequest.notifyGetMessageFailed(
           Ci.nsIMobileMessageCallback.RADIO_DISABLED_ERROR);
@@ -2546,23 +2607,28 @@ MmsService.prototype = {
                                                          aMessageRecord);
 
         gMobileMessageDatabaseService.saveReceivedMessage(aMessageRecord,
-                                                          (rv, domMessage) => {
-          let success = Components.isSuccessCode(rv);
+                                                          (aRv, aDomMessage) => {
+          let mmsMessage = null;
+          try {
+            mmsMessage = aDomMessage.QueryInterface(Ci.nsIMmsMessage);
+          } catch (e) {}
+
+          let success = Components.isSuccessCode(aRv);
           if (!success) {
             // At this point we could send a message to content to
             // notify the user that storing an incoming MMS failed, most
             // likely due to a full disk.
-            if (DEBUG) debug("Could not store MMS, error code " + rv);
+            if (DEBUG) debug("Could not store MMS, error code " + aRv);
             aRequest.notifyGetMessageFailed(
-              gMobileMessageDatabaseService.translateCrErrorToMessageCallbackError(rv));
+              gMobileMessageDatabaseService.translateCrErrorToMessageCallbackError(aRv));
             return;
           }
 
           // Notifying observers a new MMS message is retrieved.
-          this.broadcastReceivedMessageEvent(domMessage);
+          this.broadcastReceivedMessageEvent(mmsMessage);
 
           // Return the request after retrieving the MMS message successfully.
-          aRequest.notifyMessageGot(domMessage);
+          aRequest.notifyMessageGot(mmsMessage);
 
           // Cite 6.3.1 "Transaction Flow" in OMA-TS-MMS_ENC-V1_3-20110913-A:
           // If an acknowledgement is requested, the MMS Client SHALL respond
@@ -2596,7 +2662,7 @@ MmsService.prototype = {
                                url,
                                (aMmsStatus, aRetrievedMsg) =>
                                  responseNotify(aMmsStatus, aRetrievedMsg),
-                               aDomMessage);
+                               mmsMessage);
         });
     });
   },
