@@ -8,44 +8,42 @@
 #define mozilla_dom_Presentation_h
 
 #include "mozilla/DOMEventTargetHelper.h"
-#include "nsIPresentationListener.h"
 
 namespace mozilla {
 namespace dom {
 
 class Promise;
-class PresentationSession;
+class PresentationReceiver;
+class PresentationRequest;
 
 class Presentation final : public DOMEventTargetHelper
-                         , public nsIPresentationListener
 {
 public:
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(Presentation,
                                            DOMEventTargetHelper)
-  NS_DECL_NSIPRESENTATIONLISTENER
 
-  static already_AddRefed<Presentation> Create(nsPIDOMWindow* aWindow);
-  virtual JSObject*
-    WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto) override;
+  static already_AddRefed<Presentation> Create(nsPIDOMWindowInner* aWindow);
+
+  virtual JSObject* WrapObject(JSContext* aCx,
+                               JS::Handle<JSObject*> aGivenProto) override;
 
   // WebIDL (public APIs)
-  already_AddRefed<Promise> StartSession(const nsAString& aUrl,
-                                         const Optional<nsAString>& aId,
-                                         ErrorResult& aRv);
-  already_AddRefed<PresentationSession> GetSession() const;
-  bool CachedAvailable() const;
-  IMPL_EVENT_HANDLER(availablechange);
+  void SetDefaultRequest(PresentationRequest* aRequest);
+
+  already_AddRefed<PresentationRequest> GetDefaultRequest() const;
+
+  already_AddRefed<PresentationReceiver> GetReceiver();
 
 private:
-  explicit Presentation(nsPIDOMWindow* aWindow);
+  explicit Presentation(nsPIDOMWindowInner* aWindow);
+
   ~Presentation();
 
-  bool Init();
-  void Shutdown();
+  bool IsInPresentedContent() const;
 
-  bool mAvailable;
-  nsRefPtr<PresentationSession> mSession;
+  RefPtr<PresentationRequest> mDefaultRequest;
+  RefPtr<PresentationReceiver> mReceiver;
 };
 
 } // namespace dom
