@@ -97,8 +97,8 @@ GetStrokeDashData(SVGContentUtils::AutoStrokeOptions* aStrokeOptions,
       (i % 2 ? totalLengthOfGaps : totalLengthOfDashes) += dashSrc[i];
     }
   } else {
-    const nsStyleCoord *dasharray = aStyleSVG->mStrokeDasharray;
-    dashArrayLength = aStyleSVG->mStrokeDasharrayLength;
+    const nsTArray<nsStyleCoord>& dasharray = aStyleSVG->mStrokeDasharray;
+    dashArrayLength = aStyleSVG->mStrokeDasharray.Length();
     if (dashArrayLength <= 0) {
       return eContinuousStroke;
     }
@@ -173,7 +173,7 @@ SVGContentUtils::GetStrokeOptions(AutoStrokeOptions* aStrokeOptions,
                                   gfxTextContextPaint *aContextPaint,
                                   StrokeOptionFlags aFlags)
 {
-  nsRefPtr<nsStyleContext> styleContext;
+  RefPtr<nsStyleContext> styleContext;
   if (aStyleContext) {
     styleContext = aStyleContext;
   } else {
@@ -188,6 +188,7 @@ SVGContentUtils::GetStrokeOptions(AutoStrokeOptions* aStrokeOptions,
 
   const nsStyleSVG* styleSVG = styleContext->StyleSVG();
 
+  bool checkedDashAndStrokeIsDashed = false;
   if (aFlags != eIgnoreStrokeDashing) {
     DashState dashState =
       GetStrokeDashData(aStrokeOptions, aElement, styleSVG, aContextPaint);
@@ -201,6 +202,7 @@ SVGContentUtils::GetStrokeOptions(AutoStrokeOptions* aStrokeOptions,
       // Prevent our caller from wasting time looking at a pattern without gaps:
       aStrokeOptions->DiscardDashPattern();
     }
+    checkedDashAndStrokeIsDashed = (dashState == eDashedStroke);
   }
 
   aStrokeOptions->mLineWidth =
@@ -220,10 +222,12 @@ SVGContentUtils::GetStrokeOptions(AutoStrokeOptions* aStrokeOptions,
     break;
   }
 
-  if (ShapeTypeHasNoCorners(aElement)) {
+  if (ShapeTypeHasNoCorners(aElement) && !checkedDashAndStrokeIsDashed) {
+    // Note: if aFlags == eIgnoreStrokeDashing then we may be returning the
+    // wrong linecap value here, since the actual linecap used on render in this
+    // case depends on whether the stroke is dashed or not.
     aStrokeOptions->mLineCap = CapStyle::BUTT;
-  }
-  else {
+  } else {
     switch (styleSVG->mStrokeLinecap) {
       case NS_STYLE_STROKE_LINECAP_BUTT:
         aStrokeOptions->mLineCap = CapStyle::BUTT;
@@ -243,7 +247,7 @@ SVGContentUtils::GetStrokeWidth(nsSVGElement* aElement,
                                 nsStyleContext* aStyleContext,
                                 gfxTextContextPaint *aContextPaint)
 {
-  nsRefPtr<nsStyleContext> styleContext;
+  RefPtr<nsStyleContext> styleContext;
   if (aStyleContext) {
     styleContext = aStyleContext;
   } else {
@@ -271,7 +275,7 @@ SVGContentUtils::GetFontSize(Element *aElement)
   if (!aElement)
     return 1.0f;
 
-  nsRefPtr<nsStyleContext> styleContext = 
+  RefPtr<nsStyleContext> styleContext = 
     nsComputedDOMStyle::GetStyleContextForElementNoFlush(aElement,
                                                          nullptr, nullptr);
   if (!styleContext) {
@@ -309,7 +313,7 @@ SVGContentUtils::GetFontXHeight(Element *aElement)
   if (!aElement)
     return 1.0f;
 
-  nsRefPtr<nsStyleContext> styleContext = 
+  RefPtr<nsStyleContext> styleContext = 
     nsComputedDOMStyle::GetStyleContextForElementNoFlush(aElement,
                                                          nullptr, nullptr);
   if (!styleContext) {
@@ -336,9 +340,8 @@ SVGContentUtils::GetFontXHeight(nsStyleContext *aStyleContext)
   nsPresContext *presContext = aStyleContext->PresContext();
   MOZ_ASSERT(presContext, "NULL pres context in GetFontXHeight");
 
-  nsRefPtr<nsFontMetrics> fontMetrics;
-  nsLayoutUtils::GetFontMetricsForStyleContext(aStyleContext,
-                                               getter_AddRefs(fontMetrics));
+  RefPtr<nsFontMetrics> fontMetrics =
+    nsLayoutUtils::GetFontMetricsForStyleContext(aStyleContext);
 
   if (!fontMetrics) {
     // ReportToConsole
@@ -395,7 +398,7 @@ static gfx::Matrix
 GetCTMInternal(nsSVGElement *aElement, bool aScreenCTM, bool aHaveRecursed)
 {
   gfxMatrix matrix = aElement->PrependLocalTransformsTo(gfxMatrix(),
-    aHaveRecursed ? nsSVGElement::eAllTransforms : nsSVGElement::eUserSpaceToParent);
+    aHaveRecursed ? eAllTransforms : eUserSpaceToParent);
   nsSVGElement *element = aElement;
   nsIContent *ancestor = aElement->GetFlattenedTreeParent();
 
@@ -464,6 +467,45 @@ SVGContentUtils::GetCTM(nsSVGElement *aElement, bool aScreenCTM)
   return GetCTMInternal(aElement, aScreenCTM, false);
 }
 
+void
+SVGContentUtils::RectilinearGetStrokeBounds(const Rect& aRect,
+                                            const Matrix& aToBoundsSpace,
+                                            const Matrix& aToNonScalingStrokeSpace,
+                                            float aStrokeWidth,
+                                            Rect* aBounds)
+{
+  MOZ_ASSERT(aToBoundsSpace.IsRectilinear(),
+             "aToBoundsSpace must be rectilinear");
+  MOZ_ASSERT(aToNonScalingStrokeSpace.IsRectilinear(),
+             "aToNonScalingStrokeSpace must be rectilinear");
+
+  Matrix nonScalingToSource = aToNonScalingStrokeSpace.Inverse();
+  Matrix nonScalingToBounds = nonScalingToSource * aToBoundsSpace;
+
+  *aBounds = aToBoundsSpace.TransformBounds(aRect);
+
+  // Compute the amounts dx and dy that nonScalingToBounds scales a half-width
+  // stroke in the x and y directions, and then inflate aBounds by those amounts
+  // so that when aBounds is transformed back to non-scaling-stroke space
+  // it will map onto the correct stroked bounds.
+
+  Float dx = 0.0f;
+  Float dy = 0.0f;
+  // nonScalingToBounds is rectilinear, so either _12 and _21 are zero or _11
+  // and _22 are zero, and in each case the non-zero entries (from among _11,
+  // _12, _21, _22) simply scale the stroke width in the x and y directions.
+  if (FuzzyEqual(nonScalingToBounds._12, 0) &&
+      FuzzyEqual(nonScalingToBounds._21, 0)) {
+    dx = (aStrokeWidth / 2.0f) * std::abs(nonScalingToBounds._11);
+    dy = (aStrokeWidth / 2.0f) * std::abs(nonScalingToBounds._22);
+  } else {
+    dx = (aStrokeWidth / 2.0f) * std::abs(nonScalingToBounds._21);
+    dy = (aStrokeWidth / 2.0f) * std::abs(nonScalingToBounds._12);
+  }
+
+  aBounds->Inflate(dx, dy);
+}
+
 double
 SVGContentUtils::ComputeNormalizedHypotenuse(double aWidth, double aHeight)
 {
@@ -819,3 +861,38 @@ SVGContentUtils::ShapeTypeHasNoCorners(const nsIContent* aContent) {
   return aContent && aContent->IsAnyOfSVGElements(nsGkAtoms::circle,
                                                   nsGkAtoms::ellipse);
 }
+
+gfxMatrix
+SVGContentUtils::PrependLocalTransformsTo(
+  const gfxMatrix &aMatrix,
+  SVGTransformTypes aWhich,
+  const gfx::Matrix* aAnimateMotionTransform,
+  const nsSVGAnimatedTransformList* aTransforms)
+{
+  gfxMatrix result(aMatrix);
+
+  if (aWhich == eChildToUserSpace) {
+    // We don't have anything to prepend.
+    // eChildToUserSpace is not the common case, which is why we return
+    // 'result' to benefit from NRVO rather than returning aMatrix before
+    // creating 'result'.
+    return result;
+  }
+
+  MOZ_ASSERT(aWhich == eAllTransforms || aWhich == eUserSpaceToParent,
+             "Unknown TransformTypes");
+
+  // animateMotion's resulting transform is supposed to apply *on top of*
+  // any transformations from the |transform| attribute. So since we're
+  // PRE-multiplying, we need to apply the animateMotion transform *first*.
+  if (aAnimateMotionTransform) {
+    result.PreMultiply(ThebesMatrix(*aAnimateMotionTransform));
+  }
+
+  if (aTransforms) {
+    result.PreMultiply(
+      aTransforms->GetAnimValue().GetConsolidationMatrix());
+  }
+
+  return result;
+}
