@@ -32,7 +32,7 @@ XPCOMUtils.defineLazyGetter(this, "RIL", function () {
 
 // Ril quirk to always turn the radio off for the client without SIM card
 // except hw default client.
-let RILQUIRKS_RADIO_OFF_WO_CARD =
+var RILQUIRKS_RADIO_OFF_WO_CARD =
   libcutils.property_get("ro.moz.ril.radio_off_wo_card", "false") == "true";
 
 const RADIOINTERFACELAYER_CID =
@@ -392,7 +392,9 @@ DataCall.prototype = {
   ifname: null,
   addreses: null,
   dnses: null,
-  gateways: null
+  gateways: null,
+  pcscf: null,
+  mtu: -1
 };
 
 function RadioInterfaceLayer() {
@@ -453,18 +455,6 @@ RadioInterfaceLayer.prototype = {
     return this.radioInterfaces[clientId];
   },
 
-  getClientIdForEmergencyCall: function() {
-    // Select the client with sim card first.
-    for (let cid = 0; cid < this.numRadioInterfaces; ++cid) {
-      if (this.getRadioInterface(cid).isCardPresent()) {
-        return cid;
-      }
-    }
-
-    // Use the defualt client if no card presents.
-    return HW_DEFAULT_CLIENT_ID;
-  },
-
   setMicrophoneMuted: function(muted) {
     for (let clientId = 0; clientId < this.numRadioInterfaces; clientId++) {
       let radioInterface = this.radioInterfaces[clientId];
@@ -766,6 +756,13 @@ RadioInterface.prototype = {
       case "otastatuschange":
         gMobileConnectionService.notifyOtaStatusChanged(this.clientId, message.status);
         break;
+      case "deviceidentitieschange":
+        gMobileConnectionService.notifyDeviceIdentitiesChanged(this.clientId,
+                                                               message.deviceIdentities.imei,
+                                                               message.deviceIdentities.imeisv,
+                                                               message.deviceIdentities.esn,
+                                                               message.deviceIdentities.meid);
+        break;
       case "radiostatechange":
         // gRadioEnabledController should know the radio state for each client,
         // so notify gRadioEnabledController here.
