@@ -11,6 +11,7 @@
 #include "mozilla/dom/MozMobileConnectionBinding.h"
 #include "mozilla/dom/Promise.h"
 #include "mozilla/dom/TelephonyBinding.h"
+#include "mozilla/unused.h"
 
 #include "nsCharSeparatedTokenizer.h"
 #include "nsContentUtils.h"
@@ -61,14 +62,18 @@ public:
   }
 };
 
-Telephony::Telephony(nsPIDOMWindow* aOwner)
-  : DOMEventTargetHelper(aOwner)
+Telephony::Telephony(nsPIDOMWindowInner* aOwner)
+  : DOMEventTargetHelper(aOwner),
+    mIsAudioStartPlaying(false),
+    mHaveDispatchedInterruptBeginEvent(false),
+    mMuted(AudioChannelService::IsAudioChannelMutedByDefault())
 {
+  MOZ_ASSERT(aOwner);
   nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(aOwner);
   MOZ_ASSERT(global);
 
   ErrorResult rv;
-  nsRefPtr<Promise> promise = Promise::Create(global, rv);
+  RefPtr<Promise> promise = Promise::Create(global, rv);
   MOZ_ASSERT(!rv.Failed());
 
   mReadyPromise = promise;
@@ -102,7 +107,7 @@ Telephony::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
 
 // static
 already_AddRefed<Telephony>
-Telephony::Create(nsPIDOMWindow* aOwner, ErrorResult& aRv)
+Telephony::Create(nsPIDOMWindowInner* aOwner, ErrorResult& aRv)
 {
   NS_ASSERTION(aOwner, "Null owner!");
 
@@ -125,7 +130,7 @@ Telephony::Create(nsPIDOMWindow* aOwner, ErrorResult& aRv)
     return nullptr;
   }
 
-  nsRefPtr<Telephony> telephony = new Telephony(aOwner);
+  RefPtr<Telephony> telephony = new Telephony(aOwner);
 
   telephony->mService = ril;
   telephony->mListener = new Listener(telephony);
@@ -161,14 +166,6 @@ Telephony::IsValidServiceId(uint32_t aServiceId)
   return aServiceId < GetNumServices();
 }
 
-// static
-bool
-Telephony::IsActiveState(uint16_t aCallState) {
-  return aCallState == nsITelephonyService::CALL_STATE_DIALING ||
-      aCallState == nsITelephonyService::CALL_STATE_ALERTING ||
-      aCallState == nsITelephonyService::CALL_STATE_CONNECTED;
-}
-
 uint32_t
 Telephony::GetServiceId(const Optional<uint32_t>& aServiceId,
                         bool aGetIfActiveCall)
@@ -176,12 +173,12 @@ Telephony::GetServiceId(const Optional<uint32_t>& aServiceId,
   if (aServiceId.WasPassed()) {
     return aServiceId.Value();
   } else if (aGetIfActiveCall) {
-    nsTArray<nsRefPtr<TelephonyCall> > &calls = mCalls;
-    if (mGroup->CallState() == nsITelephonyService::CALL_STATE_CONNECTED) {
+    nsTArray<RefPtr<TelephonyCall> > &calls = mCalls;
+    if (mGroup->IsActive()) {
       calls = mGroup->CallsArray();
     }
     for (uint32_t i = 0; i < calls.Length(); i++) {
-      if (IsActiveState(calls[i]->CallState())) {
+      if (calls[i]->IsActive()) {
         return calls[i]->mServiceId;
       }
     }
@@ -201,7 +198,7 @@ Telephony::DialInternal(uint32_t aServiceId, const nsAString& aNumber,
     return nullptr;
   }
 
-  nsRefPtr<Promise> promise = Promise::Create(global, aRv);
+  RefPtr<Promise> promise = Promise::Create(global, aRv);
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -243,7 +240,7 @@ already_AddRefed<TelephonyCallId>
 Telephony::CreateCallId(const nsAString& aNumber, uint16_t aNumberPresentation,
                         const nsAString& aName, uint16_t aNamePresentation)
 {
-  nsRefPtr<TelephonyCallId> id =
+  RefPtr<TelephonyCallId> id =
     new TelephonyCallId(GetOwner(), aNumber, aNumberPresentation,
                         aName, aNamePresentation);
 
@@ -252,17 +249,17 @@ Telephony::CreateCallId(const nsAString& aNumber, uint16_t aNumberPresentation,
 
 already_AddRefed<TelephonyCall>
 Telephony::CreateCall(TelephonyCallId* aId, uint32_t aServiceId,
-                      uint32_t aCallIndex, uint16_t aCallState,
+                      uint32_t aCallIndex, TelephonyCallState aState,
                       bool aEmergency, bool aConference,
                       bool aSwitchable, bool aMergeable)
 {
   // We don't have to create an already ended call.
-  if (aCallState == nsITelephonyService::CALL_STATE_DISCONNECTED) {
+  if (aState == TelephonyCallState::Disconnected) {
     return nullptr;
   }
 
-  nsRefPtr<TelephonyCall> call =
-    TelephonyCall::Create(this, aId, aServiceId, aCallIndex, aCallState,
+  RefPtr<TelephonyCall> call =
+    TelephonyCall::Create(this, aId, aServiceId, aCallIndex, aState,
                           aEmergency, aConference, aSwitchable, aMergeable);
 
   NS_ASSERTION(call, "This should never fail!");
@@ -288,10 +285,10 @@ Telephony::NotifyCallsChanged(TelephonyCall* aCall)
 already_AddRefed<TelephonyCall>
 Telephony::GetCall(uint32_t aServiceId, uint32_t aCallIndex)
 {
-  nsRefPtr<TelephonyCall> call;
+  RefPtr<TelephonyCall> call;
 
   for (uint32_t i = 0; i < mCalls.Length(); i++) {
-    nsRefPtr<TelephonyCall>& tempCall = mCalls[i];
+    RefPtr<TelephonyCall>& tempCall = mCalls[i];
     if (tempCall->ServiceId() == aServiceId &&
         tempCall->CallIndex() == aCallIndex) {
       call = tempCall;
@@ -305,7 +302,7 @@ Telephony::GetCall(uint32_t aServiceId, uint32_t aCallIndex)
 already_AddRefed<TelephonyCall>
 Telephony::GetCallFromEverywhere(uint32_t aServiceId, uint32_t aCallIndex)
 {
-  nsRefPtr<TelephonyCall> call = GetCall(aServiceId, aCallIndex);
+  RefPtr<TelephonyCall> call = GetCall(aServiceId, aCallIndex);
 
   if (!call) {
     call = mGroup->GetCall(aServiceId, aCallIndex);
@@ -333,61 +330,66 @@ Telephony::HandleCallInfo(nsITelephonyCallInfo* aInfo)
   aInfo->GetIsSwitchable(&isSwitchable);
   aInfo->GetIsMergeable(&isMergeable);
 
-  nsRefPtr<TelephonyCall> call = GetCallFromEverywhere(serviceId, callIndex);
+  TelephonyCallState state = TelephonyCall::ConvertToTelephonyCallState(callState);
 
+  RefPtr<TelephonyCall> call = GetCallFromEverywhere(serviceId, callIndex);
+  // Handle a newly created call.
   if (!call) {
-    nsRefPtr<TelephonyCallId> id = CreateCallId(aInfo);
-    call = CreateCall(id, serviceId, callIndex, callState, isEmergency,
+    RefPtr<TelephonyCallId> id = CreateCallId(aInfo);
+    call = CreateCall(id, serviceId, callIndex, state, isEmergency,
                       isConference, isSwitchable, isMergeable);
-
-    if (call && callState == nsITelephonyService::CALL_STATE_INCOMING) {
+    // The newly created call is an incoming call.
+    if (call &&
+        state == TelephonyCallState::Incoming) {
       nsresult rv = DispatchCallEvent(NS_LITERAL_STRING("incoming"), call);
       NS_ENSURE_SUCCESS(rv, rv);
     }
-  } else {
-    call->UpdateEmergency(isEmergency);
-    call->UpdateSwitchable(isSwitchable);
-    call->UpdateMergeable(isMergeable);
+    return NS_OK;
+  }
 
-    nsAutoString number;
-    aInfo->GetNumber(number);
-    nsRefPtr<TelephonyCallId> id = call->Id();
-    id->UpdateNumber(number);
+  // Update an existing call
+  call->UpdateEmergency(isEmergency);
+  call->UpdateSwitchable(isSwitchable);
+  call->UpdateMergeable(isMergeable);
 
-    nsAutoString disconnectedReason;
-    aInfo->GetDisconnectedReason(disconnectedReason);
+  nsAutoString number;
+  aInfo->GetNumber(number);
+  RefPtr<TelephonyCallId> id = call->Id();
+  id->UpdateNumber(number);
 
-    // State changed.
-    if (call->CallState() != callState) {
-      if (callState == nsITelephonyService::CALL_STATE_DISCONNECTED) {
-        call->UpdateDisconnectedReason(disconnectedReason);
-        call->ChangeState(nsITelephonyService::CALL_STATE_DISCONNECTED);
-        return NS_OK;
-      }
+  nsAutoString disconnectedReason;
+  aInfo->GetDisconnectedReason(disconnectedReason);
 
-      // We don't fire the statechange event on a call in conference here.
-      // Instead, the event will be fired later in
-      // TelephonyCallGroup::ChangeState(). Thus the sequence of firing the
-      // statechange events is guaranteed: first on TelephonyCallGroup then on
-      // individual TelephonyCall objects.
-      bool fireEvent = !isConference;
-      call->ChangeStateInternal(callState, fireEvent);
+  // State changed.
+  if (call->State() != state) {
+    if (state == TelephonyCallState::Disconnected) {
+      call->UpdateDisconnectedReason(disconnectedReason);
+      call->ChangeState(TelephonyCallState::Disconnected);
+      return NS_OK;
     }
 
-    // Group changed.
-    nsRefPtr<TelephonyCallGroup> group = call->GetGroup();
+    // We don't fire the statechange event on a call in conference here.
+    // Instead, the event will be fired later in
+    // TelephonyCallGroup::ChangeState(). Thus the sequence of firing the
+    // statechange events is guaranteed: first on TelephonyCallGroup then on
+    // individual TelephonyCall objects.
+    bool fireEvent = !isConference;
+    call->ChangeStateInternal(state, fireEvent);
+  }
 
-    if (!group && isConference) {
-      // Add to conference.
-      NS_ASSERTION(mCalls.Contains(call), "Should in mCalls");
-      mGroup->AddCall(call);
-      RemoveCall(call);
-    } else if (group && !isConference) {
-      // Remove from conference.
-      NS_ASSERTION(mGroup->CallsArray().Contains(call), "Should in mGroup");
-      mGroup->RemoveCall(call);
-      AddCall(call);
-    }
+  // Group changed.
+  RefPtr<TelephonyCallGroup> group = call->GetGroup();
+
+  if (!group && isConference) {
+    // Add to conference.
+    NS_ASSERTION(mCalls.Contains(call), "Should in mCalls");
+    mGroup->AddCall(call);
+    RemoveCall(call);
+  } else if (group && !isConference) {
+    // Remove from conference.
+    NS_ASSERTION(mGroup->CallsArray().Contains(call), "Should in mGroup");
+    mGroup->RemoveCall(call);
+    AddCall(call);
   }
 
   return NS_OK;
@@ -429,7 +431,7 @@ Telephony::Dial(const nsAString& aNumber, const Optional<uint32_t>& aServiceId,
                 ErrorResult& aRv)
 {
   uint32_t serviceId = GetServiceId(aServiceId);
-  nsRefPtr<Promise> promise = DialInternal(serviceId, aNumber, false, aRv);
+  RefPtr<Promise> promise = DialInternal(serviceId, aNumber, false, aRv);
   return promise.forget();
 }
 
@@ -439,7 +441,7 @@ Telephony::DialEmergency(const nsAString& aNumber,
                          ErrorResult& aRv)
 {
   uint32_t serviceId = GetServiceId(aServiceId);
-  nsRefPtr<Promise> promise = DialInternal(serviceId, aNumber, true, aRv);
+  RefPtr<Promise> promise = DialInternal(serviceId, aNumber, true, aRv);
   return promise.forget();
 }
 
@@ -458,7 +460,7 @@ Telephony::SendTones(const nsAString& aDTMFChars,
     return nullptr;
   }
 
-  nsRefPtr<Promise> promise = Promise::Create(global, aRv);
+  RefPtr<Promise> promise = Promise::Create(global, aRv);
   if (aRv.Failed()) {
     return nullptr;
   }
@@ -518,6 +520,76 @@ Telephony::StopTone(const Optional<uint32_t>& aServiceId, ErrorResult& aRv)
   aRv = mService->StopTone(serviceId);
 }
 
+void
+Telephony::OwnAudioChannel(ErrorResult& aRv)
+{
+  if (mAudioAgent) {
+    return;
+  }
+
+  mAudioAgent = do_CreateInstance("@mozilla.org/audiochannelagent;1");
+  MOZ_ASSERT(mAudioAgent);
+  aRv = mAudioAgent->Init(GetParentObject(),
+                         (int32_t)AudioChannel::Telephony, this);
+  if (NS_WARN_IF(aRv.Failed())) {
+    return;
+  }
+  aRv = HandleAudioAgentState();
+  if (NS_WARN_IF(aRv.Failed())) {
+    return;
+  }
+}
+
+nsresult
+Telephony::HandleAudioAgentState()
+{
+  if (!mAudioAgent) {
+    return NS_OK;
+  }
+
+  Nullable<OwningTelephonyCallOrTelephonyCallGroup> activeCall;
+  GetActive(activeCall);
+  nsresult rv;
+  // Only stop the agent when there's no call.
+  if ((!mCalls.Length() && !mGroup->CallsArray().Length()) &&
+       mIsAudioStartPlaying) {
+    mIsAudioStartPlaying = false;
+    rv = mAudioAgent->NotifyStoppedPlaying();
+    mAudioAgent = nullptr;
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+  } else if (!activeCall.IsNull() && !mIsAudioStartPlaying) {
+    mIsAudioStartPlaying = true;
+    AudioPlaybackConfig config;
+    rv = mAudioAgent->NotifyStartedPlaying(&config,
+                                           AudioChannelService::AudibleState::eAudible);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+
+    // In B2G, the system app manages audio playback policy. If there is a new
+    // sound want to be playback, it must wait for the permission from the
+    // system app. It means that the sound would be muted first, and then be
+    // unmuted. For telephony, the behaviors are hold() first, then resume().
+    // However, the telephony service can't handle all these requests within a
+    // short period. The telephony service would reject our resume request,
+    // because the modem have not changed the call state yet. It causes that
+    // the telephony can't be resumed. Therefore, we don't mute the telephony
+    // at the beginning.
+    rv = WindowVolumeChanged(1.0, false);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+
+    rv = WindowSuspendChanged(config.mSuspend);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+  }
+  return NS_OK;
+}
+
 bool
 Telephony::GetMuted(ErrorResult& aRv) const
 {
@@ -551,31 +623,34 @@ Telephony::SetSpeakerEnabled(bool aEnabled, ErrorResult& aRv)
 void
 Telephony::GetActive(Nullable<OwningTelephonyCallOrTelephonyCallGroup>& aValue)
 {
-  if (mGroup->CallState() == nsITelephonyService::CALL_STATE_CONNECTED) {
+  if (mGroup->IsActive()) {
     aValue.SetValue().SetAsTelephonyCallGroup() = mGroup;
-  } else {
-    // Search the first active call.
-    for (uint32_t i = 0; i < mCalls.Length(); i++) {
-      if (IsActiveState(mCalls[i]->CallState())) {
-        aValue.SetValue().SetAsTelephonyCall() = mCalls[i];
-        return;
-      }
-    }
-    aValue.SetNull();
+    return;
   }
+
+  // Search for the active call.
+  for (uint32_t i = 0; i < mCalls.Length(); i++) {
+    if (mCalls[i]->IsActive()) {
+      aValue.SetValue().SetAsTelephonyCall() = mCalls[i];
+      return;
+    }
+  }
+
+  // Nothing active found.
+  aValue.SetNull();
 }
 
 already_AddRefed<CallsList>
 Telephony::Calls() const
 {
-  nsRefPtr<CallsList> list = mCallsList;
+  RefPtr<CallsList> list = mCallsList;
   return list.forget();
 }
 
 already_AddRefed<TelephonyCallGroup>
 Telephony::ConferenceGroup() const
 {
-  nsRefPtr<TelephonyCallGroup> group = mGroup;
+  RefPtr<TelephonyCallGroup> group = mGroup;
   return group.forget();
 }
 
@@ -587,17 +662,99 @@ Telephony::GetReady(ErrorResult& aRv) const
     return nullptr;
   }
 
-  nsRefPtr<Promise> promise = mReadyPromise;
+  RefPtr<Promise> promise = mReadyPromise;
   return promise.forget();
 }
 
+// nsIAudioChannelAgentCallback
+
+NS_IMETHODIMP
+Telephony::WindowVolumeChanged(float aVolume, bool aMuted)
+{
+  // It's impossible to put all the calls on-hold in the multi-call case.
+  if (mCalls.Length() > 1 ||
+     (mCalls.Length() == 1 && mGroup->CallsArray().Length())) {
+    return NS_ERROR_FAILURE;
+  }
+
+  ErrorResult rv;
+  nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(GetOwner());
+  RefPtr<Promise> promise = Promise::Create(global, rv);
+  if (NS_WARN_IF(rv.Failed())) {
+    return rv.StealNSResult();
+  }
+
+  bool isSingleCall = mCalls.Length();
+  if (isSingleCall && mCalls[0]->Switchable()) {
+    if (aMuted && (mCalls[0]->State() == TelephonyCallState::Connected)) {
+      Unused << mCalls[0]->Hold(rv);
+    } else if (!aMuted && (mCalls[0]->State() == TelephonyCallState::Held)) {
+      Unused << mCalls[0]->Resume(rv);
+    }
+  } else {
+    if (aMuted && (mGroup->State() == TelephonyCallGroupState::Connected)) {
+      Unused << mGroup->Hold(rv);
+    } else if (!aMuted && (mGroup->State() == TelephonyCallGroupState::Held)) {
+      Unused << mGroup->Resume(rv);
+    }
+  }
+  if (NS_WARN_IF(rv.Failed())) {
+    return rv.StealNSResult();
+  }
+
+  // These events will be triggered when the telephony is interrupted by other
+  // audio channel.
+  if (mMuted != aMuted) {
+    mMuted = aMuted;
+    // We should not dispatch "mozinterruptend" when the system app initializes
+    // the telephony audio from muted to unmuted at the first time. The event
+    // "mozinterruptend" must be dispatched after the "mozinterruptbegin".
+    if (!mHaveDispatchedInterruptBeginEvent && mMuted) {
+      DispatchTrustedEvent(NS_LITERAL_STRING("mozinterruptbegin"));
+      mHaveDispatchedInterruptBeginEvent = mMuted;
+    } else if (mHaveDispatchedInterruptBeginEvent && !mMuted) {
+      DispatchTrustedEvent(NS_LITERAL_STRING("mozinterruptend"));
+      mHaveDispatchedInterruptBeginEvent = mMuted;
+    }
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+Telephony::WindowSuspendChanged(nsSuspendedTypes aSuspend)
+{
+  // Not support yet.
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+Telephony::WindowAudioCaptureChanged(bool aCapture)
+{
+  // Do nothing, it's useless for the telephony object.
+  return NS_OK;
+}
+
 // nsITelephonyListener
 
 NS_IMETHODIMP
 Telephony::CallStateChanged(uint32_t aLength, nsITelephonyCallInfo** aAllInfo)
 {
+  // Update call state
+  nsresult rv;
   for (uint32_t i = 0; i < aLength; ++i) {
-    HandleCallInfo(aAllInfo[i]);
+    rv = HandleCallInfo(aAllInfo[i]);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+  }
+
+  // Update conference state
+  mGroup->ChangeState();
+
+  rv = HandleAudioAgentState();
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
   }
   return NS_OK;
 }
@@ -608,31 +765,13 @@ Telephony::EnumerateCallState(nsITelephonyCallInfo* aInfo)
   return HandleCallInfo(aInfo);
 }
 
-NS_IMETHODIMP
-Telephony::ConferenceCallStateChanged(uint16_t aCallState)
-{
-  mGroup->ChangeState(aCallState);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 Telephony::EnumerateCallStateComplete()
 {
   // Set conference state.
-  if (mGroup->CallsArray().Length() >= 2) {
-    const nsTArray<nsRefPtr<TelephonyCall> > &calls = mGroup->CallsArray();
-
-    uint16_t callState = calls[0]->CallState();
-    for (uint32_t i = 1; i < calls.Length(); i++) {
-      if (calls[i]->CallState() != callState) {
-        callState = nsITelephonyService::CALL_STATE_UNKNOWN;
-        break;
-      }
-    }
-
-    mGroup->ChangeState(callState);
-  }
+  mGroup->ChangeState();
 
+  HandleAudioAgentState();
   if (mReadyPromise) {
     mReadyPromise->MaybeResolve(JS::UndefinedHandleValue);
   }
@@ -648,7 +787,7 @@ Telephony::SupplementaryServiceNotification(uint32_t aServiceId,
                                             int32_t aCallIndex,
                                             uint16_t aNotification)
 {
-  nsRefPtr<TelephonyCall> associatedCall;
+  RefPtr<TelephonyCall> associatedCall;
   if (!mCalls.IsEmpty()) {
     associatedCall = GetCall(aServiceId, aCallIndex);
   }
@@ -678,10 +817,10 @@ Telephony::NotifyCdmaCallWaiting(uint32_t aServiceId, const nsAString& aNumber,
 {
   MOZ_ASSERT(mCalls.Length() == 1);
 
-  nsRefPtr<TelephonyCall> callToNotify = mCalls[0];
+  RefPtr<TelephonyCall> callToNotify = mCalls[0];
   MOZ_ASSERT(callToNotify && callToNotify->ServiceId() == aServiceId);
 
-  nsRefPtr<TelephonyCallId> id =
+  RefPtr<TelephonyCallId> id =
     new TelephonyCallId(GetOwner(), aNumber, aNumberPresentation, aName,
                         aNamePresentation);
   callToNotify->UpdateSecondId(id);
@@ -709,7 +848,7 @@ Telephony::DispatchCallEvent(const nsAString& aType,
   init.mCancelable = false;
   init.mCall = aCall;
 
-  nsRefPtr<CallEvent> event = CallEvent::Constructor(this, aType, init);
+  RefPtr<CallEvent> event = CallEvent::Constructor(this, aType, init);
 
   return DispatchTrustedEvent(event);
 }
