@@ -70,7 +70,7 @@ static void
 DoCommandCallback(Command aCommand, void* aData)
 {
   nsIDocument* doc = static_cast<nsIDocument*>(aData);
-  nsPIDOMWindow* win = doc->GetWindow();
+  nsPIDOMWindowOuter* win = doc->GetWindow();
   if (!win) {
     return;
   }
@@ -324,7 +324,11 @@ nsEditorEventListener::GetFocusedRootContent()
   nsIDocument* composedDoc = focusedContent->GetComposedDoc();
   NS_ENSURE_TRUE(composedDoc, nullptr);
 
-  return composedDoc->HasFlag(NODE_IS_EDITABLE) ? nullptr : focusedContent;
+  if (composedDoc->HasFlag(NODE_IS_EDITABLE)) {
+    return nullptr;
+  }
+
+  return focusedContent;
 }
 
 bool
@@ -357,7 +361,7 @@ nsEditorEventListener::HandleEvent(nsIDOMEvent* aEvent)
 
   nsCOMPtr<nsIEditor> kungFuDeathGrip = mEditor;
 
-  WidgetEvent* internalEvent = aEvent->GetInternalNSEvent();
+  WidgetEvent* internalEvent = aEvent->WidgetEventPtr();
 
   // Let's handle each event with the message of the internal event of the
   // coming event.  If the DOM event was created with improper interface,
@@ -448,7 +452,7 @@ nsEditorEventListener::HandleEvent(nsIDOMEvent* aEvent)
       // editor shouldn't handle this click event.
       if (mMouseDownOrUpConsumedByIME) {
         mMouseDownOrUpConsumedByIME = false;
-        mouseEvent->PreventDefault();
+        mouseEvent->AsEvent()->PreventDefault();
         return NS_OK;
       }
       return MouseClick(mouseEvent);
@@ -611,7 +615,7 @@ nsEditorEventListener::KeyPress(nsIDOMKeyEvent* aKeyEvent)
 {
   NS_ENSURE_TRUE(aKeyEvent, NS_OK);
 
-  if (!mEditor->IsAcceptableInputEvent(aKeyEvent)) {
+  if (!mEditor->IsAcceptableInputEvent(aKeyEvent->AsEvent())) {
     return NS_OK;
   }
 
@@ -622,7 +626,7 @@ nsEditorEventListener::KeyPress(nsIDOMKeyEvent* aKeyEvent)
   // below.
 
   bool defaultPrevented;
-  aKeyEvent->GetDefaultPrevented(&defaultPrevented);
+  aKeyEvent->AsEvent()->GetDefaultPrevented(&defaultPrevented);
   if (defaultPrevented) {
     return NS_OK;
   }
@@ -630,7 +634,7 @@ nsEditorEventListener::KeyPress(nsIDOMKeyEvent* aKeyEvent)
   nsresult rv = mEditor->HandleKeyPressEvent(aKeyEvent);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  aKeyEvent->GetDefaultPrevented(&defaultPrevented);
+  aKeyEvent->AsEvent()->GetDefaultPrevented(&defaultPrevented);
   if (defaultPrevented) {
     return NS_OK;
   }
@@ -641,10 +645,10 @@ nsEditorEventListener::KeyPress(nsIDOMKeyEvent* aKeyEvent)
 
   // Now, ask the native key bindings to handle the event.
   WidgetKeyboardEvent* keyEvent =
-    aKeyEvent->GetInternalNSEvent()->AsKeyboardEvent();
+    aKeyEvent->AsEvent()->WidgetEventPtr()->AsKeyboardEvent();
   MOZ_ASSERT(keyEvent,
              "DOM key event's internal event must be WidgetKeyboardEvent");
-  nsIWidget* widget = keyEvent->widget;
+  nsIWidget* widget = keyEvent->mWidget;
   // If the event is created by chrome script, the widget is always nullptr.
   if (!widget) {
     nsCOMPtr<nsIPresShell> ps = GetPresShell();
@@ -658,7 +662,7 @@ nsEditorEventListener::KeyPress(nsIDOMKeyEvent* aKeyEvent)
                            nsIWidget::NativeKeyBindingsForRichTextEditor,
                            *keyEvent, DoCommandCallback, doc);
   if (handled) {
-    aKeyEvent->PreventDefault();
+    aKeyEvent->AsEvent()->PreventDefault();
   }
   return NS_OK;
 }
@@ -668,7 +672,7 @@ nsEditorEventListener::MouseClick(nsIDOMMouseEvent* aMouseEvent)
 {
   // nothing to do if editor isn't editable or clicked on out of the editor.
   if (mEditor->IsReadonly() || mEditor->IsDisabled() ||
-      !mEditor->IsAcceptableInputEvent(aMouseEvent)) {
+      !mEditor->IsAcceptableInputEvent(aMouseEvent->AsEvent())) {
     return NS_OK;
   }
 
@@ -683,7 +687,7 @@ nsEditorEventListener::MouseClick(nsIDOMMouseEvent* aMouseEvent)
   }
 
   bool preventDefault;
-  nsresult rv = aMouseEvent->GetDefaultPrevented(&preventDefault);
+  nsresult rv = aMouseEvent->AsEvent()->GetDefaultPrevented(&preventDefault);
   if (NS_FAILED(rv) || preventDefault) {
     // We're done if 'preventdefault' is true (see for example bug 70698).
     return rv;
@@ -719,7 +723,7 @@ nsEditorEventListener::HandleMiddleClickPaste(nsIDOMMouseEvent* aMouseEvent)
     return NS_ERROR_NULL_POINTER;
   }
 
-  nsRefPtr<Selection> selection = mEditor->GetSelection();
+  RefPtr<Selection> selection = mEditor->GetSelection();
   if (selection) {
     selection->Collapse(parent, offset);
   }
@@ -754,8 +758,8 @@ nsEditorEventListener::HandleMiddleClickPaste(nsIDOMMouseEvent* aMouseEvent)
 
   // Prevent the event from propagating up to be possibly handled
   // again by the containing window:
-  aMouseEvent->StopPropagation();
-  aMouseEvent->PreventDefault();
+  aMouseEvent->AsEvent()->StopPropagation();
+  aMouseEvent->AsEvent()->PreventDefault();
 
   // We processed the event, whether drop/paste succeeded or not
   return NS_OK;
@@ -770,7 +774,7 @@ nsEditorEventListener::NotifyIMEOfMouseButtonEvent(
   }
 
   bool defaultPrevented;
-  nsresult rv = aMouseEvent->GetDefaultPrevented(&defaultPrevented);
+  nsresult rv = aMouseEvent->AsEvent()->GetDefaultPrevented(&defaultPrevented);
   NS_ENSURE_SUCCESS(rv, false);
   if (defaultPrevented) {
     return false;
@@ -840,7 +844,7 @@ nsEditorEventListener::DragOver(nsIDOMDragEvent* aDragEvent)
 
   nsCOMPtr<nsIDOMNode> parent;
   bool defaultPrevented;
-  aDragEvent->GetDefaultPrevented(&defaultPrevented);
+  aDragEvent->AsEvent()->GetDefaultPrevented(&defaultPrevented);
   if (defaultPrevented) {
     return NS_OK;
   }
@@ -850,7 +854,7 @@ nsEditorEventListener::DragOver(nsIDOMDragEvent* aDragEvent)
   NS_ENSURE_TRUE(dropParent, NS_ERROR_FAILURE);
 
   if (dropParent->IsEditable() && CanDrop(aDragEvent)) {
-    aDragEvent->PreventDefault(); // consumed
+    aDragEvent->AsEvent()->PreventDefault(); // consumed
 
     if (!mCaret) {
       return NS_OK;
@@ -869,7 +873,7 @@ nsEditorEventListener::DragOver(nsIDOMDragEvent* aDragEvent)
   if (!IsFileControlTextBox()) {
     // This is needed when dropping on an input, to prevent the editor for
     // the editable parent from receiving the event.
-    aDragEvent->StopPropagation();
+    aDragEvent->AsEvent()->StopPropagation();
   }
 
   if (mCaret) {
@@ -914,7 +918,7 @@ nsEditorEventListener::Drop(nsIDOMDragEvent* aDragEvent)
   CleanupDragDropCaret();
 
   bool defaultPrevented;
-  aDragEvent->GetDefaultPrevented(&defaultPrevented);
+  aDragEvent->AsEvent()->GetDefaultPrevented(&defaultPrevented);
   if (defaultPrevented) {
     return NS_OK;
   }
@@ -932,14 +936,14 @@ nsEditorEventListener::Drop(nsIDOMDragEvent* aDragEvent)
       // since someone else handling it might be unintentional and the
       // user could probably re-drag to be not over the disabled/readonly
       // editfields if that is what is desired.
-      return aDragEvent->StopPropagation();
+      return aDragEvent->AsEvent()->StopPropagation();
     }
     return NS_OK;
   }
 
-  aDragEvent->StopPropagation();
-  aDragEvent->PreventDefault();
-  return mEditor->InsertFromDrop(aDragEvent);
+  aDragEvent->AsEvent()->StopPropagation();
+  aDragEvent->AsEvent()->PreventDefault();
+  return mEditor->InsertFromDrop(aDragEvent->AsEvent());
 }
 
 bool
@@ -955,7 +959,7 @@ nsEditorEventListener::CanDrop(nsIDOMDragEvent* aEvent)
   nsCOMPtr<DataTransfer> dataTransfer = do_QueryInterface(domDataTransfer);
   NS_ENSURE_TRUE(dataTransfer, false);
 
-  nsRefPtr<DOMStringList> types = dataTransfer->Types();
+  RefPtr<DOMStringList> types = dataTransfer->Types();
 
   // Plaintext editors only support dropping text. Otherwise, HTML and files
   // can be dropped as well.
@@ -999,7 +1003,7 @@ nsEditorEventListener::CanDrop(nsIDOMDragEvent* aEvent)
     return true;
   }
 
-  nsRefPtr<Selection> selection = mEditor->GetSelection();
+  RefPtr<Selection> selection = mEditor->GetSelection();
   if (!selection) {
     return false;
   }
@@ -1024,7 +1028,7 @@ nsEditorEventListener::CanDrop(nsIDOMDragEvent* aEvent)
   NS_ENSURE_SUCCESS(rv, false);
 
   for (int32_t i = 0; i < rangeCount; i++) {
-    nsRefPtr<nsRange> range = selection->GetRangeAt(i);
+    RefPtr<nsRange> range = selection->GetRangeAt(i);
     if (!range) {
       // Don't bail yet, iterate through them all
       continue;
@@ -1047,7 +1051,7 @@ nsEditorEventListener::HandleStartComposition(nsIDOMEvent* aCompositionEvent)
     return NS_OK;
   }
   WidgetCompositionEvent* compositionStart =
-    aCompositionEvent->GetInternalNSEvent()->AsCompositionEvent();
+    aCompositionEvent->WidgetEventPtr()->AsCompositionEvent();
   return mEditor->BeginIMEComposition(compositionStart);
 }
 
@@ -1183,7 +1187,7 @@ nsEditorEventListener::ShouldHandleNativeKeyBindings(nsIDOMKeyEvent* aKeyEvent)
   // mouse events.
 
   nsCOMPtr<nsIDOMEventTarget> target;
-  aKeyEvent->GetTarget(getter_AddRefs(target));
+  aKeyEvent->AsEvent()->GetTarget(getter_AddRefs(target));
   nsCOMPtr<nsIContent> targetContent = do_QueryInterface(target);
   if (!targetContent) {
     return false;
