@@ -57,6 +57,15 @@ using namespace mozilla::dom;
  ********************************************************/
 
 nsTextEditRules::nsTextEditRules()
+  : mEditor(nullptr)
+  , mPasswordIMEIndex(0)
+  , mCachedSelectionOffset(0)
+  , mActionNesting(0)
+  , mLockRulesSniffing(false)
+  , mDidExplicitlySetInterline(false)
+  , mDeleteBidiImmediately(false)
+  , mLastStart(0)
+  , mLastLength(0)
 {
   InitFields();
 }
@@ -116,7 +125,7 @@ nsTextEditRules::Init(nsPlaintextEditor *aEditor)
   InitFields();
 
   mEditor = aEditor;  // we hold a non-refcounted reference back to our editor
-  nsRefPtr<Selection> selection = mEditor->GetSelection();
+  RefPtr<Selection> selection = mEditor->GetSelection();
   NS_WARN_IF_FALSE(selection, "editor cannot get selection");
 
   // Put in a magic br if needed. This method handles null selection,
@@ -183,7 +192,7 @@ nsTextEditRules::BeforeEdit(EditAction action,
 
   // get the selection and cache the position before editing
   NS_ENSURE_STATE(mEditor);
-  nsRefPtr<Selection> selection = mEditor->GetSelection();
+  RefPtr<Selection> selection = mEditor->GetSelection();
   NS_ENSURE_STATE(selection);
 
   selection->GetAnchorNode(getter_AddRefs(mCachedSelectionNode));
@@ -206,7 +215,7 @@ nsTextEditRules::AfterEdit(EditAction action,
   if (!--mActionNesting)
   {
     NS_ENSURE_STATE(mEditor);
-    nsRefPtr<Selection> selection = mEditor->GetSelection();
+    RefPtr<Selection> selection = mEditor->GetSelection();
     NS_ENSURE_STATE(selection);
 
     NS_ENSURE_STATE(mEditor);
@@ -276,7 +285,8 @@ nsTextEditRules::WillDoAction(Selection* aSelection,
     case EditAction::insertElement:
       // i had thought this would be html rules only.  but we put pre elements
       // into plaintext mail when doing quoting for reply!  doh!
-      return WillInsert(aSelection, aCancel);
+      WillInsert(*aSelection, aCancel);
+      return NS_OK;
     default:
       return NS_ERROR_FAILURE;
   }
@@ -336,25 +346,25 @@ nsTextEditRules::DocumentIsEmpty(bool *aDocumentIsEmpty)
  ********************************************************/
 
 
-nsresult
-nsTextEditRules::WillInsert(Selection* aSelection, bool* aCancel)
+void
+nsTextEditRules::WillInsert(Selection& aSelection, bool* aCancel)
 {
-  NS_ENSURE_TRUE(aSelection && aCancel, NS_ERROR_NULL_POINTER);
+  MOZ_ASSERT(aCancel);
 
-  CANCEL_OPERATION_IF_READONLY_OR_DISABLED
+  if (IsReadonly() || IsDisabled()) {
+    *aCancel = true;
+    return;
+  }
 
   // initialize out param
   *aCancel = false;
 
   // check for the magic content node and delete it if it exists
-  if (mBogusNode)
-  {
-    NS_ENSURE_STATE(mEditor);
+  if (mBogusNode) {
+    NS_ENSURE_TRUE_VOID(mEditor);
     mEditor->DeleteNode(mBogusNode);
     mBogusNode = nullptr;
   }
-
-  return NS_OK;
 }
 
 nsresult
@@ -403,8 +413,7 @@ nsTextEditRules::WillInsertBreak(Selection* aSelection,
       NS_ENSURE_SUCCESS(res, res);
     }
 
-    res = WillInsert(aSelection, aCancel);
-    NS_ENSURE_SUCCESS(res, res);
+    WillInsert(*aSelection, aCancel);
     // initialize out param
     // we want to ignore result of WillInsert()
     *aCancel = false;
@@ -488,7 +497,7 @@ GetTextNode(Selection* selection, nsEditor* editor) {
     NS_ENSURE_TRUE(node, nullptr);
     // This should be the root node, walk the tree looking for text nodes
     NodeFilterHolder filter;
-    nsRefPtr<NodeIterator> iter = new NodeIterator(node, nsIDOMNodeFilter::SHOW_TEXT, filter);
+    RefPtr<NodeIterator> iter = new NodeIterator(node, nsIDOMNodeFilter::SHOW_TEXT, filter);
     while (!editor->IsTextNode(selNode)) {
       if (NS_FAILED(res = iter->NextNode(getter_AddRefs(selNode))) || !selNode) {
         return nullptr;
@@ -642,8 +651,7 @@ nsTextEditRules::WillInsertText(EditAction aAction,
     NS_ENSURE_SUCCESS(res, res);
   }
 
-  res = WillInsert(aSelection, aCancel);
-  NS_ENSURE_SUCCESS(res, res);
+  WillInsert(*aSelection, aCancel);
   // initialize out param
   // we want to ignore result of WillInsert()
   *aCancel = false;
@@ -1084,7 +1092,7 @@ nsTextEditRules::RemoveRedundantTrailingBR()
     return NS_OK;
 
   NS_ENSURE_STATE(mEditor);
-  nsRefPtr<dom::Element> body = mEditor->GetRoot();
+  RefPtr<dom::Element> body = mEditor->GetRoot();
   if (!body)
     return NS_ERROR_NULL_POINTER;
 
@@ -1094,7 +1102,7 @@ nsTextEditRules::RemoveRedundantTrailingBR()
     return NS_OK;
   }
 
-  nsRefPtr<nsIContent> child = body->GetFirstChild();
+  RefPtr<nsIContent> child = body->GetFirstChild();
   if (!child || !child->IsElement()) {
     return NS_OK;
   }
@@ -1341,7 +1349,7 @@ nsresult nsTextEditRules::HideLastPWInput() {
   FillBufWithPWChars(&hiddenText, mLastLength);
 
   NS_ENSURE_STATE(mEditor);
-  nsRefPtr<Selection> selection = mEditor->GetSelection();
+  RefPtr<Selection> selection = mEditor->GetSelection();
   NS_ENSURE_TRUE(selection, NS_ERROR_NULL_POINTER);
   int32_t start, end;
   nsContentUtils::GetSelectionInTextControl(selection, mEditor->GetRoot(),
