@@ -22,6 +22,7 @@
 #include "mozilla/layers/CompositableClient.h"  // for CompositableClient
 #include "mozilla/layers/CompositorTypes.h"  // for TextureInfo, etc
 #include "mozilla/layers/LayersMessages.h" // for TileDescriptor
+#include "mozilla/layers/LayersTypes.h" // for TextureDumpMode
 #include "mozilla/layers/TextureClient.h"
 #include "mozilla/layers/TextureClientPool.h"
 #include "ClientLayerManager.h"
@@ -34,8 +35,6 @@
 #include "nsTArray.h"                   // for nsTArray, nsTArray_Impl, etc
 #include "nsExpirationTracker.h"
 #include "mozilla/layers/ISurfaceAllocator.h"
-#include "gfxReusableSurfaceWrapper.h"
-#include "pratom.h"                     // For PR_ATOMIC_INCREMENT/DECREMENT
 
 namespace mozilla {
 namespace layers {
@@ -43,104 +42,6 @@ namespace layers {
 class ClientTiledPaintedLayer;
 class ClientLayerManager;
 
-
-// A class to help implement copy-on-write semantics for shared tiles.
-class gfxSharedReadLock {
-protected:
-  virtual ~gfxSharedReadLock() {}
-
-public:
-  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(gfxSharedReadLock)
-
-  virtual int32_t ReadLock() = 0;
-  virtual int32_t ReadUnlock() = 0;
-  virtual int32_t GetReadCount() = 0;
-  virtual bool IsValid() const = 0;
-
-  enum gfxSharedReadLockType {
-    TYPE_MEMORY,
-    TYPE_SHMEM
-  };
-  virtual gfxSharedReadLockType GetType() = 0;
-
-protected:
-  NS_DECL_OWNINGTHREAD
-};
-
-class gfxMemorySharedReadLock : public gfxSharedReadLock {
-public:
-  gfxMemorySharedReadLock();
-
-protected:
-  ~gfxMemorySharedReadLock();
-
-public:
-  virtual int32_t ReadLock() override;
-
-  virtual int32_t ReadUnlock() override;
-
-  virtual int32_t GetReadCount() override;
-
-  virtual gfxSharedReadLockType GetType() override { return TYPE_MEMORY; }
-
-  virtual bool IsValid() const override { return true; };
-
-private:
-  int32_t mReadCount;
-};
-
-class gfxShmSharedReadLock : public gfxSharedReadLock {
-private:
-  struct ShmReadLockInfo {
-    int32_t readCount;
-  };
-
-public:
-  explicit gfxShmSharedReadLock(ISurfaceAllocator* aAllocator);
-
-protected:
-  ~gfxShmSharedReadLock();
-
-public:
-  virtual int32_t ReadLock() override;
-
-  virtual int32_t ReadUnlock() override;
-
-  virtual int32_t GetReadCount() override;
-
-  virtual bool IsValid() const override { return mAllocSuccess; };
-
-  virtual gfxSharedReadLockType GetType() override { return TYPE_SHMEM; }
-
-  mozilla::layers::ShmemSection& GetShmemSection() { return mShmemSection; }
-
-  static already_AddRefed<gfxShmSharedReadLock>
-  Open(mozilla::layers::ISurfaceAllocator* aAllocator, const mozilla::layers::ShmemSection& aShmemSection)
-  {
-    nsRefPtr<gfxShmSharedReadLock> readLock = new gfxShmSharedReadLock(aAllocator, aShmemSection);
-    return readLock.forget();
-  }
-
-private:
-  gfxShmSharedReadLock(ISurfaceAllocator* aAllocator, const mozilla::layers::ShmemSection& aShmemSection)
-    : mAllocator(aAllocator)
-    , mShmemSection(aShmemSection)
-    , mAllocSuccess(true)
-  {
-    MOZ_COUNT_CTOR(gfxShmSharedReadLock);
-  }
-
-  ShmReadLockInfo* GetShmReadLockInfoPtr()
-  {
-    return reinterpret_cast<ShmReadLockInfo*>
-      (mShmemSection.shmem().get<char>() + mShmemSection.offset());
-  }
-
-  RefPtr<ISurfaceAllocator> mAllocator;
-  mozilla::layers::ShmemSection mShmemSection;
-  bool mAllocSuccess;
-};
-
 /**
  * Represent a single tile in tiled buffer. The buffer keeps tiles,
  * each tile keeps a reference to a texture client and a read-lock. This
@@ -189,22 +90,6 @@ struct TileClient
     return mBackBuffer == nullptr && mFrontBuffer == nullptr;
   }
 
-  void ReadUnlock()
-  {
-    MOZ_ASSERT(mFrontLock, "ReadLock with no gfxSharedReadLock");
-    if (mFrontLock) {
-      mFrontLock->ReadUnlock();
-    }
-  }
-
-  void ReadLock()
-  {
-    MOZ_ASSERT(mFrontLock, "ReadLock with no gfxSharedReadLock");
-    if (mFrontLock) {
-      mFrontLock->ReadLock();
-    }
-  }
-
   void DiscardBuffers()
   {
     DiscardFrontBuffer();
@@ -225,9 +110,9 @@ struct TileClient
   */
   void Flip();
 
-  void DumpTexture(std::stringstream& aStream) {
+  void DumpTexture(std::stringstream& aStream, TextureDumpMode aCompress) {
     // TODO We should combine the OnWhite/OnBlack here an just output a single image.
-    CompositableClient::DumpTextureClient(aStream, mFrontBuffer);
+    CompositableClient::DumpTextureClient(aStream, mFrontBuffer, aCompress);
   }
 
   /**
@@ -269,12 +154,11 @@ struct TileClient
   RefPtr<TextureClient> mBackBufferOnWhite;
   RefPtr<TextureClient> mFrontBuffer;
   RefPtr<TextureClient> mFrontBufferOnWhite;
-  RefPtr<gfxSharedReadLock> mBackLock;
-  RefPtr<gfxSharedReadLock> mFrontLock;
   RefPtr<ClientLayerManager> mManager;
   RefPtr<TextureClientAllocator> mAllocator;
   gfx::IntRect mUpdateRect;
   CompositableClient* mCompositableClient;
+  bool mWasPlaceholder;
 #ifdef GFX_TILEDLAYER_DEBUG_OVERLAY
   TimeStamp        mLastUpdate;
 #endif
@@ -312,14 +196,14 @@ struct BasicTiledLayerPaintData {
    * the closest ancestor layer which scrolls, and is used to obtain
    * the composition bounds that are relevant for this layer.
    */
-  gfx::Matrix4x4 mTransformToCompBounds;
+  LayerToParentLayerMatrix4x4 mTransformToCompBounds;
 
   /*
    * The critical displayport of the content from the nearest ancestor layer
-   * that represents scrollable content with a display port set. Empty if a
-   * critical displayport is not set.
+   * that represents scrollable content with a display port set. isNothing()
+   * if a critical displayport is not set.
    */
-  LayerIntRect mCriticalDisplayPort;
+  Maybe<LayerIntRect> mCriticalDisplayPort;
 
   /*
    * The render resolution of the document that the content this layer
@@ -353,6 +237,11 @@ struct BasicTiledLayerPaintData {
    */
   bool mPaintFinished : 1;
 
+  /*
+   * Whether or not there is an async transform animation active
+   */
+  bool mHasTransformAnimation : 1;
+
   /*
    * Initializes/clears data to prepare for paint action.
    */
@@ -375,7 +264,7 @@ public:
   bool UpdateFromCompositorFrameMetrics(const LayerMetricsWrapper& aLayer,
                                         bool aHasPendingNewThebesContent,
                                         bool aLowPrecision,
-                                        ViewTransform& aViewTransform);
+                                        AsyncTransform& aViewTransform);
 
   /**
    * Determines if the compositor's upcoming composition bounds has fallen
@@ -407,13 +296,15 @@ public:
     , mCompositableClient(aCompositableClient)
     , mLastPaintContentType(gfxContentType::COLOR)
     , mLastPaintSurfaceMode(SurfaceMode::SURFACE_OPAQUE)
+    , mWasLastPaintProgressive(false)
   {}
 
   virtual void PaintThebes(const nsIntRegion& aNewValidRegion,
                    const nsIntRegion& aPaintRegion,
                    const nsIntRegion& aDirtyRegion,
                    LayerManager::DrawPaintedLayerCallback aCallback,
-                   void* aCallbackData) = 0;
+                   void* aCallbackData,
+                   bool aIsProgressive = false) = 0;
 
   virtual bool SupportsProgressiveUpdate() = 0;
   virtual bool ProgressiveUpdate(nsIntRegion& aValidRegion,
@@ -430,7 +321,8 @@ public:
 
   virtual void Dump(std::stringstream& aStream,
                     const char* aPrefix,
-                    bool aDumpHtml) {}
+                    bool aDumpHtml,
+                    TextureDumpMode aCompress) {}
 
   const CSSToParentLayerScale2D& GetFrameResolution() { return mFrameResolution; }
   void SetFrameResolution(const CSSToParentLayerScale2D& aResolution) { mFrameResolution = aResolution; }
@@ -447,6 +339,8 @@ protected:
   gfxContentType mLastPaintContentType;
   SurfaceMode mLastPaintSurfaceMode;
   CSSToParentLayerScale2D mFrameResolution;
+
+  bool mWasLastPaintProgressive;
 };
 
 class ClientMultiTiledLayerBuffer
@@ -473,7 +367,8 @@ public:
                    const nsIntRegion& aPaintRegion,
                    const nsIntRegion& aDirtyRegion,
                    LayerManager::DrawPaintedLayerCallback aCallback,
-                   void* aCallbackData) override;
+                   void* aCallbackData,
+                   bool aIsProgressive = false) override;
 
   virtual bool SupportsProgressiveUpdate() override { return true; }
   /**
@@ -507,8 +402,9 @@ public:
 
   void Dump(std::stringstream& aStream,
             const char* aPrefix,
-            bool aDumpHtml) override {
-    TiledLayerBuffer::Dump(aStream, aPrefix, aDumpHtml);
+            bool aDumpHtml,
+            TextureDumpMode aCompress) override {
+    TiledLayerBuffer::Dump(aStream, aPrefix, aDumpHtml, aCompress);
   }
 
   void ReadLock();
@@ -606,7 +502,8 @@ public:
 
   virtual void Dump(std::stringstream& aStream,
                     const char* aPrefix="",
-                    bool aDumpHtml=false);
+                    bool aDumpHtml=false,
+                    TextureDumpMode aCompress=TextureDumpMode::Compress) override;
 
   virtual TextureInfo GetTextureInfo() const override
   {
@@ -641,9 +538,8 @@ protected:
   ~MultiTiledContentClient()
   {
     MOZ_COUNT_DTOR(MultiTiledContentClient);
- 
-    mDestroyed = true;
-    mTiledBuffer.DiscardBuffers();
+
+      mTiledBuffer.DiscardBuffers();
     mLowPrecisionTiledBuffer.DiscardBuffers();
   }
 
