@@ -17,14 +17,9 @@
 #include "nsPrintfCString.h"            // for nsPrintfCString
 #include "nsString.h"                   // for nsAutoCString
 
-class nsIntRegion;
-
 #define BIAS_TIME_MS 1.0
 
 namespace mozilla {
-namespace gfx {
-class Matrix4x4;
-} // namespace gfx
 
 using namespace gfx;
 
@@ -37,6 +32,7 @@ ImageHost::ImageHost(const TextureInfo& aTextureInfo)
   , mImageContainer(nullptr)
   , mLastFrameID(-1)
   , mLastProducerID(-1)
+  , mLastInputFrameID(-1)
   , mBias(BIAS_NONE)
   , mLocked(false)
 {}
@@ -56,15 +52,6 @@ ImageHost::UseTextureHost(const nsTArray<TimedTexture>& aTextures)
 
   nsTArray<TimedImage> newImages;
 
-  // Remove all mImages without an mTextureSource to recycle.
-  for (int32_t i = mImages.Length() - 1; i >= 0; --i) {
-    if (!mImages[i].mTextureSource) {
-      mImages.RemoveElementAt(i);
-    }
-  }
-
-  // Create new TimedImage entries and recycle any mTextureSources that match
-  // our mFrontBuffers.
   for (uint32_t i = 0; i < aTextures.Length(); ++i) {
     const TimedTexture& t = aTextures[i];
     MOZ_ASSERT(t.mTexture);
@@ -77,33 +64,24 @@ ImageHost::UseTextureHost(const nsTArray<TimedTexture>& aTextures)
       continue;
     }
     TimedImage& img = *newImages.AppendElement();
-    img.mFrontBuffer = t.mTexture;
-    for (uint32_t i = 0; i < mImages.Length(); ++i) {
-      if (mImages[i].mFrontBuffer == img.mFrontBuffer) {
-        img.mTextureSource = mImages[i].mTextureSource;
-        mImages.RemoveElementAt(i);
-        break;
-      }
-    }
+    img.mTextureHost = t.mTexture;
     img.mTimeStamp = t.mTimeStamp;
     img.mPictureRect = t.mPictureRect;
     img.mFrameID = t.mFrameID;
     img.mProducerID = t.mProducerID;
+    img.mInputFrameID = t.mInputFrameID;
+    img.mTextureHost->SetCropRect(img.mPictureRect);
+    img.mTextureHost->Updated();
   }
-  // Recycle any leftover mTextureSources and call PrepareTextureSource on all
-  // images.
-  for (auto& img : newImages) {
-    if (!img.mTextureSource && !mImages.IsEmpty()) {
-      img.mTextureSource = mImages.LastElement().mTextureSource;
-      mImages.RemoveElementAt(mImages.Length() - 1);
-    }
-    // SetCropRect() affects only on a specific platform.
-    // If it is not implemented, it does nothing.
-    img.mFrontBuffer->SetCropRect(img.mPictureRect);
-    img.mFrontBuffer->Updated();
-    img.mFrontBuffer->PrepareTextureSource(img.mTextureSource);
-  }
+
   mImages.SwapElements(newImages);
+  newImages.Clear();
+
+  // If we only have one image we can upload it right away, otherwise we'll upload
+  // on-demand during composition after we have picked the proper timestamp.
+  if (mImages.Length() == 1) {
+    SetCurrentTextureHost(mImages[0].mTextureHost);
+  }
 
   // Video producers generally send replacement images with the same frameID but
   // slightly different timestamps in order to sync with the audio clock. This
@@ -122,6 +100,48 @@ ImageHost::UseTextureHost(const nsTArray<TimedTexture>& aTextures)
   }
 }
 
+void
+ImageHost::SetCurrentTextureHost(TextureHost* aTexture)
+{
+  if (aTexture == mCurrentTextureHost.get()) {
+    return;
+  }
+
+  bool swapTextureSources = !!mCurrentTextureHost && !!mCurrentTextureSource
+                            && mCurrentTextureHost->HasIntermediateBuffer();
+
+  if (swapTextureSources) {
+    auto dataSource = mCurrentTextureSource->AsDataTextureSource();
+    if (dataSource) {
+      // The current textureHost has an internal buffer in the form of the
+      // DataTextureSource. Removing the ownership of the texture source
+      // will enable the next texture host we bind to the texture source to
+      // acquire it instead of creating a new one. This is desirable in
+      // ImageHost because the current texture won't be used again with the
+      // same content. It wouldn't be desirable with ContentHost for instance,
+      // because the latter reuses the texture's valid regions.
+      dataSource->SetOwner(nullptr);
+    }
+
+    RefPtr<TextureSource> tmp = mExtraTextureSource;
+    mExtraTextureSource = mCurrentTextureSource.get();
+    mCurrentTextureSource = tmp;
+  } else {
+    mExtraTextureSource = nullptr;
+  }
+
+  mCurrentTextureHost = aTexture;
+  mCurrentTextureHost->PrepareTextureSource(mCurrentTextureSource);
+}
+
+void
+ImageHost::CleanupResources()
+{
+  mExtraTextureSource = nullptr;
+  mCurrentTextureSource = nullptr;
+  mCurrentTextureHost = nullptr;
+}
+
 void
 ImageHost::RemoveTextureHost(TextureHost* aTexture)
 {
@@ -130,13 +150,27 @@ ImageHost::RemoveTextureHost(TextureHost* aTexture)
   CompositableHost::RemoveTextureHost(aTexture);
 
   for (int32_t i = mImages.Length() - 1; i >= 0; --i) {
-    if (mImages[i].mFrontBuffer == aTexture) {
+    if (mImages[i].mTextureHost == aTexture) {
       aTexture->UnbindTextureSource();
       mImages.RemoveElementAt(i);
     }
   }
 }
 
+void
+ImageHost::UseOverlaySource(OverlaySource aOverlay,
+                            const gfx::IntRect& aPictureRect)
+{
+  if (ImageHostOverlay::IsValid(aOverlay)) {
+    if (!mImageHostOverlay) {
+      mImageHostOverlay = new ImageHostOverlay();
+    }
+    mImageHostOverlay->UseOverlaySource(aOverlay, aPictureRect);
+  } else {
+    mImageHostOverlay = nullptr;
+  }
+}
+
 static TimeStamp
 GetBiasedTime(const TimeStamp& aInput, ImageHost::Bias aBias)
 {
@@ -234,10 +268,13 @@ TextureHost*
 ImageHost::GetAsTextureHost(IntRect* aPictureRect)
 {
   TimedImage* img = ChooseImage();
+  if (img) {
+    SetCurrentTextureHost(img->mTextureHost);
+  }
   if (aPictureRect && img) {
     *aPictureRect = img->mPictureRect;
   }
-  return img ? img->mFrontBuffer.get() : nullptr;
+  return img ? img->mTextureHost.get() : nullptr;
 }
 
 void ImageHost::Attach(Layer* aLayer,
@@ -247,10 +284,9 @@ void ImageHost::Attach(Layer* aLayer,
   CompositableHost::Attach(aLayer, aCompositor, aFlags);
   for (auto& img : mImages) {
     if (GetCompositor()) {
-      img.mFrontBuffer->SetCompositor(GetCompositor());
+      img.mTextureHost->SetCompositor(GetCompositor());
     }
-    img.mFrontBuffer->Updated();
-    img.mFrontBuffer->PrepareTextureSource(img.mTextureSource);
+    img.mTextureHost->Updated();
   }
 }
 
@@ -259,8 +295,8 @@ ImageHost::Composite(LayerComposite* aLayer,
                      EffectChain& aEffectChain,
                      float aOpacity,
                      const gfx::Matrix4x4& aTransform,
-                     const gfx::Filter& aFilter,
-                     const gfx::Rect& aClipRect,
+                     const gfx::SamplingFilter aSamplingFilter,
+                     const gfx::IntRect& aClipRect,
                      const nsIntRegion* aVisibleRegion)
 {
   if (!GetCompositor()) {
@@ -269,6 +305,21 @@ ImageHost::Composite(LayerComposite* aLayer,
     // set the new compositor yet.
     return;
   }
+
+  if (mImageHostOverlay) {
+    mImageHostOverlay->Composite(GetCompositor(),
+                                 mFlashCounter,
+                                 aLayer,
+                                 aEffectChain,
+                                 aOpacity,
+                                 aTransform,
+                                 aSamplingFilter,
+                                 aClipRect,
+                                 aVisibleRegion);
+    mBias = BIAS_NONE;
+    return;
+  }
+
   int imageIndex = ChooseImageIndex();
   if (imageIndex < 0) {
     return;
@@ -279,11 +330,13 @@ ImageHost::Composite(LayerComposite* aLayer,
   }
 
   TimedImage* img = &mImages[imageIndex];
-  // Make sure the front buffer has a compositor
-  img->mFrontBuffer->SetCompositor(GetCompositor());
-  if (img->mTextureSource) {
-    img->mTextureSource->SetCompositor(GetCompositor());
+  img->mTextureHost->SetCompositor(GetCompositor());
+  // If this TextureHost will be recycled, then make sure we hold a reference to
+  // it until we're sure that the compositor has finished reading from it.
+  if (img->mTextureHost->GetFlags() & TextureFlags::RECYCLE) {
+    aLayer->GetLayerManager()->HoldTextureUntilNextComposite(img->mTextureHost);
   }
+  SetCurrentTextureHost(img->mTextureHost);
 
   {
     AutoLockCompositableHost autoLock(this);
@@ -292,21 +345,21 @@ ImageHost::Composite(LayerComposite* aLayer,
       return;
     }
 
-    if (!img->mFrontBuffer->BindTextureSource(img->mTextureSource)) {
+    if (!mCurrentTextureHost->BindTextureSource(mCurrentTextureSource)) {
       return;
     }
 
-    if (!img->mTextureSource) {
+    if (!mCurrentTextureSource) {
       // BindTextureSource above should have returned false!
       MOZ_ASSERT(false);
       return;
     }
 
     bool isAlphaPremultiplied =
-        !(img->mFrontBuffer->GetFlags() & TextureFlags::NON_PREMULTIPLIED);
+        !(mCurrentTextureHost->GetFlags() & TextureFlags::NON_PREMULTIPLIED);
     RefPtr<TexturedEffect> effect =
-        CreateTexturedEffect(img->mFrontBuffer->GetFormat(),
-            img->mTextureSource.get(), aFilter, isAlphaPremultiplied,
+        CreateTexturedEffect(mCurrentTextureHost->GetReadFormat(),
+            mCurrentTextureSource.get(), aSamplingFilter, isAlphaPremultiplied,
             GetRenderState());
     if (!effect) {
       return;
@@ -316,6 +369,13 @@ ImageHost::Composite(LayerComposite* aLayer,
       return;
     }
 
+    DiagnosticFlags diagnosticFlags = DiagnosticFlags::IMAGE;
+    if (effect->mType == EffectTypes::NV12) {
+      diagnosticFlags |= DiagnosticFlags::NV12;
+    } else if (effect->mType == EffectTypes::YCBCR) {
+      diagnosticFlags |= DiagnosticFlags::YCBCR;
+    }
+
     if (mLastFrameID != img->mFrameID || mLastProducerID != img->mProducerID) {
       if (mImageContainer) {
         aLayer->GetLayerManager()->
@@ -326,10 +386,11 @@ ImageHost::Composite(LayerComposite* aLayer,
       }
       mLastFrameID = img->mFrameID;
       mLastProducerID = img->mProducerID;
+      mLastInputFrameID = img->mInputFrameID;
     }
     aEffectChain.mPrimaryEffect = effect;
     gfx::Rect pictureRect(0, 0, img->mPictureRect.width, img->mPictureRect.height);
-    BigImageIterator* it = img->mTextureSource->AsBigImageIterator();
+    BigImageIterator* it = mCurrentTextureSource->AsBigImageIterator();
     if (it) {
 
       // This iteration does not work if we have multiple texture sources here
@@ -345,7 +406,7 @@ ImageHost::Composite(LayerComposite* aLayer,
       // the corresponding source tiles from all planes, with appropriate
       // per-plane per-tile texture coords.
       // DrawQuad currently assumes that all planes use the same texture coords.
-      MOZ_ASSERT(it->GetTileCount() == 1 || !img->mTextureSource->GetNextSibling(),
+      MOZ_ASSERT(it->GetTileCount() == 1 || !mCurrentTextureSource->GetNextSibling(),
                  "Can't handle multi-plane BigImages");
 
       it->BeginBigImageIteration();
@@ -357,34 +418,34 @@ ImageHost::Composite(LayerComposite* aLayer,
                                       Float(rect.y - tileRect.y) / tileRect.height,
                                       Float(rect.width) / tileRect.width,
                                       Float(rect.height) / tileRect.height);
-        if (img->mFrontBuffer->GetFlags() & TextureFlags::ORIGIN_BOTTOM_LEFT) {
+        if (img->mTextureHost->GetFlags() & TextureFlags::ORIGIN_BOTTOM_LEFT) {
           effect->mTextureCoords.y = effect->mTextureCoords.YMost();
           effect->mTextureCoords.height = -effect->mTextureCoords.height;
         }
         GetCompositor()->DrawQuad(rect, aClipRect, aEffectChain,
                                   aOpacity, aTransform);
-        GetCompositor()->DrawDiagnostics(DiagnosticFlags::IMAGE | DiagnosticFlags::BIGIMAGE,
+        GetCompositor()->DrawDiagnostics(diagnosticFlags | DiagnosticFlags::BIGIMAGE,
                                          rect, aClipRect, aTransform, mFlashCounter);
       } while (it->NextTile());
       it->EndBigImageIteration();
       // layer border
-      GetCompositor()->DrawDiagnostics(DiagnosticFlags::IMAGE, pictureRect,
+      GetCompositor()->DrawDiagnostics(diagnosticFlags, pictureRect,
                                        aClipRect, aTransform, mFlashCounter);
     } else {
-      IntSize textureSize = img->mTextureSource->GetSize();
+      IntSize textureSize = mCurrentTextureSource->GetSize();
       effect->mTextureCoords = Rect(Float(img->mPictureRect.x) / textureSize.width,
                                     Float(img->mPictureRect.y) / textureSize.height,
                                     Float(img->mPictureRect.width) / textureSize.width,
                                     Float(img->mPictureRect.height) / textureSize.height);
 
-      if (img->mFrontBuffer->GetFlags() & TextureFlags::ORIGIN_BOTTOM_LEFT) {
+      if (img->mTextureHost->GetFlags() & TextureFlags::ORIGIN_BOTTOM_LEFT) {
         effect->mTextureCoords.y = effect->mTextureCoords.YMost();
         effect->mTextureCoords.height = -effect->mTextureCoords.height;
       }
 
       GetCompositor()->DrawQuad(pictureRect, aClipRect, aEffectChain,
                                 aOpacity, aTransform);
-      GetCompositor()->DrawDiagnostics(DiagnosticFlags::IMAGE,
+      GetCompositor()->DrawDiagnostics(diagnosticFlags,
                                        pictureRect, aClipRect,
                                        aTransform, mFlashCounter);
     }
@@ -407,9 +468,12 @@ ImageHost::SetCompositor(Compositor* aCompositor)
 {
   if (mCompositor != aCompositor) {
     for (auto& img : mImages) {
-      img.mFrontBuffer->SetCompositor(aCompositor);
+      img.mTextureHost->SetCompositor(aCompositor);
     }
   }
+  if (mImageHostOverlay) {
+    mImageHostOverlay->SetCompositor(aCompositor);
+  }
   CompositableHost::SetCompositor(aCompositor);
 }
 
@@ -423,9 +487,13 @@ ImageHost::PrintInfo(std::stringstream& aStream, const char* aPrefix)
   pfx += "  ";
   for (auto& img : mImages) {
     aStream << "\n";
-    img.mFrontBuffer->PrintInfo(aStream, pfx.get());
+    img.mTextureHost->PrintInfo(aStream, pfx.get());
     AppendToString(aStream, img.mPictureRect, " [picture-rect=", "]");
   }
+
+  if (mImageHostOverlay) {
+    mImageHostOverlay->PrintInfo(aStream, aPrefix);
+  }
 }
 
 void
@@ -437,7 +505,7 @@ ImageHost::Dump(std::stringstream& aStream,
     aStream << aPrefix;
     aStream << (aDumpHtml ? "<ul><li>TextureHost: "
                              : "TextureHost: ");
-    DumpTextureHost(aStream, img.mFrontBuffer);
+    DumpTextureHost(aStream, img.mTextureHost);
     aStream << (aDumpHtml ? " </li></ul> " : " ");
   }
 }
@@ -445,9 +513,14 @@ ImageHost::Dump(std::stringstream& aStream,
 LayerRenderState
 ImageHost::GetRenderState()
 {
+  if (mImageHostOverlay) {
+    return mImageHostOverlay->GetRenderState();
+  }
+
   TimedImage* img = ChooseImage();
   if (img) {
-    return img->mFrontBuffer->GetRenderState();
+    SetCurrentTextureHost(img->mTextureHost);
+    return img->mTextureHost->GetRenderState();
   }
   return LayerRenderState();
 }
@@ -455,9 +528,13 @@ ImageHost::GetRenderState()
 already_AddRefed<gfx::DataSourceSurface>
 ImageHost::GetAsSurface()
 {
+  if (mImageHostOverlay) {
+    return nullptr;
+  }
+
   TimedImage* img = ChooseImage();
   if (img) {
-    return img->mFrontBuffer->GetAsSurface();
+    return img->mTextureHost->GetAsSurface();
   }
   return nullptr;
 }
@@ -470,7 +547,10 @@ ImageHost::Lock()
   if (!img) {
     return false;
   }
-  if (!img->mFrontBuffer->Lock()) {
+
+  SetCurrentTextureHost(img->mTextureHost);
+
+  if (!mCurrentTextureHost->Lock()) {
     return false;
   }
   mLocked = true;
@@ -481,9 +561,9 @@ void
 ImageHost::Unlock()
 {
   MOZ_ASSERT(mLocked);
-  TimedImage* img = ChooseImage();
-  if (img) {
-    img->mFrontBuffer->Unlock();
+
+  if (mCurrentTextureHost) {
+    mCurrentTextureHost->Unlock();
   }
   mLocked = false;
 }
@@ -491,6 +571,10 @@ ImageHost::Unlock()
 IntSize
 ImageHost::GetImageSize() const
 {
+  if (mImageHostOverlay) {
+    return mImageHostOverlay->GetImageSize();
+  }
+
   const TimedImage* img = ChooseImage();
   if (img) {
     return IntSize(img->mPictureRect.width, img->mPictureRect.height);
@@ -498,24 +582,46 @@ ImageHost::GetImageSize() const
   return IntSize();
 }
 
+bool
+ImageHost::IsOpaque()
+{
+  const TimedImage* img = ChooseImage();
+  if (!img) {
+    return false;
+  }
+
+  if (img->mPictureRect.width == 0 ||
+      img->mPictureRect.height == 0 ||
+      !img->mTextureHost) {
+    return false;
+  }
+
+  gfx::SurfaceFormat format = img->mTextureHost->GetFormat();
+  if (gfx::IsOpaque(format)) {
+    return true;
+  }
+  return false;
+}
+
 already_AddRefed<TexturedEffect>
-ImageHost::GenEffect(const gfx::Filter& aFilter)
+ImageHost::GenEffect(const gfx::SamplingFilter aSamplingFilter)
 {
   TimedImage* img = ChooseImage();
   if (!img) {
     return nullptr;
   }
-  if (!img->mFrontBuffer->BindTextureSource(img->mTextureSource)) {
+  SetCurrentTextureHost(img->mTextureHost);
+  if (!mCurrentTextureHost->BindTextureSource(mCurrentTextureSource)) {
     return nullptr;
   }
   bool isAlphaPremultiplied = true;
-  if (img->mFrontBuffer->GetFlags() & TextureFlags::NON_PREMULTIPLIED) {
+  if (mCurrentTextureHost->GetFlags() & TextureFlags::NON_PREMULTIPLIED) {
     isAlphaPremultiplied = false;
   }
 
-  return CreateTexturedEffect(img->mFrontBuffer->GetFormat(),
-                              img->mTextureSource,
-                              aFilter,
+  return CreateTexturedEffect(mCurrentTextureHost->GetReadFormat(),
+                              mCurrentTextureSource,
+                              aSamplingFilter,
                               isAlphaPremultiplied,
                               GetRenderState());
 }
@@ -532,31 +638,60 @@ ImageHost::SetImageContainer(ImageContainerParent* aImageContainer)
   }
 }
 
-#ifdef MOZ_WIDGET_GONK
-ImageHostOverlay::ImageHostOverlay(const TextureInfo& aTextureInfo)
-  : CompositableHost(aTextureInfo)
+ImageHostOverlay::ImageHostOverlay()
 {
+  MOZ_COUNT_CTOR(ImageHostOverlay);
 }
 
 ImageHostOverlay::~ImageHostOverlay()
 {
+  if (mCompositor) {
+    mCompositor->RemoveImageHostOverlay(this);
+  }
+  MOZ_COUNT_DTOR(ImageHostOverlay);
+}
+
+/* static */ bool
+ImageHostOverlay::IsValid(OverlaySource aOverlay)
+{
+  if ((aOverlay.handle().type() == OverlayHandle::Tint32_t) &&
+      aOverlay.handle().get_int32_t() != INVALID_OVERLAY) {
+    return true;
+  } else if (aOverlay.handle().type() == OverlayHandle::TGonkNativeHandle) {
+    return true;
+  }
+  return false;
 }
 
 void
-ImageHostOverlay::Composite(LayerComposite* aLayer,
+ImageHostOverlay::SetCompositor(Compositor* aCompositor)
+{
+  if (mCompositor && (mCompositor != aCompositor)) {
+    mCompositor->RemoveImageHostOverlay(this);
+  }
+  if (aCompositor) {
+    aCompositor->AddImageHostOverlay(this);
+  }
+  mCompositor = aCompositor;
+}
+
+void
+ImageHostOverlay::Composite(Compositor* aCompositor,
+                            uint32_t aFlashCounter,
+                            LayerComposite* aLayer,
                             EffectChain& aEffectChain,
                             float aOpacity,
                             const gfx::Matrix4x4& aTransform,
-                            const gfx::Filter& aFilter,
-                            const gfx::Rect& aClipRect,
+                            const gfx::SamplingFilter aSamplingFilter,
+                            const gfx::IntRect& aClipRect,
                             const nsIntRegion* aVisibleRegion)
 {
-  if (!GetCompositor()) {
+  MOZ_ASSERT(mCompositor == aCompositor);
+
+  if (mOverlay.handle().type() == OverlayHandle::Tnull_t) {
     return;
   }
 
-  if (mOverlay.handle().type() == OverlayHandle::Tnull_t)
-    return;
   Color hollow(0.0f, 0.0f, 0.0f, 0.0f);
   aEffectChain.mPrimaryEffect = new EffectSolidColor(hollow);
   aEffectChain.mSecondaryEffects[EffectTypes::BLEND_MODE] = new EffectBlendMode(CompositionOp::OP_SOURCE);
@@ -567,18 +702,24 @@ ImageHostOverlay::Composite(LayerComposite* aLayer,
   rect.SetRect(mPictureRect.x, mPictureRect.y,
                mPictureRect.width, mPictureRect.height);
 
-  mCompositor->DrawQuad(rect, aClipRect, aEffectChain, aOpacity, aTransform);
-  mCompositor->DrawDiagnostics(DiagnosticFlags::IMAGE | DiagnosticFlags::BIGIMAGE,
-                               rect, aClipRect, aTransform, mFlashCounter);
+  aCompositor->DrawQuad(rect, aClipRect, aEffectChain, aOpacity, aTransform);
+  aCompositor->DrawDiagnostics(DiagnosticFlags::IMAGE | DiagnosticFlags::BIGIMAGE,
+                               rect, aClipRect, aTransform, aFlashCounter);
 }
 
 LayerRenderState
 ImageHostOverlay::GetRenderState()
 {
   LayerRenderState state;
+#ifdef MOZ_WIDGET_GONK
   if (mOverlay.handle().type() == OverlayHandle::Tint32_t) {
     state.SetOverlayId(mOverlay.handle().get_int32_t());
+  } else if (mOverlay.handle().type() == OverlayHandle::TGonkNativeHandle) {
+    state.SetSidebandStream(mOverlay.handle().get_GonkNativeHandle());
   }
+  state.mSize.width = mPictureRect.Width();
+  state.mSize.height = mPictureRect.Height();
+#endif
   return state;
 }
 
@@ -600,7 +741,7 @@ void
 ImageHostOverlay::PrintInfo(std::stringstream& aStream, const char* aPrefix)
 {
   aStream << aPrefix;
-  aStream << nsPrintfCString("ImageHost (0x%p)", this).get();
+  aStream << nsPrintfCString("ImageHostOverlay (0x%p)", this).get();
 
   AppendToString(aStream, mPictureRect, " [picture-rect=", "]");
 
@@ -611,6 +752,5 @@ ImageHostOverlay::PrintInfo(std::stringstream& aStream, const char* aPrefix)
   }
 }
 
-#endif
 } // namespace layers
 } // namespace mozilla
