@@ -4,7 +4,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "gfxTeeSurface.h"
-#include "nsAutoPtr.h"
 #include "nsTArray.h"
 
 #include "cairo-tee.h"
@@ -28,18 +27,18 @@ gfxTeeSurface::gfxTeeSurface(gfxASurface **aSurfaces, int32_t aSurfaceCount)
 const mozilla::gfx::IntSize
 gfxTeeSurface::GetSize() const
 {
-    nsRefPtr<gfxASurface> master = Wrap(cairo_tee_surface_index(mSurface, 0));
+    RefPtr<gfxASurface> master = Wrap(cairo_tee_surface_index(mSurface, 0));
     return master->GetSize();
 }
 
 void
-gfxTeeSurface::GetSurfaces(nsTArray<nsRefPtr<gfxASurface> >* aSurfaces)
+gfxTeeSurface::GetSurfaces(nsTArray<RefPtr<gfxASurface> >* aSurfaces)
 {
     for (int32_t i = 0; ; ++i) {
         cairo_surface_t *csurf = cairo_tee_surface_index(mSurface, i);
         if (cairo_surface_status(csurf))
             break;
-        nsRefPtr<gfxASurface> *elem = aSurfaces->AppendElement();
+        RefPtr<gfxASurface> *elem = aSurfaces->AppendElement();
         if (!elem)
             return;
         *elem = Wrap(csurf);
