@@ -50,7 +50,7 @@ Vibrate(const nsTArray<uint32_t>& pattern, const WindowIdentifier &id)
 {
   HAL_LOG("Vibrate: Sending to parent process.");
 
-  AutoInfallibleTArray<uint32_t, 8> p(pattern);
+  AutoTArray<uint32_t, 8> p(pattern);
 
   WindowIdentifier newID(id);
   newID.AppendProcessID();
@@ -191,7 +191,7 @@ SetScreenBrightness(double aBrightness)
   Hal()->SendSetScreenBrightness(aBrightness);
 }
 
-void 
+void
 AdjustSystemClock(int64_t aDeltaMilliseconds)
 {
   Hal()->SendAdjustSystemClock(aDeltaMilliseconds);
@@ -201,7 +201,7 @@ void
 SetTimezone(const nsCString& aTimezoneSpec)
 {
   Hal()->SendSetTimezone(nsCString(aTimezoneSpec));
-} 
+}
 
 nsCString
 GetTimezone()
@@ -322,8 +322,8 @@ GetCurrentSwitchState(SwitchDevice aDevice)
 void
 NotifySwitchStateFromInputDevice(SwitchDevice aDevice, SwitchState aState)
 {
-  unused << aDevice;
-  unused << aState;
+  Unused << aDevice;
+  Unused << aState;
   NS_RUNTIMEABORT("Only the main process may notify switch state change.");
 }
 
@@ -466,6 +466,23 @@ bool IsHeadphoneEventFromInputDev()
   return false;
 }
 
+nsresult StartSystemService(const char* aSvcName, const char* aArgs)
+{
+  NS_RUNTIMEABORT("System services cannot be controlled from sandboxed contexts.");
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+void StopSystemService(const char* aSvcName)
+{
+  NS_RUNTIMEABORT("System services cannot be controlled from sandboxed contexts.");
+}
+
+bool SystemServiceIsRunning(const char* aSvcName)
+{
+  NS_RUNTIMEABORT("System services cannot be controlled from sandboxed contexts.");
+  return false;
+}
+
 class HalParent : public PHalParent
                 , public BatteryObserver
                 , public NetworkObserver
@@ -504,10 +521,12 @@ public:
               PBrowserParent *browserParent) override
   {
     // We give all content vibration permission.
-    TabParent *tabParent = TabParent::GetFrom(browserParent);
+    //    TabParent *tabParent = TabParent::GetFrom(browserParent);
+    /* xxxkhuey wtf
     nsCOMPtr<nsIDOMWindow> window =
       do_QueryInterface(tabParent->GetBrowserDOMWindow());
-    WindowIdentifier newID(id, window);
+    */
+    WindowIdentifier newID(id, nullptr);
     hal::Vibrate(pattern, newID);
     return true;
   }
@@ -516,10 +535,12 @@ public:
   RecvCancelVibrate(InfallibleTArray<uint64_t> &&id,
                     PBrowserParent *browserParent) override
   {
-    TabParent *tabParent = TabParent::GetFrom(browserParent);
+    //TabParent *tabParent = TabParent::GetFrom(browserParent);
+    /* XXXkhuey wtf
     nsCOMPtr<nsIDOMWindow> window =
-      do_QueryInterface(tabParent->GetBrowserDOMWindow());
-    WindowIdentifier newID(id, window);
+      tabParent->GetBrowserDOMWindow();
+    */
+    WindowIdentifier newID(id, nullptr);
     hal::CancelVibrate(newID);
     return true;
   }
@@ -545,7 +566,7 @@ public:
   }
 
   void Notify(const BatteryInformation& aBatteryInfo) override {
-    unused << SendNotifyBatteryChange(aBatteryInfo);
+    Unused << SendNotifyBatteryChange(aBatteryInfo);
   }
 
   virtual bool
@@ -568,7 +589,7 @@ public:
   }
 
   void Notify(const NetworkInformation& aNetworkInfo) override {
-    unused << SendNotifyNetworkChange(aNetworkInfo);
+    Unused << SendNotifyNetworkChange(aNetworkInfo);
   }
 
   virtual bool
@@ -610,7 +631,7 @@ public:
   }
 
   void Notify(const ScreenConfiguration& aScreenConfiguration) override {
-    unused << SendNotifyScreenConfigurationChange(aScreenConfiguration);
+    Unused << SendNotifyScreenConfigurationChange(aScreenConfiguration);
   }
 
   virtual bool
@@ -703,14 +724,14 @@ public:
     return true;
   }
 
-  virtual bool 
+  virtual bool
   RecvSetTimezone(const nsCString& aTimezoneSpec) override
   {
     if (!AssertAppProcessPermission(this, "time")) {
       return false;
     }
     hal::SetTimezone(aTimezoneSpec);
-    return true;  
+    return true;
   }
 
   virtual bool
@@ -768,15 +789,15 @@ public:
     hal::RegisterSensorObserver(aSensor, this);
     return true;
   }
-   
+
   virtual bool
   RecvDisableSensorNotifications(const SensorType &aSensor) override {
     hal::UnregisterSensorObserver(aSensor, this);
     return true;
   }
-  
+
   void Notify(const SensorData& aSensorData) override {
-    unused << SendNotifySensorChange(aSensorData);
+    Unused << SendNotifySensorChange(aSensorData);
   }
 
   virtual bool
@@ -799,7 +820,7 @@ public:
     hal::RegisterWakeLockObserver(this);
     return true;
   }
-   
+
   virtual bool
   RecvDisableWakeLockNotifications() override
   {
@@ -813,10 +834,10 @@ public:
     hal::GetWakeLockInfo(aTopic, aWakeLockInfo);
     return true;
   }
-  
+
   void Notify(const WakeLockInformation& aWakeLockInfo) override
   {
-    unused << SendNotifyWakeLockChange(aWakeLockInfo);
+    Unused << SendNotifyWakeLockChange(aWakeLockInfo);
   }
 
   virtual bool
@@ -836,7 +857,7 @@ public:
 
   void Notify(const SwitchEvent& aSwitchEvent) override
   {
-    unused << SendNotifySwitchChange(aSwitchEvent);
+    Unused << SendNotifySwitchChange(aSwitchEvent);
   }
 
   virtual bool
@@ -849,12 +870,12 @@ public:
 
   void Notify(const int64_t& aClockDeltaMS) override
   {
-    unused << SendNotifySystemClockChange(aClockDeltaMS);
+    Unused << SendNotifySystemClockChange(aClockDeltaMS);
   }
 
   void Notify(const SystemTimezoneChangeInformation& aSystemTimezoneChangeInfo) override
   {
-    unused << SendNotifySystemTimezoneChange(aSystemTimezoneChangeInfo);
+    Unused << SendNotifySystemTimezoneChange(aSystemTimezoneChangeInfo);
   }
 
   virtual bool
@@ -951,7 +972,7 @@ public:
 bool
 HalChild::RecvNotifySensorChange(const hal::SensorData &aSensorData) {
   hal::NotifySensorChange(aSensorData);
-  
+
   return true;
 }
 
