@@ -18,13 +18,14 @@
 static bool gDisableOptimize = false;
 
 #include "GeckoProfiler.h"
-#include "mozilla/Likely.h"
 #include "MainThreadUtils.h"
-#include "mozilla/MemoryReporting.h"
-#include "nsMargin.h"
-#include "nsThreadUtils.h"
 #include "mozilla/CheckedInt.h"
 #include "mozilla/gfx/Tools.h"
+#include "mozilla/Likely.h"
+#include "mozilla/MemoryReporting.h"
+#include "mozilla/Telemetry.h"
+#include "nsMargin.h"
+#include "nsThreadUtils.h"
 
 
 namespace mozilla {
@@ -33,8 +34,6 @@ using namespace gfx;
 
 namespace image {
 
-static UserDataKey kVolatileBuffer;
-
 static void
 VolatileBufferRelease(void* vbuf)
 {
@@ -58,14 +57,17 @@ CreateLockedSurface(VolatileBuffer* vbuf,
   MOZ_ASSERT(!vbufptr->WasBufferPurged(), "Expected image data!");
 
   int32_t stride = VolatileSurfaceStride(size, format);
+
+  // The VolatileBufferPtr is held by this DataSourceSurface.
   RefPtr<DataSourceSurface> surf =
-    Factory::CreateWrappingDataSourceSurface(*vbufptr, stride, size, format);
+    Factory::CreateWrappingDataSourceSurface(*vbufptr, stride, size, format,
+                                             &VolatileBufferRelease,
+                                             static_cast<void*>(vbufptr));
   if (!surf) {
     delete vbufptr;
     return nullptr;
   }
 
-  surf->AddUserData(&kVolatileBuffer, vbufptr, VolatileBufferRelease);
   return surf.forget();
 }
 
@@ -75,7 +77,7 @@ AllocateBufferForImage(const IntSize& size, SurfaceFormat format)
   int32_t stride = VolatileSurfaceStride(size, format);
   RefPtr<VolatileBuffer> buf = new VolatileBuffer();
   if (buf->Init(stride * size.height,
-                1 << gfxAlphaRecovery::GoodAlignmentLog2())) {
+                size_t(1) << gfxAlphaRecovery::GoodAlignmentLog2())) {
     return buf.forget();
   }
 
@@ -140,6 +142,7 @@ imgFrame::imgFrame()
   , mBlendMethod(BlendMethod::OVER)
   , mHasNoAlpha(false)
   , mAborted(false)
+  , mFinished(false)
   , mOptimizable(false)
   , mPalettedImageData(nullptr)
   , mPaletteDepth(0)
@@ -160,7 +163,8 @@ imgFrame::~imgFrame()
 {
 #ifdef DEBUG
   MonitorAutoLock lock(mMonitor);
-  MOZ_ASSERT(mAborted || IsImageCompleteInternal());
+  MOZ_ASSERT(mAborted || AreAllPixelsWritten());
+  MOZ_ASSERT(mAborted || mFinished);
 #endif
 
   free(mPalettedImageData);
@@ -183,8 +187,7 @@ imgFrame::InitForDecoder(const nsIntSize& aImageSize,
   }
 
   mImageSize = aImageSize;
-  mOffset.MoveTo(aRect.x, aRect.y);
-  mSize.SizeTo(aRect.width, aRect.height);
+  mFrameRect = aRect;
 
   mFormat = aFormat;
   mPaletteDepth = aPaletteDepth;
@@ -201,27 +204,26 @@ imgFrame::InitForDecoder(const nsIntSize& aImageSize,
 
     // Use the fallible allocator here. Paletted images always use 1 byte per
     // pixel, so calculating the amount of memory we need is straightforward.
-    mPalettedImageData =
-      static_cast<uint8_t*>(malloc(PaletteDataLength() +
-                                   (mSize.width * mSize.height)));
+    size_t dataSize = PaletteDataLength() + mFrameRect.Area();
+    mPalettedImageData = static_cast<uint8_t*>(calloc(dataSize, sizeof(uint8_t)));
     if (!mPalettedImageData) {
-      NS_WARNING("malloc for paletted image data should succeed");
+      NS_WARNING("Call to calloc for paletted image data should succeed");
     }
     NS_ENSURE_TRUE(mPalettedImageData, NS_ERROR_OUT_OF_MEMORY);
   } else {
     MOZ_ASSERT(!mImageSurface, "Called imgFrame::InitForDecoder() twice?");
 
-    mVBuf = AllocateBufferForImage(mSize, mFormat);
+    mVBuf = AllocateBufferForImage(mFrameRect.Size(), mFormat);
     if (!mVBuf) {
       mAborted = true;
       return NS_ERROR_OUT_OF_MEMORY;
     }
     if (mVBuf->OnHeap()) {
-      int32_t stride = VolatileSurfaceStride(mSize, mFormat);
+      int32_t stride = VolatileSurfaceStride(mFrameRect.Size(), mFormat);
       VolatileBufferPtr<uint8_t> ptr(mVBuf);
-      memset(ptr, 0, stride * mSize.height);
+      memset(ptr, 0, stride * mFrameRect.height);
     }
-    mImageSurface = CreateLockedSurface(mVBuf, mSize, mFormat);
+    mImageSurface = CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
 
     if (!mImageSurface) {
       NS_WARNING("Failed to create VolatileDataSourceSurface");
@@ -237,7 +239,7 @@ nsresult
 imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
                            const nsIntSize& aSize,
                            const SurfaceFormat aFormat,
-                           GraphicsFilter aFilter,
+                           SamplingFilter aSamplingFilter,
                            uint32_t aImageFlags)
 {
   // Assert for properties that should be verified by decoders,
@@ -249,8 +251,7 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
   }
 
   mImageSize = aSize;
-  mOffset.MoveTo(0, 0);
-  mSize.SizeTo(aSize.width, aSize.height);
+  mFrameRect = IntRect(IntPoint(0, 0), aSize);
 
   mFormat = aFormat;
   mPaletteDepth = 0;
@@ -265,25 +266,25 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
     // get away with using volatile buffers.
     MOZ_ASSERT(!mImageSurface, "Called imgFrame::InitWithDrawable() twice?");
 
-    mVBuf = AllocateBufferForImage(mSize, mFormat);
+    mVBuf = AllocateBufferForImage(mFrameRect.Size(), mFormat);
     if (!mVBuf) {
       mAborted = true;
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    int32_t stride = VolatileSurfaceStride(mSize, mFormat);
+    int32_t stride = VolatileSurfaceStride(mFrameRect.Size(), mFormat);
     VolatileBufferPtr<uint8_t> ptr(mVBuf);
     if (!ptr) {
       mAborted = true;
       return NS_ERROR_OUT_OF_MEMORY;
     }
     if (mVBuf->OnHeap()) {
-      memset(ptr, 0, stride * mSize.height);
+      memset(ptr, 0, stride * mFrameRect.height);
     }
-    mImageSurface = CreateLockedSurface(mVBuf, mSize, mFormat);
+    mImageSurface = CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
 
     target = gfxPlatform::GetPlatform()->
-      CreateDrawTargetForData(ptr, mSize, stride, mFormat);
+      CreateDrawTargetForData(ptr, mFrameRect.Size(), stride, mFormat);
   } else {
     // We can't use data surfaces for content, so we'll create an offscreen
     // surface instead.  This means if someone later calls RawAccessRef(), we
@@ -292,20 +293,20 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
     MOZ_ASSERT(!mOptSurface, "Called imgFrame::InitWithDrawable() twice?");
 
     target = gfxPlatform::GetPlatform()->
-      CreateOffscreenContentDrawTarget(mSize, mFormat);
+      CreateOffscreenContentDrawTarget(mFrameRect.Size(), mFormat);
   }
 
-  if (!target) {
+  if (!target || !target->IsValid()) {
     mAborted = true;
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
   // Draw using the drawable the caller provided.
-  nsIntRect imageRect(0, 0, mSize.width, mSize.height);
-  nsRefPtr<gfxContext> ctx = new gfxContext(target);
-  gfxUtils::DrawPixelSnapped(ctx, aDrawable, mSize,
-                             ImageRegion::Create(imageRect),
-                             mFormat, aFilter, aImageFlags);
+  RefPtr<gfxContext> ctx = gfxContext::ForDrawTarget(target);
+  MOZ_ASSERT(ctx);  // Already checked the draw target above.
+  gfxUtils::DrawPixelSnapped(ctx, aDrawable, mFrameRect.Size(),
+                             ImageRegion::Create(ThebesRect(mFrameRect)),
+                             mFormat, aSamplingFilter, aImageFlags);
 
   if (canUseDataSurface && !mImageSurface) {
     NS_WARNING("Failed to create VolatileDataSourceSurface");
@@ -321,7 +322,12 @@ imgFrame::InitWithDrawable(gfxDrawable* aDrawable,
 
   // If we reach this point, we should regard ourselves as complete.
   mDecoded = GetRect();
-  MOZ_ASSERT(IsImageComplete());
+  mFinished = true;
+
+#ifdef DEBUG
+  MonitorAutoLock lock(mMonitor);
+  MOZ_ASSERT(AreAllPixelsWritten());
+#endif
 
   return NS_OK;
 }
@@ -356,10 +362,10 @@ imgFrame::Optimize()
   /* Figure out if the entire image is a constant color */
 
   if (gfxPrefs::ImageSingleColorOptimizationEnabled() &&
-      mImageSurface->Stride() == mSize.width * 4) {
+      mImageSurface->Stride() == mFrameRect.width * 4) {
     uint32_t* imgData = (uint32_t*) ((uint8_t*) mVBufPtr);
     uint32_t firstPixel = * (uint32_t*) imgData;
-    uint32_t pixelCount = mSize.width * mSize.height + 1;
+    uint32_t pixelCount = mFrameRect.Area() + 1;
 
     while (--pixelCount && *imgData++ == firstPixel)
       ;
@@ -391,20 +397,27 @@ imgFrame::Optimize()
     // moment
   }
 
+  const bool usedSingleColorOptimizationUsefully = mSinglePixel &&
+                                                   mFrameRect.Area() > 1;
+  Telemetry::Accumulate(Telemetry::IMAGE_OPTIMIZE_TO_SINGLE_COLOR_USED,
+                        usedSingleColorOptimizationUsefully);
+
 #ifdef ANDROID
   SurfaceFormat optFormat = gfxPlatform::GetPlatform()
     ->Optimal2DFormatForContent(gfxContentType::COLOR);
 
   if (mFormat != SurfaceFormat::B8G8R8A8 &&
-      optFormat == SurfaceFormat::R5G6B5) {
+      optFormat == SurfaceFormat::R5G6B5_UINT16) {
+    Telemetry::Accumulate(Telemetry::IMAGE_OPTIMIZE_TO_565_USED, true);
+
     RefPtr<VolatileBuffer> buf =
-      AllocateBufferForImage(mSize, optFormat);
+      AllocateBufferForImage(mFrameRect.Size(), optFormat);
     if (!buf) {
       return NS_OK;
     }
 
     RefPtr<DataSourceSurface> surf =
-      CreateLockedSurface(buf, mSize, optFormat);
+      CreateLockedSurface(buf, mFrameRect.Size(), optFormat);
     if (!surf) {
       return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -418,7 +431,7 @@ imgFrame::Optimize()
     RefPtr<DrawTarget> target =
       Factory::CreateDrawTargetForData(BackendType::CAIRO,
                                        mapping.mData,
-                                       mSize,
+                                       mFrameRect.Size(),
                                        mapping.mStride,
                                        optFormat);
 
@@ -426,7 +439,7 @@ imgFrame::Optimize()
       gfxWarning() << "imgFrame::Optimize failed in CreateDrawTargetForData";
       return NS_ERROR_OUT_OF_MEMORY;
     }
-    Rect rect(0, 0, mSize.width, mSize.height);
+    Rect rect(0, 0, mFrameRect.width, mFrameRect.height);
     target->DrawSurface(mImageSurface, rect, rect);
     target->Flush();
     surf->Unmap();
@@ -522,7 +535,7 @@ imgFrame::SurfaceForDrawing(bool               aDoPadding,
                        DrawOptions(1.0f, CompositionOp::OP_SOURCE));
     } else {
       SurfacePattern pattern(aSurface,
-                             ExtendMode::REPEAT,
+                             aRegion.GetExtendMode(),
                              Matrix::Translation(mDecoded.x, mDecoded.y));
       target->FillRect(ToRect(aRegion.Intersect(available).Rect()), pattern);
     }
@@ -537,15 +550,15 @@ imgFrame::SurfaceForDrawing(bool               aDoPadding,
   gfxPoint paddingTopLeft(aPadding.left, aPadding.top);
   aRegion = aRegion.Intersect(available) - paddingTopLeft;
   aContext->Multiply(gfxMatrix::Translation(paddingTopLeft));
-  aImageRect = gfxRect(0, 0, mSize.width, mSize.height);
+  aImageRect = gfxRect(0, 0, mFrameRect.width, mFrameRect.height);
 
-  gfxIntSize availableSize(mDecoded.width, mDecoded.height);
+  IntSize availableSize(mDecoded.width, mDecoded.height);
   return SurfaceWithFormat(new gfxSurfaceDrawable(aSurface, availableSize),
                            mFormat);
 }
 
 bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
-                    GraphicsFilter aFilter, uint32_t aImageFlags)
+                    SamplingFilter aSamplingFilter, uint32_t aImageFlags)
 {
   PROFILER_LABEL("imgFrame", "Draw",
     js::ProfileEntry::Category::GRAPHICS);
@@ -559,13 +572,13 @@ bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
 
   MonitorAutoLock lock(mMonitor);
 
-  nsIntMargin padding(mOffset.y,
-                      mImageSize.width - (mOffset.x + mSize.width),
-                      mImageSize.height - (mOffset.y + mSize.height),
-                      mOffset.x);
+  nsIntMargin padding(mFrameRect.y,
+                      mImageSize.width - mFrameRect.XMost(),
+                      mImageSize.height - mFrameRect.YMost(),
+                      mFrameRect.x);
 
   bool doPadding = padding != nsIntMargin(0,0,0,0);
-  bool doPartialDecode = !IsImageCompleteInternal();
+  bool doPartialDecode = !AreAllPixelsWritten();
 
   if (mSinglePixel && !doPadding && !doPartialDecode) {
     if (mSinglePixelColor.a == 0.0) {
@@ -574,8 +587,7 @@ bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
     RefPtr<DrawTarget> dt = aContext->GetDrawTarget();
     dt->FillRect(ToRect(aRegion.Rect()),
                  ColorPattern(mSinglePixelColor),
-                 DrawOptions(1.0f,
-                             CompositionOpForOp(aContext->CurrentOperator())));
+                 DrawOptions(1.0f, aContext->CurrentOp()));
     return true;
   }
 
@@ -587,6 +599,7 @@ bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
   gfxRect imageRect(0, 0, mImageSize.width, mImageSize.height);
   bool doTile = !imageRect.Contains(aRegion.Rect()) &&
                 !(aImageFlags & imgIContainer::FLAG_CLAMP);
+
   ImageRegion region(aRegion);
   // SurfaceForDrawing changes the current transform, and we need it to still
   // be changed when we call gfxUtils::DrawPixelSnapped. We still need to
@@ -602,7 +615,7 @@ bool imgFrame::Draw(gfxContext* aContext, const ImageRegion& aRegion,
   if (surfaceResult.IsValid()) {
     gfxUtils::DrawPixelSnapped(aContext, surfaceResult.mDrawable,
                                imageRect.Size(), region, surfaceResult.mFormat,
-                               aFilter, aImageFlags);
+                               aSamplingFilter, aImageFlags);
   }
   return true;
 }
@@ -621,15 +634,9 @@ imgFrame::ImageUpdatedInternal(const nsIntRect& aUpdateRect)
 
   mDecoded.UnionRect(mDecoded, aUpdateRect);
 
-  // clamp to bounds, in case someone sends a bogus updateRect (I'm looking at
-  // you, gif decoder)
-  nsIntRect boundsRect(mOffset, mSize);
-  mDecoded.IntersectRect(mDecoded, boundsRect);
-
-  // If the image is now complete, wake up anyone who's waiting.
-  if (IsImageCompleteInternal()) {
-    mMonitor.NotifyAll();
-  }
+  // Clamp to the frame rect to ensure that decoder bugs don't result in a
+  // decoded rect that extends outside the bounds of the frame rect.
+  mDecoded.IntersectRect(mDecoded, mFrameRect);
 
   return NS_OK;
 }
@@ -643,7 +650,7 @@ imgFrame::Finish(Opacity aFrameOpacity /* = Opacity::SOME_TRANSPARENCY */,
   MonitorAutoLock lock(mMonitor);
   MOZ_ASSERT(mLockCount > 0, "Image data should be locked");
 
-  if (aFrameOpacity == Opacity::OPAQUE) {
+  if (aFrameOpacity == Opacity::FULLY_OPAQUE) {
     mHasNoAlpha = true;
   }
 
@@ -651,31 +658,10 @@ imgFrame::Finish(Opacity aFrameOpacity /* = Opacity::SOME_TRANSPARENCY */,
   mTimeout = aRawTimeout;
   mBlendMethod = aBlendMethod;
   ImageUpdatedInternal(GetRect());
-}
+  mFinished = true;
 
-nsIntRect
-imgFrame::GetRect() const
-{
-  return gfx::IntRect(mOffset, mSize);
-}
-
-int32_t
-imgFrame::GetStride() const
-{
-  mMonitor.AssertCurrentThreadOwns();
-
-  if (mImageSurface) {
-    return mImageSurface->Stride();
-  }
-
-  return VolatileSurfaceStride(mSize, mFormat);
-}
-
-SurfaceFormat
-imgFrame::GetFormat() const
-{
-  MonitorAutoLock lock(mMonitor);
-  return mFormat;
+  // The image is now complete, wake up anyone who's waiting.
+  mMonitor.NotifyAll();
 }
 
 uint32_t
@@ -684,11 +670,11 @@ imgFrame::GetImageBytesPerRow() const
   mMonitor.AssertCurrentThreadOwns();
 
   if (mVBuf) {
-    return mSize.width * BytesPerPixel(mFormat);
+    return mFrameRect.width * BytesPerPixel(mFormat);
   }
 
   if (mPaletteDepth) {
-    return mSize.width;
+    return mFrameRect.width;
   }
 
   return 0;
@@ -697,7 +683,7 @@ imgFrame::GetImageBytesPerRow() const
 uint32_t
 imgFrame::GetImageDataLength() const
 {
-  return GetImageBytesPerRow() * mSize.height;
+  return GetImageBytesPerRow() * mFrameRect.height;
 }
 
 void
@@ -809,7 +795,7 @@ imgFrame::AssertImageDataLocked() const
 #endif
 }
 
-class UnlockImageDataRunnable : public nsRunnable
+class UnlockImageDataRunnable : public Runnable
 {
 public:
   explicit UnlockImageDataRunnable(imgFrame* aTarget)
@@ -821,7 +807,7 @@ public:
   NS_IMETHOD Run() { return mTarget->UnlockImageData(); }
 
 private:
-  nsRefPtr<imgFrame> mTarget;
+  RefPtr<imgFrame> mTarget;
 };
 
 nsresult
@@ -834,8 +820,8 @@ imgFrame::UnlockImageData()
     return NS_ERROR_FAILURE;
   }
 
-  MOZ_ASSERT(mLockCount > 1 || IsImageCompleteInternal() || mAborted,
-             "Should have marked complete or aborted before unlocking");
+  MOZ_ASSERT(mLockCount > 1 || mFinished || mAborted,
+             "Should have Finish()'d or aborted before unlocking");
 
   // If we're about to become unlocked, we don't need to hold on to our data
   // surface anymore. (But we don't need to do anything for paletted images,
@@ -853,7 +839,7 @@ imgFrame::UnlockImageData()
     // allows DrawTargets to avoid blending when drawing known opaque images.
     if (mHasNoAlpha && mFormat == SurfaceFormat::B8G8R8A8 && mImageSurface) {
       mFormat = SurfaceFormat::B8G8R8X8;
-      mImageSurface = CreateLockedSurface(mVBuf, mSize, mFormat);
+      mImageSurface = CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
     }
 
     // Convert the data surface to a GPU surface or a single color if possible.
@@ -926,24 +912,7 @@ imgFrame::GetSurfaceInternal()
     return nullptr;
   }
 
-  return CreateLockedSurface(mVBuf, mSize, mFormat);
-}
-
-already_AddRefed<DrawTarget>
-imgFrame::GetDrawTarget()
-{
-  MonitorAutoLock lock(mMonitor);
-
-  uint8_t* data;
-  uint32_t length;
-  GetImageDataInternal(&data, &length);
-  if (!data) {
-    return nullptr;
-  }
-
-  int32_t stride = GetStride();
-  return gfxPlatform::GetPlatform()->
-    CreateDrawTargetForData(data, mSize, stride, mFormat);
+  return CreateLockedSurface(mVBuf, mFrameRect.Size(), mFormat);
 }
 
 AnimationData
@@ -966,20 +935,6 @@ imgFrame::GetAnimationData() const
                        mBlendMethod, mDisposalMethod, hasAlpha);
 }
 
-ScalingData
-imgFrame::GetScalingData() const
-{
-  MonitorAutoLock lock(mMonitor);
-  MOZ_ASSERT(mLockCount > 0, "Image data should be locked");
-  MOZ_ASSERT(!GetIsPaletted(), "GetScalingData can't handle paletted images");
-
-  uint8_t* data;
-  uint32_t length;
-  GetImageDataInternal(&data, &length);
-
-  return ScalingData(data, mSize, GetImageBytesPerRow(), mFormat);
-}
-
 void
 imgFrame::Abort()
 {
@@ -992,20 +947,27 @@ imgFrame::Abort()
 }
 
 bool
-imgFrame::IsImageComplete() const
+imgFrame::IsAborted() const
 {
   MonitorAutoLock lock(mMonitor);
-  return IsImageCompleteInternal();
+  return mAborted;
+}
+
+bool
+imgFrame::IsFinished() const
+{
+  MonitorAutoLock lock(mMonitor);
+  return mFinished;
 }
 
 void
-imgFrame::WaitUntilComplete() const
+imgFrame::WaitUntilFinished() const
 {
   MonitorAutoLock lock(mMonitor);
 
   while (true) {
     // Return if we're aborted or complete.
-    if (mAborted || IsImageCompleteInternal()) {
+    if (mAborted || mFinished) {
       return;
     }
 
@@ -1015,11 +977,10 @@ imgFrame::WaitUntilComplete() const
 }
 
 bool
-imgFrame::IsImageCompleteInternal() const
+imgFrame::AreAllPixelsWritten() const
 {
   mMonitor.AssertCurrentThreadOwns();
-  return mDecoded.IsEqualInterior(nsIntRect(mOffset.x, mOffset.y,
-                                            mSize.width, mSize.height));
+  return mDecoded.IsEqualInterior(mFrameRect);
 }
 
 bool imgFrame::GetCompositingFailed() const
