@@ -49,7 +49,7 @@ class RequestBehaviour : public ProxyBehaviour
     mOwner = aOwner;
 
     if (mOwner) {
-      nsRefPtr<ProgressTracker> ownerProgressTracker = GetProgressTracker();
+      RefPtr<ProgressTracker> ownerProgressTracker = GetProgressTracker();
       mOwnerHasImage = ownerProgressTracker && ownerProgressTracker->HasImage();
     } else {
       mOwnerHasImage = false;
@@ -63,7 +63,7 @@ class RequestBehaviour : public ProxyBehaviour
   // imgRequestProxy::~imgRequestProxy unregisters the proxy as an observer
   // from whatever request it was registered with (if any). This, in turn,
   // means that imgRequest::mObservers will not have any stale pointers in it.
-  nsRefPtr<imgRequest> mOwner;
+  RefPtr<imgRequest> mOwner;
 
   bool mOwnerHasImage;
 };
@@ -74,7 +74,7 @@ RequestBehaviour::GetImage() const
   if (!mOwnerHasImage) {
     return nullptr;
   }
-  nsRefPtr<ProgressTracker> progressTracker = GetProgressTracker();
+  RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
   return progressTracker->GetImage();
 }
 
@@ -160,7 +160,7 @@ imgRequestProxy::Init(imgRequest* aOwner,
   NS_PRECONDITION(!GetOwner() && !mListener,
                   "imgRequestProxy is already initialized");
 
-  LOG_SCOPE_WITH_PARAM(GetImgLog(), "imgRequestProxy::Init", "request",
+  LOG_SCOPE_WITH_PARAM(gImgLog, "imgRequestProxy::Init", "request",
                        aOwner);
 
   MOZ_ASSERT(mAnimationConsumers == 0, "Cannot have animation before Init");
@@ -208,15 +208,6 @@ imgRequestProxy::ChangeOwner(imgRequest* aNewOwner)
   uint32_t oldAnimationConsumers = mAnimationConsumers;
   ClearAnimationConsumers();
 
-  // Were we decoded before?
-  bool wasDecoded = false;
-  nsRefPtr<ProgressTracker> progressTracker = GetProgressTracker();
-  if (progressTracker->HasImage() &&
-      progressTracker->GetImageStatus() &
-        imgIRequest::STATUS_FRAME_COMPLETE) {
-    wasDecoded = true;
-  }
-
   GetOwner()->RemoveProxy(this, NS_IMAGELIB_CHANGING_OWNER);
 
   mBehaviour->SetOwner(aNewOwner);
@@ -235,9 +226,9 @@ imgRequestProxy::ChangeOwner(imgRequest* aNewOwner)
 
   GetOwner()->AddProxy(this);
 
-  // If we were decoded, or if we'd previously requested a decode, request a
-  // decode on the new image
-  if (wasDecoded || mDecodeRequested) {
+  // If we'd previously requested a synchronous decode, request a decode on the
+  // new image.
+  if (mDecodeRequested) {
     StartDecoding();
   }
 
@@ -312,7 +303,7 @@ imgRequestProxy::Cancel(nsresult status)
     return NS_ERROR_FAILURE;
   }
 
-  LOG_SCOPE(GetImgLog(), "imgRequestProxy::Cancel");
+  LOG_SCOPE(gImgLog, "imgRequestProxy::Cancel");
 
   mCanceled = true;
 
@@ -343,7 +334,7 @@ imgRequestProxy::CancelAndForgetObserver(nsresult aStatus)
     return NS_ERROR_FAILURE;
   }
 
-  LOG_SCOPE(GetImgLog(), "imgRequestProxy::CancelAndForgetObserver");
+  LOG_SCOPE(gImgLog, "imgRequestProxy::CancelAndForgetObserver");
 
   mCanceled = true;
 
@@ -358,9 +349,7 @@ imgRequestProxy::CancelAndForgetObserver(nsresult aStatus)
   mIsInLoadGroup = oldIsInLoadGroup;
 
   if (mIsInLoadGroup) {
-    nsCOMPtr<nsIRunnable> ev =
-      NS_NewRunnableMethod(this, &imgRequestProxy::DoRemoveFromLoadGroup);
-    NS_DispatchToCurrentThread(ev);
+    NS_DispatchToCurrentThread(NewRunnableMethod(this, &imgRequestProxy::DoRemoveFromLoadGroup));
   }
 
   NullOutListener();
@@ -374,42 +363,23 @@ imgRequestProxy::StartDecoding()
   // Flag this, so we know to transfer the request if our owner changes
   mDecodeRequested = true;
 
-  nsRefPtr<Image> image = GetImage();
+  RefPtr<Image> image = GetImage();
   if (image) {
     return image->StartDecoding();
   }
 
   if (GetOwner()) {
-    GetOwner()->RequestDecode();
+    GetOwner()->StartDecoding();
   }
 
   return NS_OK;
 }
 
-NS_IMETHODIMP
-imgRequestProxy::RequestDecode()
-{
-  // Flag this, so we know to transfer the request if our owner changes
-  mDecodeRequested = true;
-
-  nsRefPtr<Image> image = GetImage();
-  if (image) {
-    return image->RequestDecode();
-  }
-
-  if (GetOwner()) {
-    GetOwner()->RequestDecode();
-  }
-
-  return NS_OK;
-}
-
-
 NS_IMETHODIMP
 imgRequestProxy::LockImage()
 {
   mLockCount++;
-  nsRefPtr<Image> image = GetImage();
+  RefPtr<Image> image = GetImage();
   if (image) {
     return image->LockImage();
   }
@@ -422,7 +392,7 @@ imgRequestProxy::UnlockImage()
   MOZ_ASSERT(mLockCount > 0, "calling unlock but no locks!");
 
   mLockCount--;
-  nsRefPtr<Image> image = GetImage();
+  RefPtr<Image> image = GetImage();
   if (image) {
     return image->UnlockImage();
   }
@@ -432,7 +402,7 @@ imgRequestProxy::UnlockImage()
 NS_IMETHODIMP
 imgRequestProxy::RequestDiscard()
 {
-  nsRefPtr<Image> image = GetImage();
+  RefPtr<Image> image = GetImage();
   if (image) {
     return image->RequestDiscard();
   }
@@ -443,7 +413,7 @@ NS_IMETHODIMP
 imgRequestProxy::IncrementAnimationConsumers()
 {
   mAnimationConsumers++;
-  nsRefPtr<Image> image = GetImage();
+  RefPtr<Image> image = GetImage();
   if (image) {
     image->IncrementAnimationConsumers();
   }
@@ -461,7 +431,7 @@ imgRequestProxy::DecrementAnimationConsumers()
   // early, but not the observer.)
   if (mAnimationConsumers > 0) {
     mAnimationConsumers--;
-    nsRefPtr<Image> image = GetImage();
+    RefPtr<Image> image = GetImage();
     if (image) {
       image->DecrementAnimationConsumers();
     }
@@ -525,7 +495,7 @@ imgRequestProxy::GetImage(imgIContainer** aImage)
   // that'll happen if we get Canceled before the owner instantiates its image
   // (because Canceling unregisters us as a listener on mOwner). If we're
   // in that situation, just grab the image off of mOwner.
-  nsRefPtr<Image> image = GetImage();
+  RefPtr<Image> image = GetImage();
   nsCOMPtr<imgIContainer> imageToReturn;
   if (image) {
     imageToReturn = do_QueryInterface(image);
@@ -545,7 +515,7 @@ imgRequestProxy::GetImage(imgIContainer** aImage)
 NS_IMETHODIMP
 imgRequestProxy::GetImageStatus(uint32_t* aStatus)
 {
-  nsRefPtr<ProgressTracker> progressTracker = GetProgressTracker();
+  RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
   *aStatus = progressTracker->GetImageStatus();
 
   return NS_OK;
@@ -628,7 +598,7 @@ imgRequestProxy* NewStaticProxy(imgRequestProxy* aThis)
 {
   nsCOMPtr<nsIPrincipal> currentPrincipal;
   aThis->GetImagePrincipal(getter_AddRefs(currentPrincipal));
-  nsRefPtr<Image> image = aThis->GetImage();
+  RefPtr<Image> image = aThis->GetImage();
   return new imgRequestProxyStatic(image, currentPrincipal);
 }
 
@@ -656,10 +626,10 @@ imgRequestProxy::PerformClone(imgINotificationObserver* aObserver,
 {
   NS_PRECONDITION(aClone, "Null out param");
 
-  LOG_SCOPE(GetImgLog(), "imgRequestProxy::Clone");
+  LOG_SCOPE(gImgLog, "imgRequestProxy::Clone");
 
   *aClone = nullptr;
-  nsRefPtr<imgRequestProxy> clone = aAllocFn(this);
+  RefPtr<imgRequestProxy> clone = aAllocFn(this);
 
   // It is important to call |SetLoadFlags()| before calling |Init()| because
   // |Init()| adds the request to the loadgroup.
@@ -806,7 +776,7 @@ imgRequestProxy::Notify(int32_t aType, const mozilla::gfx::IntRect* aRect)
   MOZ_ASSERT(aType != imgINotificationObserver::LOAD_COMPLETE,
              "Should call OnLoadComplete");
 
-  LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::Notify", "type",
+  LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::Notify", "type",
                       NotificationTypeToString(aType));
 
   if (!mListener || mCanceled) {
@@ -822,10 +792,10 @@ imgRequestProxy::Notify(int32_t aType, const mozilla::gfx::IntRect* aRect)
 void
 imgRequestProxy::OnLoadComplete(bool aLastPart)
 {
-  if (MOZ_LOG_TEST(GetImgLog(), LogLevel::Debug)) {
+  if (MOZ_LOG_TEST(gImgLog, LogLevel::Debug)) {
     nsAutoCString name;
     GetName(name);
-    LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::OnLoadComplete",
+    LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::OnLoadComplete",
                         "name", name.get());
   }
 
@@ -868,10 +838,10 @@ imgRequestProxy::OnLoadComplete(bool aLastPart)
 void
 imgRequestProxy::BlockOnload()
 {
-  if (MOZ_LOG_TEST(GetImgLog(), LogLevel::Debug)) {
+  if (MOZ_LOG_TEST(gImgLog, LogLevel::Debug)) {
     nsAutoCString name;
     GetName(name);
-    LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::BlockOnload",
+    LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::BlockOnload",
                         "name", name.get());
   }
 
@@ -884,10 +854,10 @@ imgRequestProxy::BlockOnload()
 void
 imgRequestProxy::UnblockOnload()
 {
-  if (MOZ_LOG_TEST(GetImgLog(), LogLevel::Debug)) {
+  if (MOZ_LOG_TEST(gImgLog, LogLevel::Debug)) {
     nsAutoCString name;
     GetName(name);
-    LOG_FUNC_WITH_PARAM(GetImgLog(), "imgRequestProxy::UnblockOnload",
+    LOG_FUNC_WITH_PARAM(gImgLog, "imgRequestProxy::UnblockOnload",
                         "name", name.get());
   }
 
@@ -928,7 +898,7 @@ nsresult
 imgRequestProxy::GetStaticRequest(imgRequestProxy** aReturn)
 {
   *aReturn = nullptr;
-  nsRefPtr<Image> image = GetImage();
+  RefPtr<Image> image = GetImage();
 
   bool animated;
   if (!image || (NS_SUCCEEDED(image->GetAnimated(&animated)) && !animated)) {
@@ -945,12 +915,12 @@ imgRequestProxy::GetStaticRequest(imgRequestProxy** aReturn)
   }
 
   // We are animated. We need to create a frozen version of this image.
-  nsRefPtr<Image> frozenImage = ImageOps::Freeze(image);
+  RefPtr<Image> frozenImage = ImageOps::Freeze(image);
 
   // Create a static imgRequestProxy with our new extracted frame.
   nsCOMPtr<nsIPrincipal> currentPrincipal;
   GetImagePrincipal(getter_AddRefs(currentPrincipal));
-  nsRefPtr<imgRequestProxy> req = new imgRequestProxyStatic(frozenImage,
+  RefPtr<imgRequestProxy> req = new imgRequestProxyStatic(frozenImage,
                                                             currentPrincipal);
   req->Init(nullptr, nullptr, mURI, nullptr);
 
@@ -967,7 +937,7 @@ imgRequestProxy::NotifyListener()
   // processing when we receive notifications (like OnStopRequest()), and we
   // need to check mCanceled everywhere too.
 
-  nsRefPtr<ProgressTracker> progressTracker = GetProgressTracker();
+  RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
   if (GetOwner()) {
     // Send the notifications to our listener asynchronously.
     progressTracker->Notify(this);
@@ -988,16 +958,16 @@ imgRequestProxy::SyncNotifyListener()
   // processing when we receive notifications (like OnStopRequest()), and we
   // need to check mCanceled everywhere too.
 
-  nsRefPtr<ProgressTracker> progressTracker = GetProgressTracker();
+  RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
   progressTracker->SyncNotify(this);
 }
 
 void
 imgRequestProxy::SetHasImage()
 {
-  nsRefPtr<ProgressTracker> progressTracker = GetProgressTracker();
+  RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
   MOZ_ASSERT(progressTracker);
-  nsRefPtr<Image> image = progressTracker->GetImage();
+  RefPtr<Image> image = progressTracker->GetImage();
   MOZ_ASSERT(image);
 
   // Force any private status related to the owner to reflect
@@ -1033,7 +1003,7 @@ RequestBehaviour::HasImage() const
   if (!mOwnerHasImage) {
     return false;
   }
-  nsRefPtr<ProgressTracker> progressTracker = GetProgressTracker();
+  RefPtr<ProgressTracker> progressTracker = GetProgressTracker();
   return progressTracker ? progressTracker->HasImage() : false;
 }
 
@@ -1058,7 +1028,7 @@ public:
 
   virtual already_AddRefed<mozilla::image::Image>
   GetImage() const override {
-    nsRefPtr<mozilla::image::Image> image = mImage;
+    RefPtr<mozilla::image::Image> image = mImage;
     return image.forget();
   }
 
@@ -1083,14 +1053,14 @@ public:
 private:
   // Our image. We have to hold a strong reference here, because that's normally
   // the job of the underlying request.
-  nsRefPtr<mozilla::image::Image> mImage;
+  RefPtr<mozilla::image::Image> mImage;
 };
 
 imgRequestProxyStatic::imgRequestProxyStatic(mozilla::image::Image* aImage,
                                              nsIPrincipal* aPrincipal)
 : mPrincipal(aPrincipal)
 {
-  mBehaviour = new StaticBehaviour(aImage);
+  mBehaviour = mozilla::MakeUnique<StaticBehaviour>(aImage);
 }
 
 NS_IMETHODIMP
