@@ -78,7 +78,8 @@ nrappkit copyright:
    ekr@rtfm.com  Thu Dec 20 20:14:49 2001
 */
 #include "logging.h"
-#include "mozilla/Scoped.h"
+#include "mozilla/UniquePtr.h"
+#include "mozilla/unused.h"
 #include "databuffer.h"
 
 extern "C" {
@@ -92,7 +93,7 @@ extern "C" {
 #include "local_addr.h"
 #include "stun_util.h"
 #include "registry.h"
-#include "nr_socket_multi_tcp.h"
+#include "nr_socket_buffered_stun.h"
 }
 
 #include "stunserver.h"
@@ -131,11 +132,15 @@ static int nr_socket_wrapped_sendto(void *obj, const void *msg, size_t len, int
 
 static int nr_socket_wrapped_recvfrom(void *obj, void * restrict buf, size_t maxlen,
     size_t *len, int flags, nr_transport_addr *addr) {
-  MOZ_CRASH();
+  nr_socket_wrapped *wrapped = static_cast<nr_socket_wrapped *>(obj);
+
+  return nr_socket_recvfrom(wrapped->sock_, buf, maxlen, len, flags, addr);
 }
 
 static int nr_socket_wrapped_getfd(void *obj, NR_SOCKET *fd) {
-  MOZ_CRASH();
+  nr_socket_wrapped *wrapped = static_cast<nr_socket_wrapped *>(obj);
+
+  return nr_socket_getfd(wrapped->sock_, fd);
 }
 
 static int nr_socket_wrapped_getaddr(void *obj, nr_transport_addr *addrp) {
@@ -170,7 +175,7 @@ static nr_socket_vtbl nr_socket_wrapped_vtbl = {
 };
 
 int nr_socket_wrapped_create(nr_socket *inner, nr_socket **outp) {
-  ScopedDeletePtr<nr_socket_wrapped> wrapped(new nr_socket_wrapped());
+  auto wrapped = MakeUnique<nr_socket_wrapped>();
 
   wrapped->sock_ = inner;
 
@@ -178,7 +183,7 @@ int nr_socket_wrapped_create(nr_socket *inner, nr_socket **outp) {
   if (r)
     return r;
 
-  wrapped.forget();
+  Unused << wrapped.release();
   return 0;
 }
 
@@ -186,10 +191,10 @@ int nr_socket_wrapped_create(nr_socket *inner, nr_socket **outp) {
 // Instance static.
 // Note: Calling Create() at static init time is not going to be safe, since
 // we have no reason to expect this will be initted to a nullptr yet.
-TestStunServer* TestStunServer::instance;
-TestStunTcpServer* TestStunTcpServer::instance;
-TestStunServer* TestStunServer::instance6;
-TestStunTcpServer* TestStunTcpServer::instance6;
+TestStunServer *TestStunServer::instance;
+TestStunTcpServer *TestStunTcpServer::instance;
+TestStunServer *TestStunServer::instance6;
+TestStunTcpServer *TestStunTcpServer::instance6;
 uint16_t TestStunServer::instance_port = 3478;
 uint16_t TestStunTcpServer::instance_port = 3478;
 
@@ -216,7 +221,7 @@ TestStunServer::~TestStunServer() {
   delete response_addr_;
 }
 
-int TestStunServer::SetInternalPort(nr_local_addr* addr, uint16_t port) {
+int TestStunServer::SetInternalPort(nr_local_addr *addr, uint16_t port) {
   if (nr_transport_addr_set_port(&addr->addr, port)) {
     MOZ_MTLOG(ML_ERROR, "Couldn't set port");
     return R_INTERNAL;
@@ -230,7 +235,7 @@ int TestStunServer::SetInternalPort(nr_local_addr* addr, uint16_t port) {
   return 0;
 }
 
-int TestStunServer::TryOpenListenSocket(nr_local_addr* addr, uint16_t port) {
+int TestStunServer::TryOpenListenSocket(nr_local_addr *addr, uint16_t port) {
 
   int r = SetInternalPort(addr, port);
 
@@ -319,10 +324,10 @@ int TestStunServer::Initialize(int address_family) {
   return 0;
 }
 
-TestStunServer* TestStunServer::Create(int address_family) {
+UniquePtr<TestStunServer> TestStunServer::Create(int address_family) {
   NR_reg_init(NR_REG_MODE_LOCAL);
 
-  ScopedDeletePtr<TestStunServer> server(new TestStunServer());
+  UniquePtr<TestStunServer> server(new TestStunServer());
 
   if (server->Initialize(address_family))
     return nullptr;
@@ -336,7 +341,7 @@ TestStunServer* TestStunServer::Create(int address_family) {
 
   NR_ASYNC_WAIT(fd, NR_ASYNC_WAIT_READ, &TestStunServer::readable_cb, server.get());
 
-  return server.forget();
+  return server;
 }
 
 void TestStunServer::ConfigurePort(uint16_t port) {
@@ -347,13 +352,13 @@ TestStunServer* TestStunServer::GetInstance(int address_family) {
   switch (address_family) {
     case AF_INET:
       if (!instance)
-        instance = Create(address_family);
+        instance = Create(address_family).release();
 
       MOZ_ASSERT(instance);
       return instance;
     case AF_INET6:
       if (!instance6)
-        instance6 = Create(address_family);
+        instance6 = Create(address_family).release();
 
       return instance6;
     default:
@@ -372,21 +377,30 @@ void TestStunServer::ShutdownInstance() {
 struct DeferredStunOperation {
   DeferredStunOperation(TestStunServer *server,
                         const char *data, size_t len,
-                        nr_transport_addr *addr) :
+                        nr_transport_addr *addr,
+                        nr_socket *sock) :
       server_(server),
-      buffer_(reinterpret_cast<const uint8_t *>(data), len) {
+      buffer_(reinterpret_cast<const uint8_t *>(data), len),
+      sock_(sock) {
     nr_transport_addr_copy(&addr_, addr);
   }
 
   TestStunServer *server_;
   DataBuffer buffer_;
   nr_transport_addr addr_;
+  nr_socket *sock_;
 };
 
-void TestStunServer::Process(const uint8_t *msg, size_t len, nr_transport_addr *addr) {
+void TestStunServer::Process(const uint8_t *msg, size_t len, nr_transport_addr *addr, nr_socket *sock) {
+
+  if (!sock) {
+    sock = send_sock_;
+  }
+
   // Set the wrapped address so that the response goes to the right place.
-  nr_socket_wrapped_set_send_addr(send_sock_, addr);
-  nr_stun_server_process_request(stun_server_, send_sock_,
+  nr_socket_wrapped_set_send_addr(sock, addr);
+
+  nr_stun_server_process_request(stun_server_, sock,
                                  const_cast<char *>(reinterpret_cast<const char *>(msg)),
                                  len,
                                  response_addr_ ?
@@ -397,32 +411,43 @@ void TestStunServer::Process(const uint8_t *msg, size_t len, nr_transport_addr *
 void TestStunServer::process_cb(NR_SOCKET s, int how, void *cb_arg) {
   DeferredStunOperation *op = static_cast<DeferredStunOperation *>(cb_arg);
   op->server_->timer_handle_ = nullptr;
-  op->server_->Process(op->buffer_.data(), op->buffer_.len(), &op->addr_);
+  op->server_->Process(op->buffer_.data(), op->buffer_.len(), &op->addr_, op->sock_);
 
   delete op;
 }
 
-void TestStunServer::readable_cb(NR_SOCKET s, int how, void *cb_arg) {
-  TestStunServer* server = static_cast<TestStunServer*>(cb_arg);
+nr_socket* TestStunServer::GetReceivingSocket(NR_SOCKET s) {
+  return listen_sock_;
+}
 
-  char message[4096];
+nr_socket* TestStunServer::GetSendingSocket(nr_socket *sock) {
+  return send_sock_;
+}
+
+void TestStunServer::readable_cb(NR_SOCKET s, int how, void *cb_arg) {
+  TestStunServer *server = static_cast<TestStunServer*>(cb_arg);
+
+  char message[max_stun_message_size];
   size_t message_len;
   nr_transport_addr addr;
+  nr_socket *recv_sock = server->GetReceivingSocket(s);
+  if (!recv_sock) {
+    MOZ_MTLOG(ML_ERROR, "Failed to lookup receiving socket");
+    return;
+  }
+  nr_socket *send_sock = server->GetSendingSocket(recv_sock);
 
-  int r = nr_socket_recvfrom(server->listen_sock_, message, sizeof(message),
-    &message_len, 0, &addr);
+  /* Re-arm. */
+  NR_ASYNC_WAIT(s, NR_ASYNC_WAIT_READ, &TestStunServer::readable_cb, server);
 
-  if (r) {
+  if (nr_socket_recvfrom(recv_sock, message, sizeof(message),
+      &message_len, 0, &addr)) {
     MOZ_MTLOG(ML_ERROR, "Couldn't read STUN message");
     return;
   }
 
   MOZ_MTLOG(ML_DEBUG, "Received data of length " << message_len);
 
-  // Re-arm.
-  NR_ASYNC_WAIT(s, NR_ASYNC_WAIT_READ, &TestStunServer::readable_cb, server);
-
-
   // If we have initial dropping set, check at this point.
   std::string key(addr.as_string);
 
@@ -444,10 +469,11 @@ void TestStunServer::readable_cb(NR_SOCKET s, int how, void *cb_arg) {
                        new DeferredStunOperation(
                            server,
                            message, message_len,
-                           &addr),
+                           &addr, send_sock),
                        &server->timer_handle_);
   } else {
-    server->Process(reinterpret_cast<const uint8_t *>(message), message_len, &addr);
+    server->Process(reinterpret_cast<const uint8_t *>(message), message_len,
+                    &addr, send_sock);
   }
 }
 
@@ -496,6 +522,7 @@ void TestStunServer::Reset() {
   }
   delete response_addr_;
   response_addr_ = nullptr;
+  received_ct_.clear();
 }
 
 
@@ -509,13 +536,13 @@ TestStunTcpServer* TestStunTcpServer::GetInstance(int address_family) {
   switch (address_family) {
     case AF_INET:
       if (!instance)
-        instance = Create(address_family);
+        instance = Create(address_family).release();
 
       MOZ_ASSERT(instance);
       return instance;
     case AF_INET6:
       if (!instance6)
-        instance6 = Create(address_family);
+        instance6 = Create(address_family).release();
 
       return instance6;
     default:
@@ -525,11 +552,12 @@ TestStunTcpServer* TestStunTcpServer::GetInstance(int address_family) {
 
 void TestStunTcpServer::ShutdownInstance() {
   delete instance;
-
   instance = nullptr;
+  delete instance6;
+  instance6 = nullptr;
 }
 
-int TestStunTcpServer::TryOpenListenSocket(nr_local_addr* addr, uint16_t port) {
+int TestStunTcpServer::TryOpenListenSocket(nr_local_addr *addr, uint16_t port) {
 
   addr->addr.protocol=IPPROTO_TCP;
 
@@ -538,17 +566,15 @@ int TestStunTcpServer::TryOpenListenSocket(nr_local_addr* addr, uint16_t port) {
   if (r)
     return r;
 
-  if (ice_ctx_ == NULL)
-    ice_ctx_ = NrIceCtx::Create("stun", false, false, false, false, false);
+  nr_socket *sock;
+  if (nr_socket_local_create(nullptr, &addr->addr, &sock)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't create listen tcp socket");
+    return R_ALREADY;
+  }
 
-  //TODO (nils@mozilla.com) can we replace this with a more basic TCP socket
-  // alternative which would allow us to remove the framing argument from the
-  // nr_socket_multi_tcp_create() call?
-  if(nr_socket_multi_tcp_create(ice_ctx_->ctx(),
-     &addr->addr, TCP_TYPE_PASSIVE, 0, 0, 2048,
-     &listen_sock_)) {
-     MOZ_MTLOG(ML_ERROR, "Couldn't create listen socket");
-     return R_ALREADY;
+  if (nr_socket_buffered_stun_create(sock, 2048, TURN_TCP_FRAMING, &listen_sock_)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't create listen tcp socket");
+    return R_ALREADY;
   }
 
   if(nr_socket_listen(listen_sock_, 10)) {
@@ -559,24 +585,80 @@ int TestStunTcpServer::TryOpenListenSocket(nr_local_addr* addr, uint16_t port) {
   return 0;
 }
 
-TestStunTcpServer* TestStunTcpServer::Create(int address_family) {
+nr_socket* TestStunTcpServer::GetReceivingSocket(NR_SOCKET s) {
+  return connections_[s];
+}
+
+nr_socket* TestStunTcpServer::GetSendingSocket(nr_socket *sock) {
+  return sock;
+}
+
+void TestStunTcpServer::accept_cb(NR_SOCKET s, int how, void *cb_arg) {
+  TestStunTcpServer *server = static_cast<TestStunTcpServer*>(cb_arg);
+  nr_socket *newsock, *bufsock, *wrapsock;
+  nr_transport_addr remote_addr;
+  NR_SOCKET fd;
+
+  /* rearm */
+  NR_ASYNC_WAIT(s, NR_ASYNC_WAIT_READ, &TestStunTcpServer::accept_cb, cb_arg);
+
+  /* accept */
+  if (nr_socket_accept(server->listen_sock_, &remote_addr, &newsock)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't accept incoming tcp connection");
+    return;
+  }
+
+  if(nr_socket_buffered_stun_create(newsock, 2048, TURN_TCP_FRAMING, &bufsock)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't create connected tcp socket");
+    nr_socket_destroy(&newsock);
+    return;
+  }
+
+  nr_socket_buffered_set_connected_to(bufsock, &remote_addr);
+
+  if(nr_socket_wrapped_create(bufsock, &wrapsock)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't wrap connected tcp socket");
+    nr_socket_destroy(&bufsock);
+    return;
+  }
+
+  if(nr_socket_getfd(wrapsock, &fd)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't get fd from connected tcp socket");
+    nr_socket_destroy(&wrapsock);
+    return;
+  }
+
+  server->connections_[fd] = wrapsock;
+
+  NR_ASYNC_WAIT(fd, NR_ASYNC_WAIT_READ, &TestStunServer::readable_cb, server);
+}
+
+  UniquePtr<TestStunTcpServer> TestStunTcpServer::Create(int address_family) {
   NR_reg_init(NR_REG_MODE_LOCAL);
 
-  ScopedDeletePtr<TestStunTcpServer> server(new TestStunTcpServer());
+  UniquePtr<TestStunTcpServer> server(new TestStunTcpServer());
 
   if (server->Initialize(address_family)) {
     return nullptr;
   }
 
-  nr_socket_multi_tcp_set_readable_cb(server->listen_sock_,
-    &TestStunServer::readable_cb, server.get());
+  NR_SOCKET fd;
+  if(nr_socket_getfd(server->listen_sock_, &fd)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't get tcp fd");
+    return nullptr;
+  }
 
-  return server.forget();
+  NR_ASYNC_WAIT(fd, NR_ASYNC_WAIT_READ, &TestStunTcpServer::accept_cb, server.get());
+
+  return server;
 }
 
 TestStunTcpServer::~TestStunTcpServer() {
-  ice_ctx_ = nullptr;
-  nr_socket_destroy(&listen_sock_);
+  for (auto it = connections_.begin(); it != connections_.end();) {
+    NR_ASYNC_CANCEL(it->first, NR_ASYNC_WAIT_READ);
+    nr_socket_destroy(&it->second);
+    connections_.erase(it++);
+  }
 }
 
 }  // close namespace
