@@ -26,7 +26,7 @@
 
 #include "databuffer.h"
 #include "dtlsidentity.h"
-#include "nricectx.h"
+#include "nricectxhandler.h"
 #include "nricemediastream.h"
 #include "transportflow.h"
 #include "transportlayer.h"
@@ -35,7 +35,6 @@
 #include "transportlayerlog.h"
 #include "transportlayerloopback.h"
 
-#include "mtransport_test_utils.h"
 #include "runnable_utils.h"
 
 #define GTEST_HAS_RTTI 0
@@ -45,8 +44,6 @@
 using namespace mozilla;
 MOZ_MTLOG_MODULE("mtransport")
 
-MtransportTestUtils *test_utils;
-
 
 const uint8_t kTlsChangeCipherSpecType = 0x14;
 const uint8_t kTlsHandshakeType =        0x16;
@@ -91,8 +88,6 @@ class TransportLayerDummy : public TransportLayer {
   bool *destroyed_;
 };
 
-class TransportLayerLossy;
-
 class Inspector {
  public:
   virtual ~Inspector() {}
@@ -438,29 +433,30 @@ class TlsServerKeyExchangeECDHE {
 namespace {
 class TransportTestPeer : public sigslot::has_slots<> {
  public:
-  TransportTestPeer(nsCOMPtr<nsIEventTarget> target, std::string name)
+  TransportTestPeer(nsCOMPtr<nsIEventTarget> target, std::string name, MtransportTestUtils* utils)
       : name_(name), target_(target),
-        received_(0), flow_(new TransportFlow(name)),
+        received_packets_(0),received_bytes_(0),flow_(new TransportFlow(name)),
         loopback_(new TransportLayerLoopback()),
         logging_(new TransportLayerLogging()),
         lossy_(new TransportLayerLossy()),
         dtls_(new TransportLayerDtls()),
         identity_(DtlsIdentity::Generate()),
-        ice_ctx_(NrIceCtx::Create(name,
-                                  name == "P2" ?
-                                  TransportLayerDtls::CLIENT :
-                                  TransportLayerDtls::SERVER)),
+        ice_ctx_(NrIceCtxHandler::Create(name,
+                                         name == "P2" ?
+                                         TransportLayerDtls::CLIENT :
+                                         TransportLayerDtls::SERVER)),
         streams_(), candidates_(),
         peer_(nullptr),
         gathering_complete_(false),
         enabled_cipersuites_(),
         disabled_cipersuites_(),
-        reuse_dhe_key_(false) {
+        reuse_dhe_key_(false),
+        test_utils_(utils) {
     std::vector<NrIceStunServer> stun_servers;
     UniquePtr<NrIceStunServer> server(NrIceStunServer::Create(
         std::string((char *)"stun.services.mozilla.com"), 3478));
     stun_servers.push_back(*server);
-    EXPECT_TRUE(NS_SUCCEEDED(ice_ctx_->SetStunServers(stun_servers)));
+    EXPECT_TRUE(NS_SUCCEEDED(ice_ctx_->ctx()->SetStunServers(stun_servers)));
 
     dtls_->SetIdentity(identity_);
     dtls_->SetRole(name == "P2" ?
@@ -476,7 +472,7 @@ class TransportTestPeer : public sigslot::has_slots<> {
   }
 
   ~TransportTestPeer() {
-    test_utils->sts_target()->Dispatch(
+    test_utils_->sts_target()->Dispatch(
       WrapRunnable(this, &TransportTestPeer::DestroyFlow),
       NS_DISPATCH_SYNC);
   }
@@ -593,7 +589,7 @@ class TransportTestPeer : public sigslot::has_slots<> {
   }
 
   void ConnectSocket(TransportTestPeer *peer) {
-    RUN_ON_THREAD(test_utils->sts_target(),
+    RUN_ON_THREAD(test_utils_->sts_target(),
                   WrapRunnable(this, & TransportTestPeer::ConnectSocket_s,
                                peer),
                   NS_DISPATCH_SYNC);
@@ -603,7 +599,7 @@ class TransportTestPeer : public sigslot::has_slots<> {
     nsresult res;
 
     // Attach our slots
-    ice_ctx_->SignalGatheringStateChange.
+    ice_ctx_->ctx()->SignalGatheringStateChange.
         connect(this, &TransportTestPeer::GatheringStateChange);
 
     char name[100];
@@ -611,11 +607,11 @@ class TransportTestPeer : public sigslot::has_slots<> {
              (int)streams_.size());
 
     // Create the media stream
-    mozilla::RefPtr<NrIceMediaStream> stream =
+    RefPtr<NrIceMediaStream> stream =
         ice_ctx_->CreateStream(static_cast<char *>(name), 1);
 
     ASSERT_TRUE(stream != nullptr);
-    ice_ctx_->SetStream(streams_.size(), stream);
+    ice_ctx_->ctx()->SetStream(streams_.size(), stream);
     streams_.push_back(stream);
 
     // Listen for candidates
@@ -624,7 +620,7 @@ class TransportTestPeer : public sigslot::has_slots<> {
 
     // Create the transport layer
     ice_ = new TransportLayerIce(name);
-    ice_->SetParameters(ice_ctx_, stream, 1);
+    ice_->SetParameters(ice_ctx_->ctx(), stream, 1);
 
     // Assemble the stack
     nsAutoPtr<std::queue<mozilla::TransportLayer *> > layers(
@@ -632,7 +628,7 @@ class TransportTestPeer : public sigslot::has_slots<> {
     layers->push(ice_);
     layers->push(dtls_);
 
-    test_utils->sts_target()->Dispatch(
+    test_utils_->sts_target()->Dispatch(
       WrapRunnableRet(&res, flow_, &TransportFlow::PushLayers, layers),
       NS_DISPATCH_SYNC);
 
@@ -643,8 +639,8 @@ class TransportTestPeer : public sigslot::has_slots<> {
     flow_->SignalStateChange.connect(this, &TransportTestPeer::StateChanged);
 
     // Start gathering
-    test_utils->sts_target()->Dispatch(
-        WrapRunnableRet(&res, ice_ctx_, &NrIceCtx::StartGathering),
+    test_utils_->sts_target()->Dispatch(
+        WrapRunnableRet(&res, ice_ctx_->ctx(), &NrIceCtx::StartGathering),
         NS_DISPATCH_SYNC);
     ASSERT_TRUE(NS_SUCCEEDED(res));
   }
@@ -683,15 +679,15 @@ class TransportTestPeer : public sigslot::has_slots<> {
     }
 
     // First send attributes
-    test_utils->sts_target()->Dispatch(
-      WrapRunnableRet(&res, peer_->ice_ctx_,
+    test_utils_->sts_target()->Dispatch(
+      WrapRunnableRet(&res, peer_->ice_ctx_->ctx(),
                       &NrIceCtx::ParseGlobalAttributes,
-                      ice_ctx_->GetGlobalAttributes()),
+                      ice_ctx_->ctx()->GetGlobalAttributes()),
       NS_DISPATCH_SYNC);
     ASSERT_TRUE(NS_SUCCEEDED(res));
 
     for (size_t i=0; i<streams_.size(); ++i) {
-      test_utils->sts_target()->Dispatch(
+      test_utils_->sts_target()->Dispatch(
         WrapRunnableRet(&res, peer_->streams_[i], &NrIceMediaStream::ParseAttributes,
                         candidates_[streams_[i]->name()]), NS_DISPATCH_SYNC);
 
@@ -699,15 +695,15 @@ class TransportTestPeer : public sigslot::has_slots<> {
     }
 
     // Start checks on the other peer.
-    test_utils->sts_target()->Dispatch(
-      WrapRunnableRet(&res, peer_->ice_ctx_, &NrIceCtx::StartChecks),
+    test_utils_->sts_target()->Dispatch(
+      WrapRunnableRet(&res, peer_->ice_ctx_->ctx(), &NrIceCtx::StartChecks),
       NS_DISPATCH_SYNC);
     ASSERT_TRUE(NS_SUCCEEDED(res));
   }
 
   TransportResult SendPacket(const unsigned char* data, size_t len) {
     TransportResult ret;
-    test_utils->sts_target()->Dispatch(
+    test_utils_->sts_target()->Dispatch(
       WrapRunnableRet(&ret, flow_, &TransportFlow::SendPacket, data, len),
       NS_DISPATCH_SYNC);
 
@@ -724,13 +720,18 @@ class TransportTestPeer : public sigslot::has_slots<> {
   void PacketReceived(TransportFlow * flow, const unsigned char* data,
                       size_t len) {
     std::cerr << "Received " << len << " bytes" << std::endl;
-    ++received_;
+    ++received_packets_;
+    received_bytes_ += len;
   }
 
   void SetLoss(uint32_t loss) {
     lossy_->SetLoss(loss);
   }
 
+  void SetCombinePackets(bool combine) {
+    loopback_->CombinePackets(combine);
+  }
+
   void SetInspector(UniquePtr<Inspector> inspector) {
     lossy_->SetInspector(Move(inspector));
   }
@@ -754,7 +755,7 @@ class TransportTestPeer : public sigslot::has_slots<> {
   TransportLayer::State state() {
     TransportLayer::State tstate;
 
-    RUN_ON_THREAD(test_utils->sts_target(),
+    RUN_ON_THREAD(test_utils_->sts_target(),
                   WrapRunnableRet(&tstate, flow_, &TransportFlow::state));
 
     return tstate;
@@ -768,12 +769,14 @@ class TransportTestPeer : public sigslot::has_slots<> {
     return state() == TransportLayer::TS_ERROR;
   }
 
-  size_t received() { return received_; }
+  size_t receivedPackets() { return received_packets_; }
+
+  size_t receivedBytes() { return received_bytes_; }
 
   uint16_t cipherSuite() const {
     nsresult rv;
     uint16_t cipher;
-    RUN_ON_THREAD(test_utils->sts_target(),
+    RUN_ON_THREAD(test_utils_->sts_target(),
                   WrapRunnableRet(&rv, dtls_, &TransportLayerDtls::GetCipherSuite,
                                   &cipher));
 
@@ -786,7 +789,7 @@ class TransportTestPeer : public sigslot::has_slots<> {
   uint16_t srtpCipher() const {
     nsresult rv;
     uint16_t cipher;
-    RUN_ON_THREAD(test_utils->sts_target(),
+    RUN_ON_THREAD(test_utils_->sts_target(),
                   WrapRunnableRet(&rv, dtls_, &TransportLayerDtls::GetSrtpCipher,
                                   &cipher));
     if (NS_FAILED(rv)) {
@@ -798,16 +801,17 @@ class TransportTestPeer : public sigslot::has_slots<> {
  private:
   std::string name_;
   nsCOMPtr<nsIEventTarget> target_;
-  size_t received_;
-    mozilla::RefPtr<TransportFlow> flow_;
+  size_t received_packets_;
+  size_t received_bytes_;
+    RefPtr<TransportFlow> flow_;
   TransportLayerLoopback *loopback_;
   TransportLayerLogging *logging_;
   TransportLayerLossy *lossy_;
   TransportLayerDtls *dtls_;
   TransportLayerIce *ice_;
-  mozilla::RefPtr<DtlsIdentity> identity_;
-  mozilla::RefPtr<NrIceCtx> ice_ctx_;
-  std::vector<mozilla::RefPtr<NrIceMediaStream> > streams_;
+  RefPtr<DtlsIdentity> identity_;
+  RefPtr<NrIceCtxHandler> ice_ctx_;
+  std::vector<RefPtr<NrIceMediaStream> > streams_;
   std::map<std::string, std::vector<std::string> > candidates_;
   TransportTestPeer *peer_;
   bool gathering_complete_;
@@ -816,23 +820,25 @@ class TransportTestPeer : public sigslot::has_slots<> {
   std::vector<uint16_t> enabled_cipersuites_;
   std::vector<uint16_t> disabled_cipersuites_;
   bool reuse_dhe_key_;
+  MtransportTestUtils* test_utils_;
 };
 
 
-class TransportTest : public ::testing::Test {
+class TransportTest : public MtransportTest {
  public:
   TransportTest() {
     fds_[0] = nullptr;
     fds_[1] = nullptr;
   }
 
-  ~TransportTest() {
+  void TearDown() override {
     delete p1_;
     delete p2_;
 
     //    Can't detach these
     //    PR_Close(fds_[0]);
     //    PR_Close(fds_[1]);
+    MtransportTest::TearDown();
   }
 
   void DestroyPeerFlows() {
@@ -840,7 +846,9 @@ class TransportTest : public ::testing::Test {
     p2_->DisconnectDestroyFlow();
   }
 
-  void SetUp() {
+  void SetUp() override {
+    MtransportTest::SetUp();
+
     nsresult rv;
     target_ = do_GetService(NS_SOCKETTRANSPORTSERVICE_CONTRACTID, &rv);
     ASSERT_TRUE(NS_SUCCEEDED(rv));
@@ -849,8 +857,8 @@ class TransportTest : public ::testing::Test {
   }
 
   void Reset() {
-    p1_ = new TransportTestPeer(target_, "P1");
-    p2_ = new TransportTestPeer(target_, "P2");
+    p1_ = new TransportTestPeer(target_, "P1", test_utils_);
+    p2_ = new TransportTestPeer(target_, "P2", test_utils_);
   }
 
   void SetupSrtp() {
@@ -919,8 +927,8 @@ class TransportTest : public ::testing::Test {
     ASSERT_TRUE_WAIT(p2_->connected(), 10000);
   }
 
-  void TransferTest(size_t count) {
-    unsigned char buf[1000];
+  void TransferTest(size_t count, size_t bytes = 1024) {
+    unsigned char buf[bytes];
 
     for (size_t i= 0; i<count; ++i) {
       memset(buf, count & 0xff, sizeof(buf));
@@ -928,16 +936,17 @@ class TransportTest : public ::testing::Test {
       ASSERT_TRUE(rv > 0);
     }
 
-    std::cerr << "Received == " << p2_->received() << std::endl;
-    ASSERT_TRUE_WAIT(count == p2_->received(), 10000);
+    std::cerr << "Received == " << p2_->receivedPackets() << " packets" << std::endl;
+    ASSERT_TRUE_WAIT(count == p2_->receivedPackets(), 10000);
+    ASSERT_TRUE((count * sizeof(buf)) == p2_->receivedBytes());
   }
 
  protected:
   void ConnectSocketInternal() {
-    test_utils->sts_target()->Dispatch(
+    test_utils_->sts_target()->Dispatch(
       WrapRunnable(p1_, &TransportTestPeer::ConnectSocket, p2_),
       NS_DISPATCH_SYNC);
-    test_utils->sts_target()->Dispatch(
+    test_utils_->sts_target()->Dispatch(
       WrapRunnable(p2_, &TransportTestPeer::ConnectSocket, p1_),
       NS_DISPATCH_SYNC);
   }
@@ -953,8 +962,19 @@ TEST_F(TransportTest, TestNoDtlsVerificationSettings) {
   ConnectSocketExpectFail();
 }
 
+static void DisableChaCha(TransportTestPeer* peer) {
+  // On ARM, ChaCha20Poly1305 might be preferred; disable it for the tests that
+  // want to check the cipher suite.  It doesn't matter which peer disables the
+  // suite, disabling on either side has the same effect.
+  std::vector<uint16_t> chachaSuites;
+  chachaSuites.push_back(TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256);
+  chachaSuites.push_back(TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256);
+  peer->SetCipherSuiteChanges(std::vector<uint16_t>(), chachaSuites);
+}
+
 TEST_F(TransportTest, TestConnect) {
   SetDtlsPeer();
+  DisableChaCha(p1_);
   ConnectSocket();
 
   // check that we got the right suite
@@ -967,6 +987,7 @@ TEST_F(TransportTest, TestConnect) {
 TEST_F(TransportTest, TestConnectSrtp) {
   SetupSrtp();
   SetDtlsPeer();
+  DisableChaCha(p2_);
   ConnectSocket();
 
   ASSERT_EQ(TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256, p1_->cipherSuite());
@@ -1143,6 +1164,28 @@ TEST_F(TransportTest, TestTransfer) {
   TransferTest(1);
 }
 
+TEST_F(TransportTest, TestTransferMaxSize) {
+  SetDtlsPeer();
+  ConnectSocket();
+  /* transportlayerdtls uses a 9216 bytes buffer - as this test uses the
+   * loopback implementation it does not have to take into account the extra
+   * bytes added by the DTLS layer below. */
+  TransferTest(1, 9216);
+}
+
+TEST_F(TransportTest, TestTransferMultiple) {
+  SetDtlsPeer();
+  ConnectSocket();
+  TransferTest(3);
+}
+
+TEST_F(TransportTest, TestTransferCombinedPackets) {
+  SetDtlsPeer();
+  ConnectSocket();
+  p2_->SetCombinePackets(true);
+  TransferTest(3);
+}
+
 TEST_F(TransportTest, TestConnectLoseFirst) {
   SetDtlsPeer();
   p1_->SetLoss(0);
@@ -1155,10 +1198,32 @@ TEST_F(TransportTest, TestConnectIce) {
   ConnectIce();
 }
 
-TEST_F(TransportTest, TestTransferIce) {
+TEST_F(TransportTest, TestTransferIceMaxSize) {
   SetDtlsPeer();
   ConnectIce();
-  TransferTest(1);
+  /* nICEr and transportlayerdtls both use 9216 bytes buffers. But the DTLS
+   * layer add extra bytes to the packet, which size depends on chosen cipher
+   * etc. Sending more then 9216 bytes works, but on the receiving side the call
+   * to PR_recvfrom() will truncate any packet bigger then nICEr's buffer size
+   * of 9216 bytes, which then results in the DTLS layer discarding the packet.
+   * Therefore we leave some headroom (according to
+   * https://bugzilla.mozilla.org/show_bug.cgi?id=1214269#c29 256 bytes should
+   * be save choice) here for the DTLS bytes to make it safely into the 
+   * receiving buffer in nICEr. */
+  TransferTest(1, 8960);
+}
+
+TEST_F(TransportTest, TestTransferIceMultiple) {
+  SetDtlsPeer();
+  ConnectIce();
+  TransferTest(3);
+}
+
+TEST_F(TransportTest, TestTransferIceCombinedPackets) {
+  SetDtlsPeer();
+  ConnectIce();
+  p2_->SetCombinePackets(true);
+  TransferTest(3);
 }
 
 // test the default configuration against a peer that supports only
@@ -1221,7 +1286,7 @@ TEST_F(TransportTest, TestDheOnlyFails) {
 }
 
 TEST(PushTests, LayerFail) {
-  mozilla::RefPtr<TransportFlow> flow = new TransportFlow();
+  RefPtr<TransportFlow> flow = new TransportFlow();
   nsresult rv;
   bool destroyed1, destroyed2;
 
@@ -1241,7 +1306,7 @@ TEST(PushTests, LayerFail) {
 }
 
 TEST(PushTests, LayersFail) {
-  mozilla::RefPtr<TransportFlow> flow = new TransportFlow();
+  RefPtr<TransportFlow> flow = new TransportFlow();
   nsresult rv;
   bool destroyed1, destroyed2, destroyed3;
 
@@ -1273,17 +1338,3 @@ TEST(PushTests, LayersFail) {
 }
 
 }  // end namespace
-
-int main(int argc, char **argv)
-{
-  test_utils = new MtransportTestUtils();
-
-  NSS_NoDB_Init(nullptr);
-  NSS_SetDomesticPolicy();
-  // Start the tests
-  ::testing::InitGoogleTest(&argc, argv);
-
-  int rv = RUN_ALL_TESTS();
-  delete test_utils;
-  return rv;
-}
