@@ -84,6 +84,7 @@ extern "C" {
 #include "stun_msg.h" // for NR_STUN_MAX_MESSAGE_SIZE
 #include "nr_api.h"
 #include "async_wait.h"
+#include "async_timer.h"
 #include "nr_socket.h"
 #include "nr_socket_local.h"
 #include "stun_hint.h"
@@ -137,6 +138,21 @@ static nr_socket_factory_vtbl test_nat_socket_factory_vtbl = {
   test_nat_socket_factory_destroy
 };
 
+/* static */
+TestNat::NatBehavior
+TestNat::ToNatBehavior(const std::string& type) {
+  if (!type.compare("ENDPOINT_INDEPENDENT")) {
+    return TestNat::ENDPOINT_INDEPENDENT;
+  } else if (!type.compare("ADDRESS_DEPENDENT")) {
+    return TestNat::ADDRESS_DEPENDENT;
+  } else if (!type.compare("PORT_DEPENDENT")) {
+    return TestNat::PORT_DEPENDENT;
+  }
+
+  MOZ_ASSERT(false, "Invalid NAT behavior");
+  return TestNat::ENDPOINT_INDEPENDENT;
+}
+
 bool TestNat::has_port_mappings() const {
   for (TestNrSocket *sock : sockets_) {
     if (sock->has_port_mappings()) {
@@ -184,7 +200,8 @@ int TestNat::create_socket_factory(nr_socket_factory **factorypp) {
 }
 
 TestNrSocket::TestNrSocket(TestNat *nat)
-  : nat_(nat) {
+  : nat_(nat),
+    timer_handle_(nullptr) {
   nat_->insert_socket(this);
 }
 
@@ -192,7 +209,7 @@ TestNrSocket::~TestNrSocket() {
   nat_->erase_socket(this);
 }
 
-nsRefPtr<NrSocket> TestNrSocket::create_external_socket(
+RefPtr<NrSocketBase> TestNrSocket::create_external_socket(
     const nr_transport_addr &dest_addr) const {
   MOZ_ASSERT(nat_->enabled_);
   MOZ_ASSERT(!nat_->is_an_internal_tuple(dest_addr));
@@ -202,8 +219,9 @@ nsRefPtr<NrSocket> TestNrSocket::create_external_socket(
 
   // Open the socket on an arbitrary port, on the same address.
   // TODO(bug 1170299): Remove const_cast when no longer necessary
-  if ((r = nr_transport_addr_copy(&nat_external_addr,
-                                  const_cast<nr_transport_addr*>(&my_addr_)))) {
+  if ((r = nr_transport_addr_copy(
+             &nat_external_addr,
+             const_cast<nr_transport_addr*>(&internal_socket_->my_addr())))) {
     r_log(LOG_GENERIC,LOG_CRIT, "%s: Failure in nr_transport_addr_copy: %d",
                                 __FUNCTION__, r);
     return nullptr;
@@ -215,9 +233,10 @@ nsRefPtr<NrSocket> TestNrSocket::create_external_socket(
     return nullptr;
   }
 
-  nsRefPtr<NrSocket> external_socket = new NrSocket;
+  RefPtr<NrSocketBase> external_socket;
+  r = NrSocketBase::CreateSocket(&nat_external_addr, &external_socket);
 
-  if ((r = external_socket->create(&nat_external_addr))) {
+  if (r) {
     r_log(LOG_GENERIC,LOG_CRIT, "%s: Failure in NrSocket::create: %d",
                                 __FUNCTION__, r);
     return nullptr;
@@ -226,13 +245,83 @@ nsRefPtr<NrSocket> TestNrSocket::create_external_socket(
   return external_socket;
 }
 
+int TestNrSocket::create(nr_transport_addr *addr) {
+  return NrSocketBase::CreateSocket(addr, &internal_socket_);
+}
+
+int TestNrSocket::getaddr(nr_transport_addr *addrp) {
+  return internal_socket_->getaddr(addrp);
+}
+
+void TestNrSocket::close() {
+  if (timer_handle_) {
+    NR_async_timer_cancel(timer_handle_);
+    timer_handle_ = 0;
+  }
+  internal_socket_->close();
+  for (RefPtr<PortMapping>& port_mapping : port_mappings_) {
+    port_mapping->external_socket_->close();
+  }
+}
+
+int TestNrSocket::listen(int backlog) {
+  MOZ_ASSERT(internal_socket_->my_addr().protocol == IPPROTO_TCP);
+  r_log(LOG_GENERIC, LOG_DEBUG,
+        "TestNrSocket %s listening",
+        internal_socket_->my_addr().as_string);
+
+  return internal_socket_->listen(backlog);
+}
+
+int TestNrSocket::accept(nr_transport_addr *addrp, nr_socket **sockp) {
+  MOZ_ASSERT(internal_socket_->my_addr().protocol == IPPROTO_TCP);
+  int r = internal_socket_->accept(addrp, sockp);
+  if (r) {
+    return r;
+  }
+
+  if (nat_->enabled_ && !nat_->is_an_internal_tuple(*addrp)) {
+    nr_socket_destroy(sockp);
+    return R_IO_ERROR;
+  }
+
+  return 0;
+}
+
+void TestNrSocket::process_delayed_cb(NR_SOCKET s, int how, void *cb_arg) {
+  DeferredPacket *op = static_cast<DeferredPacket *>(cb_arg);
+  op->socket_->timer_handle_ = nullptr;
+  r_log(LOG_GENERIC, LOG_DEBUG,
+        "TestNrSocket %s sending delayed STUN response",
+        op->internal_socket_->my_addr().as_string);
+  op->internal_socket_->sendto(op->buffer_.data(), op->buffer_.len(),
+                               op->flags_, &op->to_);
+
+  delete op;
+}
+
 int TestNrSocket::sendto(const void *msg, size_t len,
                          int flags, nr_transport_addr *to) {
-  MOZ_ASSERT(my_addr_.protocol != IPPROTO_TCP);
-  ASSERT_ON_THREAD(ststhread_);
+  MOZ_ASSERT(internal_socket_->my_addr().protocol != IPPROTO_TCP);
 
+  UCHAR *buf = static_cast<UCHAR*>(const_cast<void*>(msg));
+  if (nat_->block_stun_ &&
+      nr_is_stun_message(buf, len)) {
+    return 0;
+  }
+
+  /* TODO: improve the functionality of this in bug 1253657 */
   if (!nat_->enabled_ || nat_->is_an_internal_tuple(*to)) {
-    return NrSocket::sendto(msg, len, flags, to);
+    if (nat_->delay_stun_resp_ms_ &&
+        nr_is_stun_response_message(buf, len)) {
+      NR_ASYNC_TIMER_SET(nat_->delay_stun_resp_ms_,
+                         process_delayed_cb,
+                         new DeferredPacket(this, msg, len, flags, to,
+                                            internal_socket_),
+                         &timer_handle_);
+      return 0;
+    }
+    return internal_socket_->sendto(msg, len, flags, to);
   }
 
   destroy_stale_port_mappings();
@@ -251,7 +340,7 @@ int TestNrSocket::sendto(const void *msg, size_t len,
     // See if we have already made the external socket we need to use.
     PortMapping *similar_port_mapping =
       get_port_mapping(*to, nat_->mapping_type_);
-    nsRefPtr<NrSocket> external_socket;
+    RefPtr<NrSocketBase> external_socket;
 
     if (similar_port_mapping) {
       external_socket = similar_port_mapping->external_socket_;
@@ -270,7 +359,7 @@ int TestNrSocket::sendto(const void *msg, size_t len,
       // Make sure the new port mapping is ready to receive traffic if the
       // TestNrSocket is already waiting.
       port_mapping->async_wait(NR_ASYNC_WAIT_READ,
-                              port_mapping_readable_callback,
+                              socket_readable_callback,
                               this,
                               (char*)__FUNCTION__,
                               __LINE__);
@@ -285,8 +374,7 @@ int TestNrSocket::sendto(const void *msg, size_t len,
 int TestNrSocket::recvfrom(void *buf, size_t maxlen,
                            size_t *len, int flags,
                            nr_transport_addr *from) {
-  MOZ_ASSERT(my_addr_.protocol != IPPROTO_TCP);
-  ASSERT_ON_THREAD(ststhread_);
+  MOZ_ASSERT(internal_socket_->my_addr().protocol != IPPROTO_TCP);
 
   int r;
   bool ingress_allowed = false;
@@ -298,15 +386,21 @@ int TestNrSocket::recvfrom(void *buf, size_t maxlen,
     if (!r) {
       PortMapping *port_mapping_used;
       ingress_allowed = allow_ingress(*from, &port_mapping_used);
-      if (ingress_allowed && nat_->refresh_on_ingress_ && port_mapping_used) {
-        port_mapping_used->last_used_ = PR_IntervalNow();
+      if (ingress_allowed) {
+        r_log(LOG_GENERIC, LOG_DEBUG, "TestNrSocket %s received from %s via %s",
+              internal_socket_->my_addr().as_string,
+              from->as_string,
+              port_mapping_used->external_socket_->my_addr().as_string);
+        if (nat_->refresh_on_ingress_) {
+          port_mapping_used->last_used_ = PR_IntervalNow();
+        }
       }
     }
   } else {
     // If no external socket has data, see if there's any data that was sent
     // directly to the TestNrSocket, and eat it if it isn't supposed to get
     // through.
-    r = NrSocket::recvfrom(buf, maxlen, len, flags, from);
+    r = internal_socket_->recvfrom(buf, maxlen, len, flags, from);
     if (!r) {
       // We do not use allow_ingress() here because that only handles traffic
       // landing on an external port.
@@ -314,9 +408,13 @@ int TestNrSocket::recvfrom(void *buf, size_t maxlen,
                          nat_->is_an_internal_tuple(*from));
       if (!ingress_allowed) {
         r_log(LOG_GENERIC, LOG_INFO, "TestNrSocket %s denying ingress from %s: "
-                                     "Not behind the same NAT",
-                                     my_addr_.as_string,
-                                     from->as_string);
+              "Not behind the same NAT",
+              internal_socket_->my_addr().as_string,
+              from->as_string);
+      } else {
+        r_log(LOG_GENERIC, LOG_DEBUG, "TestNrSocket %s received from %s",
+              internal_socket_->my_addr().as_string,
+              from->as_string);
       }
     }
   }
@@ -337,18 +435,15 @@ int TestNrSocket::recvfrom(void *buf, size_t maxlen,
 
 bool TestNrSocket::allow_ingress(const nr_transport_addr &from,
                                  PortMapping **port_mapping_used) const {
-  *port_mapping_used = nullptr;
-  if (!nat_->enabled_)
-    return true;
-
-  if (nat_->is_an_internal_tuple(from))
-    return true;
+  // This is only called for traffic arriving at a port mapping
+  MOZ_ASSERT(nat_->enabled_);
+  MOZ_ASSERT(!nat_->is_an_internal_tuple(from));
 
   *port_mapping_used = get_port_mapping(from, nat_->filtering_type_);
   if (!(*port_mapping_used)) {
     r_log(LOG_GENERIC, LOG_INFO, "TestNrSocket %s denying ingress from %s: "
                                  "Filtered",
-                                 my_addr_.as_string,
+                                 internal_socket_->my_addr().as_string,
                                  from.as_string);
     return false;
   }
@@ -356,7 +451,7 @@ bool TestNrSocket::allow_ingress(const nr_transport_addr &from,
   if (is_port_mapping_stale(**port_mapping_used)) {
     r_log(LOG_GENERIC, LOG_INFO, "TestNrSocket %s denying ingress from %s: "
                                  "Stale port mapping",
-                                 my_addr_.as_string,
+                                 internal_socket_->my_addr().as_string,
                                  from.as_string);
     return false;
   }
@@ -364,7 +459,7 @@ bool TestNrSocket::allow_ingress(const nr_transport_addr &from,
   if (!nat_->allow_hairpinning_ && nat_->is_my_external_tuple(from)) {
     r_log(LOG_GENERIC, LOG_INFO, "TestNrSocket %s denying ingress from %s: "
                                  "Hairpinning disallowed",
-                                 my_addr_.as_string,
+                                 internal_socket_->my_addr().as_string,
                                  from.as_string);
     return false;
   }
@@ -373,7 +468,6 @@ bool TestNrSocket::allow_ingress(const nr_transport_addr &from,
 }
 
 int TestNrSocket::connect(nr_transport_addr *addr) {
-  ASSERT_ON_THREAD(ststhread_);
 
   if (connect_invoked_ || !port_mappings_.empty()) {
     MOZ_CRASH("TestNrSocket::connect() called more than once!");
@@ -386,17 +480,21 @@ int TestNrSocket::connect(nr_transport_addr *addr) {
                                       // we don't normally connect on UDP.
       || nat_->is_an_internal_tuple(*addr)) {
     // This will set connect_invoked_
-    return NrSocket::connect(addr);
+    return internal_socket_->connect(addr);
   }
 
-  nsRefPtr<NrSocket> external_socket(create_external_socket(*addr));
+  RefPtr<NrSocketBase> external_socket(create_external_socket(*addr));
   if (!external_socket) {
     return R_INTERNAL;
   }
 
   PortMapping *port_mapping = create_port_mapping(*addr, external_socket);
   port_mappings_.push_back(port_mapping);
-  port_mapping->external_socket_->connect(addr);
+  int r = port_mapping->external_socket_->connect(addr);
+  if (r && r != R_WOULDBLOCK) {
+    return r;
+  }
+
   port_mapping->last_used_ = PR_IntervalNow();
 
   if (poll_flags() & PR_POLL_READ) {
@@ -407,62 +505,90 @@ int TestNrSocket::connect(nr_transport_addr *addr) {
                              __LINE__);
   }
 
-  return 0;
+  return r;
 }
 
 int TestNrSocket::write(const void *msg, size_t len, size_t *written) {
-  ASSERT_ON_THREAD(ststhread_);
 
   if (port_mappings_.empty()) {
     // The no-nat case, just pass call through.
-    r_log(LOG_GENERIC, LOG_INFO, "TestNrSocket %s writing",
+    r_log(LOG_GENERIC, LOG_DEBUG, "TestNrSocket %s writing",
           my_addr().as_string);
 
-    return NrSocket::write(msg, len, written);
+    return internal_socket_->write(msg, len, written);
   } else {
+    destroy_stale_port_mappings();
+    if (port_mappings_.empty()) {
+      return -1;
+    }
     // This is TCP only
     MOZ_ASSERT(port_mappings_.size() == 1);
-    r_log(LOG_GENERIC, LOG_INFO,
+    r_log(LOG_GENERIC, LOG_DEBUG,
           "PortMapping %s -> %s writing",
           port_mappings_.front()->external_socket_->my_addr().as_string,
           port_mappings_.front()->remote_address_.as_string);
 
+    port_mappings_.front()->last_used_ = PR_IntervalNow();
     return port_mappings_.front()->external_socket_->write(msg, len, written);
   }
 }
 
 int TestNrSocket::read(void *buf, size_t maxlen, size_t *len) {
-  ASSERT_ON_THREAD(ststhread_);
 
   if (port_mappings_.empty()) {
-    return NrSocket::read(buf, maxlen, len);
+    return internal_socket_->read(buf, maxlen, len);
   } else {
     MOZ_ASSERT(port_mappings_.size() == 1);
-    return port_mappings_.front()->external_socket_->read(buf, maxlen, len);
+    int bytesRead =
+      port_mappings_.front()->external_socket_->read(buf, maxlen, len);
+    if (bytesRead > 0 && nat_->refresh_on_ingress_) {
+      port_mappings_.front()->last_used_ = PR_IntervalNow();
+    }
+    return bytesRead;
   }
 }
 
 int TestNrSocket::async_wait(int how, NR_async_cb cb, void *cb_arg,
                              char *function, int line) {
-  ASSERT_ON_THREAD(ststhread_);
+  r_log(LOG_GENERIC, LOG_DEBUG, "TestNrSocket %s waiting for %s",
+                                internal_socket_->my_addr().as_string,
+                                how == NR_ASYNC_WAIT_READ ? "read" : "write");
 
-  // Make sure we're waiting on the socket for the internal address
-  int r = NrSocket::async_wait(how, cb, cb_arg, function, line);
+  int r;
 
-  if (r) {
-    return r;
+  if (how == NR_ASYNC_WAIT_READ) {
+    NrSocketBase::async_wait(how, cb, cb_arg, function, line);
+
+    // Make sure we're waiting on the socket for the internal address
+    r = internal_socket_->async_wait(how,
+                                     socket_readable_callback,
+                                     this,
+                                     function,
+                                     line);
+  } else {
+    // For write, just use the readiness of the internal socket, since we queue
+    // everything for the port mappings.
+    r = internal_socket_->async_wait(how,
+                                     cb,
+                                     cb_arg,
+                                     function,
+                                     line);
   }
 
-  r_log(LOG_GENERIC, LOG_DEBUG, "TestNrSocket %s waiting for %s",
-                                my_addr_.as_string,
-                                how == NR_ASYNC_WAIT_READ ? "read" : "write");
+  if (r) {
+    r_log(LOG_GENERIC, LOG_ERR, "TestNrSocket %s failed to async_wait for "
+                                "internal socket: %d\n",
+                                internal_socket_->my_addr().as_string,
+                                r);
+    return r;
+  }
 
   if (is_tcp_connection_behind_nat()) {
     // Bypass all port-mapping related logic
     return 0;
   }
 
-  if (my_addr_.protocol == IPPROTO_TCP) {
+  if (internal_socket_->my_addr().protocol == IPPROTO_TCP) {
     // For a TCP connection through a simulated NAT, these signals are
     // just passed through.
     MOZ_ASSERT(port_mappings_.size() == 1);
@@ -478,11 +604,15 @@ int TestNrSocket::async_wait(int how, NR_async_cb cb, void *cb_arg,
     for (PortMapping *port_mapping : port_mappings_) {
       // Be ready to receive traffic on our port mappings
       r = port_mapping->async_wait(how,
-                                   port_mapping_readable_callback,
+                                   socket_readable_callback,
                                    this,
                                    function,
                                    line);
       if (r) {
+        r_log(LOG_GENERIC, LOG_ERR, "TestNrSocket %s failed to async_wait for "
+                                    "port mapping: %d\n",
+                                    internal_socket_->my_addr().as_string,
+                                    r);
         return r;
       }
     }
@@ -498,18 +628,18 @@ void TestNrSocket::cancel_port_mapping_async_wait(int how) {
 }
 
 int TestNrSocket::cancel(int how) {
-  ASSERT_ON_THREAD(ststhread_);
 
   r_log(LOG_GENERIC, LOG_DEBUG, "TestNrSocket %s stop waiting for %s",
-        my_addr_.as_string,
+        internal_socket_->my_addr().as_string,
         how == NR_ASYNC_WAIT_READ ? "read" : "write");
 
   // Writable callbacks are decoupled except for the TCP case
-  if (how == NR_ASYNC_WAIT_READ || my_addr_.protocol == IPPROTO_TCP) {
+  if (how == NR_ASYNC_WAIT_READ ||
+      internal_socket_->my_addr().protocol == IPPROTO_TCP) {
     cancel_port_mapping_async_wait(how);
   }
 
-  return NrSocket::cancel(how);
+  return internal_socket_->cancel(how);
 }
 
 bool TestNrSocket::has_port_mappings() const {
@@ -548,7 +678,7 @@ void TestNrSocket::destroy_stale_port_mappings() {
     if (is_port_mapping_stale(**temp)) {
       r_log(LOG_GENERIC, LOG_INFO,
             "TestNrSocket %s destroying port mapping %s -> %s",
-            my_addr_.as_string,
+            internal_socket_->my_addr().as_string,
             (*temp)->external_socket_->my_addr().as_string,
             (*temp)->remote_address_.as_string);
 
@@ -557,34 +687,27 @@ void TestNrSocket::destroy_stale_port_mappings() {
   }
 }
 
-void TestNrSocket::port_mapping_readable_callback(void *ext_sock_v,
-                                                  int how,
-                                                  void *test_sock_v) {
+void TestNrSocket::socket_readable_callback(void *real_sock_v,
+                                             int how,
+                                             void *test_sock_v) {
   TestNrSocket *test_socket = static_cast<TestNrSocket*>(test_sock_v);
-  NrSocket *external_socket = static_cast<NrSocket*>(ext_sock_v);
+  NrSocket *real_socket = static_cast<NrSocket*>(real_sock_v);
 
-  test_socket->on_port_mapping_readable(external_socket);
+  test_socket->on_socket_readable(real_socket);
 }
 
-void TestNrSocket::on_port_mapping_readable(NrSocket *external_socket) {
-  if (!readable_socket_) {
-    readable_socket_ = external_socket;
+void TestNrSocket::on_socket_readable(NrSocketBase *real_socket) {
+  if (!readable_socket_ && (real_socket != internal_socket_)) {
+    readable_socket_ = real_socket;
   }
 
-  // None of our port mappings should be waiting for readable callbacks
-  // if nobody is waiting for readable callbacks from us.
-  MOZ_ASSERT(poll_flags() & PR_POLL_READ);
-
   fire_readable_callback();
 }
 
 void TestNrSocket::fire_readable_callback() {
   MOZ_ASSERT(poll_flags() & PR_POLL_READ);
-  // Stop listening on all mapped sockets; we will start listening again
-  // if the app starts listening to us again.
-  cancel_port_mapping_async_wait(NR_ASYNC_WAIT_READ);
   r_log(LOG_GENERIC, LOG_DEBUG, "TestNrSocket %s ready for read",
-        my_addr_.as_string);
+        internal_socket_->my_addr().as_string);
   fire_callback(NR_ASYNC_WAIT_READ);
 }
 
@@ -592,13 +715,13 @@ void TestNrSocket::port_mapping_writeable_callback(void *ext_sock_v,
                                                    int how,
                                                    void *test_sock_v) {
   TestNrSocket *test_socket = static_cast<TestNrSocket*>(test_sock_v);
-  NrSocket *external_socket = static_cast<NrSocket*>(ext_sock_v);
+  NrSocketBase *external_socket = static_cast<NrSocketBase*>(ext_sock_v);
 
   test_socket->write_to_port_mapping(external_socket);
 }
 
-void TestNrSocket::write_to_port_mapping(NrSocket *external_socket) {
-  MOZ_ASSERT(my_addr_.protocol != IPPROTO_TCP);
+void TestNrSocket::write_to_port_mapping(NrSocketBase *external_socket) {
+  MOZ_ASSERT(internal_socket_->my_addr().protocol != IPPROTO_TCP);
 
   int r = 0;
   for (PortMapping *port_mapping : port_mappings_) {
@@ -624,17 +747,18 @@ void TestNrSocket::port_mapping_tcp_passthrough_callback(void *ext_sock_v,
                                                          int how,
                                                          void *test_sock_v) {
   TestNrSocket *test_socket = static_cast<TestNrSocket*>(test_sock_v);
-  r_log(LOG_GENERIC, LOG_INFO,
+  r_log(LOG_GENERIC, LOG_DEBUG,
         "TestNrSocket %s firing %s callback",
-        test_socket->my_addr().as_string,
+        test_socket->internal_socket_->my_addr().as_string,
         how == NR_ASYNC_WAIT_READ ? "readable" : "writeable");
 
 
-  test_socket->fire_callback(how);
+  test_socket->internal_socket_->fire_callback(how);
 }
 
 bool TestNrSocket::is_tcp_connection_behind_nat() const {
-  return my_addr_.protocol == IPPROTO_TCP && port_mappings_.empty();
+  return internal_socket_->my_addr().protocol == IPPROTO_TCP &&
+         port_mappings_.empty();
 }
 
 TestNrSocket::PortMapping* TestNrSocket::get_port_mapping(
@@ -665,9 +789,9 @@ TestNrSocket::PortMapping* TestNrSocket::get_port_mapping(
 
 TestNrSocket::PortMapping* TestNrSocket::create_port_mapping(
     const nr_transport_addr &remote_address,
-    const nsRefPtr<NrSocket> &external_socket) const {
+    const RefPtr<NrSocketBase> &external_socket) const {
   r_log(LOG_GENERIC, LOG_INFO, "TestNrSocket %s creating port mapping %s -> %s",
-        my_addr_.as_string,
+        internal_socket_->my_addr().as_string,
         external_socket->my_addr().as_string,
         remote_address.as_string);
 
@@ -676,7 +800,7 @@ TestNrSocket::PortMapping* TestNrSocket::create_port_mapping(
 
 TestNrSocket::PortMapping::PortMapping(
     const nr_transport_addr &remote_address,
-    const nsRefPtr<NrSocket> &external_socket) :
+    const RefPtr<NrSocketBase> &external_socket) :
   external_socket_(external_socket) {
   // TODO(bug 1170299): Remove const_cast when no longer necessary
   nr_transport_addr_copy(&remote_address_,
@@ -689,7 +813,7 @@ int TestNrSocket::PortMapping::send_from_queue() {
 
   while (!send_queue_.empty()) {
     UdpPacket &packet = *send_queue_.front();
-    r_log(LOG_GENERIC, LOG_INFO,
+    r_log(LOG_GENERIC, LOG_DEBUG,
           "PortMapping %s -> %s sending from queue to %s",
           external_socket_->my_addr().as_string,
           remote_address_.as_string,
@@ -706,7 +830,7 @@ int TestNrSocket::PortMapping::send_from_queue() {
               __FUNCTION__, r);
         send_queue_.clear();
       } else {
-        r_log(LOG_GENERIC, LOG_INFO, "Would block, will retry later");
+        r_log(LOG_GENERIC, LOG_DEBUG, "Would block, will retry later");
       }
       break;
     }
@@ -721,7 +845,7 @@ int TestNrSocket::PortMapping::sendto(const void *msg,
                                       size_t len,
                                       const nr_transport_addr &to) {
   MOZ_ASSERT(remote_address_.protocol != IPPROTO_TCP);
-  r_log(LOG_GENERIC, LOG_INFO,
+  r_log(LOG_GENERIC, LOG_DEBUG,
         "PortMapping %s -> %s sending to %s",
         external_socket_->my_addr().as_string,
         remote_address_.as_string,
@@ -733,8 +857,8 @@ int TestNrSocket::PortMapping::sendto(const void *msg,
       const_cast<nr_transport_addr*>(&to));
 
   if (r == R_WOULDBLOCK) {
-    r_log(LOG_GENERIC, LOG_INFO, "Enqueueing UDP packet to %s", to.as_string);
-    send_queue_.push_back(nsRefPtr<UdpPacket>(new UdpPacket(msg, len, to)));
+    r_log(LOG_GENERIC, LOG_DEBUG, "Enqueueing UDP packet to %s", to.as_string);
+    send_queue_.push_back(RefPtr<UdpPacket>(new UdpPacket(msg, len, to)));
     return 0;
   } else if (r) {
     r_log(LOG_GENERIC,LOG_ERR, "Error: %d", r);
