@@ -113,13 +113,11 @@ void
 SdpFmtpAttributeList::Serialize(std::ostream& os) const
 {
   for (auto i = mFmtps.begin(); i != mFmtps.end(); ++i) {
-    os << "a=" << mType << ":" << i->format << " ";
     if (i->parameters) {
+      os << "a=" << mType << ":" << i->format << " ";
       i->parameters->Serialize(os);
-    } else {
-      os << i->parameters_string;
+      os << CRLF;
     }
-    os << CRLF;
   }
 }
 
@@ -147,6 +145,34 @@ void SdpIdentityAttribute::Serialize(std::ostream& os) const
 }
 #endif
 
+// Class to help with omitting a leading delimiter for the first item in a list
+class SkipFirstDelimiter
+{
+  public:
+    explicit SkipFirstDelimiter(const std::string& delim) :
+      mDelim(delim),
+      mFirst(true)
+    {}
+
+    std::ostream& print(std::ostream& os)
+    {
+      if (!mFirst) {
+        os << mDelim;
+      }
+      mFirst = false;
+      return os;
+    }
+
+  private:
+    std::string mDelim;
+    bool mFirst;
+};
+
+static std::ostream& operator<<(std::ostream& os, SkipFirstDelimiter& delim)
+{
+  return delim.print(os);
+}
+
 void
 SdpImageattrAttributeList::XYRange::Serialize(std::ostream& os) const
 {
@@ -160,13 +186,9 @@ SdpImageattrAttributeList::XYRange::Serialize(std::ostream& os) const
     os << discreteValues.front();
   } else {
     os << "[";
-    bool first = true;
+    SkipFirstDelimiter comma(",");
     for (auto value : discreteValues) {
-      if (!first) {
-        os << ",";
-      }
-      first = false;
-      os << value;
+      os << comma << value;
     }
     os << "]";
   }
@@ -442,13 +464,9 @@ SdpImageattrAttributeList::SRange::Serialize(std::ostream& os) const
     os << discreteValues.front();
   } else {
     os << "[";
-    bool first = true;
+    SkipFirstDelimiter comma(",");
     for (auto value : discreteValues) {
-      if (!first) {
-        os << ",";
-      }
-      first = false;
-      os << value;
+      os << comma << value;
     }
     os << "]";
   }
@@ -461,19 +479,28 @@ SdpImageattrAttributeList::PRange::Serialize(std::ostream& os) const
   os << "[" << min << "-" << max << "]";
 }
 
+static std::string ParseToken(std::istream& is,
+                              const std::string& delims,
+                              std::string* error)
+{
+  std::string token;
+  while (is) {
+    unsigned char c = PeekChar(is, error);
+    if (!c || (delims.find(c) != std::string::npos)) {
+      break;
+    }
+    token.push_back(std::tolower(is.get()));
+  }
+  return token;
+}
+
 static std::string ParseKey(std::istream& is, std::string* error)
 {
-  is >> std::ws;
-  std::string key;
-  while (is && PeekChar(is, error) != '=') {
-    key.push_back(std::tolower(is.get()));
-  }
-
+  std::string token = ParseToken(is, "=", error);
   if (!SkipChar(is, '=', error)) {
     return "";
   }
-
-  return key;
+  return token;
 }
 
 static bool SkipBraces(std::istream& is, std::string* error)
@@ -630,31 +657,11 @@ SdpImageattrAttributeList::Set::Serialize(std::ostream& os) const
   os << "]";
 }
 
-static std::string
-GetLowercaseToken(std::istream& is, std::string* error)
-{
-  is >> std::ws;
-  std::string token;
-  while (true) {
-    switch (PeekChar(is, error)) {
-      case '\0':
-      case ' ':
-      case '\t':
-        return token;
-      default:
-        token.push_back(std::tolower(is.get()));
-    }
-  }
-
-  MOZ_ASSERT_UNREACHABLE("Unexpected break in loop");
-  return "";
-}
-
 bool
 SdpImageattrAttributeList::Imageattr::ParseSets(std::istream& is,
                                                 std::string* error)
 {
-  std::string type = GetLowercaseToken(is, error);
+  std::string type = ParseToken(is, " \t", error);
 
   bool* isAll = nullptr;
   std::vector<Set>* sets = nullptr;
@@ -827,6 +834,225 @@ SdpRemoteCandidatesAttribute::Serialize(std::ostream& os) const
   os << CRLF;
 }
 
+bool
+SdpRidAttributeList::Rid::ParseParameters(std::istream& is, std::string* error)
+{
+  if (!PeekChar(is, error)) {
+    // No parameters
+    return true;
+  }
+
+  do {
+    is >> std::ws;
+    std::string key = ParseKey(is, error);
+    if (key.empty()) {
+      return false; // Illegal trailing cruft
+    }
+
+    // This allows pt= to appear anywhere, instead of only at the beginning, but
+    // this ends up being significantly less code.
+    if (key == "pt") {
+      if (!ParseFormats(is, error)) {
+        return false;
+      }
+    } else if (key == "max-width") {
+      if (!GetUnsigned<uint32_t>(
+            is, 0, UINT32_MAX, &constraints.maxWidth, error)) {
+        return false;
+      }
+    } else if (key == "max-height") {
+      if (!GetUnsigned<uint32_t>(
+            is, 0, UINT32_MAX, &constraints.maxHeight, error)) {
+        return false;
+      }
+    } else if (key == "max-fps") {
+      if (!GetUnsigned<uint32_t>(
+            is, 0, UINT32_MAX, &constraints.maxFps, error)) {
+        return false;
+      }
+    } else if (key == "max-fs") {
+      if (!GetUnsigned<uint32_t>(
+            is, 0, UINT32_MAX, &constraints.maxFs, error)) {
+        return false;
+      }
+    } else if (key == "max-br") {
+      if (!GetUnsigned<uint32_t>(
+            is, 0, UINT32_MAX, &constraints.maxBr, error)) {
+        return false;
+      }
+    } else if (key == "max-pps") {
+      if (!GetUnsigned<uint32_t>(
+            is, 0, UINT32_MAX, &constraints.maxPps, error)) {
+        return false;
+      }
+    } else if (key == "depend") {
+      if (!ParseDepend(is, error)) {
+        return false;
+      }
+    } else {
+      (void) ParseToken(is, ";", error);
+    }
+  } while (SkipChar(is, ';', error));
+  return true;
+}
+
+bool
+SdpRidAttributeList::Rid::ParseDepend(
+    std::istream& is,
+    std::string* error)
+{
+  do {
+    std::string id = ParseToken(is, ",;", error);
+    if (id.empty()) {
+      return false;
+    }
+    dependIds.push_back(id);
+  } while(SkipChar(is, ',', error));
+
+  return true;
+}
+
+bool
+SdpRidAttributeList::Rid::ParseFormats(
+    std::istream& is,
+    std::string* error)
+{
+  do {
+    uint16_t fmt;
+    if (!GetUnsigned<uint16_t>(is, 0, 127, &fmt, error)) {
+      return false;
+    }
+    formats.push_back(fmt);
+  } while (SkipChar(is, ',', error));
+
+  return true;
+}
+
+void
+SdpRidAttributeList::Rid::SerializeParameters(std::ostream& os) const
+{
+  if (!HasParameters()) {
+    return;
+  }
+
+  os << " ";
+
+  SkipFirstDelimiter semic(";");
+
+  if (!formats.empty()) {
+    os << semic << "pt=";
+    SkipFirstDelimiter comma(",");
+    for (uint16_t fmt : formats) {
+      os << comma << fmt;
+    }
+  }
+
+  if (constraints.maxWidth) {
+    os << semic << "max-width=" << constraints.maxWidth;
+  }
+
+  if (constraints.maxHeight) {
+    os << semic << "max-height=" << constraints.maxHeight;
+  }
+
+  if (constraints.maxFps) {
+    os << semic << "max-fps=" << constraints.maxFps;
+  }
+
+  if (constraints.maxFs) {
+    os << semic << "max-fs=" << constraints.maxFs;
+  }
+
+  if (constraints.maxBr) {
+    os << semic << "max-br=" << constraints.maxBr;
+  }
+
+  if (constraints.maxPps) {
+    os << semic << "max-pps=" << constraints.maxPps;
+  }
+
+  if (!dependIds.empty()) {
+    os << semic << "depend=";
+    SkipFirstDelimiter comma(",");
+    for (const std::string& id : dependIds) {
+      os << comma << id;
+    }
+  }
+}
+
+bool
+SdpRidAttributeList::Rid::Parse(std::istream& is, std::string* error)
+{
+  id = ParseToken(is, " ", error);
+  if (id.empty()) {
+    return false;
+  }
+
+  is >> std::ws;
+  std::string directionToken = ParseToken(is, " ", error);
+  if (directionToken == "send") {
+    direction = sdp::kSend;
+  } else if (directionToken == "recv") {
+    direction = sdp::kRecv;
+  } else {
+    *error = "Invalid direction, must be either send or recv";
+    return false;
+  }
+
+  return ParseParameters(is, error);
+}
+
+void
+SdpRidAttributeList::Rid::Serialize(std::ostream& os) const
+{
+  os << id << " " << direction;
+  SerializeParameters(os);
+}
+
+bool
+SdpRidAttributeList::Rid::HasFormat(const std::string& format) const
+{
+  uint16_t formatAsInt;
+  if (!SdpHelper::GetPtAsInt(format, &formatAsInt)) {
+    return false;
+  }
+
+  if (formats.empty()) {
+    return true;
+  }
+
+  return (std::find(formats.begin(), formats.end(), formatAsInt) !=
+         formats.end());
+}
+
+void
+SdpRidAttributeList::Serialize(std::ostream& os) const
+{
+  for (const Rid& rid : mRids) {
+    os << "a=" << mType << ":";
+    rid.Serialize(os);
+    os << CRLF;
+  }
+}
+
+bool
+SdpRidAttributeList::PushEntry(const std::string& raw,
+                               std::string* error,
+                               size_t* errorPos)
+{
+  std::istringstream is(raw);
+
+  Rid rid;
+  if (!rid.Parse(is, error)) {
+    is.clear();
+    *errorPos = is.tellg();
+    return false;
+  }
+
+  mRids.push_back(rid);
+  return true;
+}
+
 void
 SdpRtcpAttribute::Serialize(std::ostream& os) const
 {
@@ -899,11 +1125,8 @@ void
 SdpSctpmapAttributeList::Serialize(std::ostream& os) const
 {
   for (auto i = mSctpmaps.begin(); i != mSctpmaps.end(); ++i) {
-    os << "a=" << mType << ":" << i->pt << " " << i->name;
-    if (i->streams) {
-      os << " " << i->streams;
-    }
-    os << CRLF;
+    os << "a=" << mType << ":" << i->pt << " " << i->name << " " << i->streams
+      << CRLF;
   }
 }
 
@@ -916,15 +1139,9 @@ SdpSetupAttribute::Serialize(std::ostream& os) const
 void
 SdpSimulcastAttribute::Version::Serialize(std::ostream& os) const
 {
-  bool first = true;
-  for (uint16_t format : choices) {
-    if (first) {
-      first = false;
-    } else {
-      os << ",";
-    }
-
-    os << format;
+  SkipFirstDelimiter comma(",");
+  for (const std::string& choice : choices) {
+    os << comma << choice;
   }
 }
 
@@ -932,8 +1149,8 @@ bool
 SdpSimulcastAttribute::Version::Parse(std::istream& is, std::string* error)
 {
   do {
-    uint16_t value;
-    if (!GetUnsigned<uint16_t>(is, 0, UINT16_MAX, &value, error)) {
+    std::string value = ParseToken(is, ",; ", error);
+    if (value.empty()) {
       return false;
     }
     choices.push_back(value);
@@ -942,44 +1159,39 @@ SdpSimulcastAttribute::Version::Parse(std::istream& is, std::string* error)
   return true;
 }
 
-void
-SdpSimulcastAttribute::Version::AppendAsStrings(
-    std::vector<std::string>* formats) const
+bool
+SdpSimulcastAttribute::Version::GetChoicesAsFormats(
+    std::vector<uint16_t>* formats) const
 {
-  for (uint16_t pt : choices) {
-    std::ostringstream os;
-    os << pt;
-    formats->push_back(os.str());
-  }
-}
-
-void
-SdpSimulcastAttribute::Version::AddChoice(const std::string& pt)
-{
-  uint16_t ptAsInt;
-  if (!SdpHelper::GetPtAsInt(pt, &ptAsInt)) {
-    MOZ_ASSERT(false);
-    return;
+  for (const std::string& choice : choices) {
+    uint16_t format;
+    if (!SdpHelper::GetPtAsInt(choice, &format) || (format > 127)) {
+      return false;
+    }
+    formats->push_back(format);
   }
 
-  choices.push_back(ptAsInt);
+  return true;
 }
 
 void
 SdpSimulcastAttribute::Versions::Serialize(std::ostream& os) const
 {
-  bool first = true;
+  switch (type) {
+    case kRid:
+      os << "rid=";
+      break;
+    case kPt:
+      os << "pt=";
+      break;
+  }
+
+  SkipFirstDelimiter semic(";");
   for (const Version& version : *this) {
     if (!version.IsSet()) {
       continue;
     }
-
-    if (first) {
-      first = false;
-    } else {
-      os << ";";
-    }
-
+    os << semic;
     version.Serialize(os);
   }
 }
@@ -987,11 +1199,35 @@ SdpSimulcastAttribute::Versions::Serialize(std::ostream& os) const
 bool
 SdpSimulcastAttribute::Versions::Parse(std::istream& is, std::string* error)
 {
+  std::string rawType = ParseKey(is, error);
+  if (rawType.empty()) {
+    return false;
+  }
+
+  if (rawType == "pt") {
+    type = kPt;
+  } else if (rawType == "rid") {
+    type = kRid;
+  } else {
+    *error = "Unknown simulcast identification type ";
+    error->append(rawType);
+    return false;
+  }
+
   do {
     Version version;
     if (!version.Parse(is, error)) {
       return false;
     }
+
+    if (type == kPt) {
+      std::vector<uint16_t> formats;
+      if (!version.GetChoicesAsFormats(&formats)) {
+        *error = "Invalid payload type";
+        return false;
+      }
+    }
+
     push_back(version);
   } while(SkipChar(is, ';', error));
 
@@ -1001,9 +1237,7 @@ SdpSimulcastAttribute::Versions::Parse(std::istream& is, std::string* error)
 void
 SdpSimulcastAttribute::Serialize(std::ostream& os) const
 {
-  MOZ_ASSERT(sendVersions.IsSet() ||
-             recvVersions.IsSet() ||
-             sendrecvVersions.IsSet());
+  MOZ_ASSERT(sendVersions.IsSet() || recvVersions.IsSet());
 
   os << "a=" << mType << ":";
 
@@ -1017,11 +1251,6 @@ SdpSimulcastAttribute::Serialize(std::ostream& os) const
     recvVersions.Serialize(os);
   }
 
-  if (sendrecvVersions.IsSet()) {
-    os << " sendrecv ";
-    sendrecvVersions.Serialize(os);
-  }
-
   os << CRLF;
 }
 
@@ -1030,10 +1259,10 @@ SdpSimulcastAttribute::Parse(std::istream& is, std::string* error)
 {
   bool gotRecv = false;
   bool gotSend = false;
-  bool gotSendrecv = false;
 
   while (true) {
-    std::string token = GetLowercaseToken(is, error);
+    is >> std::ws;
+    std::string token = ParseToken(is, " \t", error);
     if (token.empty()) {
       break;
     }
@@ -1060,24 +1289,13 @@ SdpSimulcastAttribute::Parse(std::istream& is, std::string* error)
       if (!recvVersions.Parse(is, error)) {
         return false;
       }
-    } else if (token == "sendrecv") {
-      if (gotSendrecv) {
-        *error = "Already got a sendrecv list";
-        return false;
-      }
-      gotSendrecv = true;
-
-      is >> std::ws;
-      if (!sendrecvVersions.Parse(is, error)) {
-        return false;
-      }
     } else {
-      *error = "Type must be either 'send', 'recv', or 'sendrecv'";
+      *error = "Type must be either 'send' or 'recv'";
       return false;
     }
   }
 
-  if (!gotSend && !gotRecv && !gotSendrecv) {
+  if (!gotSend && !gotRecv) {
     *error = "Empty simulcast attribute";
     return false;
   }
@@ -1219,6 +1437,8 @@ SdpAttribute::IsAllowedAtMediaLevel(AttributeType type)
       return true;
     case kRemoteCandidatesAttribute:
       return true;
+    case kRidAttribute:
+      return true;
     case kRtcpAttribute:
       return true;
     case kRtcpFbAttribute:
@@ -1301,6 +1521,8 @@ SdpAttribute::IsAllowedAtSessionLevel(AttributeType type)
       return true;
     case kRemoteCandidatesAttribute:
       return false;
+    case kRidAttribute:
+      return false;
     case kRtcpAttribute:
       return false;
     case kRtcpFbAttribute:
@@ -1381,6 +1603,8 @@ SdpAttribute::GetAttributeTypeString(AttributeType type)
       return "recvonly";
     case kRemoteCandidatesAttribute:
       return "remote-candidates";
+    case kRidAttribute:
+      return "rid";
     case kRtcpAttribute:
       return "rtcp";
     case kRtcpFbAttribute:
