@@ -22,6 +22,7 @@
 #include "mozilla/Maybe.h"
 
 #include "signaling/src/sdp/SdpEnum.h"
+#include "signaling/src/common/EncodingConstraints.h"
 
 namespace mozilla
 {
@@ -59,6 +60,7 @@ public:
     kPtimeAttribute,
     kRecvonlyAttribute,
     kRemoteCandidatesAttribute,
+    kRidAttribute,
     kRtcpAttribute,
     kRtcpFbAttribute,
     kRtcpMuxAttribute,
@@ -771,6 +773,101 @@ public:
   std::vector<Candidate> mCandidates;
 };
 
+/*
+a=rid, draft-pthatcher-mmusic-rid-01
+
+   rid-syntax        = "a=rid:" rid-identifier SP rid-dir
+                       [ rid-pt-param-list / rid-param-list ]
+
+   rid-identifier    = 1*(alpha-numeric / "-" / "_")
+
+   rid-dir           = "send" / "recv"
+
+   rid-pt-param-list = SP rid-fmt-list *(";" rid-param)
+
+   rid-param-list    = SP rid-param *(";" rid-param)
+
+   rid-fmt-list      = "pt=" fmt *( "," fmt )
+                        ; fmt defined in {{RFC4566}}
+
+   rid-param         = rid-width-param
+                       / rid-height-param
+                       / rid-fps-param
+                       / rid-fs-param
+                       / rid-br-param
+                       / rid-pps-param
+                       / rid-depend-param
+                       / rid-param-other
+
+   rid-width-param   = "max-width" [ "=" int-param-val ]
+
+   rid-height-param  = "max-height" [ "=" int-param-val ]
+
+   rid-fps-param     = "max-fps" [ "=" int-param-val ]
+
+   rid-fs-param      = "max-fs" [ "=" int-param-val ]
+
+   rid-br-param      = "max-br" [ "=" int-param-val ]
+
+   rid-pps-param     = "max-pps" [ "=" int-param-val ]
+
+   rid-depend-param  = "depend=" rid-list
+
+   rid-param-other   = 1*(alpha-numeric / "-") [ "=" param-val ]
+
+   rid-list          = rid-identifier *( "," rid-identifier )
+
+   int-param-val     = 1*DIGIT
+
+   param-val         = *( %x20-58 / %x60-7E )
+                       ; Any printable character except semicolon
+*/
+class SdpRidAttributeList : public SdpAttribute
+{
+public:
+  explicit SdpRidAttributeList()
+    : SdpAttribute(kRidAttribute)
+  {}
+
+  struct Rid
+  {
+    Rid() :
+      direction(sdp::kSend)
+    {}
+
+    bool Parse(std::istream& is, std::string* error);
+    bool ParseParameters(std::istream& is, std::string* error);
+    bool ParseDepend(std::istream& is, std::string* error);
+    bool ParseFormats(std::istream& is, std::string* error);
+    void Serialize(std::ostream& os) const;
+    void SerializeParameters(std::ostream& os) const;
+    bool HasFormat(const std::string& format) const;
+    bool HasParameters() const
+    {
+      return !formats.empty() ||
+        constraints.maxWidth ||
+        constraints.maxHeight ||
+        constraints.maxFps ||
+        constraints.maxFs ||
+        constraints.maxBr ||
+        constraints.maxPps ||
+        !dependIds.empty();
+    }
+
+
+    std::string id;
+    sdp::Direction direction;
+    std::vector<uint16_t> formats; // Empty implies all
+    EncodingConstraints constraints;
+    std::vector<std::string> dependIds;
+  };
+
+  virtual void Serialize(std::ostream& os) const override;
+  bool PushEntry(const std::string& raw, std::string* error, size_t* errorPos);
+
+  std::vector<Rid> mRids;
+};
+
 ///////////////////////////////////////////////////////////////////////////
 // a=rtcp, RFC3605
 //-------------------------------------------------------------------------
@@ -845,7 +942,7 @@ class SdpRtcpFbAttributeList : public SdpAttribute
 public:
   SdpRtcpFbAttributeList() : SdpAttribute(kRtcpFbAttribute) {}
 
-  enum Type { kAck, kApp, kCcm, kNack, kTrrInt };
+  enum Type { kAck, kApp, kCcm, kNack, kTrrInt, kRemb };
 
   static const char* pli;
   static const char* sli;
@@ -896,6 +993,9 @@ inline std::ostream& operator<<(std::ostream& os,
     case SdpRtcpFbAttributeList::kTrrInt:
       os << "trr-int";
       break;
+    case SdpRtcpFbAttributeList::kRemb:
+      os << "goog-remb";
+      break;
     default:
       MOZ_ASSERT(false);
       os << "?";
@@ -1139,21 +1239,45 @@ public:
     unsigned int max_fr;
   };
 
+  class OpusParameters : public Parameters
+  {
+  public:
+    enum { kDefaultMaxPlaybackRate = 48000,
+           kDefaultStereo = 0 };
+    OpusParameters() :
+      Parameters(SdpRtpmapAttributeList::kOpus),
+      maxplaybackrate(kDefaultMaxPlaybackRate),
+      stereo(kDefaultStereo)
+    {}
+
+    Parameters*
+    Clone() const override
+    {
+      return new OpusParameters(*this);
+    }
+
+    void
+    Serialize(std::ostream& os) const override
+    {
+      os << "maxplaybackrate=" << maxplaybackrate << ";"
+         << "stereo=" << stereo;
+    }
+
+    unsigned int maxplaybackrate;
+    unsigned int stereo;
+  };
+
   class Fmtp
   {
   public:
-    Fmtp(const std::string& aFormat, const std::string& aParametersString,
-         UniquePtr<Parameters> aParameters)
+    Fmtp(const std::string& aFormat, UniquePtr<Parameters> aParameters)
         : format(aFormat),
-          parameters_string(aParametersString),
           parameters(Move(aParameters))
     {
     }
 
-    Fmtp(const std::string& aFormat, const std::string& aParametersString,
-         const Parameters& aParameters)
+    Fmtp(const std::string& aFormat, const Parameters& aParameters)
         : format(aFormat),
-          parameters_string(aParametersString),
           parameters(aParameters.Clone())
     {
     }
@@ -1165,34 +1289,28 @@ public:
     {
       if (this != &rhs) {
         format = rhs.format;
-        parameters_string = rhs.parameters_string;
         parameters.reset(rhs.parameters ? rhs.parameters->Clone() : nullptr);
       }
       return *this;
     }
 
     // The contract around these is as follows:
-    // * |format| and |parameters_string| are always set
     // * |parameters| is only set if we recognized the media type and had
     //   a subclass of Parameters to represent that type of parameters
     // * |parameters| is a best-effort representation; it might be missing
     //   stuff
-    // * if |parameters| is set, it determines the serialized form,
-    //   otherwise |parameters_string| is used
     // * Parameters::codec_type tells you the concrete class, eg
     //   kH264 -> H264Parameters
     std::string format;
-    std::string parameters_string;
     UniquePtr<Parameters> parameters;
   };
 
   virtual void Serialize(std::ostream& os) const override;
 
   void
-  PushEntry(const std::string& format, const std::string& parameters_string,
-            UniquePtr<Parameters> parameters)
+  PushEntry(const std::string& format, UniquePtr<Parameters> parameters)
   {
-    mFmtps.push_back(Fmtp(format, parameters_string, Move(parameters)));
+    mFmtps.push_back(Fmtp(format, Move(parameters)));
   }
 
   std::vector<Fmtp> mFmtps;
@@ -1301,16 +1419,16 @@ inline std::ostream& operator<<(std::ostream& os, SdpSetupAttribute::Role r)
   return os;
 }
 
-// Note: This ABNF is buggy since it does not include a ':' after "a=simulcast"
-// The authors have said this will be fixed in a subsequent version.
-// TODO(bug 1191986): Update this ABNF once the new version is out.
-// simulcast-attribute = "a=simulcast" 1*3( WSP sc-dir-list )
-// sc-dir-list         = sc-dir WSP sc-fmt-list *( ";" sc-fmt-list )
-// sc-dir              = "send" / "recv" / "sendrecv"
-// sc-fmt-list         = sc-fmt *( "," sc-fmt )
-// sc-fmt              = fmt
+// sc-attr     = "a=simulcast:" 1*2( WSP sc-str-list ) [WSP sc-pause-list]
+// sc-str-list = sc-dir WSP sc-id-type "=" sc-alt-list *( ";" sc-alt-list )
+// sc-pause-list = "paused=" sc-alt-list
+// sc-dir      = "send" / "recv"
+// sc-id-type  = "pt" / "rid" / token
+// sc-alt-list = sc-id *( "," sc-id )
+// sc-id       = fmt / rid-identifier / token
 // ; WSP defined in [RFC5234]
-// ; fmt defined in [RFC4566]
+// ; fmt, token defined in [RFC4566]
+// ; rid-identifier defined in [I-D.pthatcher-mmusic-rid]
 class SdpSimulcastAttribute : public SdpAttribute
 {
 public:
@@ -1328,15 +1446,20 @@ public:
         return !choices.empty();
       }
       bool Parse(std::istream& is, std::string* error);
-      void AppendAsStrings(std::vector<std::string>* formats) const;
-      void AddChoice(const std::string& pt);
+      bool GetChoicesAsFormats(std::vector<uint16_t>* formats) const;
 
-      std::vector<uint16_t> choices;
+      std::vector<std::string> choices;
   };
 
   class Versions : public std::vector<Version>
   {
     public:
+      enum Type {
+        kPt,
+        kRid
+      };
+
+      Versions() : type(kRid) {}
       void Serialize(std::ostream& os) const;
       bool IsSet() const
       {
@@ -1352,12 +1475,13 @@ public:
 
         return false;
       }
+
       bool Parse(std::istream& is, std::string* error);
+      Type type;
   };
 
   Versions sendVersions;
   Versions recvVersions;
-  Versions sendrecvVersions;
 };
 
 ///////////////////////////////////////////////////////////////////////////
