@@ -106,8 +106,6 @@ SipccSdpAttributeList::LoadSimpleStrings(sdp_t* sdp, uint16_t level,
                    errorHolder);
   LoadSimpleString(sdp, level, SDP_ATTR_LABEL, SdpAttribute::kLabelAttribute,
                    errorHolder);
-  LoadSimpleString(sdp, level, SDP_ATTR_IDENTITY,
-                   SdpAttribute::kIdentityAttribute, errorHolder);
 }
 
 void
@@ -513,8 +511,8 @@ SipccSdpAttributeList::LoadImageattr(sdp_t* sdp,
     std::string error;
     size_t errorPos;
     if (!imageattrs->PushEntry(imageattrRaw, &error, &errorPos)) {
-      std::ostringstream fullError(error + " at column ");
-      fullError << errorPos;
+      std::ostringstream fullError;
+      fullError << error << " at column " << errorPos;
       errorHolder.AddParseError(
         sdp_attr_line_number(sdp, SDP_ATTR_IMAGEATTR, level, 0, i),
         fullError.str());
@@ -548,9 +546,8 @@ SipccSdpAttributeList::LoadSimulcast(sdp_t* sdp,
   std::istringstream is(simulcastRaw);
   std::string error;
   if (!simulcast->Parse(is, &error)) {
-    is.clear();
-    std::ostringstream fullError(error + " at column ");
-    fullError << is.tellg();
+    std::ostringstream fullError;
+    fullError << error << " at column " << is.tellg();
     errorHolder.AddParseError(
       sdp_attr_line_number(sdp, SDP_ATTR_SIMULCAST, level, 0, 1),
       fullError.str());
@@ -648,6 +645,16 @@ SipccSdpAttributeList::LoadMsidSemantics(sdp_t* sdp, uint16_t level,
   return true;
 }
 
+void
+SipccSdpAttributeList::LoadIdentity(sdp_t* sdp, uint16_t level)
+{
+  const char* val = sdp_attr_get_long_string(sdp, SDP_ATTR_IDENTITY, level, 0, 1);
+  if (val) {
+    SetAttribute(new SdpStringAttribute(SdpAttribute::kIdentityAttribute,
+                                        std::string(val)));
+  }
+}
+
 void
 SipccSdpAttributeList::LoadFmtp(sdp_t* sdp, uint16_t level)
 {
@@ -667,21 +674,6 @@ SipccSdpAttributeList::LoadFmtp(sdp_t* sdp, uint16_t level)
     // payload_num is the number in the fmtp attribute, verbatim
     osPayloadType << fmtp->payload_num;
 
-    // Get the serialized form of the parameters
-    flex_string fs;
-    flex_string_init(&fs);
-
-    // Very lame, but we need direct access so we can get the serialized form
-    sdp_result_e sdpres = sdp_build_attr_fmtp_params(sdp, fmtp, &fs);
-
-    if (sdpres != SDP_SUCCESS) {
-      flex_string_free(&fs);
-      continue;
-    }
-
-    std::string paramsString(fs.buffer);
-    flex_string_free(&fs);
-
     // Get parsed form of parameters, if supported
     UniquePtr<SdpFmtpAttributeList::Parameters> parameters;
 
@@ -700,14 +692,7 @@ SipccSdpAttributeList::LoadFmtp(sdp_t* sdp, uint16_t level)
             !!(fmtp->level_asymmetry_allowed);
 
         h264Parameters->packetization_mode = fmtp->packetization_mode;
-// Copied from VcmSIPCCBinding
-#ifdef _WIN32
-        sscanf_s(fmtp->profile_level_id, "%x",
-                 &h264Parameters->profile_level_id, sizeof(unsigned*));
-#else
-        sscanf(fmtp->profile_level_id, "%xu",
-               &h264Parameters->profile_level_id);
-#endif
+        sscanf(fmtp->profile_level_id, "%x", &h264Parameters->profile_level_id);
         h264Parameters->max_mbps = fmtp->max_mbps;
         h264Parameters->max_fs = fmtp->max_fs;
         h264Parameters->max_cpb = fmtp->max_cpb;
@@ -736,11 +721,18 @@ SipccSdpAttributeList::LoadFmtp(sdp_t* sdp, uint16_t level)
 
         parameters.reset(vp8Parameters);
       } break;
+      case RTP_OPUS: {
+        SdpFmtpAttributeList::OpusParameters* opusParameters(
+            new SdpFmtpAttributeList::OpusParameters);
+        opusParameters->maxplaybackrate = fmtp->maxplaybackrate;
+        opusParameters->stereo = fmtp->stereo;
+        parameters.reset(opusParameters);
+      } break;
       default: {
       }
     }
 
-    fmtps->PushEntry(osPayloadType.str(), paramsString, Move(parameters));
+    fmtps->PushEntry(osPayloadType.str(), Move(parameters));
   }
 
   if (!fmtps->mFmtps.empty()) {
@@ -783,6 +775,41 @@ SipccSdpAttributeList::LoadMsids(sdp_t* sdp, uint16_t level,
   }
 }
 
+bool
+SipccSdpAttributeList::LoadRid(sdp_t* sdp,
+                                     uint16_t level,
+                                     SdpErrorHolder& errorHolder)
+{
+  UniquePtr<SdpRidAttributeList> rids(new SdpRidAttributeList);
+
+  for (uint16_t i = 1; i < UINT16_MAX; ++i) {
+    const char* ridRaw = sdp_attr_get_simple_string(sdp,
+                                                    SDP_ATTR_RID,
+                                                    level,
+                                                    0,
+                                                    i);
+    if (!ridRaw) {
+      break;
+    }
+
+    std::string error;
+    size_t errorPos;
+    if (!rids->PushEntry(ridRaw, &error, &errorPos)) {
+      std::ostringstream fullError;
+      fullError << error << " at column " << errorPos;
+      errorHolder.AddParseError(
+        sdp_attr_line_number(sdp, SDP_ATTR_RID, level, 0, i),
+        fullError.str());
+      return false;
+    }
+  }
+
+  if (!rids->mRids.empty()) {
+    SetAttribute(rids.release());
+  }
+  return true;
+}
+
 void
 SipccSdpAttributeList::LoadExtmap(sdp_t* sdp, uint16_t level,
                                   SdpErrorHolder& errorHolder)
@@ -904,6 +931,9 @@ SipccSdpAttributeList::LoadRtcpFb(sdp_t* sdp, uint16_t level,
         os << rtcpfb->param.trr_int;
         parameter = os.str();
       } break;
+      case SDP_RTCP_FB_REMB: {
+        type = SdpRtcpFbAttributeList::kRemb;
+      } break;
       default:
         // Type we don't care about, ignore.
         continue;
@@ -977,6 +1007,8 @@ SipccSdpAttributeList::Load(sdp_t* sdp, uint16_t level,
     if (!LoadMsidSemantics(sdp, level, errorHolder)) {
       return false;
     }
+
+    LoadIdentity(sdp, level);
   } else {
     sdp_media_e mtype = sdp_get_media_type(sdp, level);
     if (mtype == SDP_MEDIA_APPLICATION) {
@@ -1000,6 +1032,9 @@ SipccSdpAttributeList::Load(sdp_t* sdp, uint16_t level,
     if (!LoadSimulcast(sdp, level, errorHolder)) {
       return false;
     }
+    if (!LoadRid(sdp, level, errorHolder)) {
+      return false;
+    }
   }
 
   LoadIceAttributes(sdp, level);
@@ -1224,6 +1259,16 @@ SipccSdpAttributeList::GetMsidSemantic() const
   return *static_cast<const SdpMsidSemanticAttributeList*>(attr);
 }
 
+const SdpRidAttributeList&
+SipccSdpAttributeList::GetRid() const
+{
+  if (!HasAttribute(SdpAttribute::kRidAttribute)) {
+    MOZ_CRASH();
+  }
+  const SdpAttribute* attr = GetAttribute(SdpAttribute::kRidAttribute);
+  return *static_cast<const SdpRidAttributeList*>(attr);
+}
+
 uint32_t
 SipccSdpAttributeList::GetPtime() const
 {
