@@ -19,6 +19,7 @@
 #include "mozilla/RefPtr.h"
 #include "FakeMediaStreams.h"
 #include "FakeMediaStreamsImpl.h"
+#include "FakeLogging.h"
 #include "MediaConduitErrors.h"
 #include "MediaConduitInterface.h"
 #include "MediaPipeline.h"
@@ -35,10 +36,15 @@
 
 #include "webrtc/modules/interface/module_common_types.h"
 
+#include "FakeIPC.h"
+#include "FakeIPC.cpp"
+
 #define GTEST_HAS_RTTI 0
 #include "gtest/gtest.h"
 #include "gtest_utils.h"
 
+#include "TestHarness.h"
+
 using namespace mozilla;
 MOZ_MTLOG_MODULE("mediapipeline")
 
@@ -123,7 +129,7 @@ class TransportInfo {
     flow_ = nullptr;
   }
 
-  mozilla::RefPtr<TransportFlow> flow_;
+  RefPtr<TransportFlow> flow_;
   TransportLayerLoopback *loopback_;
   TransportLayerDtls *dtls_;
 };
@@ -131,7 +137,7 @@ class TransportInfo {
 class TestAgent {
  public:
   TestAgent() :
-      audio_config_(109, "opus", 48000, 960, 2, 64000),
+      audio_config_(109, "opus", 48000, 960, 2, 64000, false),
       audio_conduit_(mozilla::AudioSessionConduit::Create()),
       audio_(),
       audio_pipeline_() {
@@ -186,7 +192,7 @@ class TestAgent {
     audio_rtcp_transport_.Shutdown();
     bundle_transport_.Shutdown();
     if (audio_pipeline_)
-      audio_pipeline_->ShutdownTransport_s();
+      audio_pipeline_->DetachTransport_s();
   }
 
   void Shutdown() {
@@ -228,12 +234,12 @@ class TestAgent {
 
  protected:
   mozilla::AudioCodecConfig audio_config_;
-  mozilla::RefPtr<mozilla::MediaSessionConduit> audio_conduit_;
-  nsRefPtr<DOMMediaStream> audio_;
+  RefPtr<mozilla::MediaSessionConduit> audio_conduit_;
+  RefPtr<DOMMediaStream> audio_;
   // TODO(bcampen@mozilla.com): Right now this does not let us test RTCP in
   // both directions; only the sender's RTCP is sent, but the receiver should
   // be sending it too.
-  mozilla::RefPtr<mozilla::MediaPipeline> audio_pipeline_;
+  RefPtr<mozilla::MediaPipeline> audio_pipeline_;
   TransportInfo audio_rtp_transport_;
   TransportInfo audio_rtcp_transport_;
   TransportInfo bundle_transport_;
@@ -245,6 +251,11 @@ class TestAgentSend : public TestAgent {
 
   virtual void CreatePipelines_s(bool aIsRtcpMux) {
     audio_ = new Fake_DOMMediaStream(new Fake_AudioStreamSource());
+    audio_->SetHintContents(Fake_DOMMediaStream::HINT_CONTENTS_AUDIO);
+
+    nsTArray<RefPtr<Fake_MediaStreamTrack>> tracks;
+    audio_->GetAudioTracks(tracks);
+    ASSERT_EQ(1U, tracks.Length());
 
     mozilla::MediaConduitErrorCode err =
         static_cast<mozilla::AudioSessionConduit *>(audio_conduit_.get())->
@@ -269,10 +280,9 @@ class TestAgentSend : public TestAgent {
         test_pc,
         nullptr,
         test_utils->sts_target(),
-        audio_,
+        tracks[0],
         "audio_track_fake_uuid",
         1,
-        false,
         audio_conduit_,
         rtp,
         rtcp,
@@ -320,12 +330,11 @@ class TestAgentReceive : public TestAgent {
         test_pc,
         nullptr,
         test_utils->sts_target(),
-        audio_->GetStream(), "audio_track_fake_uuid", 1, 1,
+        audio_->GetStream()->AsSourceStream(), "audio_track_fake_uuid", 1, 1,
         static_cast<mozilla::AudioSessionConduit *>(audio_conduit_.get()),
         audio_rtp_transport_.flow_,
         audio_rtcp_transport_.flow_,
-        bundle_filter_,
-        false);
+        bundle_filter_);
 
     audio_pipeline_->Init();
   }
@@ -396,9 +405,9 @@ class MediaPipelineTest : public ::testing::Test {
     // Setup transport flows
     InitTransports(aIsRtcpMux);
 
-    mozilla::SyncRunnable::DispatchToThread(
-      test_utils->sts_target(),
-      WrapRunnable(&p1_, &TestAgent::CreatePipelines_s, aIsRtcpMux));
+    NS_DispatchToMainThread(
+      WrapRunnable(&p1_, &TestAgent::CreatePipelines_s, aIsRtcpMux),
+      NS_DISPATCH_SYNC);
 
     mozilla::SyncRunnable::DispatchToThread(
       test_utils->sts_target(),
@@ -560,7 +569,6 @@ TEST_F(MediaPipelineFilterTest, TestFilterReport0CountTruncated) {
 TEST_F(MediaPipelineFilterTest, TestFilterReport1SSRCTruncated) {
   MediaPipelineFilter filter;
   filter.AddRemoteSSRC(16);
-  filter.AddLocalSSRC(17);
   const unsigned char sr[] = {
     RTCP_TYPEINFO(1, MediaPipelineFilter::SENDER_REPORT_T, 12),
     REPORT_FRAGMENT(16),
@@ -572,7 +580,6 @@ TEST_F(MediaPipelineFilterTest, TestFilterReport1SSRCTruncated) {
 TEST_F(MediaPipelineFilterTest, TestFilterReport1BigSSRC) {
   MediaPipelineFilter filter;
   filter.AddRemoteSSRC(0x01020304);
-  filter.AddLocalSSRC(0x11121314);
   const unsigned char sr[] = {
     RTCP_TYPEINFO(1, MediaPipelineFilter::SENDER_REPORT_T, 12),
     SSRC(0x01020304),
@@ -597,7 +604,6 @@ TEST_F(MediaPipelineFilterTest, TestFilterReportNoMatch) {
 
 TEST_F(MediaPipelineFilterTest, TestFilterUnknownRTCPType) {
   MediaPipelineFilter filter;
-  filter.AddLocalSSRC(18);
   ASSERT_FALSE(filter.FilterSenderReport(unknown_type, sizeof(unknown_type)));
 }
 
@@ -698,6 +704,7 @@ TEST_F(MediaPipelineTest, TestAudioSendEmptyBundleFilter) {
 
 
 int main(int argc, char **argv) {
+  ScopedXPCOM xpcom("mediapipeline_unittest");
   test_utils = new MtransportTestUtils();
   // Start the tests
   NSS_NoDB_Init(nullptr);
