@@ -17,6 +17,7 @@
 #include "nsProxyRelease.h"
 #include "SerializedLoadContext.h"
 #include "nsNetUtil.h"
+#include "nsIFileURL.h"
 
 // needed to alloc/free NSPR file descriptors
 #include "private/pprio.h"
@@ -34,7 +35,7 @@ namespace net {
 // Helper class to dispatch events async on windows/OSX
 //-----------------------------------------------------------------------------
 
-class CallsListenerInNewEvent : public nsRunnable
+class CallsListenerInNewEvent : public Runnable
 {
 public:
     CallsListenerInNewEvent(nsIRemoteOpenFileListener *aListener, nsresult aRv)
@@ -106,35 +107,10 @@ RemoteOpenFileChild::~RemoteOpenFileChild()
       NotifyListener(NS_ERROR_UNEXPECTED);
     }
   } else {
-    nsCOMPtr<nsIThread> mainThread = do_GetMainThread();
-    if (mainThread) {
-      MOZ_ALWAYS_TRUE(NS_SUCCEEDED(NS_ProxyRelease(mainThread, mURI, true)));
-      MOZ_ALWAYS_TRUE(NS_SUCCEEDED(NS_ProxyRelease(mainThread, mAppURI, true)));
-      MOZ_ALWAYS_TRUE(NS_SUCCEEDED(NS_ProxyRelease(mainThread, mListener,
-                                                   true)));
-
-      TabChild* tabChild;
-      mTabChild.forget(&tabChild);
-
-      if (tabChild) {
-        nsCOMPtr<nsIRunnable> runnable =
-          NS_NewNonOwningRunnableMethod(tabChild, &TabChild::Release);
-        MOZ_ASSERT(runnable);
-
-        MOZ_ALWAYS_TRUE(NS_SUCCEEDED(mainThread->Dispatch(runnable,
-                                                          NS_DISPATCH_NORMAL)));
-      }
-    } else {
-      using mozilla::unused;
-
-      NS_WARNING("RemoteOpenFileChild released after thread shutdown, leaking "
-                 "its members!");
-
-      unused << mURI.forget();
-      unused << mAppURI.forget();
-      unused << mListener.forget();
-      unused << mTabChild.forget();
-    }
+    NS_ReleaseOnMainThread(mURI.forget(), true);
+    NS_ReleaseOnMainThread(mAppURI.forget(), true);
+    NS_ReleaseOnMainThread(mListener.forget(), true);
+    NS_ReleaseOnMainThread(mTabChild.forget(), true);
   }
 
   if (mNSPRFileDesc) {
@@ -210,14 +186,10 @@ RemoteOpenFileChild::AsyncRemoteFileOpen(int32_t aFlags,
 
   mTabChild = static_cast<TabChild*>(aTabChild);
 
-  if (MissingRequiredTabChild(mTabChild, "remoteopenfile")) {
-    return NS_ERROR_ILLEGAL_VALUE;
-  }
-
 #if defined(XP_WIN) || defined(MOZ_WIDGET_COCOA)
   // Windows/OSX desktop builds skip remoting, and just open file in child
   // process when asked for NSPR handle
-  nsRefPtr<CallsListenerInNewEvent> runnable =
+  RefPtr<CallsListenerInNewEvent> runnable =
     new CallsListenerInNewEvent(aListener, NS_OK);
   runnable->Dispatch();
 
@@ -229,6 +201,8 @@ RemoteOpenFileChild::AsyncRemoteFileOpen(int32_t aFlags,
     MOZ_CRASH("Couldn't get path from file!");
   }
 
+  mListener = aListener;
+
   if (mTabChild) {
     if (mTabChild->GetCachedFileDescriptor(path, this)) {
       // The file descriptor was found in the cache and OnCachedFileDescriptor()
@@ -248,7 +222,6 @@ RemoteOpenFileChild::AsyncRemoteFileOpen(int32_t aFlags,
   // The chrome process now has a logical ref to us until it calls Send__delete.
   AddIPDLReference();
 
-  mListener = aListener;
   mAsyncOpenCalled = true;
   return NS_OK;
 #endif
@@ -296,7 +269,7 @@ RemoteOpenFileChild::HandleFileDescriptorAndNotifyListener(
     // descriptor callback or through the normal messaging mechanism). Close the
     // file descriptor if it is valid.
     if (aFD.IsValid()) {
-      nsRefPtr<CloseFileRunnable> runnable = new CloseFileRunnable(aFD);
+      RefPtr<CloseFileRunnable> runnable = new CloseFileRunnable(aFD);
       runnable->Dispatch();
     }
     return;
@@ -304,7 +277,7 @@ RemoteOpenFileChild::HandleFileDescriptorAndNotifyListener(
 
   MOZ_ASSERT(!mNSPRFileDesc);
 
-  nsRefPtr<TabChild> tabChild;
+  RefPtr<TabChild> tabChild;
   mTabChild.swap(tabChild);
 
   // If RemoteOpenFile reply (Recv__delete__) for app's application.zip comes
@@ -338,7 +311,7 @@ RemoteOpenFileChild::NotifyListener(nsresult aResult)
   mListener->OnRemoteFileOpenComplete(aResult);
   mListener = nullptr;     // release ref to listener
 
-  nsRefPtr<nsJARProtocolHandler> handler(gJarHandler);
+  RefPtr<nsJARProtocolHandler> handler(gJarHandler);
   NS_WARN_IF_FALSE(handler, "nsJARProtocolHandler is already gone!");
 
   if (handler) {
