@@ -4,7 +4,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-
 #include "PackagedAppService.h"
 #include "nsICacheStorage.h"
 #include "LoadContextInfo.h"
@@ -13,15 +12,28 @@
 #include "nsIMultiPartChannel.h"
 #include "../../cache2/CacheFileUtils.h"
 #include "nsStreamUtils.h"
-#include "mozilla/Logging.h"
 #include "mozilla/DebugOnly.h"
+#include "nsIHttpHeaderVisitor.h"
+#include "mozilla/LoadContext.h"
+#include "nsIInstallPackagedWebapp.h"
+#include "mozilla/Logging.h"
+#include "nsHttpResponseHead.h"
+#include "nsICachingChannel.h"
+#include "nsStringStream.h"
+#include "nsIOutputStream.h"
+#include "nsIInputStream.h"
+#include "nsNetUtil.h"
+#include "nsICacheEntryOpenCallback.h"
+#include "nsIURL.h"
+#include "nsContentUtils.h"
+#include "nsIStreamConverterService.h"
+#include "nsMimeTypes.h"
 
 namespace mozilla {
 namespace net {
 
 static PackagedAppService *gPackagedAppService = nullptr;
-
-static PRLogModuleInfo *gPASLog = nullptr;
+static LazyLogModule gPASLog("PackagedAppService");
 #undef LOG
 #define LOG(args) MOZ_LOG(gPASLog, mozilla::LogLevel::Debug, args)
 
@@ -57,7 +69,7 @@ PackagedAppService::CacheEntryWriter::Create(nsIURI *aURI,
                                              nsICacheStorage *aStorage,
                                              CacheEntryWriter **aResult)
 {
-  nsRefPtr<CacheEntryWriter> writer = new CacheEntryWriter();
+  RefPtr<CacheEntryWriter> writer = new CacheEntryWriter();
   nsresult rv = aStorage->OpenTruncate(aURI, EmptyCString(),
                                        getter_AddRefs(writer->mEntry));
   if (NS_FAILED(rv)) {
@@ -89,9 +101,155 @@ PackagedAppService::CacheEntryWriter::CopySecurityInfo(nsIChannel *aChannel)
   return NS_OK;
 }
 
+namespace { // anon
+
+class HeaderCopier final : public nsIHttpHeaderVisitor
+{
+public:
+  NS_DECL_ISUPPORTS
+  NS_DECL_NSIHTTPHEADERVISITOR
+
+  explicit HeaderCopier(nsHttpResponseHead* aHead)
+    : mHead(aHead)
+  {
+  }
+
+private:
+  ~HeaderCopier() {}
+  bool ShouldCopy(const nsACString& aHeader) const;
+
+  nsHttpResponseHead* mHead;
+};
+
+NS_IMPL_ISUPPORTS(HeaderCopier, nsIHttpHeaderVisitor)
+
+NS_IMETHODIMP
+HeaderCopier::VisitHeader(const nsACString& aHeader, const nsACString& aValue)
+{
+  if (!ShouldCopy(aHeader)) {
+    return NS_OK;
+  }
+
+  return mHead->SetHeader(nsHttp::ResolveAtom(aHeader), aValue);
+}
+
+bool
+HeaderCopier::ShouldCopy(const nsACString &aHeader) const
+{
+  nsHttpAtom header = nsHttp::ResolveAtom(aHeader);
+
+  // Don't overwrite the existing headers.
+  if (mHead->HasHeader(header)) {
+    return false;
+  }
+
+  // A black list of headers we shouldn't copy.
+  static const nsHttpAtom kHeadersCopyBlacklist[] = {
+    nsHttp::Authentication,
+    nsHttp::Cache_Control,
+    nsHttp::Connection,
+    nsHttp::Content_Disposition,
+    nsHttp::Content_Encoding,
+    nsHttp::Content_Language,
+    nsHttp::Content_Length,
+    nsHttp::Content_Location,
+    nsHttp::Content_MD5,
+    nsHttp::Content_Range,
+    nsHttp::Content_Type,
+    nsHttp::ETag,
+    nsHttp::Last_Modified,
+    nsHttp::Proxy_Authenticate,
+    nsHttp::Proxy_Connection,
+    nsHttp::Set_Cookie,
+    nsHttp::Set_Cookie2,
+    nsHttp::TE,
+    nsHttp::Trailer,
+    nsHttp::Transfer_Encoding,
+    nsHttp::Vary,
+    nsHttp::WWW_Authenticate,
+  };
+
+  // Loop through the black list to check if we should copy this header.
+  for (uint32_t i = 0; i < mozilla::ArrayLength(kHeadersCopyBlacklist); i++) {
+    if (header == kHeadersCopyBlacklist[i]) {
+      return false;
+    }
+  }
+
+  return true;
+}
+
+// Helper function to get the package cache entry from the request. The request
+// could be from multipart channel or the package channel.
+static already_AddRefed<nsICacheEntry>
+GetPackageCacheEntry(nsIRequest *aRequest)
+{
+  nsCOMPtr<nsIChannel> baseChannel;
+
+  nsCOMPtr<nsIMultiPartChannel> multiChannel(do_QueryInterface(aRequest));
+  if (multiChannel) {
+    // If it's a request from multipart channel, get the base channel from it.
+    multiChannel->GetBaseChannel(getter_AddRefs(baseChannel));
+  } else {
+    // Otherwise, the request is from the package channel.
+    baseChannel = do_QueryInterface(aRequest);
+  }
+
+  if (!baseChannel) {
+    return nullptr;
+  }
+
+  nsCOMPtr<nsICachingChannel> cachingChannel = do_QueryInterface(baseChannel);
+  if (!cachingChannel) {
+    return nullptr;
+  }
+
+  nsCOMPtr<nsISupports> cacheToken;
+  cachingChannel->GetCacheToken(getter_AddRefs(cacheToken));
+  if (!cacheToken) {
+    return nullptr;
+  }
+
+  nsCOMPtr<nsICacheEntry> entry(do_QueryInterface(cacheToken));
+
+  return entry.forget();
+}
+
+// Create nsIInputStream based on the given string which doesn't have to
+// be null-terminated. Note that the string data is shared.
+static already_AddRefed<nsIInputStream>
+CreateSharedStringStream(const char* aData, uint32_t aCount)
+{
+  nsresult rv;
+  nsCOMPtr<nsIStringInputStream> stream =
+    do_CreateInstance("@mozilla.org/io/string-input-stream;1", &rv);
+  NS_ENSURE_SUCCESS(rv, nullptr);
+
+  rv = stream->ShareData((char*)aData, aCount);
+  NS_ENSURE_SUCCESS(rv, nullptr);
+
+  return stream.forget();
+}
+
+// Get the original HTTP response header from the request.
+static bool
+GetOriginalResponseHeader(nsIRequest* aRequest, nsACString& aHeader)
+{
+  nsCOMPtr<nsIMultiPartChannel> multiPartChannel(do_QueryInterface(aRequest));
+  if (!multiPartChannel) {
+    return false;
+  }
+
+  multiPartChannel->GetOriginalResponseHeader(aHeader);
+
+  return true;
+}
+
+} // anon
+
 /* static */ nsresult
 PackagedAppService::CacheEntryWriter::CopyHeadersFromChannel(nsIChannel *aChannel,
-                                                  nsHttpResponseHead *aHead)
+                                                             nsHttpResponseHead *aHead)
 {
   if (!aChannel || !aHead) {
     return NS_ERROR_INVALID_ARG;
@@ -102,25 +260,17 @@ PackagedAppService::CacheEntryWriter::CopyHeadersFromChannel(nsIChannel *aChanne
     return NS_ERROR_FAILURE;
   }
 
-  nsAutoCString value;
-  httpChan->GetResponseHeader(NS_LITERAL_CSTRING("Content-Security-Policy"), value);
-  aHead->SetHeader(nsHttp::ResolveAtom("Content-Security-Policy"), value);
-
-  return NS_OK;
+  RefPtr<HeaderCopier> headerCopier = new HeaderCopier(aHead);
+  return httpChan->VisitResponseHeaders(headerCopier);
 }
 
 NS_METHOD
-PackagedAppService::CacheEntryWriter::ConsumeData(nsIInputStream *aStream,
-                                                  void *aClosure,
-                                                  const char *aFromRawSegment,
-                                                  uint32_t aToOffset,
+PackagedAppService::CacheEntryWriter::ConsumeData(const char *aBuf,
                                                   uint32_t aCount,
                                                   uint32_t *aWriteCount)
 {
-  MOZ_ASSERT(aClosure, "The closure must not be null");
-  CacheEntryWriter *self = static_cast<CacheEntryWriter*>(aClosure);
-  MOZ_ASSERT(self->mOutputStream, "The stream should not be null");
-  return self->mOutputStream->Write(aFromRawSegment, aCount, aWriteCount);
+  MOZ_ASSERT(mOutputStream, "The stream should not be null");
+  return mOutputStream->Write(aBuf, aCount, aWriteCount);
 }
 
 NS_IMETHODIMP
@@ -196,12 +346,8 @@ PackagedAppService::CacheEntryWriter::OnDataAvailable(nsIRequest *aRequest,
                                                       uint64_t aOffset,
                                                       uint32_t aCount)
 {
-  if (!aInputStream) {
-    return NS_ERROR_INVALID_ARG;
-  }
-  // Calls ConsumeData to read the data into the cache entry
-  uint32_t n;
-  return aInputStream->ReadSegments(ConsumeData, this, aCount, &n);
+  MOZ_ASSERT_UNREACHABLE("This function should never ever be called");
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -247,11 +393,14 @@ PackagedAppService::PackagedAppChannelListener::OnDataAvailable(nsIRequest *aReq
 
 ////////////////////////////////////////////////////////////////////////////////
 
-NS_IMPL_ISUPPORTS(PackagedAppService::PackagedAppDownloader, nsIStreamListener)
+NS_IMPL_ISUPPORTS(PackagedAppService::PackagedAppDownloader,
+                  nsIStreamListener,
+                  nsIPackagedAppVerifierListener)
 
 nsresult
 PackagedAppService::PackagedAppDownloader::Init(nsILoadContextInfo* aInfo,
-                                                const nsCString& aKey)
+                                                const nsCString& aKey,
+                                                const nsACString& aPackageOrigin)
 {
   nsresult rv;
   nsCOMPtr<nsICacheStorageService> cacheStorageService =
@@ -267,9 +416,31 @@ PackagedAppService::PackagedAppDownloader::Init(nsILoadContextInfo* aInfo,
   }
 
   mPackageKey = aKey;
+  mPackageOrigin = aPackageOrigin;
+  mProcessingFirstRequest = true;
+
   return NS_OK;
 }
 
+void
+PackagedAppService::PackagedAppDownloader::EnsureVerifier(nsIRequest *aRequest)
+{
+  if (mVerifier) {
+    return;
+  }
+
+  LOG(("Creating PackagedAppVerifier."));
+
+  nsCOMPtr<nsIMultiPartChannel> multiChannel(do_QueryInterface(aRequest));
+  nsCString signature = GetSignatureFromChannel(multiChannel);
+  nsCOMPtr<nsICacheEntry> packageCacheEntry = GetPackageCacheEntry(aRequest);
+
+  mVerifier = new PackagedAppVerifier(this,
+                                      mPackageOrigin,
+                                      signature,
+                                      packageCacheEntry);
+}
+
 NS_IMETHODIMP
 PackagedAppService::PackagedAppDownloader::OnStartRequest(nsIRequest *aRequest,
                                                           nsISupports *aContext)
@@ -295,7 +466,24 @@ PackagedAppService::PackagedAppDownloader::OnStartRequest(nsIRequest *aRequest,
   MOZ_ASSERT(mWriter);
   rv = mWriter->OnStartRequest(aRequest, aContext);
   NS_WARN_IF(NS_FAILED(rv));
-  return NS_OK;
+
+  EnsureVerifier(aRequest);
+
+  if (!mVerifier->WouldVerify()) {
+    // It means there's no signature or the signed app is disabled.
+    return NS_OK;
+  }
+
+  mVerifier->OnStartRequest(nullptr, uri);
+
+  // Since the header is considered as a part of the streaming data,
+  // we need to feed the header as data to the verifier.
+  nsCString header;
+  if (!GetOriginalResponseHeader(aRequest, header)) {
+    return NS_ERROR_FAILURE;
+  }
+  nsCOMPtr<nsIInputStream> stream = CreateSharedStringStream(header.get(), header.Length());
+  return mVerifier->OnDataAvailable(nullptr, nullptr, stream, 0, header.Length());
 }
 
 nsresult
@@ -334,8 +522,15 @@ PackagedAppService::PackagedAppDownloader::GetSubresourceURI(nsIRequest * aReque
 
   path += PACKAGED_APP_TOKEN;
 
-  // TODO: make sure the path is normalized
-  if (StringBeginsWith(contentLocation, NS_LITERAL_CSTRING("/"))) {
+  {
+    // We use this temp URI to generate a path that is relative
+    // to the package URI and not to the root of the domain.
+    nsCOMPtr<nsIURI> tempURI;
+    NS_NewURI(getter_AddRefs(tempURI), "http://temp-domain.local/");
+    tempURI->SetPath(contentLocation);
+    // The path is now normalized.
+    tempURI->GetPath(contentLocation);
+    // Remove the leading slash.
     contentLocation = Substring(contentLocation, 1);
   }
 
@@ -356,6 +551,61 @@ PackagedAppService::PackagedAppDownloader::GetSubresourceURI(nsIRequest * aReque
   return NS_OK;
 }
 
+void
+PackagedAppService::PackagedAppDownloader::OnError(EErrorType aError)
+{
+  // TODO: Handler verification error properly.
+  LOG(("PackagedAppDownloader::OnError > %d", aError));
+
+  FinalizeDownload(NS_ERROR_SIGNED_APP_MANIFEST_INVALID);
+}
+
+void
+PackagedAppService::PackagedAppDownloader::FinalizeDownload(nsresult aStatusCode)
+{
+  // If this is the last part of the package, it means the requested resources
+  // have not been found in the package so we return an appropriate error.
+  // If the package response comes from the cache, we want to preserve the
+  // statusCode, so ClearCallbacks looks for the resource in the cache, instead
+  // of returning NS_ERROR_FILE_NOT_FOUND.
+  if (NS_SUCCEEDED(aStatusCode) && !mIsFromCache) {
+    aStatusCode = NS_ERROR_FILE_NOT_FOUND;
+  }
+
+  RefPtr<PackagedAppDownloader> kungFuDeathGrip(this);
+  // NotifyPackageDownloaded removes the ref from the array. Keep a temp ref
+  if (gPackagedAppService) {
+    gPackagedAppService->NotifyPackageDownloaded(mPackageKey);
+  }
+  ClearCallbacks(aStatusCode);
+
+  // Explicity remove the downloader from the verifier. The downloader
+  // will die after exiting this function but the verifier may still be
+  // alive for a while since some resources maybe being verified.
+  if (mVerifier) {
+    mVerifier->ClearListener();
+  }
+}
+
+nsCString
+PackagedAppService::PackagedAppDownloader::GetSignatureFromChannel(nsIMultiPartChannel* aMulitChannel)
+{
+  if (mIsFromCache) {
+    // We don't need the signature if the resource is loaded from cache.
+    return EmptyCString();
+  }
+
+  if (!aMulitChannel) {
+    LOG(("The package is either not loaded from cache or malformed."));
+    return EmptyCString();
+  }
+
+  nsCString packageHeader;
+  aMulitChannel->GetPreamble(packageHeader);
+
+  return packageHeader;
+}
+
 NS_IMETHODIMP
 PackagedAppService::PackagedAppDownloader::OnStopRequest(nsIRequest *aRequest,
                                                          nsISupports *aContext,
@@ -367,55 +617,108 @@ PackagedAppService::PackagedAppDownloader::OnStopRequest(nsIRequest *aRequest,
   LOG(("[%p] PackagedAppDownloader::OnStopRequest > status:%X multiChannel:%p\n",
        this, aStatusCode, multiChannel.get()));
 
-  // The request is normally a multiPartChannel. If it isn't, it generally means
-  // an error has occurred in nsMultiMixedConv.
-  // If an error occurred in OnStartRequest, mWriter could be null.
-  if (multiChannel && mWriter) {
-    mWriter->OnStopRequest(aRequest, aContext, aStatusCode);
-
-    nsCOMPtr<nsIURI> uri;
-    rv = GetSubresourceURI(aRequest, getter_AddRefs(uri));
-    if (NS_WARN_IF(NS_FAILED(rv))) {
-      return NS_OK;
-    }
-
-    nsCOMPtr<nsICacheEntry> entry;
-    mWriter->mEntry.swap(entry);
-
-    // We don't need the writer anymore - this will close its stream
-    mWriter = nullptr;
-    CallCallbacks(uri, entry, aStatusCode);
-  }
+  mProcessingFirstRequest = false;
 
   // lastPart will be true if this is the last part in the package,
   // or if aRequest isn't a multipart channel
   bool lastPart = true;
   if (multiChannel) {
-    rv = multiChannel->GetIsLastPart(&lastPart);
-    if (NS_SUCCEEDED(rv) && !lastPart) {
-      // If this isn't the last part, we don't do the cleanup yet
-      return NS_OK;
+    multiChannel->GetIsLastPart(&lastPart);
+  }
+
+  // The request is normally a multiPartChannel. If it isn't, it generally means
+  // an error has occurred in nsMultiMixedConv.
+  // If an error occurred in OnStartRequest, mWriter could be null.
+  if (!multiChannel || !mWriter) {
+    LOG(("Either the package was loaded from cache or malformed"));
+    if (lastPart) {
+      // Chances to get here:
+      //   1) Very likely the package has been cached or
+      //   2) Less likely the package is malformed.
+      if (!mVerifier || !mVerifier->WouldVerify()) {
+        FinalizeDownload(aStatusCode);
+      } else {
+        // We've got a broken last part and some resources might be still
+        // in the verification queue. Send a dummy ResourceCacheInfo to the
+        // verifier so this broken resource will be processed in the correct
+        // order.
+        mVerifier->SetHasBrokenLastPart(aStatusCode);
+      }
     }
+    return NS_OK;
   }
 
-  // If this is the last part of the package, it means the requested resources
-  // have not been found in the package so we return an appropriate error.
-  // If the package response comes from the cache, we want to preserve the
-  // statusCode, so ClearCallbacks looks for the resource in the cache, instead
-  // of returning NS_ERROR_FILE_NOT_FOUND.
-  if (NS_SUCCEEDED(aStatusCode) && lastPart && !mIsFromCache) {
-    aStatusCode = NS_ERROR_FILE_NOT_FOUND;
+  LOG(("We are going to finish the resource and process it in the verifier."));
+
+  // We've got a resource downloaded. Finalize this resource cache and delegate to
+  // PackagedAppVerifier rather than serving this resource right away.
+  mWriter->OnStopRequest(aRequest, aContext, aStatusCode);
+
+  nsCOMPtr<nsIURI> uri;
+  rv = GetSubresourceURI(aRequest, getter_AddRefs(uri));
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return NS_OK;
   }
 
-  nsRefPtr<PackagedAppDownloader> kungFuDeathGrip(this);
-  // NotifyPackageDownloaded removes the ref from the array. Keep a temp ref
-  if (gPackagedAppService) {
-    gPackagedAppService->NotifyPackageDownloaded(mPackageKey);
+  nsCOMPtr<nsICacheEntry> entry;
+  mWriter->mEntry.swap(entry);
+
+  // We don't need the writer anymore - this will close its stream
+  mWriter = nullptr;
+
+  // The downloader only needs to focus on PackagedAppVerifierListener callback.
+  // The PackagedAppVerifier would handle the manifest/resource verification.
+  RefPtr<ResourceCacheInfo> info =
+    new ResourceCacheInfo(uri, entry, aStatusCode, lastPart);
+
+  if (!mVerifier->WouldVerify()) {
+    // No manifest at all. Everything is simply a resource.
+    OnResourceVerified(info, true);
+    return NS_OK;
   }
-  ClearCallbacks(aStatusCode);
+
+  mVerifier->OnStopRequest(nullptr, info, aStatusCode);
+
   return NS_OK;
 }
 
+NS_METHOD
+PackagedAppService::PackagedAppDownloader::ConsumeData(nsIInputStream *aStream,
+                                                       void *aClosure,
+                                                       const char *aFromRawSegment,
+                                                       uint32_t aToOffset,
+                                                       uint32_t aCount,
+                                                       uint32_t *aWriteCount)
+{
+  MOZ_ASSERT(aClosure, "The closure must not be null");
+
+  if (!aStream) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  PackagedAppDownloader *self = static_cast<PackagedAppDownloader*>(aClosure);
+
+  if (!self->mWriter) {
+    *aWriteCount = aCount;
+    return NS_OK;
+  }
+
+  self->mWriter->ConsumeData(aFromRawSegment, aCount, aWriteCount);
+
+  if (!self->mVerifier->WouldVerify()) {
+    // No signature or signed app support is disabled.
+    return NS_OK;
+  }
+
+  if (self->mProcessingFirstRequest) {
+    // mProcessingFirstRequest will be set to false on the first OnStopRequest.
+    self->mManifestContent.Append(aFromRawSegment, aCount);
+  }
+
+  nsCOMPtr<nsIInputStream> stream = CreateSharedStringStream(aFromRawSegment, aCount);
+  return self->mVerifier->OnDataAvailable(nullptr, nullptr, stream, 0, aCount);
+}
+
 NS_IMETHODIMP
 PackagedAppService::PackagedAppDownloader::OnDataAvailable(nsIRequest *aRequest,
                                                            nsISupports *aContext,
@@ -423,12 +726,8 @@ PackagedAppService::PackagedAppDownloader::OnDataAvailable(nsIRequest *aRequest,
                                                            uint64_t aOffset,
                                                            uint32_t aCount)
 {
-  if (!mWriter) {
-    uint32_t n;
-    return aInputStream->ReadSegments(NS_DiscardSegment, nullptr, aCount, &n);
-  }
-  return mWriter->OnDataAvailable(aRequest, aContext, aInputStream, aOffset,
-                                  aCount);
+  uint32_t n;
+  return aInputStream->ReadSegments(ConsumeData, this, aCount, &n);
 }
 
 nsresult
@@ -437,7 +736,7 @@ PackagedAppService::PackagedAppDownloader::AddCallback(nsIURI *aURI,
 {
   MOZ_RELEASE_ASSERT(NS_IsMainThread(), "mCallbacks hashtable is not thread safe");
   nsAutoCString spec;
-  aURI->GetAsciiSpec(spec);
+  aURI->GetSpecIgnoringRef(spec);
 
   LogURI("PackagedAppDownloader::AddCallback", this, aURI);
   LOG(("[%p]    > callback: %p\n", this, aCallback));
@@ -450,6 +749,7 @@ PackagedAppService::PackagedAppDownloader::AddCallback(nsIURI *aURI,
       // need to wait for it to be inserted in the cache and we can serve it
       // right now, directly.  See also the CallCallbacks method bellow.
       LOG(("[%p]    > already downloaded\n", this));
+
       mCacheStorage->AsyncOpenURI(aURI, EmptyCString(),
                                   nsICacheStorage::OPEN_READONLY, aCallback);
     } else {
@@ -466,6 +766,7 @@ PackagedAppService::PackagedAppDownloader::AddCallback(nsIURI *aURI,
     newArray->AppendObject(aCallback);
     mCallbacks.Put(spec, newArray);
   }
+
   return NS_OK;
 }
 
@@ -486,6 +787,7 @@ PackagedAppService::PackagedAppDownloader::CallCallbacks(nsIURI *aURI,
 
   nsCOMArray<nsICacheEntryOpenCallback>* array = mCallbacks.Get(spec);
   if (array) {
+    uint32_t callbacksNum = array->Length();
     // Call all the callbacks for this URI
     for (uint32_t i = 0; i < array->Length(); ++i) {
       nsCOMPtr<nsICacheEntryOpenCallback> callback(array->ObjectAt(i));
@@ -497,7 +799,7 @@ PackagedAppService::PackagedAppDownloader::CallCallbacks(nsIURI *aURI,
     // An empty array means that the resource was already downloaded, and a
     // new call to AddCallback can simply return it from the cache.
     array->Clear();
-    LOG(("[%p]    > called callbacks\n", this));
+    LOG(("[%p]    > called callbacks (%d)\n", this, callbacksNum));
   } else {
     // There were no listeners waiting for this resource, but we insert a new
     // empty array into the hashtable so if any new callbacks are added while
@@ -519,6 +821,8 @@ PackagedAppService::PackagedAppDownloader::ClearCallbacks(nsresult aResult)
   LOG(("[%p] PackagedAppService::PackagedAppDownloader::ClearCallbacks > packageKey:%s status:%X\n",
        this, mPackageKey.get(), aResult));
 
+  // Clear the registered callbacks which are not called at all. If the package is already
+  // in the cache, the requested resource will be called back here.
   for (auto iter = mCallbacks.Iter(); !iter.Done(); iter.Next()) {
     const nsACString& key = iter.Key();
     const nsCOMArray<nsICacheEntryOpenCallback>* callbackArray = iter.UserData();
@@ -554,12 +858,136 @@ PackagedAppService::PackagedAppDownloader::ClearCallbacks(nsresult aResult)
   return NS_OK;
 }
 
+static bool
+AddPackageIdToOrigin(nsACString& aOrigin, const nsACString& aPackageId)
+{
+  nsAutoCString originNoSuffix;
+  mozilla::NeckoOriginAttributes attrs;
+  if (!attrs.PopulateFromOrigin(aOrigin, originNoSuffix)) {
+    return false;
+  }
+
+  attrs.mSignedPkg = NS_ConvertUTF8toUTF16(aPackageId);
+  nsAutoCString suffixWithPackageId;
+  attrs.CreateSuffix(suffixWithPackageId);
+  aOrigin = originNoSuffix + suffixWithPackageId;
+  return true;
+}
+
+void
+PackagedAppService::PackagedAppDownloader::InstallSignedPackagedApp(const ResourceCacheInfo* aInfo)
+{
+  // TODO: Bug 1178533 to register permissions, system messages etc on navigation to
+  //       signed packages.
+  LOG(("Install this packaged app."));
+  bool isSuccess = false;
+
+  nsCOMPtr<nsIInstallPackagedWebapp> installer =
+    do_GetService("@mozilla.org/newapps/installpackagedwebapp;1");
+
+  if (!installer) {
+    LOG(("InstallSignedPackagedApp: fail to get InstallPackagedWebapp service"));
+    return OnError(ERROR_GET_INSTALLER_FAILED);
+  }
+
+  nsCString manifestURL;
+  aInfo->mURI->GetAsciiSpec(manifestURL);
+
+
+  nsCString originWithPackageId = mPackageOrigin;
+  if (!AddPackageIdToOrigin(originWithPackageId, mVerifier->GetPackageIdentifier())) {
+    NS_WARNING("mPackageOrigin is malformed.");
+  }
+
+  installer->InstallPackagedWebapp(mManifestContent.get(),
+                                   originWithPackageId.get(),
+                                   manifestURL.get(),
+                                   &isSuccess);
+  if (!isSuccess) {
+    LOG(("InstallSignedPackagedApp: failed to install permissions"));
+    return OnError(ERROR_INSTALL_RESOURCE_FAILED);
+  }
+
+  LOG(("InstallSignedPackagedApp: success."));
+}
+
+//------------------------------------------------------------------
+// nsIPackagedAppVerifierListener
+//------------------------------------------------------------------
+NS_IMETHODIMP
+PackagedAppService::PackagedAppDownloader::OnVerified(bool aIsManifest,
+                                                      nsIURI* aUri,
+                                                      nsICacheEntry* aCacheEntry,
+                                                      nsresult aStatusCode,
+                                                      bool aIsLastPart,
+                                                      bool aVerificationSuccess)
+{
+  if (!aUri) {
+    NS_WARNING("We've got a broken last part.");
+    FinalizeDownload(aStatusCode);
+    return NS_OK;
+  }
+
+  RefPtr<ResourceCacheInfo> info =
+    new ResourceCacheInfo(aUri, aCacheEntry, aStatusCode, aIsLastPart);
+
+  aIsManifest ? OnManifestVerified(info, aVerificationSuccess)
+              : OnResourceVerified(info, aVerificationSuccess);
+
+  return NS_OK;
+}
+
+void
+PackagedAppService::PackagedAppDownloader::OnManifestVerified(const ResourceCacheInfo* aInfo,
+                                                              bool aSuccess)
+{
+  if (!aSuccess) {
+    // The signature is found but not verified.
+    return OnError(ERROR_MANIFEST_VERIFIED_FAILED);
+  }
+
+  // TODO: If we disallow the request for the manifest file, do NOT callback here.
+  CallCallbacks(aInfo->mURI, aInfo->mCacheEntry, aInfo->mStatusCode);
+
+  if (aInfo->mIsLastPart) {
+    NS_WARNING("This package has manifest only.");
+    FinalizeDownload(aInfo->mStatusCode);
+    return;
+  }
+
+  bool isPackagedSigned;
+  mVerifier->GetIsPackageSigned(&isPackagedSigned);
+  if (!isPackagedSigned) {
+    // A verified but unsigned manifest means this package has no signature.
+    LOG(("No signature in the package. Just run normally."));
+    return;
+  }
+
+  InstallSignedPackagedApp(aInfo);
+}
+
+void
+PackagedAppService::PackagedAppDownloader::OnResourceVerified(const ResourceCacheInfo* aInfo,
+                                                              bool aSuccess)
+{
+  if (!aSuccess) {
+    return OnError(ERROR_RESOURCE_VERIFIED_FAILED);
+  }
+
+  // Serve this resource to all listeners.
+  CallCallbacks(aInfo->mURI, aInfo->mCacheEntry, aInfo->mStatusCode);
+
+  if (aInfo->mIsLastPart) {
+    LOG(("This is the last part. FinalizeDownload (%d)", aInfo->mStatusCode));
+    FinalizeDownload(aInfo->mStatusCode);
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 
 PackagedAppService::PackagedAppService()
 {
   gPackagedAppService = this;
-  gPASLog = PR_NewLogModule("PackagedAppService");
   LOG(("[%p] Created PackagedAppService\n", this));
 }
 
@@ -605,29 +1033,63 @@ PackagedAppService::GetPackageURI(nsIURI *aURI, nsIURI **aPackageURI)
 }
 
 NS_IMETHODIMP
-PackagedAppService::RequestURI(nsIURI *aURI,
-                               nsILoadContextInfo *aInfo,
-                               nsICacheEntryOpenCallback *aCallback)
+PackagedAppService::GetResource(nsIChannel *aChannel,
+                                nsICacheEntryOpenCallback *aCallback)
 {
-  // Check arguments are not null
-  if (!aURI || !aCallback || !aInfo) {
+  MOZ_RELEASE_ASSERT(NS_IsMainThread(), "mDownloadingPackages hashtable is not thread safe");
+  LOG(("[%p] PackagedAppService::GetResource(aChannel: %p, aCallback: %p)\n",
+       this, aChannel, aCallback));
+
+  if (!aChannel || !aCallback) {
     return NS_ERROR_INVALID_ARG;
   }
 
-
   nsresult rv;
-  LogURI("PackagedAppService::RequestURI", this, aURI, aInfo);
+  nsIScriptSecurityManager *securityManager =
+    nsContentUtils::GetSecurityManager();
+  if (!securityManager) {
+    LOG(("[%p]    > No securityManager\n", this));
+    return NS_ERROR_UNEXPECTED;
+  }
+  nsCOMPtr<nsIPrincipal> principal;
+  rv = securityManager->GetChannelURIPrincipal(aChannel, getter_AddRefs(principal));
+  if (NS_FAILED(rv) || !principal) {
+    LOG(("[%p]    > Error getting principal rv=%X principal=%p\n",
+         this, rv, principal.get()));
+    return NS_FAILED(rv) ? rv : NS_ERROR_NULL_POINTER;
+  }
 
-  MOZ_RELEASE_ASSERT(NS_IsMainThread(), "mDownloadingPackages hashtable is not thread safe");
+  nsCOMPtr<nsILoadContextInfo> loadContextInfo = GetLoadContextInfo(aChannel);
+  if (!loadContextInfo) {
+    LOG(("[%p]    > Channel has no loadContextInfo\n", this));
+    return NS_ERROR_NULL_POINTER;
+  }
 
+  nsLoadFlags loadFlags = 0;
+  rv = aChannel->GetLoadFlags(&loadFlags);
+  if (NS_FAILED(rv)) {
+    LOG(("[%p]    > Error calling GetLoadFlags rv=%X\n", this, rv));
+    return rv;
+  }
+
+  nsCOMPtr<nsILoadInfo> loadInfo = aChannel->GetLoadInfo();
+
+  nsCOMPtr<nsIURI> uri;
+  rv = aChannel->GetURI(getter_AddRefs(uri));
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    LOG(("[%p]    > Error calling GetURI rv=%X\n", this, rv));
+    return rv;
+  }
+
+  LogURI("PackagedAppService::GetResource", this, uri, loadContextInfo);
   nsCOMPtr<nsIURI> packageURI;
-  rv = GetPackageURI(aURI, getter_AddRefs(packageURI));
+  rv = GetPackageURI(uri, getter_AddRefs(packageURI));
   if (NS_FAILED(rv)) {
     return rv;
   }
 
   nsAutoCString key;
-  CacheFileUtils::AppendKeyPrefix(aInfo, key);
+  CacheFileUtils::AppendKeyPrefix(loadContextInfo, key);
 
   {
     nsAutoCString spec;
@@ -636,29 +1098,21 @@ PackagedAppService::RequestURI(nsIURI *aURI,
     key += spec;
   }
 
-  nsRefPtr<PackagedAppDownloader> downloader;
+  RefPtr<PackagedAppDownloader> downloader;
   if (mDownloadingPackages.Get(key, getter_AddRefs(downloader))) {
     // We have determined that the file is not in the cache.
     // If we find that the package that the file belongs to is currently being
     // downloaded, we will add the callback to the package's queue, and it will
     // be called once the file is processed and saved in the cache.
-
-    downloader->AddCallback(aURI, aCallback);
+    downloader->AddCallback(uri, aCallback);
     return NS_OK;
   }
 
-  // We need to set this flag, because the package metadata
-  // needs to have a separate entry for anonymous channels.
-  uint32_t extra_flags = 0;
-  if (aInfo->IsAnonymous()) {
-    extra_flags = nsIRequest::LOAD_ANONYMOUS;
-  }
-
   nsCOMPtr<nsIChannel> channel;
-  rv = NS_NewChannel(
-    getter_AddRefs(channel), packageURI, nsContentUtils::GetSystemPrincipal(),
-    nsILoadInfo::SEC_NORMAL, nsIContentPolicy::TYPE_OTHER, nullptr, nullptr,
-    nsIRequest::LOAD_NORMAL | extra_flags);
+  rv = NS_NewChannelInternal(
+    getter_AddRefs(channel), packageURI,
+    loadInfo,
+    nullptr, nullptr, loadFlags);
 
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
@@ -673,12 +1127,14 @@ PackagedAppService::RequestURI(nsIURI *aURI,
   }
 
   downloader = new PackagedAppDownloader();
-  rv = downloader->Init(aInfo, key);
+  nsCString packageOrigin;
+  principal->GetOrigin(packageOrigin);
+  rv = downloader->Init(loadContextInfo, key, packageOrigin);
   if (NS_WARN_IF(NS_FAILED(rv))) {
     return rv;
   }
 
-  downloader->AddCallback(aURI, aCallback);
+  downloader->AddCallback(uri, aCallback);
 
   nsCOMPtr<nsIStreamConverterService> streamconv =
     do_GetService("@mozilla.org/streamConverters;1", &rv);
@@ -696,9 +1152,19 @@ PackagedAppService::RequestURI(nsIURI *aURI,
   // Add the package to the hashtable.
   mDownloadingPackages.Put(key, downloader);
 
-  nsRefPtr<PackagedAppChannelListener> listener =
+  RefPtr<PackagedAppChannelListener> listener =
     new PackagedAppChannelListener(downloader, mimeConverter);
 
+  nsCOMPtr<nsIInterfaceRequestor> loadContext;
+  aChannel->GetNotificationCallbacks(getter_AddRefs(loadContext));
+  if (loadContext) {
+    channel->SetNotificationCallbacks(loadContext);
+  }
+
+  if (loadInfo && loadInfo->GetEnforceSecurity()) {
+    return channel->AsyncOpen2(listener);
+  }
+
   return channel->AsyncOpen(listener, nullptr);
 }
 
