@@ -17,6 +17,9 @@
 #include <algorithm>
 #include "nsContentSecurityManager.h"
 #include "nsHttp.h"
+#include "nsNetUtil.h"
+#include "nsIURI.h"
+#include "nsHttpHeaderArray.h"
 
 //
 // Helper function for determining the length of data bytes up to
@@ -418,7 +421,8 @@ nsPartChannel::VisitResponseHeaders(nsIHttpHeaderVisitor *visitor)
 {
     if (!mResponseHead)
         return NS_ERROR_NOT_AVAILABLE;
-    return mResponseHead->Headers().VisitHeaders(visitor);
+    return mResponseHead->VisitHeaders(visitor,
+        mozilla::net::nsHttpHeaderArray::eFilterResponse);
 }
 
 //
@@ -459,6 +463,31 @@ nsPartChannel::GetBaseChannel(nsIChannel ** aReturn)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPartChannel::GetPreamble(nsACString & aPreamble)
+{
+    aPreamble = mPreamble;
+    return NS_OK;
+}
+
+void
+nsPartChannel::SetPreamble(const nsACString& aPreamble)
+{
+    mPreamble = aPreamble;
+}
+
+NS_IMETHODIMP
+nsPartChannel::GetOriginalResponseHeader(nsACString & aOriginalResponseHeader)
+{
+    aOriginalResponseHeader = mOriginalResponseHeader;
+    return NS_OK;
+}
+
+void
+nsPartChannel::SetOriginalResponseHeader(const nsACString& aOriginalResponseHeader)
+{
+    mOriginalResponseHeader = aOriginalResponseHeader;
+}
 
 // nsISupports implementation
 NS_IMPL_ISUPPORTS(nsMultiMixedConv,
@@ -522,6 +551,49 @@ private:
   char *mBuffer;
 };
 
+char*
+nsMultiMixedConv::ProbeToken(char* aBuffer, uint32_t& aTokenLen)
+{
+    // To sign a packaged web app in the new security model, we need
+    // to add the signature to the package header. The header is the
+    // data before the first token and the header format is
+    //
+    // [field-name]: [field-value] CR LF
+    //
+    // So the package may look like:
+    //
+    // manifest-signature: MRjdkly...
+    // --gc0pJq0M:08jU534c0p
+    // Content-Location: /someapp.webmanifest
+    // Content-Type: application/manifest
+    //
+    // {
+    // "name": "My App",
+    // "description":"A great app!"
+    // ...
+    //
+    //
+    // We search for the first '\r\n--' and assign the subsquent chars
+    // to the token until another '\r\n'. '--' will be included in the
+    // token we probed. If the second '\r\n' is not found, we still treat
+    // the token is not found and more data will be requested.
+
+    char* posCRLFDashDash = PL_strstr(aBuffer, "\r\n--");
+    if (!posCRLFDashDash) {
+        return nullptr;
+    }
+
+    char* tokenStart = posCRLFDashDash + 2; // Skip "\r\n".
+    char* tokenEnd = PL_strstr(tokenStart, "\r\n");
+    if (!tokenEnd) {
+        return nullptr;
+    }
+
+    aTokenLen = tokenEnd - tokenStart;
+
+    return tokenStart;
+}
+
 // nsIStreamListener implementation
 NS_IMETHODIMP
 nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
@@ -584,24 +656,40 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         } else if (mPackagedApp) {
             // We need to check the line starts with --
             if (!StringBeginsWith(firstBuffer, NS_LITERAL_CSTRING("--"))) {
-                return NS_ERROR_FAILURE;
-            }
+                char* tokenPos = ProbeToken(buffer, mTokenLen);
+                if (!tokenPos) {
+                    // No token is found. We need more data.
+                    mFirstOnData = true;
+                } else {
+                    // Token is probed.
+                    mToken = Substring(tokenPos, mTokenLen);
+                    mPreamble = nsCString(Substring(buffer, tokenPos));
 
-            // If the boundary was set in the header,
-            // we need to check it matches with the one in the file.
-            if (mTokenLen &&
-                !StringBeginsWith(Substring(firstBuffer, 2), mToken)) {
-                return NS_ERROR_FAILURE;
-            }
+                    // Push the cursor to the token so that the while loop below will
+                    // find token from the right position.
+                    cursor = tokenPos;
 
-            // Save the token.
-            if (!mTokenLen) {
-                mToken = nsCString(Substring(firstBuffer, 2).BeginReading(),
-                                   posCR - 2);
-                mTokenLen = mToken.Length();
-            }
+                    // Update bufLen to exlude the preamble. Otherwise, the first
+                    // |SendData| would claim longer buffer length.
+                    bufLen -= mPreamble.Length();
+                }
+            } else {
+                // If the boundary was set in the header,
+                // we need to check it matches with the one in the file.
+                if (mTokenLen &&
+                    !StringBeginsWith(Substring(firstBuffer, 2), mToken)) {
+                    return NS_ERROR_FAILURE;
+                }
 
-            cursor = buffer;
+                // Save the token.
+                if (!mTokenLen) {
+                    mToken = nsCString(Substring(firstBuffer, 2).BeginReading(),
+                                       posCR - 2);
+                    mTokenLen = mToken.Length();
+                }
+
+                cursor = buffer;
+            }
         } else if (!PL_strnstr(cursor, token, mTokenLen + 2)) {
             char *newBuffer = (char *) realloc(buffer, bufLen + mTokenLen + 1);
             if (!newBuffer)
@@ -629,9 +717,15 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         // for this "part" given the previous buffer given to 
         // us in the previous OnDataAvailable callback.
         bool done = false;
+        const char* originalCursor = cursor;
         rv = ParseHeaders(channel, cursor, bufLen, &done);
         if (NS_FAILED(rv)) return rv;
 
+        // Append the content to the original header.
+        if (cursor > originalCursor) {
+            mOriginalResponseHeader.Append(originalCursor, cursor - originalCursor);
+        }
+
         if (done) {
             mProcessingHeaders = false;
             rv = SendStart(channel);
@@ -669,9 +763,16 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
             // parse headers
             mNewPart = false;
             cursor = token;
-            bool done = false; 
+            bool done = false;
+            const char* originalCursor = cursor;
             rv = ParseHeaders(channel, cursor, bufLen, &done);
             if (NS_FAILED(rv)) return rv;
+
+            // Append the content to the original header.
+            if (cursor > originalCursor) {
+                mOriginalResponseHeader.Append(originalCursor, cursor - originalCursor);
+            }
+
             if (done) {
                 rv = SendStart(channel);
                 if (NS_FAILED(rv)) return rv;
@@ -965,6 +1066,13 @@ nsMultiMixedConv::SendStart(nsIChannel *aChannel) {
     // Set up the new part channel...
     mPartChannel = newChannel;
 
+    // Pass preamble to the channel.
+    mPartChannel->SetPreamble(mPreamble);
+
+    // Pass original http header.
+    mPartChannel->SetOriginalResponseHeader(mOriginalResponseHeader);
+    mOriginalResponseHeader = EmptyCString();
+
     // We pass the headers to the nsPartChannel
     mPartChannel->SetResponseHead(mResponseHead.forget());
 
