@@ -54,8 +54,6 @@ class DebianBootstrapper(BaseBootstrapper):
         'libdbus-1-dev',
         'libdbus-glib-1-dev',
         'libgconf2-dev',
-        'libgstreamer0.10-dev',
-        'libgstreamer-plugins-base0.10-dev',
         'libgtk2.0-dev',
         'libgtk-3-dev',
         'libiw-dev',
@@ -76,7 +74,6 @@ class DebianBootstrapper(BaseBootstrapper):
     MOBILE_ANDROID_COMMON_PACKAGES = [
         'zlib1g-dev',  # mobile/android requires system zlib.
         'openjdk-7-jdk',
-        'ant',
         'wget',  # For downloading the Android SDK and NDK.
         'libncurses5:i386',  # See comments about i386 below.
         'libstdc++6:i386',
@@ -104,11 +101,17 @@ class DebianBootstrapper(BaseBootstrapper):
         self.apt_install(*self.browser_packages)
 
     def install_mobile_android_packages(self):
+        self.ensure_mobile_android_packages()
+
+    def install_mobile_android_artifact_mode_packages(self):
+        self.ensure_mobile_android_packages(artifact_mode=True)
+
+    def ensure_mobile_android_packages(self, artifact_mode=False):
         import android
 
         # Multi-part process:
         # 1. System packages.
-        # 2. Android SDK and NDK.
+        # 2. Android SDK. Android NDK only if we are not in artifact mode.
         # 3. Android packages.
 
         # 1. This is hard to believe, but the Android SDK binaries are 32-bit
@@ -123,32 +126,31 @@ class DebianBootstrapper(BaseBootstrapper):
 
         # 2. The user may have an external Android SDK (in which case we save
         # them a lengthy download), or they may have already completed the
-        # download. We unpack to ~/.mozbuild/{android-sdk-linux, android-ndk-r8e}.
+        # download. We unpack to ~/.mozbuild/{android-sdk-linux, android-ndk-r11b}.
         mozbuild_path = os.environ.get('MOZBUILD_STATE_PATH', os.path.expanduser(os.path.join('~', '.mozbuild')))
         self.sdk_path = os.environ.get('ANDROID_SDK_HOME', os.path.join(mozbuild_path, 'android-sdk-linux'))
-        self.ndk_path = os.environ.get('ANDROID_NDK_HOME', os.path.join(mozbuild_path, 'android-ndk-r8e'))
+        self.ndk_path = os.environ.get('ANDROID_NDK_HOME', os.path.join(mozbuild_path, 'android-ndk-r11b'))
         self.sdk_url = 'https://dl.google.com/android/android-sdk_r24.0.1-linux.tgz'
-        is_64bits = sys.maxsize > 2**32
-        if is_64bits:
-            self.ndk_url = 'https://dl.google.com/android/ndk/android-ndk-r8e-linux-x86_64.tar.bz2'
-        else:
-            self.ndk_url = 'https://dl.google.com/android/ndk/android-ndk-r8e-linux-x86.tar.bz2'
+        self.ndk_url = android.android_ndk_url('linux')
+
         android.ensure_android_sdk_and_ndk(path=mozbuild_path,
                                            sdk_path=self.sdk_path, sdk_url=self.sdk_url,
-                                           ndk_path=self.ndk_path, ndk_url=self.ndk_url)
+                                           ndk_path=self.ndk_path, ndk_url=self.ndk_url,
+                                           artifact_mode=artifact_mode)
 
         # 3. We expect the |android| tool to at
         # ~/.mozbuild/android-sdk-linux/tools/android.
         android_tool = os.path.join(self.sdk_path, 'tools', 'android')
         android.ensure_android_packages(android_tool=android_tool)
 
-    def suggest_mobile_android_mozconfig(self):
+    def suggest_mobile_android_mozconfig(self, artifact_mode=False):
         import android
+        android.suggest_mozconfig(sdk_path=self.sdk_path,
+                                  ndk_path=self.ndk_path,
+                                  artifact_mode=artifact_mode)
 
-        # The SDK path that mozconfig wants includes platforms/android-21.
-        sdk_path = os.path.join(self.sdk_path, 'platforms', android.ANDROID_PLATFORM)
-        android.suggest_mozconfig(sdk_path=sdk_path,
-                                  ndk_path=self.ndk_path)
+    def suggest_mobile_android_artifact_mode_mozconfig(self):
+        self.suggest_mobile_android_mozconfig(artifact_mode=True)
 
     def _update_package_manager(self):
         self.apt_update()
