@@ -7,34 +7,47 @@ from __future__ import absolute_import, unicode_literals
 import itertools
 import json
 import os
-import re
 
 import mozpack.path as mozpath
-import mozwebidlcodegen
 
-from .base import BuildBackend
+from mozbuild.backend.base import BuildBackend
 
-from ..frontend.data import (
+from mozbuild.frontend.context import (
+    Context,
+    Path,
+    RenamedSourcePath,
+    VARIABLES,
+)
+from mozbuild.frontend.data import (
+    BaseProgram,
+    ChromeManifestEntry,
     ConfigFileSubstitution,
     ExampleWebIDLInterface,
-    HeaderFileSubstitution,
     IPDLFile,
+    FinalTargetPreprocessedFiles,
+    FinalTargetFiles,
     GeneratedEventWebIDLFile,
     GeneratedWebIDLFile,
     PreprocessedTestWebIDLFile,
     PreprocessedWebIDLFile,
+    RustRlibLibrary,
+    SharedLibrary,
     TestManifest,
     TestWebIDLFile,
     UnifiedSources,
     XPIDLFile,
     WebIDLFile,
 )
+from mozbuild.jar import (
+    DeprecatedJarManifest,
+    JarManifestParser,
+)
+from mozbuild.preprocessor import Preprocessor
+from mozpack.chrome.manifest import parse_manifest_line
 
 from collections import defaultdict
 
-from ..util import (
-    group_unified_files,
-)
+from mozbuild.util import group_unified_files
 
 class XPIDLManager(object):
     """Helps manage XPCOM IDLs in the context of the build system."""
@@ -161,16 +174,14 @@ class TestManager(object):
         self.topsrcdir = mozpath.normpath(config.topsrcdir)
 
         self.tests_by_path = defaultdict(list)
+        self.installs_by_path = defaultdict(list)
+        self.deferred_installs = set()
+        self.manifest_default_support_files = {}
 
-    def add(self, t, flavor=None, topsrcdir=None):
+    def add(self, t, flavor, topsrcdir, default_supp_files):
         t = dict(t)
         t['flavor'] = flavor
 
-        if topsrcdir is None:
-            topsrcdir = self.topsrcdir
-        else:
-            topsrcdir = mozpath.normpath(topsrcdir)
-
         path = mozpath.normpath(t['path'])
         assert mozpath.basedir(path, [topsrcdir])
 
@@ -178,8 +189,37 @@ class TestManager(object):
         t['file_relpath'] = key
         t['dir_relpath'] = mozpath.dirname(key)
 
+        # Support files are propagated from the default section to individual
+        # tests by the manifest parser, but we end up storing a lot of
+        # redundant data due to the huge number of support files.
+        # So if we have support files that are the same as the manifest default
+        # we track that separately, per-manifest instead of per-test, to save
+        # space.
+        supp_files = t.get('support-files')
+        if supp_files and supp_files == default_supp_files:
+            self.manifest_default_support_files[t['manifest']] = default_supp_files
+            del t['support-files']
+
         self.tests_by_path[key].append(t)
 
+    def add_installs(self, obj, topsrcdir):
+        for src, (dest, _) in obj.installs.iteritems():
+            key = src[len(topsrcdir)+1:]
+            self.installs_by_path[key].append((src, dest))
+        for src, pat, dest in obj.pattern_installs:
+            key = mozpath.join(src[len(topsrcdir)+1:], pat)
+            self.installs_by_path[key].append((src, pat, dest))
+        for path in obj.deferred_installs:
+            self.deferred_installs.add(path[2:])
+
+
+class BinariesCollection(object):
+    """Tracks state of binaries produced by the build."""
+
+    def __init__(self):
+        self.shared_libraries = []
+        self.programs = []
+
 
 class CommonBackend(BuildBackend):
     """Holds logic common to all build backends."""
@@ -188,6 +228,7 @@ class CommonBackend(BuildBackend):
         self._idl_manager = XPIDLManager(self.environment)
         self._test_manager = TestManager(self.environment)
         self._webidls = WebIDLCollection()
+        self._binaries = BinariesCollection()
         self._configs = set()
         self._ipdl_sources = set()
 
@@ -196,64 +237,108 @@ class CommonBackend(BuildBackend):
 
         if isinstance(obj, TestManifest):
             for test in obj.tests:
-                self._test_manager.add(test, flavor=obj.flavor,
-                    topsrcdir=obj.topsrcdir)
+                self._test_manager.add(test, obj.flavor, obj.topsrcdir,
+                                       obj.default_support_files)
+            self._test_manager.add_installs(obj, obj.topsrcdir)
 
         elif isinstance(obj, XPIDLFile):
+            # TODO bug 1240134 tracks not processing XPIDL files during
+            # artifact builds.
             self._idl_manager.register_idl(obj)
 
         elif isinstance(obj, ConfigFileSubstitution):
             # Do not handle ConfigFileSubstitution for Makefiles. Leave that
             # to other
             if mozpath.basename(obj.output_path) == 'Makefile':
-                return
+                return False
             with self._get_preprocessor(obj) as pp:
                 pp.do_include(obj.input_path)
             self.backend_input_files.add(obj.input_path)
 
-        elif isinstance(obj, HeaderFileSubstitution):
-            self._create_config_header(obj)
-            self.backend_input_files.add(obj.input_path)
-
         # We should consider aggregating WebIDL types in emitter.py.
         elif isinstance(obj, WebIDLFile):
+            # WebIDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._webidls.sources.add(mozpath.join(obj.srcdir, obj.basename))
 
         elif isinstance(obj, GeneratedEventWebIDLFile):
+            # WebIDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._webidls.generated_events_sources.add(mozpath.join(
                 obj.srcdir, obj.basename))
 
         elif isinstance(obj, TestWebIDLFile):
+            # WebIDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._webidls.test_sources.add(mozpath.join(obj.srcdir,
                 obj.basename))
 
         elif isinstance(obj, PreprocessedTestWebIDLFile):
+            # WebIDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._webidls.preprocessed_test_sources.add(mozpath.join(
                 obj.srcdir, obj.basename))
 
         elif isinstance(obj, GeneratedWebIDLFile):
+            # WebIDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._webidls.generated_sources.add(mozpath.join(obj.srcdir,
                 obj.basename))
 
         elif isinstance(obj, PreprocessedWebIDLFile):
+            # WebIDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._webidls.preprocessed_sources.add(mozpath.join(
                 obj.srcdir, obj.basename))
 
         elif isinstance(obj, ExampleWebIDLInterface):
+            # WebIDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._webidls.example_interfaces.add(obj.name)
 
         elif isinstance(obj, IPDLFile):
+            # IPDL isn't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             self._ipdl_sources.add(mozpath.join(obj.srcdir, obj.basename))
 
         elif isinstance(obj, UnifiedSources):
+            # Unified sources aren't relevant to artifact builds.
+            if self.environment.is_artifact_build:
+                return True
+
             if obj.have_unified_mapping:
                 self._write_unified_files(obj.unified_source_mapping, obj.objdir)
             if hasattr(self, '_process_unified_sources'):
                 self._process_unified_sources(obj)
-        else:
-            return
 
-        obj.ack()
+        elif isinstance(obj, BaseProgram):
+            self._binaries.programs.append(obj)
+            return False
+
+        elif isinstance(obj, SharedLibrary):
+            self._binaries.shared_libraries.append(obj)
+            return False
+
+        else:
+            return False
+
+        return True
 
     def consume_finished(self):
         if len(self._idl_manager.idls):
@@ -290,10 +375,26 @@ class CommonBackend(BuildBackend):
             self.backend_input_files.add(config.source)
 
         # Write out a machine-readable file describing every test.
-        path = mozpath.join(self.environment.topobjdir, 'all-tests.json')
+        topobjdir = self.environment.topobjdir
+        with self._write_file(mozpath.join(topobjdir, 'all-tests.json')) as fh:
+            json.dump([self._test_manager.tests_by_path,
+                       self._test_manager.manifest_default_support_files], fh)
+
+        path = mozpath.join(self.environment.topobjdir, 'test-installs.json')
         with self._write_file(path) as fh:
-            s = json.dumps(self._test_manager.tests_by_path)
-            fh.write(s)
+            json.dump({k: v for k, v in self._test_manager.installs_by_path.items()
+                       if k in self._test_manager.deferred_installs},
+                      fh,
+                      sort_keys=True,
+                      indent=4)
+
+        # Write out a machine-readable file describing binaries.
+        with self._write_file(mozpath.join(topobjdir, 'binaries.json')) as fh:
+            d = {
+                'shared_libraries': [s.to_dict() for s in self._binaries.shared_libraries],
+                'programs': [p.to_dict() for p in self._binaries.programs],
+            }
+            json.dump(d, fh, sort_keys=True, indent=4)
 
     def _handle_webidl_collection(self, webidls):
         if not webidls.all_stems():
@@ -321,6 +422,8 @@ class CommonBackend(BuildBackend):
         with self._write_file(file_lists) as fh:
             json.dump(o, fh, sort_keys=True, indent=2)
 
+        import mozwebidlcodegen
+
         manager = mozwebidlcodegen.create_build_system_manager(
             self.environment.topsrcdir,
             self.environment.topobjdir,
@@ -373,39 +476,96 @@ class CommonBackend(BuildBackend):
             self._write_unified_file(unified_file, source_filenames,
                                      output_directory, poison_windows_h)
 
-    def _create_config_header(self, obj):
-        '''Creates the given config header. A config header is generated by
-        taking the corresponding source file and replacing some #define/#undef
-        occurences:
-            "#undef NAME" is turned into "#define NAME VALUE"
-            "#define NAME" is unchanged
-            "#define NAME ORIGINAL_VALUE" is turned into "#define NAME VALUE"
-            "#undef UNKNOWN_NAME" is turned into "/* #undef UNKNOWN_NAME */"
-            Whitespaces are preserved.
-        '''
-        with self._write_file(obj.output_path) as fh, \
-             open(obj.input_path, 'rU') as input:
-            r = re.compile('^\s*#\s*(?P<cmd>[a-z]+)(?:\s+(?P<name>\S+)(?:\s+(?P<value>\S+))?)?', re.U)
-            for l in input:
-                m = r.match(l)
-                if m:
-                    cmd = m.group('cmd')
-                    name = m.group('name')
-                    value = m.group('value')
-                    if name:
-                        if name in obj.config.defines:
-                            if cmd == 'define' and value:
-                                l = l[:m.start('value')] \
-                                    + str(obj.config.defines[name]) \
-                                    + l[m.end('value'):]
-                            elif cmd == 'undef':
-                                l = l[:m.start('cmd')] \
-                                    + 'define' \
-                                    + l[m.end('cmd'):m.end('name')] \
-                                    + ' ' \
-                                    + str(obj.config.defines[name]) \
-                                    + l[m.end('name'):]
-                        elif cmd == 'undef':
-                           l = '/* ' + l[:m.end('name')] + ' */' + l[m.end('name'):]
+    def _consume_jar_manifest(self, obj):
+        # Ideally, this would all be handled somehow in the emitter, but
+        # this would require all the magic surrounding l10n and addons in
+        # the recursive make backend to die, which is not going to happen
+        # any time soon enough.
+        # Notably missing:
+        # - DEFINES from config/config.mk
+        # - L10n support
+        # - The equivalent of -e when USE_EXTENSION_MANIFEST is set in
+        #   moz.build, but it doesn't matter in dist/bin.
+        pp = Preprocessor()
+        if obj.defines:
+            pp.context.update(obj.defines.defines)
+        pp.context.update(self.environment.defines)
+        pp.context.update(
+            AB_CD='en-US',
+            BUILD_FASTER=1,
+        )
+        pp.out = JarManifestParser()
+        try:
+            pp.do_include(obj.path.full_path)
+        except DeprecatedJarManifest as e:
+            raise DeprecatedJarManifest('Parsing error while processing %s: %s'
+                                        % (obj.path.full_path, e.message))
+        self.backend_input_files |= pp.includes
 
-                fh.write(l)
+        for jarinfo in pp.out:
+            jar_context = Context(
+                allowed_variables=VARIABLES, config=obj._context.config)
+            jar_context.push_source(obj._context.main_path)
+            jar_context.push_source(obj.path.full_path)
+
+            install_target = obj.install_target
+            if jarinfo.base:
+                install_target = mozpath.normpath(
+                    mozpath.join(install_target, jarinfo.base))
+            jar_context['FINAL_TARGET'] = install_target
+            if obj.defines:
+                jar_context['DEFINES'] = obj.defines.defines
+            files = jar_context['FINAL_TARGET_FILES']
+            files_pp = jar_context['FINAL_TARGET_PP_FILES']
+
+            for e in jarinfo.entries:
+                if e.is_locale:
+                    if jarinfo.relativesrcdir:
+                        src = '/%s' % jarinfo.relativesrcdir
+                    else:
+                        src = ''
+                    src = mozpath.join(src, 'en-US', e.source)
+                else:
+                    src = e.source
+
+                src = Path(jar_context, src)
+
+                if '*' not in e.source and not os.path.exists(src.full_path):
+                    if e.is_locale:
+                        raise Exception(
+                            '%s: Cannot find %s' % (obj.path, e.source))
+                    if e.source.startswith('/'):
+                        src = Path(jar_context, '!' + e.source)
+                    else:
+                        # This actually gets awkward if the jar.mn is not
+                        # in the same directory as the moz.build declaring
+                        # it, but it's how it works in the recursive make,
+                        # not that anything relies on that, but it's simpler.
+                        src = Path(obj._context, '!' + e.source)
+
+                output_basename = mozpath.basename(e.output)
+                if output_basename != src.target_basename:
+                    src = RenamedSourcePath(jar_context,
+                                            (src, output_basename))
+                path = mozpath.dirname(mozpath.join(jarinfo.name, e.output))
+
+                if e.preprocess:
+                    if '*' in e.source:
+                        raise Exception('%s: Wildcards are not supported with '
+                                        'preprocessing' % obj.path)
+                    files_pp[path] += [src]
+                else:
+                    files[path] += [src]
+
+            if files:
+                self.consume_object(FinalTargetFiles(jar_context, files))
+            if files_pp:
+                self.consume_object(
+                    FinalTargetPreprocessedFiles(jar_context, files_pp))
+
+            for m in jarinfo.chrome_manifests:
+                entry = parse_manifest_line(
+                    mozpath.dirname(jarinfo.name),
+                    m.replace('%', mozpath.basename(jarinfo.name) + '/'))
+                self.consume_object(ChromeManifestEntry(
+                    jar_context, '%s.manifest' % jarinfo.name, entry))
