@@ -200,7 +200,7 @@ FileSystemDataSource::Create(nsISupports* aOuter, const nsIID& aIID, void **aRes
 {
     NS_ENSURE_NO_AGGREGATION(aOuter);
 
-    nsRefPtr<FileSystemDataSource> self = new FileSystemDataSource();
+    RefPtr<FileSystemDataSource> self = new FileSystemDataSource();
     if (!self)
         return NS_ERROR_OUT_OF_MEMORY;
      
@@ -333,7 +333,7 @@ FileSystemDataSource::GetTarget(nsIRDFResource *source,
             url->GetValueConst(&uni);
             if (!uni)   return(NS_RDF_NO_VALUE);
             nsAutoString    urlStr;
-            urlStr.Assign(NS_LITERAL_STRING(NS_MOZICON_SCHEME).get());
+            urlStr.AssignLiteral(NS_MOZICON_SCHEME);
             urlStr.Append(uni);
 
             rv = mRDFService->GetLiteral(urlStr.get(), getter_AddRefs(url));
@@ -851,7 +851,7 @@ FileSystemDataSource::GetVolumeList(nsISimpleEnumerator** aResult)
 
     for (volNum = 0; volNum < 26; volNum++)
     {
-        swprintf( drive, L"%c:\\", volNum + (char16_t)'A');
+        swprintf_s(drive, 32, L"%c:\\", volNum + (char16_t)'A');
 
         driveType = GetDriveTypeW(drive);
         if (driveType != DRIVE_UNKNOWN && driveType != DRIVE_NO_ROOT_DIR)
@@ -1007,16 +1007,14 @@ FileSystemDataSource::GetFolderList(nsIRDFResource *source, bool allowHidden,
             fullURI.Append('/');
         }
 
-        char    *escLeafStr = nsEscape(NS_ConvertUTF16toUTF8(leafStr).get(), url_Path);
+        nsAutoCString leaf;
+        bool escaped = NS_Escape(NS_ConvertUTF16toUTF8(leafStr), leaf, url_Path);
         leafStr.Truncate();
 
-        if (!escLeafStr)
+        if (!escaped) {
             continue;
+        }
   
-        nsAutoCString           leaf(escLeafStr);
-        free(escLeafStr);
-        escLeafStr = nullptr;
-
         // using nsEscape() [above] doesn't escape slashes, so do that by hand
         int32_t         aOffset;
         while ((aOffset = leaf.FindChar('/')) >= 0)
