@@ -3,7 +3,7 @@
 
 "use strict";
 
-const Cm = Components.manager;
+var Cm = Components.manager;
 
 Cu.import("resource://gre/modules/FxAccounts.jsm");
 Cu.import("resource://gre/modules/FxAccountsCommon.js");
@@ -102,6 +102,7 @@ FxAccountsManager._fxAccounts = {
   _error: 'error',
   _assertion: 'assertion',
   _keys: 'keys',
+  _profile: 'aprofile',
   _signedInUser: null,
 
   _reset: function() {
@@ -140,6 +141,13 @@ FxAccountsManager._fxAccounts = {
     return deferred.promise;
   },
 
+  getSignedInUserProfile: function() {
+    let deferred = Promise.defer();
+    this._reject ? deferred.reject(this._error)
+                 : deferred.resolve(this._profile);
+    return deferred.promise;
+  },
+
   getKeys: function() {
     let deferred = Promise.defer();
     this._reject ? deferred.reject(this._error)
@@ -204,9 +212,10 @@ var FakeFxAccountsClient = {
     return deferred.promise;
   },
 
-  signIn: function(user, password) {
+  signIn: function(user, password, getKeys) {
     this._signInCalled = true;
     this._password = password;
+    this._keyFetchToken = getKeys ? "token" : null;
     let deferred = Promise.defer();
     this._reject ? deferred.reject()
                  : deferred.resolve({ email: user,
@@ -635,11 +644,13 @@ add_test(function(test_getAssertion_refresh_auth_no_refresh) {
 
 add_test(function(test_getAccount_existing_verified_session) {
   do_print("= getAccount, existing verified session =");
+  FxAccountsManager._activeSession = null;
   FxAccountsManager.getAccount().then(
     result => {
-      do_check_false(FxAccountsManager._fxAccounts._getSignedInUserCalled);
+      do_check_true(FxAccountsManager._fxAccounts._getSignedInUserCalled);
       do_check_eq(result.email, FxAccountsManager._user.email);
       do_check_eq(result.verified, FxAccountsManager._user.verified);
+      do_check_eq(result.profile, "aprofile");
       run_next_test();
     },
     error => {
@@ -806,6 +817,50 @@ add_test(function(test_signIn_already_signed_user) {
   );
 });
 
+add_test(function(test_signIn_getKeys_true) {
+  do_print("= signIn, getKeys true =");
+  FxAccountsManager.signOut().then(() => {
+    FxAccountsManager.signIn("user@domain.org", "password", true).then(
+      result => {
+        do_check_true(FakeFxAccountsClient._signInCalled);
+        do_check_true(FxAccountsManager._fxAccounts._getSignedInUserCalled);
+        do_check_eq(FxAccountsManager._fxAccounts._signedInUser.email, "user@domain.org");
+        do_check_eq(FakeFxAccountsClient._password, "password");
+        do_check_eq(FakeFxAccountsClient._keyFetchToken, "token");
+        do_check_eq(result.user.email, "user@domain.org");
+        FakeFxAccountsClient._reset();
+        FxAccountsManager._fxAccounts._reset();
+        run_next_test();
+      },
+      error => {
+        do_throw("Unexpected error");
+      }
+    );
+  });
+});
+
+add_test(function(test_signIn_getKeys_false) {
+  do_print("= signIn, getKeys false =");
+  FxAccountsManager.signOut().then(() => {
+    FxAccountsManager.signIn("user@domain.org", "password", false).then(
+      result => {
+        do_check_true(FakeFxAccountsClient._signInCalled);
+        do_check_true(FxAccountsManager._fxAccounts._getSignedInUserCalled);
+        do_check_eq(FxAccountsManager._fxAccounts._signedInUser.email, "user@domain.org");
+        do_check_eq(FakeFxAccountsClient._password, "password");
+        do_check_eq(FakeFxAccountsClient._keyFetchToken, null);
+        do_check_eq(result.user.email, "user@domain.org");
+        FakeFxAccountsClient._reset();
+        FxAccountsManager._fxAccounts._reset();
+        run_next_test();
+      },
+      error => {
+        do_throw("Unexpected error");
+      }
+    );
+  });
+});
+
 add_test(function(test_resendVerificationEmail_error_handling) {
   do_print("= resendVerificationEmail smoke test =");
   let user = FxAccountsManager._fxAccounts._signedInUser;
