@@ -10,12 +10,11 @@ this.EXPORTED_SYMBOLS = [
   "Store"
 ];
 
-const {classes: Cc, interfaces: Ci, results: Cr, utils: Cu} = Components;
+var {classes: Cc, interfaces: Ci, results: Cr, utils: Cu} = Components;
 
 Cu.import("resource://services-common/async.js");
 Cu.import("resource://gre/modules/Log.jsm");
 Cu.import("resource://services-common/observers.js");
-Cu.import("resource://services-common/utils.js");
 Cu.import("resource://services-sync/constants.js");
 Cu.import("resource://services-sync/identity.js");
 Cu.import("resource://services-sync/record.js");
@@ -299,17 +298,20 @@ Store.prototype = {
    */
   applyIncomingBatch: function (records) {
     let failed = [];
-    for each (let record in records) {
+    for (let record of records) {
       try {
         this.applyIncoming(record);
-      } catch (ex if (ex.code == Engine.prototype.eEngineAbortApplyIncoming)) {
-        // This kind of exception should have a 'cause' attribute, which is an
-        // originating exception.
-        // ex.cause will carry its stack with it when rethrown.
-        throw ex.cause;
-      } catch (ex if !Async.isShutdownException(ex)) {
-        this._log.warn("Failed to apply incoming record " + record.id);
-        this._log.warn("Encountered exception: " + Utils.exceptionStr(ex));
+      } catch (ex) {
+        if (ex.code == Engine.prototype.eEngineAbortApplyIncoming) {
+          // This kind of exception should have a 'cause' attribute, which is an
+          // originating exception.
+          // ex.cause will carry its stack with it when rethrown.
+          throw ex.cause;
+        }
+        if (Async.isShutdownException(ex)) {
+          throw ex;
+        }
+        this._log.warn("Failed to apply incoming record " + record.id, ex);
         this.engine._noteApplyFailure();
         failed.push(record.id);
       }
@@ -484,7 +486,11 @@ EngineManager.prototype = {
   },
 
   getAll: function () {
-    return [engine for ([name, engine] in Iterator(this._engines))];
+    let engines = [];
+    for (let [name, engine] in Iterator(this._engines)) {
+      engines.push(engine);
+    }
+    return engines;
   },
 
   /**
@@ -497,7 +503,7 @@ EngineManager.prototype = {
   },
 
   get enabledEngineNames() {
-    return [e.name for each (e in this.getEnabled())];
+    return this.getEnabled().map(e => e.name);
   },
 
   persistDeclined: function () {
@@ -574,16 +580,11 @@ EngineManager.prototype = {
         this._engines[name] = engine;
       }
     } catch (ex) {
-      this._log.error(CommonUtils.exceptionStr(ex));
-
-      let mesg = ex.message ? ex.message : ex;
       let name = engineObject || "";
       name = name.prototype || "";
       name = name.name || "";
 
-      let out = "Could not initialize engine '" + name + "': " + mesg;
-      this._log.error(out);
-
+      this._log.error(`Could not initialize engine ${name}`, ex);
       return engineObject;
     }
   },
@@ -813,7 +814,13 @@ SyncEngine.prototype = {
     return this._previousFailed;
   },
   set previousFailed(val) {
-    let cb = (error) => this._log.error(Utils.exceptionStr(error));
+    let cb = (error) => {
+      if (error) {
+        this._log.error("Failed to set previousFailed", error);
+      } else {
+        this._log.debug("Successfully wrote previousFailed.");
+      }
+    }
     // Coerce the array to a string for more efficient comparison.
     if (val + "" == this._previousFailed) {
       return;
@@ -993,11 +1000,13 @@ SyncEngine.prototype = {
       this._tracker.ignoreAll = true;
       try {
         failed = failed.concat(this._store.applyIncomingBatch(applyBatch));
-      } catch (ex if !Async.isShutdownException(ex)) {
+      } catch (ex) {
+        if (Async.isShutdownException(ex)) {
+          throw ex;
+        }
         // Catch any error that escapes from applyIncomingBatch. At present
         // those will all be abort events.
-        this._log.warn("Got exception " + Utils.exceptionStr(ex) +
-                       ", aborting processIncoming.");
+        this._log.warn("Got exception, aborting processIncoming", ex);
         aborting = ex;
       }
       this._tracker.ignoreAll = false;
@@ -1042,7 +1051,10 @@ SyncEngine.prototype = {
       try {
         try {
           item.decrypt(key);
-        } catch (ex if Utils.isHMACMismatch(ex)) {
+        } catch (ex) {
+          if (!Utils.isHMACMismatch(ex)) {
+            throw ex;
+          }
           let strategy = self.handleHMACMismatch(item, true);
           if (strategy == SyncEngine.kRecoveryStrategy.retry) {
             // You only get one retry.
@@ -1052,7 +1064,10 @@ SyncEngine.prototype = {
               key = self.service.collectionKeys.keyForCollection(self.name);
               item.decrypt(key);
               strategy = null;
-            } catch (ex if Utils.isHMACMismatch(ex)) {
+            } catch (ex) {
+              if (!Utils.isHMACMismatch(ex)) {
+                throw ex;
+              }
               strategy = self.handleHMACMismatch(item, false);
             }
           }
@@ -1065,7 +1080,7 @@ SyncEngine.prototype = {
               self._log.debug("Ignoring second retry suggestion.");
               // Fall through to error case.
             case SyncEngine.kRecoveryStrategy.error:
-              self._log.warn("Error decrypting record: " + Utils.exceptionStr(ex));
+              self._log.warn("Error decrypting record", ex);
               self._noteApplyFailure();
               failed.push(item.id);
               return;
@@ -1076,7 +1091,7 @@ SyncEngine.prototype = {
           }
         }
       } catch (ex) {
-        self._log.warn("Error decrypting record: " + Utils.exceptionStr(ex));
+        self._log.warn("Error decrypting record", ex);
         self._noteApplyFailure();
         failed.push(item.id);
         return;
@@ -1085,17 +1100,20 @@ SyncEngine.prototype = {
       let shouldApply;
       try {
         shouldApply = self._reconcile(item);
-      } catch (ex if (ex.code == Engine.prototype.eEngineAbortApplyIncoming)) {
-        self._log.warn("Reconciliation failed: aborting incoming processing.");
-        self._noteApplyFailure();
-        failed.push(item.id);
-        aborting = ex.cause;
-      } catch (ex if !Async.isShutdownException(ex)) {
-        self._log.warn("Failed to reconcile incoming record " + item.id);
-        self._log.warn("Encountered exception: " + Utils.exceptionStr(ex));
-        self._noteApplyFailure();
-        failed.push(item.id);
-        return;
+      } catch (ex) {
+        if (ex.code == Engine.prototype.eEngineAbortApplyIncoming) {
+          self._log.warn("Reconciliation failed: aborting incoming processing.");
+          self._noteApplyFailure();
+          failed.push(item.id);
+          aborting = ex.cause;
+        } else if (!Async.isShutdownException(ex)) {
+          self._log.warn("Failed to reconcile incoming record " + item.id, ex);
+          self._noteApplyFailure();
+          failed.push(item.id);
+          return;
+        } else {
+          throw ex;
+        }
       }
 
       if (shouldApply) {
@@ -1217,13 +1235,11 @@ SyncEngine.prototype = {
   },
 
   _noteApplyFailure: function () {
-    Services.telemetry.getKeyedHistogramById(
-      "WEAVE_ENGINE_APPLY_FAILURES").add(this.name);
+    // here would be a good place to record telemetry...
   },
 
   _noteApplyNewFailure: function () {
-    Services.telemetry.getKeyedHistogramById(
-      "WEAVE_ENGINE_APPLY_NEW_FAILURES").add(this.name);
+    // here would be a good place to record telemetry...
   },
 
   /**
@@ -1439,13 +1455,7 @@ SyncEngine.prototype = {
 
       // collection we'll upload
       let up = new Collection(this.engineURL, null, this.service);
-      let count = 0;
-
-      // Upload what we've got so far in the collection
-      let doUpload = Utils.bind2(this, function(desc) {
-        this._log.info("Uploading " + desc + " of " + modifiedIDs.length +
-                       " records");
-        let resp = up.post();
+      let handleResponse = resp => {
         if (!resp.success) {
           this._log.debug("Uploading records failed: " + resp);
           resp.failureCode = ENGINE_UPLOAD_FAIL;
@@ -1464,35 +1474,36 @@ SyncEngine.prototype = {
                           + failed_ids.join(", "));
 
         // Clear successfully uploaded objects.
-        for each (let id in resp.obj.success) {
+        for (let key in resp.obj.success) {
+          let id = resp.obj.success[key];
           delete this._modified[id];
         }
+      }
 
-        up.clearRecords();
-      });
+      let postQueue = up.newPostQueue(this._log, handleResponse);
 
-      for each (let id in modifiedIDs) {
+      for (let id of modifiedIDs) {
+        let out;
+        let ok = false;
         try {
-          let out = this._createRecord(id);
+          out = this._createRecord(id);
           if (this._log.level <= Log.Level.Trace)
             this._log.trace("Outgoing: " + out);
 
           out.encrypt(this.service.collectionKeys.keyForCollection(this.name));
-          up.pushData(out);
-        } catch (ex if !Async.isShutdownException(ex)) {
-          this._log.warn("Error creating record: " + Utils.exceptionStr(ex));
+          ok = true;
+        } catch (ex) {
+          if (Async.isShutdownException(ex)) {
+            throw ex;
+          }
+          this._log.warn("Error creating record", ex);
+        }
+        if (ok) {
+          postQueue.enqueue(out);
         }
-
-        // Partial upload
-        if ((++count % MAX_UPLOAD_RECORDS) == 0)
-          doUpload((count - MAX_UPLOAD_RECORDS) + " - " + count + " out");
-
         this._store._sleep(0);
       }
-
-      // Final upload
-      if (count % MAX_UPLOAD_RECORDS > 0)
-        doUpload(count >= MAX_UPLOAD_RECORDS ? "last batch" : "all");
+      postQueue.flush();
     }
   },
 
@@ -1570,8 +1581,11 @@ SyncEngine.prototype = {
     try {
       this._log.trace("Trying to decrypt a record from the server..");
       test.get();
-    } catch (ex if !Async.isShutdownException(ex)) {
-      this._log.debug("Failed test decrypt: " + Utils.exceptionStr(ex));
+    } catch (ex) {
+      if (Async.isShutdownException(ex)) {
+        throw ex;
+      }
+      this._log.debug("Failed test decrypt", ex);
     }
 
     return canDecrypt;
