@@ -23,6 +23,7 @@ class MercurialUpdater(object):
     def __init__(self, state_dir):
         self.state_dir = os.path.normpath(state_dir)
         self.vcs_tools_dir = os.path.join(self.state_dir, 'version-control-tools')
+        self.hgwatchman_dir = os.path.join(self.state_dir, 'hgwatchman')
 
     def update_all(self):
         hg = get_hg_path()
@@ -34,16 +35,41 @@ class MercurialUpdater(object):
             self.vcs_tools_dir,
             'default',
             'Ensuring version-control-tools is up to date...')
+        self.update_mercurial_repo(
+            hg,
+            'https://bitbucket.org/facebook/hgwatchman',
+            self.hgwatchman_dir,
+            '5ca0f920df7ec8a93d322f06d554f778184cdcd1',
+            'Ensuring hgwatchman is up to date...')
         if repo_existed:
             print(FINISHED)
         return 0
 
     def update_mercurial_repo(self, hg, url, dest, branch, msg):
+        # Disable common extensions whose older versions may cause `hg`
+        # invocations to abort.
+        disable_exts = [
+            'bzexport',
+            'bzpost',
+            'firefoxtree',
+            'hgwatchman',
+            'mozext',
+            'mqext',
+            'qimportbz',
+            'push-to-try',
+            'reviewboard',
+        ]
+        global_args = []
+        for ext in disable_exts:
+            global_args.extend(['--config', 'extensions.%s=!' % ext])
+
         # We always pass the host fingerprints that we "know" to be canonical
         # because the existing config may have outdated fingerprints and this
         # may cause Mercurial to abort.
         return self._update_repo(hg, url, dest, branch, msg,
-            update_mercurial_repo, hostfingerprints=HOST_FINGERPRINTS)
+                                 update_mercurial_repo,
+                                 hostfingerprints=HOST_FINGERPRINTS,
+                                 global_args=global_args)
 
     def _update_repo(self, binary, url, dest, branch, msg, fn, *args, **kwargs):
         print('=' * 80)
