@@ -8,6 +8,7 @@ import difflib
 import errno
 import os
 import shutil
+import ssl
 import stat
 import sys
 import subprocess
@@ -41,12 +42,18 @@ are up to date and you won't have to do anything.
 To begin, press the enter/return key.
 '''.strip()
 
-OLDEST_NON_LEGACY_VERSION = LooseVersion('3.2.4')
+# This should match MODERN_MERCURIAL_VERSION in
+# python/mozboot/mozboot/base.py.
+OLDEST_NON_LEGACY_VERSION = LooseVersion('3.7.3')
 LEGACY_MERCURIAL = '''
 You are running an out of date Mercurial client (%s).
 
 For a faster and better Mercurial experience, we HIGHLY recommend you
 upgrade.
+
+Legacy versions of Mercurial have known security vulnerabilities. Failure
+to upgrade may leave you exposed. You are highly encouraged to upgrade
+in case you aren't running a patched version.
 '''.strip()
 
 MISSING_USERNAME = '''
@@ -64,18 +71,6 @@ not configured to produce patches in that format.
 (Relevant config options: diff.git, diff.showfunc, diff.unified)
 '''.strip()
 
-MQ_INFO = '''
-The mq extension manages patches as separate files. It provides an
-alternative to the recommended bookmark-based development workflow.
-
-If you are a newcomer to Mercurial or are coming from Git, it is
-recommended to avoid mq.
-
-(Relevant config option: extensions.mq)
-
-Would you like to activate the mq extension
-'''.strip()
-
 BZEXPORT_INFO = '''
 If you plan on uploading patches to Mozilla, there is an extension called
 bzexport that makes it easy to upload patches from the command line via the
@@ -87,39 +82,12 @@ https://hg.mozilla.org/hgcustom/version-control-tools/file/default/hgext/bzexpor
 Would you like to activate bzexport
 '''.strip()
 
-MQEXT_INFO = '''
-The mqext extension adds a number of features, including automatically committing
-changes to your mq patch queue. More info is available at
-https://hg.mozilla.org/hgcustom/version-control-tools/file/default/hgext/mqext/README.txt
-
-(Relevant config option: extensions.mqext)
-
-Would you like to activate mqext
-'''.strip()
-
-QIMPORTBZ_INFO = '''
-The qimportbz extension
-(https://hg.mozilla.org/hgcustom/version-control-tools/file/default/hgext/qimportbz/README) makes it possible to
-import patches from Bugzilla using a friendly bz:// URL handler. e.g.
-|hg qimport bz://123456|.
-
-(Relevant config option: extensions.qimportbz)
-
-Would you like to activate qimportbz
-'''.strip()
-
-QNEWCURRENTUSER_INFO = '''
-The mercurial queues command |hg qnew|, which creates new patches in your patch
-queue does not set patch author information by default. Author information
-should be included when uploading for review.
-'''.strip()
-
 FINISHED = '''
 Your Mercurial should now be properly configured and recommended extensions
 should be up to date!
 '''.strip()
 
-REVIEWBOARD_MINIMUM_VERSION = LooseVersion('3.1')
+REVIEWBOARD_MINIMUM_VERSION = LooseVersion('3.5')
 
 REVIEWBOARD_INCOMPATIBLE = '''
 Your Mercurial is too old to use the reviewboard extension, which is necessary
@@ -161,7 +129,7 @@ if you have enabled 2 Factor Authentication in Bugzilla.
 All consumers formerly looking at these options should support API Keys.
 '''.lstrip()
 
-BZPOST_MINIMUM_VERSION = LooseVersion('3.1')
+BZPOST_MINIMUM_VERSION = LooseVersion('3.5')
 
 BZPOST_INFO = '''
 The bzpost extension automatically records the URLs of pushed commits to
@@ -172,7 +140,7 @@ referenced Bugzilla bugs after push.
 Would you like to activate bzpost
 '''.strip()
 
-FIREFOXTREE_MINIMUM_VERSION = LooseVersion('3.1')
+FIREFOXTREE_MINIMUM_VERSION = LooseVersion('3.5')
 
 FIREFOXTREE_INFO = '''
 The firefoxtree extension makes interacting with the multiple Firefox
@@ -198,7 +166,7 @@ b) perform head/bookmark-based development (as opposed to mq)
 Would you like to activate firefoxtree
 '''.strip()
 
-PUSHTOTRY_MINIMUM_VERSION = LooseVersion('3.3')
+PUSHTOTRY_MINIMUM_VERSION = LooseVersion('3.5')
 
 PUSHTOTRY_INFO = '''
 The push-to-try extension generates a temporary commit with a given
@@ -211,6 +179,15 @@ they can push to try without depending on mq or other workarounds.
 Would you like to activate push-to-try
 '''.strip()
 
+CLONEBUNDLES_INFO = '''
+Mercurial 3.6 and hg.mozilla.org support transparently cloning from a CDN,
+making clones faster and more reliable.
+
+(Relevant config option: experimental.clonebundles)
+
+Would you like to activate this feature and have faster clones
+'''.strip()
+
 BUNDLECLONE_MINIMUM_VERSION = LooseVersion('3.1')
 
 BUNDLECLONE_INFO = '''
@@ -223,6 +200,47 @@ We highly recommend you activate this extension.
 Would you like to activate bundleclone
 '''.strip()
 
+WIP_INFO = '''
+It is common to want a quick view of changesets that are in progress.
+
+The ``hg wip`` command provides should a view.
+
+Example Usage:
+
+  $ hg wip
+  o   4084:fcfa34d0387b dminor  @
+  |  mozreview: use repository name when displaying treeherder results (bug 1230548) r=mcote
+  | @   4083:786baf6d476a gps
+  | |  mozreview: create child review requests from batch API
+  | o   4082:3f100fa4a94f gps
+  | |  mozreview: copy more read-only processing code; r?smacleod
+  | o   4081:939417680cbe gps
+  |/   mozreview: add web API to submit an entire series of commits (bug 1229468); r?smacleod
+
+(Not shown are the colors that help denote the state each changeset
+is in.)
+
+(Relevant config options: alias.wip, revsetalias.wip, templates.wip)
+
+Would you like to install the `hg wip` alias?
+'''.strip()
+
+HGWATCHMAN_MINIMUM_VERSION = LooseVersion('3.5.2')
+FSMONITOR_MINIMUM_VERSION = LooseVersion('3.8')
+
+FSMONITOR_INFO = '''
+Filesystem monitor integrates the watchman filesystem watching
+tool with Mercurial. Commands like `hg status`, `hg diff`, and
+`hg commit` that need to examine filesystem state can query watchman
+and obtain filesystem state nearly instantaneously. The result is much
+faster command execution.
+
+When enabled, the filesystem monitor tool will launch a background
+watchman file watching daemon for accessed Mercurial repositories.
+
+Would you like to enable filesystem monitor tool
+'''.strip()
+
 FILE_PERMISSIONS_WARNING = '''
 Your hgrc file is currently readable by others.
 
@@ -337,7 +355,34 @@ class MercurialSetupWizard(object):
             'rewriting via the "histedit" command (similar to '
             '`git rebase -i`)')
 
-        self.prompt_native_extension(c, 'mq', MQ_INFO)
+        # hgwatchman is provided by MozillaBuild and we don't yet support
+        # Linux/BSD.
+        # Note that the hgwatchman project has been renamed to fsmonitor and has
+        # been moved into Mercurial core, as of version 3.8. So, if your Mercurial
+        # version is modern enough (>=3.8), you could set fsmonitor in hgrc file
+        # directly.
+        if ('hgwatchman' not in c.extensions
+            and sys.platform.startswith('darwin')
+            and hg_version >= HGWATCHMAN_MINIMUM_VERSION
+            and self._prompt_yn(FSMONITOR_INFO)):
+            if (hg_version >= FSMONITOR_MINIMUM_VERSION):
+                c.activate_extension('fsmonitor')
+            else:
+                # Unlike other extensions, we need to run an installer
+                # to compile a Python C extension.
+                try:
+                    subprocess.check_output(
+                        ['make', 'local'],
+                        cwd=self.updater.hgwatchman_dir,
+                        stderr=subprocess.STDOUT)
+
+                    ext_path = os.path.join(self.updater.hgwatchman_dir,
+                                            'hgwatchman')
+                    if self.can_use_extension(c, 'hgwatchman', ext_path):
+                        c.activate_extension('hgwatchman', ext_path)
+                except subprocess.CalledProcessError as e:
+                    print('Error compiling hgwatchman; will not install hgwatchman')
+                    print(e.output)
 
         if 'reviewboard' not in c.extensions:
             if hg_version < REVIEWBOARD_MINIMUM_VERSION:
@@ -359,30 +404,21 @@ class MercurialSetupWizard(object):
         if hg_version >= FIREFOXTREE_MINIMUM_VERSION:
             self.prompt_external_extension(c, 'firefoxtree', FIREFOXTREE_INFO)
 
-        if hg_version >= BUNDLECLONE_MINIMUM_VERSION:
+        # Functionality from bundleclone is experimental in Mercurial 3.6.
+        # There was a bug in 3.6, so look for 3.6.1.
+        if hg_version >= LooseVersion('3.6.1'):
+            if not c.have_clonebundles() and self._prompt_yn(CLONEBUNDLES_INFO):
+                c.activate_clonebundles()
+                print('Enabled the clonebundles feature.\n')
+        elif hg_version >= BUNDLECLONE_MINIMUM_VERSION:
             self.prompt_external_extension(c, 'bundleclone', BUNDLECLONE_INFO)
 
         if hg_version >= PUSHTOTRY_MINIMUM_VERSION:
             self.prompt_external_extension(c, 'push-to-try', PUSHTOTRY_INFO)
 
-        if 'mq' in c.extensions:
-            self.prompt_external_extension(c, 'mqext', MQEXT_INFO)
-
-            if 'mqext' in c.extensions and not c.have_mqext_autocommit_mq():
-                if self._prompt_yn('Would you like to configure mqext to '
-                    'automatically commit changes as you modify patches'):
-                    c.ensure_mqext_autocommit_mq()
-                    print('Configured mqext to auto-commit.\n')
-
-            self.prompt_external_extension(c, 'qimportbz', QIMPORTBZ_INFO)
-
-            if not c.have_qnew_currentuser_default():
-                print(QNEWCURRENTUSER_INFO)
-                if self._prompt_yn('Would you like qnew to set patch author by '
-                                   'default'):
-                    c.ensure_qnew_currentuser_default()
-                    print('Configured qnew to set patch author by default.')
-                    print('')
+        if not c.have_wip():
+            if self._prompt_yn(WIP_INFO):
+                c.install_wip_alias()
 
         if 'reviewboard' in c.extensions or 'bzpost' in c.extensions:
             bzuser, bzpass, bzuserid, bzcookie, bzapikey = c.get_bugzilla_credentials()
@@ -422,7 +458,20 @@ class MercurialSetupWizard(object):
                     print('Cleaning up old repository: %s' % path)
                     shutil.rmtree(path)
 
-        c.add_mozilla_host_fingerprints()
+        # Python + Mercurial didn't have terrific TLS handling until Python
+        # 2.7.9 and Mercurial 3.4. For this reason, it was recommended to pin
+        # certificates in Mercurial config files. In modern versions of
+        # Mercurial, the system CA store is used and old, legacy TLS protocols
+        # are disabled. The default connection/security setting should
+        # be sufficient and pinning certificates is no longer needed.
+        have_modern_ssl = hasattr(ssl, 'SSLContext')
+        if hg_version < LooseVersion('3.4') or not have_modern_ssl:
+            c.add_mozilla_host_fingerprints()
+
+        # We always update fingerprints if they are present. We /could/ offer to
+        # remove fingerprints if running modern Python and Mercurial. But that
+        # just adds more UI complexity and isn't worth it.
+        c.update_mozilla_host_fingerprints()
 
         # References to multiple version-control-tools checkouts can confuse
         # version-control-tools, since various Mercurial extensions resolve
@@ -523,7 +572,10 @@ class MercurialSetupWizard(object):
                 print('')
                 return
         if not path:
-            path = os.path.join(self.vcs_tools_dir, 'hgext', name)
+            # We replace the user's home directory with ~ so the
+            # config file doesn't depend on the path to the home
+            # directory
+            path = os.path.join(self.vcs_tools_dir.replace(os.path.expanduser('~'), '~'), 'hgext', name)
         c.activate_extension(name, path)
         print('Activated %s extension.\n' % name)
 
